@@ -52,12 +52,23 @@
 /* Does the file command work well with - option for stdin? */
 #undef FILE_STDIN
 
+/* Does the grep command work well with - option for stdin? */
+#undef GREP_STDIN
+
 /* Is the program using the GPM library? */
 #undef HAVE_LIBGPM
 
 /* Is the program using the distributed slang library? */
 #undef HAVE_SLANG
 
+/* Is the program using a system-installed slang library? */
+#undef HAVE_SYSTEM_SLANG
+
+/* Define if the slang.h header file is inside a directory slang
+** in the standard directories
+*/
+#undef SLANG_H_INSIDE_SLANG_DIR
+
 /* Does the program have subshell support? */
 #undef HAVE_SUBSHELL_SUPPORT
 
@@ -140,7 +151,7 @@
 /* Defined if your CPP understands ## macro token pasting method */
 #undef HAVE_PORTABLE_TOKEN_PASTING
 
-/* Define if you want to use SCO pty opening routines */
+/* Define if you want to turn on SCO-specific code */
 #undef SCO_FLAVOR
 
 /* Define if your system has struct linger */
@@ -179,6 +190,15 @@
 /* Define if your system has socketpair */
 #undef HAVE_SOCKETPAIR
 
+/* Do we have posix signals? */
+#undef HAVE_SIGACTION
+#undef HAVE_SIGPROCMASK
+#undef HAVE_SIGEMPTYSET
+#undef HAVE_SIGADDSET
+
+/* Version of ncurses */
+#undef NCURSES_970530
+
 @BOTTOM@
 
 #ifdef HAVE_LIBPT
@@ -189,6 +209,12 @@
 #    include <xvmain.h>
 #endif
 
+#if defined(HAVE_SIGADDSET) && defined(HAVE_SIGEMPTYSET)
+# if defined(HAVE_SIGACTION) && defined(HAVE_SIGPROCMASK)
+#  define SLANG_POSIX_SIGNALS
+# endif
+#endif
+
 #ifdef __os2__
 #    define OS2_NT 1
 #    define S_ISFIFO(x) 0
@@ -197,3 +223,11 @@
 #ifdef _OS_NT
 #    define OS2_NT 1
 #endif
+
+#ifndef OS2_NT
+/* some Unices do not define this, and slang requires it: */
+#ifndef unix
+#    define unix
+#endif
+#endif
+
