@@ -1,8 +1,8 @@
 /* editor low level data handling and cursor fundamentals.
 
-   Copyright (C) 1996 the Free Software Foundation
+   Copyright (C) 1996, 1997 the Free Software Foundation
    
-   Authors: 1996 Paul Sheer
+   Authors: 1996, 1997 Paul Sheer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -61,13 +61,16 @@ fin.
 
 */
 
-int (*user_defined_key_function) (unsigned int state, unsigned int keycode) = 0;
+#ifndef MIDNIGHT
+int (*user_defined_key_function) (unsigned int state, unsigned int keycode, KeySym keysym) = 0;
 
-void edit_set_user_key_function (int (*user_def_key_func) (unsigned int, unsigned int))
+void edit_set_user_key_function (int (*user_def_key_func) (unsigned int, unsigned int, KeySym))
 {
     user_defined_key_function = user_def_key_func;
 }
 
+#endif
+
 /*
    returns a byte from any location in the file.
    Returns '\n' if out of bounds.
@@ -104,7 +107,12 @@ int init_dynamic_edit_buffers (WEdit * edit, const char *filename, const char *t
     }
 
     if (filename)
+#if defined(OS2_NT) && defined(MIDNIGHT)
+	/* .ado: because of ^M */
+	if ((file = open ((char *) filename, O_RDONLY | O_TEXT)) == -1) {
+#else
 	if ((file = open ((char *) filename, O_RDONLY)) == -1) {
+#endif
 	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error (" Fail trying to open edit file for reading. "));
 	    return 1;
 	}
@@ -145,37 +153,48 @@ int load_edit_file (WEdit * edit, const char *filename, const char *text)
 	filename = NULL;
     } else {
 	if ((file = open ((char *) filename, O_RDONLY)) < 0) {
-	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error (" Fail trying to open edit file for reading. "));
+	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error (catstrs (" Fail trying to open the file, ", filename, ", for reading ", 0)));
 	    return 1;
 	}
 	if (stat ((char *) filename, &s) < 0) {
 	    close (file);
-	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error(" Cannot get info on this file. "));
+	    Cerrordialogue (0, 0, 0, " Error ", get_sys_error (catstrs (" Cannot get size/permissions info on the file, ", filename, " ", 0)));
 	    return 1;
 	}
 	if (S_ISDIR (s.st_mode) || S_ISSOCK (s.st_mode)
 	    || S_ISFIFO (s.st_mode)) {
 	    close (file);
-	    Cerrordialogue (0, 0, 0, " Error ", " This is not a text file ");
+	    Cerrordialogue (0, 0, 0, " Error ", catstrs (" ", filename, " is not an ordinary file ", 0));
 	    return 1;
 	}
 	if (s.st_size >= SIZE_LIMIT) {
 	    close (file);
-	    Cerrordialogue (0, 0, 0, " Error ", " File is to large. Increase edit.h:MAXBUF and recompile ");
+	    Cerrordialogue (0, 0, 0, " Error ", catstrs (" The file, ", filename, ", is too large. \n Increase edit.h:MAXBUF and recompile the editor ", 0));
 	    return 1;
 	}
 	close (file);
 	edit->last_byte = s.st_size;
+	edit->stat = s;
     }
 
     return init_dynamic_edit_buffers (edit, filename, text);
 }
 
+#ifdef MIDNIGHT
+#define space_width 1
+#else
+static int space_width = 0;
+extern unsigned char per_char[256];
+#endif
 
 /* fills in the edit struct. returns 0 on fail. Pass e as NULL for this function to do an malloc for you */
 WEdit *edit_init (WEdit * e, int lines, int columns, const char *filename, const char *text, const char *dir)
 {
     char *f;
+#ifndef MIDNIGHT
+    if (!space_width)
+	space_width = per_char[' '];
+#endif
     if (!e)
 	e = malloc (sizeof (WEdit));
     if (!e) {
@@ -185,6 +204,10 @@ WEdit *edit_init (WEdit * e, int lines, int columns, const char *filename, const
     memset (&(e->from_here), 0, (unsigned long) &(e->to_here) - (unsigned long) &(e->from_here));
     e->num_widget_lines = lines;
     e->num_widget_columns = columns;
+    e->stat.st_mode = S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH;
+    e->stat.st_uid = getuid ();
+    e->stat.st_gid = getgid ();
+    e->bracket = -1;
     if (!dir)
 	dir = "";
     f = (char *) filename;
@@ -740,7 +763,11 @@ long edit_eol (WEdit * edit, long current)
 {
     if (current < edit->last_byte) {
 	for (;; current++)
+#if 0
 	    if (current == edit->last_byte || edit_get_byte (edit, current) == '\n')
+#else
+	    if (edit_get_byte (edit, current) == '\n')
+#endif
 		break;
     } else
 	return edit->last_byte;
@@ -752,7 +779,11 @@ long edit_bol (WEdit * edit, long current)
 {
     if (current > 0) {
 	for (;; current--)
+#if 0
 	    if (current == 0 || edit_get_byte (edit, current - 1) == '\n')
+#else
+	    if (edit_get_byte (edit, current - 1) == '\n')
+#endif
 		break;
     } else
 	return 0;
@@ -807,9 +838,10 @@ long edit_move_backward (WEdit * edit, long current, int lines)
     return current;
 }
 
+#ifdef MIDNIGHT
 /* If cols is zero this returns the count of columns from current to upto. */
 /* If upto is zero returns index of cols across from current. */
-float edit_move_forward3 (WEdit * edit, long current, int cols, long upto)
+long edit_move_forward3 (WEdit * edit, long current, int cols, long upto)
 {
     long p, q;
     int col = 0;
@@ -829,12 +861,9 @@ float edit_move_forward3 (WEdit * edit, long current, int cols, long upto)
 		return p - 1;
 	}
 	c = edit_get_byte (edit, p);
-
-#ifdef MIDNIGHT
 	if (c == '\r')
 	    continue;
 	else
-#endif
 	if (c == '\t')
 	    col += TAB_SIZE - col % TAB_SIZE;
 	else
@@ -849,7 +878,7 @@ float edit_move_forward3 (WEdit * edit, long current, int cols, long upto)
     }
     return (float) col;
 }
-
+#endif
 
 /* returns the current column position of the cursor */
 int edit_get_col (WEdit * edit)
@@ -880,7 +909,7 @@ void edit_scroll_upward (WEdit * edit, unsigned long i)
 	edit->start_line -= i;
 	edit->start_display = edit_move_backward (edit, edit->start_display, i);
 	edit->force |= REDRAW_PAGE;
-	edit->force &= (0xff - REDRAW_CHAR_ONLY);
+	edit->force &= (0xfff - REDRAW_CHAR_ONLY);
     }
     update_curs_row(edit);
 }
@@ -897,20 +926,18 @@ void edit_scroll_downward (WEdit * edit, int i)
 	edit->start_line += i;
 	edit->start_display = edit_move_forward (edit, edit->start_display, i, 0);
 	edit->force |= REDRAW_PAGE;
-	edit->force &= (0xff - REDRAW_CHAR_ONLY);
+	edit->force &= (0xfff - REDRAW_CHAR_ONLY);
     }
     update_curs_row(edit);
 }
 
-
 void edit_scroll_right (WEdit * edit, int i)
 {
     edit->force |= REDRAW_PAGE;
-    edit->force &= (0xff - REDRAW_CHAR_ONLY);
+    edit->force &= (0xfff - REDRAW_CHAR_ONLY);
     edit->start_col -= i;
 }
 
-
 void edit_scroll_left (WEdit * edit, int i)
 {
     if (edit->start_col) {
@@ -918,7 +945,7 @@ void edit_scroll_left (WEdit * edit, int i)
 	if (edit->start_col > 0)
 	    edit->start_col = 0;
 	edit->force |= REDRAW_PAGE;
-	edit->force &= (0xff - REDRAW_CHAR_ONLY);
+	edit->force &= (0xfff - REDRAW_CHAR_ONLY);
     }
 }
 
@@ -933,16 +960,21 @@ static int is_in_indent (WEdit *edit)
     return 1;
 }
 
-static void edit_move_to_prev_col (WEdit *edit, long p)
-{
-    edit_cursor_move (edit, (int) edit_move_forward3 (edit, p, edit->prev_col, 0) - edit->curs1);
+static int left_of_four_spaces (WEdit *edit);
 
-    if (is_in_indent(edit) && option_fake_half_tabs) {
+static void edit_move_to_prev_col (WEdit * edit, long p)
+{
+    edit_cursor_move (edit, edit_move_forward3 (edit, p, edit->prev_col, 0) - edit->curs1);
+
+    if (is_in_indent (edit) && option_fake_half_tabs) {
 	update_curs_col (edit);
-	if (edit->curs_col % HALF_TAB_SIZE) {
-	    edit->curs_col -= (edit->curs_col % HALF_TAB_SIZE);
+	if (edit->curs_col % (HALF_TAB_SIZE * space_width)) {
+	    int q = edit->curs_col;
+	    edit->curs_col -= (edit->curs_col % (HALF_TAB_SIZE * space_width));
 	    p = edit_bol (edit, edit->curs1);
-	    edit_cursor_move (edit, (int) edit_move_forward3 (edit, p, edit->curs_col, 0) - edit->curs1);
+	    edit_cursor_move (edit, edit_move_forward3 (edit, p, edit->curs_col, 0) - edit->curs1);
+	    if (!left_of_four_spaces (edit))
+		edit_cursor_move (edit, edit_move_forward3 (edit, p, q, 0) - edit->curs1);
 	}
     }
 }
@@ -970,6 +1002,61 @@ static void edit_move_up (WEdit * edit, unsigned long i, int scroll)
     }
 }
 
+int is_blank (WEdit * edit, long offset)
+{
+    long s, f;
+    int c;
+    s = edit_bol (edit, offset);
+    f = edit_eol (edit, offset) - 1;
+    while (s <= f) {
+	c = edit_get_byte (edit, s++);
+	if ((c > ' ' && c <= '~') || c >= 160)	/* non-printables on a line are considered "blank" */
+	    return 0;
+    }
+    return 1;
+}
+
+int line_is_blank (WEdit * edit, long line)
+{
+    long p;
+    if (line < edit->curs_line)
+	p = edit_move_backward (edit, edit->curs1, edit->curs_line - line);
+    else if (line > edit->curs_line)
+	p = edit_move_forward (edit, edit->curs1, line - edit->curs_line, 0);
+    else
+	p = edit->curs1;
+    return is_blank (edit, p);
+}
+
+/* moves up until a blank line is reached, or until just 
+   before a non-blank line is reached */
+static void edit_move_up_paragraph (WEdit * edit, int scroll)
+{
+    int i;
+    if (edit->curs_line <= 1) {
+	i = 0;
+    } else {
+	if (line_is_blank (edit, edit->curs_line)) {
+	    if (line_is_blank (edit, edit->curs_line - 1)) {
+		for (i = edit->curs_line - 1; i; i--)
+		    if (!line_is_blank (edit, i)) {
+			i++;
+			break;
+		    }
+	    } else {
+		for (i = edit->curs_line - 1; i; i--)
+		    if (line_is_blank (edit, i))
+			break;
+	    }
+	} else {
+	    for (i = edit->curs_line - 1; i; i--)
+		if (line_is_blank (edit, i))
+		    break;
+	}
+    }
+    edit_move_up (edit, edit->curs_line - i, scroll);
+}
+
 /* move i lines */
 static void edit_move_down (WEdit * edit, int i, int scroll)
 {
@@ -991,6 +1078,35 @@ static void edit_move_down (WEdit * edit, int i, int scroll)
     }
 }
 
+/* moves down until a blank line is reached, or until just
+   before a non-blank line is reached */
+static void edit_move_down_paragraph (WEdit * edit, int scroll)
+{
+    int i;
+    if (edit->curs_line >= edit->total_lines - 1) {
+	i = edit->total_lines;
+    } else {
+	if (line_is_blank (edit, edit->curs_line)) {
+	    if (line_is_blank (edit, edit->curs_line + 1)) {
+		for (i = edit->curs_line + 1; i; i++)
+		    if (!line_is_blank (edit, i)) {
+			i--;
+			break;
+		    }
+	    } else {
+		for (i = edit->curs_line + 1; i; i++)
+		    if (line_is_blank (edit, i))
+			break;
+	    }
+	} else {
+	    for (i = edit->curs_line + 1; i; i++)
+		if (line_is_blank (edit, i))
+		    break;
+	}
+    }
+    edit_move_down (edit, i - edit->curs_line, scroll);
+}
+
 static void edit_begin_page (WEdit *edit)
 {
     update_curs_row (edit);
@@ -1075,26 +1191,27 @@ void edit_push_markers (WEdit * edit)
 /* return -1 on nothing to store or error, zero otherwise */
 void edit_get_selection (WEdit * edit)
 {
-    long start_mark, end_mark, i;
-    if (eval_marks (edit, &start_mark, &end_mark)) {
-	if (selection.text)
-	    free (selection.text);
-	selection.text = malloc (1);
-	selection.len = 0;
+    long start_mark, end_mark;
+    if (eval_marks (edit, &start_mark, &end_mark))
 	return;
-    }
-    selection.len = end_mark - start_mark;
-    if (selection.text)
-	free (selection.text);
-    selection.text = malloc (selection.len + 1);
-    if (!selection.text) {
-	selection.text = malloc (1);
-	selection.len = 0;
+    if (selection_history[current_selection].len < 4096)  /* large selections should not be held -- to save memory */
+	current_selection = (current_selection + 1) % NUM_SELECTION_HISTORY;
+    selection_history[current_selection].len = end_mark - start_mark;
+    if (selection_history[current_selection].text)
+	free (selection_history[current_selection].text);
+    selection_history[current_selection].text = malloc (selection_history[current_selection].len + 1);
+    if (!selection_history[current_selection].text) {
+	selection_history[current_selection].text = malloc (1);
+	*selection_history[current_selection].text= 0;
+	selection_history[current_selection].len = 0;
     } else {
-	for (i = 0; i < selection.len; i++, start_mark++)
-	    selection.text[i] = edit_get_byte (edit, start_mark);
+	unsigned char *p = selection_history[current_selection].text;
+	for (; start_mark < end_mark; start_mark++)
+	    *p++ = edit_get_byte (edit, start_mark);
+	*p = 0;
     }
-    selection.text[selection.len] = 0;
+    selection.text = selection_history[current_selection].text;
+    selection.len = selection_history[current_selection].len;
 }
 
 void edit_set_markers (WEdit * edit, long m1, long m2)
@@ -1204,7 +1321,7 @@ void edit_do_undo (WEdit * edit)
     push_action_disabled = 1;	/* don't record undo's onto undo stack! */
 
     while ((ac = pop_action (edit)) < KEY_PRESS) {
-	switch (ac) {
+	switch ((int) ac) {
 	case STACK_BOTTOM:
 	    goto done_undo;
 	case CURS_RIGHT:
@@ -1250,11 +1367,24 @@ void edit_do_undo (WEdit * edit)
 
 static void edit_delete_to_line_end (WEdit * edit)
 {
-    int c;
-    do {
-	c = edit_delete (edit);
-    } while (c != '\n' && c);
-    edit_insert_ahead (edit, '\n');
+    for (;;) {
+	if (edit_get_byte (edit, edit->curs1) == '\n')
+	    break;
+	if (!edit->curs2)
+	    break;
+	edit_delete (edit);
+    }
+}
+
+static void edit_delete_to_line_begin (WEdit * edit)
+{
+    for (;;) {
+	if (edit_get_byte (edit, edit->curs1 - 1) == '\n')
+	    break;
+	if (!edit->curs1)
+	    break;
+	edit_backspace (edit);
+    }
 }
 
 static void edit_delete_line (WEdit * edit)
@@ -1280,7 +1410,7 @@ static void insert_spaces_tab (WEdit * edit)
 static int is_aligned_on_a_tab (WEdit * edit)
 {
     update_curs_col (edit);
-    if ((edit->curs_col % TAB_SIZE) && edit->curs_col % TAB_SIZE != HALF_TAB_SIZE)
+    if ((edit->curs_col % (TAB_SIZE * space_width)) && edit->curs_col % (TAB_SIZE * space_width) != (HALF_TAB_SIZE * space_width))
 	return 0;		/* not alligned on a tab */
     return 1;
 }
@@ -1319,7 +1449,11 @@ static void edit_auto_indent (WEdit *edit, int always)
     while (strchr ("\t ", edit_get_byte (edit, q)) && q < edit->last_byte - 1)	/* move to the end of the leading whitespace of the line */
 	q++;
 
-    indent = (int) edit_move_forward3 (edit, p, 0, q);	/* count the number of columns of indentation */
+    indent = edit_move_forward3 (edit, p, 0, q);	/* count the number of columns of indentation */
+
+#ifndef MIDNIGHT
+    indent /= per_char[' '];
+#endif
 
      /* insert that amount: */
     if (!option_fill_tabs_with_spaces) {
@@ -1370,7 +1504,11 @@ static void check_and_wrap_line (WEdit * edit)
     if (option_word_wrap_line_length < 2)
 	return;
     update_curs_col (edit);
+#ifdef MIDNIGHT
     if (edit->curs_col < option_word_wrap_line_length)
+#else
+    if (edit->curs_col < option_word_wrap_line_length * TEXT_M_WIDTH)
+#endif
 	return;
     curs = edit->curs1;
     for (;;) {
@@ -1423,6 +1561,44 @@ void push_key_press (WEdit * edit)
 	push_action (edit, MARK_1 + edit->mark1);
 }
 
+/* this find the matching bracket in either direction, and sets edit->bracket */
+void edit_find_bracket (WEdit * edit)
+{
+    if (option_find_bracket) {
+	const char *b = "{}{[][()(", *p;
+	static int last_bracket = -1;
+	int i = 1, a, inc = -1, c, d, n = 0, j = 0;
+	long q;
+
+	edit->bracket = -1;
+	c = edit_get_byte (edit, edit->curs1);
+	p = strchr (b, c);
+	update_curs_row (edit);
+	if (p) {
+	    d = p[1];
+	    if (strchr ("{[(", c))
+		inc = 1;
+	    for (q = edit->curs1 + inc;; q += inc) {
+		if (q >= edit->last_byte || q < edit->start_display || j++ > 10000)
+		    break;
+		a = edit_get_byte (edit, q);
+		if (inc > 0 && a == '\n')
+		    n++;
+		if (n >= edit->num_widget_lines - edit->curs_row)	/* out of screen */
+		    break;
+		i += (a == c) - (a == d);
+		if (!i) {
+		    edit->bracket = q;
+		    break;
+		}
+	    }
+	}
+	if (last_bracket != edit->bracket)
+	    edit->force |= REDRAW_PAGE;
+	last_bracket = edit->bracket;
+    }
+}
+
 
 /* this executes a command as though the user initiated it through a key press. */
 /* callback with WIDGET_KEY as a message calls this after translating the key
@@ -1461,8 +1637,15 @@ int edit_execute_key_command (WEdit * edit, int command, int char_for_insertion)
     return r;
 }
 
-
+#ifdef MIDNIGHT
 static const char *shell_cmd[] = SHELL_COMMANDS_i
+#else
+static void (*user_commamd) (WEdit *, int) = 0;
+void edit_set_user_command (void (*func) (WEdit *, int))
+{
+    user_commamd = func;
+}
+#endif
 
 
 /* 
@@ -1476,6 +1659,7 @@ static const char *shell_cmd[] = SHELL_COMMANDS_i
 int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 {
     int result = 1;
+    edit->force |= REDRAW_LINE;
     if (edit->found_len)
 /* the next key press will unhighlight the found string, so update whole page */
 	edit->force |= REDRAW_PAGE;
@@ -1487,10 +1671,8 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	}
 	edit->highlight = 1;
     } else {			/* any other command */
-	if (edit->highlight) {
-	    edit->highlight = 0;
+	if (edit->highlight)
 	    edit_mark_cmd (edit, 0);	/* clear */
-	}
 	edit->highlight = 0;
     }
 
@@ -1513,6 +1695,7 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	edit->found_len = 0;
 	edit->prev_col = edit_get_col (edit);
 	edit->search_start = edit->curs1;
+	edit_find_bracket (edit);
 	return 1;
     }
     switch (command) {
@@ -1580,6 +1763,9 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
     case CK_Delete_To_Line_End:
 	edit_delete_to_line_end (edit);
 	break;
+    case CK_Delete_To_Line_Begin:
+	edit_delete_to_line_begin (edit);
+	break;
     case CK_Return:
 	edit_insert (edit, '\n');
 	if (option_return_does_auto_indent)
@@ -1602,7 +1788,7 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	if (option_fake_half_tabs) {
 	    if (is_in_indent (edit) && right_of_four_spaces (edit)) {
 		edit_cursor_move (edit, -HALF_TAB_SIZE);
-		edit->force &= (0xFF - REDRAW_CHAR_ONLY);
+		edit->force &= (0xFFF - REDRAW_CHAR_ONLY);
 		break;
 	    }
 	}
@@ -1613,7 +1799,7 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	if (option_fake_half_tabs) {
 	    if (is_in_indent (edit) && left_of_four_spaces (edit)) {
 		edit_cursor_move (edit, HALF_TAB_SIZE);
-		edit->force &= (0xFF - REDRAW_CHAR_ONLY);
+		edit->force &= (0xFFF - REDRAW_CHAR_ONLY);
 		break;
 	    }
 	}
@@ -1643,6 +1829,14 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
     case CK_Down_Highlight:
 	edit_move_down (edit, 1, 0);
 	break;
+    case CK_Paragraph_Up:
+    case CK_Paragraph_Up_Highlight:
+	edit_move_up_paragraph (edit, 0);
+	break;
+    case CK_Paragraph_Down:
+    case CK_Paragraph_Down_Highlight:
+	edit_move_down_paragraph (edit, 0);
+	break;
     case CK_Scroll_Up:
     case CK_Scroll_Up_Highlight:
 	edit_move_up (edit, 1, 1);
@@ -1704,6 +1898,9 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
     case CK_XPaste:
 	paste_from_X_buf_cmd (edit);
 	break;
+    case CK_Selection_History:
+	edit_paste_from_history (edit);
+	break;
 
     case CK_Save_As:
 	edit_save_as_cmd (edit);
@@ -1759,24 +1956,31 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
     case CK_Goto:
 	edit_goto_cmd (edit);
 	break;
+    case CK_Delete_Macro:
+	edit_delete_macro_cmd (edit);
+	break;
+#ifdef MIDNIGHT
     case CK_Sort:
 	edit_sort_cmd (edit);
 	break;
+#endif
     }
 
 
-
+#ifdef MIDNIGHT
 /* CK_Pipe_Block */
-    if ((command / 1000) == 1) {	/* a shell command */
+    if ((command / 1000) == 1)	/* a shell command */
 	edit_block_process_cmd (edit, shell_cmd[command - 1000], 1);
-    }
+#else
+    if ((command / 1000) == 1)	/* a user defined command */
+	if (user_commamd)
+	    (*user_commamd) (edit, command - 1000);
+#endif
+
     if (command > CK_Macro (0) && command <= CK_Last_Macro) {	/* a macro command */
-	int s[2];
 	struct macro m[MAX_MACRO_LENGTH];
 	int nm;
-	s[0] = command - 2000;
-	s[1] = 0;
-	if ((result = edit_load_macro_cmd (edit, m, &nm, s)))
+	if ((result = edit_load_macro_cmd (edit, m, &nm, command - 2000)))
 	    edit_execute_macro (edit, m, nm);
     }
 /* keys which must set the col position, and the search vars */
@@ -1789,13 +1993,28 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	return 1;
 	break;
     case CK_Up:
+    case CK_Up_Highlight:
     case CK_Down:
+    case CK_Down_Highlight:
     case CK_Page_Up:
+    case CK_Page_Up_Highlight:
     case CK_Page_Down:
+    case CK_Page_Down_Highlight:
     case CK_Beginning_Of_Text:
+    case CK_Beginning_Of_Text_Highlight:
     case CK_End_Of_Text:
+    case CK_End_Of_Text_Highlight:
+    case CK_Paragraph_Up:
+    case CK_Paragraph_Up_Highlight:
+    case CK_Paragraph_Down:
+    case CK_Paragraph_Down_Highlight:
+    case CK_Scroll_Up:
+    case CK_Scroll_Up_Highlight:
+    case CK_Scroll_Down:
+    case CK_Scroll_Down_Highlight:
 	edit->search_start = edit->curs1;
 	edit->found_len = 0;
+	edit_find_bracket (edit);
 	return 1;
 	break;
     default:
@@ -1803,7 +2022,7 @@ int edit_execute_command (WEdit * edit, int command, int char_for_insertion)
 	edit->prev_col = edit_get_col (edit);
 	edit->search_start = edit->curs1;
     }
-
+    edit_find_bracket (edit);
     return result;
 }
 
