@@ -4,6 +4,7 @@
 #ifdef MIDNIGHT
 
 #include <stdio.h>
+#include <stdarg.h>
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -23,6 +24,7 @@
 
 #include "global.h"
 #include <stdio.h>
+#include <stdarg.h>
 #include <sys/types.h>
 
 #ifdef HAVE_UNISTD_H
@@ -93,19 +95,21 @@
 #define BLOCK_FILE "/cedit/cooledit.block"
 #define ERROR_FILE "/cedit/cooledit.error"
 #define TEMP_FILE "/cedit/cooledit.temp"
+#define SCRIPT_FILE "/cedit/cooledit.script"
 #define EDIT_DIR "/cedit"
 
 #define EDIT_KEY_EMULATION_NORMAL 0
 #define EDIT_KEY_EMULATION_EMACS  1
 
-#define REDRAW_LINE_ABOVE    (1 << 0)
-#define REDRAW_LINE_BELOW    (1 << 1)
-#define REDRAW_AFTER_CURSOR  (1 << 2)
-#define REDRAW_BEFORE_CURSOR (1 << 3)
-#define REDRAW_PAGE          (1 << 4)
-#define REDRAW_IN_BOUNDS     (1 << 5)
-#define REDRAW_CHAR_ONLY     (1 << 6)
-#define REDRAW_COMPLETELY    (1 << 7)
+#define REDRAW_LINE          (1 << 0)
+#define REDRAW_LINE_ABOVE    (1 << 1)
+#define REDRAW_LINE_BELOW    (1 << 2)
+#define REDRAW_AFTER_CURSOR  (1 << 3)
+#define REDRAW_BEFORE_CURSOR (1 << 4)
+#define REDRAW_PAGE          (1 << 5)
+#define REDRAW_IN_BOUNDS     (1 << 6)
+#define REDRAW_CHAR_ONLY     (1 << 7)
+#define REDRAW_COMPLETELY    (1 << 8)
 
 #define MOD_ABNORMAL 1
 #define MOD_UNDERLINED 2
@@ -126,12 +130,12 @@
 
 #endif
 
-#define EDIT_RIGHT_EXTREME 0
-#define EDIT_LEFT_EXTREME 0
-#define EDIT_TOP_EXTREME 0
-#define EDIT_BOTTOM_EXTREME 0
+#define EDIT_RIGHT_EXTREME option_edit_right_extreme
+#define EDIT_LEFT_EXTREME option_edit_left_extreme
+#define EDIT_TOP_EXTREME option_edit_top_extreme
+#define EDIT_BOTTOM_EXTREME option_edit_bottom_extreme
 
-#define MAX_MACRO_LENGTH 256
+#define MAX_MACRO_LENGTH 1024
 
 /*there are a maximum of ... */
 #define MAXBUFF 1024
@@ -216,7 +220,7 @@ struct editor_widget {
     long start_col;		/* First displayed column, negative */
     long curs_row;		/*row position of curser on the screen */
     long curs_col;		/*column position on screen */
-    unsigned char force;	/* how much of the screen do we redraw? */
+    int force;			/* how much of the screen do we redraw? */
     unsigned char overwrite;
     unsigned char modified;	/*has the file been changed?: 1 if char inserted or
 				   deleted at all since last load or save */
@@ -230,13 +234,13 @@ struct editor_widget {
     long total_lines;		/*total lines in the file */
     long mark1;			/*position of highlight start */
     long mark2;			/*position of highlight end */
-/* user options */
-    int wrap_len;
+    long bracket;		/*position of a matching bracket */
 
 /* undo stack and pointers */
     unsigned long stack_pointer;
     long *undo_stack;
     unsigned long stack_bottom;
+    struct stat stat;
 
     int to_here;		/* dummy marker */
 
@@ -250,20 +254,24 @@ typedef struct editor_widget WEdit;
 #ifndef MIDNIGHT
 
 void edit_render_expose (WEdit * edit, XExposeEvent * xexpose);
-void edit_render_tidbits (struct cool_widget *widget);
+void edit_render_tidbits (struct cool_widget *w);
 int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent);
 void edit_draw_menus (Window parent, int x, int y);
 void edit_run_make (void);
 void edit_change_directory (void);
-void xprint_to_widget (Window win, long row, int start_col, float start_col_real, long end_col, unsigned short line[]);
 int edit_man_page_cmd (WEdit * edit);
 void edit_search_replace_dialog (Window parent, int x, int y, char **search_text, char **replace_text, char **arg_order, char *heading, int option);
 void edit_search_dialog (WEdit * edit, char **search_text);
-long edit_find (long search_start, char *exp, int *len, long last_byte, int (*get_byte) (void *, long index), void *data);
+long edit_find (long search_start, unsigned char *exp, int *len, long last_byte, int (*get_byte) (void *, long), void *data);
 void edit_set_foreground_colors (unsigned long normal, unsigned long bold, unsigned long italic);
 void edit_set_background_colors (unsigned long normal, unsigned long abnormal, unsigned long marked, unsigned long marked_abnormal, unsigned long highlighted);
 void edit_set_cursor_color (unsigned long c);
 void draw_options_dialog (Window parent, int x, int y);
+void CRefreshEditor (WEdit * edit);
+void edit_set_user_command (void (*func) (WEdit *, int));
+void edit_draw_this_line_proportional (WEdit * edit, long b, int curs_row, int start_column, int end_column);
+unsigned char get_international_character (unsigned char key_press);
+void edit_set_user_key_function (int (*user_def_key_func) (unsigned int, unsigned int, KeySym keysym));
 
 #else
 
@@ -273,7 +281,7 @@ void edit_init_menu_emacs (void);
 void edit_init_menu_normal (void);
 void edit_done_menu (void);
 int edit_raw_key_query (char *heading, char *query, int cancel);
-char *strcasechr (const char *s, int c);
+char *strcasechr (const unsigned char *s, int c);
 int edit (const char *_file);
 
 #endif
@@ -282,7 +290,7 @@ int edit_get_byte (WEdit * edit, long byte_index);
 int load_edit_file (WEdit * edit, const char *filename, const char *text);
 int edit_count_lines (WEdit * edit, long current, int upto);
 long edit_move_forward (WEdit * edit, long current, int lines, long upto);
-float edit_move_forward3 (WEdit * edit, long current, int cols, long upto);
+long edit_move_forward3 (WEdit * edit, long current, int cols, long upto);
 long edit_move_backward (WEdit * edit, long current, int lines);
 int edit_translate_key (WEdit * edit, unsigned int x_keycode, long x_key, int x_state, int *cmd, int *ch);
 void edit_scroll_screen_over_cursor (WEdit * edit);
@@ -349,17 +357,21 @@ void edit_right_word_move (WEdit * edit);
 void edit_get_selection (WEdit * edit);
 
 int edit_save_macro_cmd (WEdit * edit, struct macro macro[], int n);
-int edit_load_macro_cmd (WEdit * edit, struct macro macro[], int *n, int *k);
+int edit_load_macro_cmd (WEdit * edit, struct macro macro[], int *n, int k);
+void edit_delete_macro_cmd (WEdit * edit);
 
 int edit_copy_to_X_buf_cmd (WEdit * edit);
 int edit_cut_to_X_buf_cmd (WEdit * edit);
 void paste_from_X_buf_cmd (WEdit * edit);
-void edit_set_user_key_function (int (*user_def_key_func) (unsigned int, unsigned int));
+
+void edit_paste_from_history (WEdit *edit);
 
 void split_filename (WEdit * edit, char *name);
 
 #ifdef MIDNIGHT
 
+/* put OS2/NT/WIN95 defines here */
+
 #define TEXT_PIX_PER_LINE 1
 #define TEXT_M_WIDTH 1
 
@@ -369,6 +381,12 @@ void split_filename (WEdit * edit, char *name);
 #define read(f,b,c) mc_read(f,b,c)
 #define write(f,b,c) mc_write(f,b,c)
 #define stat(f,s) mc_stat(f,s)
+#if defined(OS2_NT)
+#  define mkdir(s,m) mc_mkdir(s)
+#else
+#  define mkdir(s,m) mc_mkdir(s,m)
+#endif
+#define itoa MY_itoa
 
 #define Cgetloadfile(w,x,y,d,f,h) input_dialog (h, " Enter file name: ", f)
 #define Cgetsavefile(w,x,y,d,f,h) input_dialog (h, " Enter file name: ", f)
@@ -393,17 +411,34 @@ extern char *edit_init_error_msg;
 #else				/* ! MIDNIGHT */
 
 #define Cerrordialogue(w,x,y,h,s) Cerrordialog(w,x,y,h,s)
-#define Cmessagedialogue(w,x,y,h,s) Cmessagedialog(w,x,y,h,s)
-#define Cmessagedialogue(w,x,y,h,s) Cmessagedialog(w,x,y,h,s)
+#define Cmessagedialogue(w,x,y,h,s) Cmessagedialog(w,x,y,0,h,s)
 #define Cquerydialogue Cquerydialog
 
 #endif				/* ! MIDNIGHT */
 
 extern char *home_dir;
 
+#define NUM_SELECTION_HISTORY 32
+
 #ifdef _EDIT_C
 
-struct selection selection = {0, 0};
+struct selection selection =
+{0, 0};
+int current_selection = 0;
+/* Note: selection.text = selection_history[current_selection].text */
+struct selection selection_history[NUM_SELECTION_HISTORY] =
+{
+    {0, 0},
+    {0, 0},
+    {0, 0},
+    {0, 0},
+    {0, 0},
+    {0, 0},
+    {0, 0},
+    {0, 0},
+    {0, 0},
+    {0, 0},
+};
 
 #ifdef MIDNIGHT
 /*
@@ -420,11 +455,19 @@ int option_fill_tabs_with_spaces = 0;
 int option_return_does_auto_indent = 1;
 int option_backspace_through_tabs = 0;
 int option_fake_half_tabs = 1;
+int option_save_mode = 0;
+int option_backup_ext_int = -1;
+int option_find_bracket = 1;
 
 int option_editor_fg_normal = 26;
 int option_editor_fg_bold = 8;
 int option_editor_fg_italic = 10;
 
+int option_edit_right_extreme = 0;
+int option_edit_left_extreme = 0;
+int option_edit_top_extreme = 0;
+int option_edit_bottom_extreme = 0;
+
 int option_editor_bg_normal = 1;
 int option_editor_bg_abnormal = 0;
 int option_editor_bg_marked = 2;
@@ -434,10 +477,13 @@ int option_editor_fg_cursor = 18;
 
 char *option_whole_chars_search = "0123456789abcdefghijklmnopqrstuvwxyz_";
 char *option_whole_chars_move = "0123456789abcdefghijklmnopqrstuvwxyz_; ,[](){}";
+char *option_backup_ext = "~";
 
 #else				/* ! _EDIT_C */
 
 extern struct selection selection;
+extern struct selection selection_history[];
+extern int current_selection;
 
 #ifdef MIDNIGHT
 /*
@@ -454,11 +500,19 @@ extern int option_fill_tabs_with_spaces;
 extern int option_return_does_auto_indent;
 extern int option_backspace_through_tabs;
 extern int option_fake_half_tabs;
+extern int option_save_mode;
+extern int option_backup_ext_int;
+extern int option_find_bracket;
 
 extern int option_editor_fg_normal;
 extern int option_editor_fg_bold;
 extern int option_editor_fg_italic;
 
+extern int option_edit_right_extreme;
+extern int option_edit_left_extreme;
+extern int option_edit_top_extreme;
+extern int option_edit_bottom_extreme;
+
 extern int option_editor_bg_normal;
 extern int option_editor_bg_abnormal;
 extern int option_editor_bg_marked;
@@ -468,6 +522,7 @@ extern int option_editor_fg_cursor;
 
 extern char *option_whole_chars_search;
 extern char *option_whole_chars_move;
+extern char *option_backup_ext;
 
 #endif				/* ! _EDIT_C */
 
