@@ -30,6 +30,8 @@
 #define CK_End_Page		22
 #define CK_Delete_Word_Left	23
 #define CK_Delete_Word_Right	24
+#define CK_Paragraph_Up		25
+#define CK_Paragraph_Down		26
 
 
 /* file commands */
@@ -62,13 +64,14 @@
 #define CK_Goto			407
 #define CK_Delete_Line		408
 #define CK_Delete_To_Line_End	409
-#define CK_Run_Make		410
+#define CK_Delete_To_Line_Begin	410
 #define CK_Man_Page		411
 #define CK_Sort			412
 
 /* macro */
 #define CK_Begin_Record_Macro	501
 #define CK_End_Record_Macro	502
+#define CK_Delete_Macro		503
 
 /* highlight commands */
 #define CK_Page_Up_Highlight 		604
@@ -87,14 +90,17 @@
 #define CK_End_Page_Highlight		617
 #define CK_Scroll_Up_Highlight		618
 #define CK_Scroll_Down_Highlight	619
+#define CK_Paragraph_Up_Highlight		620
+#define CK_Paragraph_Down_Highlight	621
 
 /* X clipboard operations */
 
 #define CK_XStore		701
 #define CK_XCut			702
 #define CK_XPaste		703
+#define CK_Selection_History	704
 
-
+#ifdef MIDNIGHT		/* cooledit now has its own full-featured script editor and executor */
 /*
    Process a block through a shell command: CK_Pipe_Block(i) executes shell_cmd[i].
    shell_cmd[i] must process the file ~/cooledit.block and output ~/cooledit.block
@@ -105,12 +111,15 @@
    it into the text, which is annoying.
  */
 #define CK_Pipe_Block(i)	(1000+(i))
+#define SHELL_COMMANDS_i {"/cedit/edit.indent.rc", "/cedit/edit.spell.rc", /* and so on */ 0};
+#else
+#define CK_User_Command(i)	(1000+(i))
+#endif
 
 /* execute a macro */
 #define CK_Macro(i)		(2000+(i))
 #define CK_Last_Macro		CK_Macro(0x7FFF)
 
-#define SHELL_COMMANDS_i {"/cedit/edit.indent.rc", "/cedit/edit.spell.rc", "/cedit/edit.sort.rc", /* and so on */ 0};
 
 #endif
 
