@@ -1,8 +1,8 @@
 /* editor text drawing.
 
-   Copyright (C) 1996 the Free Software Foundation
+   Copyright (C) 1996, 1997 the Free Software Foundation
 
-   Authors: 1996 Paul Sheer
+   Authors: 1996, 1997 Paul Sheer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #endif
 
 
-static void status_string (WEdit * edit, char *s, int w)
+static void status_string (WEdit * edit, char *s, int w, int fill, int font_width)
 {
     int i;
     char t[160];		/* 160 just to be sure */
@@ -41,7 +41,7 @@ static void status_string (WEdit * edit, char *s, int w)
 	     edit->mark1 != edit->mark2 ? 'B' : '-',
 	     edit->modified ? 'M' : '-', edit->macro_i < 0 ? '-' : 'R',
 	     edit->overwrite == 0 ? '-' : 'O',
-	     edit->curs_col, edit->start_line + 1, edit->curs_row,
+	     edit->curs_col / font_width, edit->start_line + 1, edit->curs_row,
 	     edit->curs_line + 1, edit->total_lines + 1, edit->curs1,
 	     edit->last_byte, edit->curs1 < edit->last_byte
 	     ? edit_get_byte (edit, edit->curs1) : -1);
@@ -52,9 +52,9 @@ static void status_string (WEdit * edit, char *s, int w)
     do {
 	if (strchr (" +-*=/:b", s[i]))	/* chop off the last word/number */
 	    break;
-	s[i] = ' ';
+	s[i] = fill;
     } while (i--);
-    s[i] = ' ';
+    s[i] = fill;
     s[w] = 0;
 }
 
@@ -82,7 +82,7 @@ void edit_status (WEdit * edit)
 	t = w - 20;
 	if (t < 0)
 	    t = 0;
-	status_string (edit, s + 20, t);
+	status_string (edit, s + 20, t, ' ', 1);
     } else {
 	s[w] = 0;
     }
@@ -94,31 +94,46 @@ void edit_status (WEdit * edit)
 
 #else
 
+extern int fixed_font;
+
+void Crerendertext (CWidget * wdt);
+    
 void edit_status (WEdit * edit)
 {
-    int w, i, t;
-    char *s;
-    w = edit->num_widget_columns - 1;
-    if (w < 0)
-	w = 0;
-/*    if (w > 70)
-   w = 70; */
-    s = malloc (3 * w / 2 + 80);
-    memset (s, ' ', w);
-    if (w > 1) {
-	i = w > 24 ? 18 : w - 6;
-	i = i < 13 ? 13 : i;
-	sprintf (s, "%s", name_trunc (edit->filename ? edit->filename : "", i));
-	i = strlen (s);
-	s[i] = ' ';
-	t = w - i - 2;
-	if (t < 0)
-	    t = 0;
-	status_string (edit, s + i + 2, t);
+    if ((edit->widget->options & EDITOR_NO_TEXT)) {
+	return;
+    } else {
+	int w, i, t;
+	CWidget *wdt;
+	char id[33];
+	char s[160];
+	w = edit->num_widget_columns - 1;
+	if (w > 150)
+	    w = 150;
+	if (w < 0)
+	    w = 0;
+	memset (s, 0, w);
+	if (w > 1) {
+	    i = w > 24 ? 18 : w - 6;
+	    i = i < 13 ? 13 : i;
+	    sprintf (s, "%s", name_trunc (edit->filename ? edit->filename : "", i));
+	    i = strlen (s);
+	    s[i] = ' ';
+	    s[i+1] = ' ';
+	    t = w - i - 2;
+	    if (t < 0)
+		t = 0;
+	    status_string (edit, s + i + 2, t, 0, TEXT_M_WIDTH);
+	}
+	s[w] = 0;
+	strcpy (id, edit->widget->ident);
+	strcat (id, ".text");
+	wdt = Cwidget (id);
+	free (wdt->text);
+	wdt->text = strdup (s);
+	Csetwidgetsize (id, edit->widget->width, wdt->height);
+	Crerendertext (wdt);
     }
-    s[w] = 0;
-    Credrawtext (catstrs (edit->widget->ident, ".text", 0), s);
-    free (s);
 }
 
 
@@ -154,17 +169,25 @@ int cursor_out_of_screen (WEdit * edit)
     return 0;
 }
 
-
 /* this scrolls the text so that cursor is on the screen */
 void edit_scroll_screen_over_cursor (WEdit * edit)
 {
-    int p = edit_get_col (edit);
+    int p;
     int outby;
+    p = edit_get_col (edit);
     update_curs_row (edit);
+#ifdef MIDNIGHT
     outby = p + edit->start_col - edit->num_widget_columns + 1 + EDIT_RIGHT_EXTREME;
+#else
+    outby = p + edit->start_col - edit->widget->width + 7 + (EDIT_RIGHT_EXTREME + 1) * TEXT_M_WIDTH;
+#endif
     if (outby > 0)
 	edit_scroll_right (edit, outby);
+#ifdef MIDNIGHT
     outby = EDIT_LEFT_EXTREME - p - edit->start_col;
+#else
+    outby = EDIT_LEFT_EXTREME * TEXT_M_WIDTH - p - edit->start_col;
+#endif
     if (outby > 0)
 	edit_scroll_left (edit, outby);
     p = edit->curs_row;
@@ -178,13 +201,13 @@ void edit_scroll_screen_over_cursor (WEdit * edit)
 }
 
 
-
 #ifndef MIDNIGHT
 
 #define CACHE_WIDTH 256
 #define CACHE_HEIGHT 128
 
 int EditExposeRedraw = 0;
+int EditClear = 0;
 
 /* background colors: marked is refers to mouse highlighting, highlighted refers to a found string. */
 unsigned long edit_abnormal_color, edit_marked_abnormal_color;
@@ -219,98 +242,6 @@ void edit_set_cursor_color (unsigned long c)
     edit_cursor_color = c;
 }
 
-void xprint_to_widget (Window win, long row, int start_col, float start_col_real, long end_col, unsigned short line[])
-{
-    static unsigned short cache[CACHE_WIDTH][CACHE_HEIGHT];
-    static Window last = 0;
-    int M_width = TEXT_M_WIDTH;
-    int M_height = TEXT_PIX_PER_LINE;
-    int x, y = row * M_height + EDIT_TEXT_VERTICAL_OFFSET;
-    unsigned short *p;
-    static char text[MAX_LINE_LEN];
-    unsigned int style;
-    int i = 0;
-    unsigned long bg, fg;
-    int cursor_found = 0;
-
-    if (row >= CACHE_HEIGHT || end_col >= CACHE_WIDTH)	/* this will look ugly to users who want gian-normou-huge edit windows */
-	return;
-
-/* find the end of the line and, */
-    while (line[i++]);
-
-/* discard the part of the line we don't need to draw */
-    line += (start_col - (int) start_col_real);
-    i -= start_col - (int) start_col_real + 1;
-    if (i <= 0)
-	i = 0;
-
-/* if its not the same window, reset the screen rememberer */
-    if (last != win) {
-	last = win;
-	memset (cache, -1, CACHE_WIDTH * CACHE_HEIGHT * sizeof (short));
-    }
-/* fill from the end of the line with spaces up to the required width */
-    while (i <= end_col - start_col && i < CACHE_WIDTH)
-	line[i++] = ' ';
-    line[i] = 0;
-
-    if (!EditExposeRedraw) {	/* except for exposes (when we don't know what was erased) */
-/* discard part of the line that is the same on the screen, where the cache remembers it: */
-	for (i = 0, x = start_col; x <= end_col; x++, i++)
-	    if (cache[x][row] != line[i] || cache[x][row] & MOD_CURSOR * 256)
-		break;
-
-	for (x = end_col; x >= start_col && x >= i; line[(x--) - start_col] = 0)
-	    if (cache[x][row] != line[x - start_col] || cache[x][row] & MOD_CURSOR * 256)
-		break;
-	start_col += i;
-	line += i;
-    }
-    x = start_col * M_width + EDIT_TEXT_HORIZONTAL_OFFSET;	/* pixel pos of line start */
-    p = line;
-
-    while (*p) {
-	i = 0;
-	style = (*p) & 0xFF00;
-	if (style & (MOD_ABNORMAL * 256)) {
-	    bg = edit_abnormal_color;
-	    if (style & (MOD_MARKED * 256))
-		bg = edit_marked_abnormal_color;
-	} else if (style & (MOD_HIGHLIGHTED * 256))
-	    bg = edit_highlighted_color;
-	else if (style & (MOD_MARKED * 256))
-	    bg = edit_marked_color;
-	else
-	    bg = edit_normal_background_color;
-	fg = edit_normal_foreground_color;
-	if (style & (MOD_BOLD * 256))
-	    fg = edit_bold_color;
-	if (style & (MOD_ITALIC * 256))
-	    fg = edit_italic_color;
-/* print all characters with the same style (i.e. same foreground and background color) at once: */
-	while (style == ((*p) & 0xFF00) && *p) {
-	    text[i++] = (char) *p;
-	    cache[start_col++][row] = *(p++);
-	}
-	Csetbackcolor (bg);
-	Csetcolor (fg);
-	XDrawImageString (CDisplay, win, CGC, x + FONT_OFFSET_X, y + FONT_OFFSET_Y, text, i);
-/* if we printed a cursor: */
-	if (style & (MOD_CURSOR * 256)) {
-	    Csetcolor (edit_cursor_color);
-	    Cline (win, x, y + CFontStruct->descent + IN_FONT_OFFSET_Y,
-		   x, y + M_height - 1 + IN_FONT_OFFSET_Y);	/* non focussed cursor form */
-	    Cline (win, x + 1, y + CFontStruct->descent + IN_FONT_OFFSET_Y,
-	    x + M_width - 1, y + CFontStruct->descent + IN_FONT_OFFSET_Y);
-	    cursor_found = 1;
-	    Csetcursor (win, x, y, M_width, M_height, EDITOR_CURSOR, *text, bg, fg);	/* widget library's flashing cursor */
-	}
-	x += M_width * i;
-    }
-}
-
-
 #else
 
 #define BOLD_COLOR        MARKED_COLOR
@@ -365,29 +296,6 @@ static void print_to_widget (WEdit * edit, long row, int start_col, float start_
     }
 }
 
-#endif
-
-
-
-#ifndef MIDNIGHT
-
-/* feel free to add your own translation table (XWindows version only) */
-static char basic_translation[256] =
-{"0ABCDEFGHIJKL\005NOPQRSTUVWXYZ[\\]^_"
- " !\"#$%&'()*+,-./0123456789:;<=>?"
- "@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_"
- "`abcdefghijklmnopqrstuvwxyz{|}~ "
- "\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024\024"
- "\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023\023"
- "\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022\022"
- "\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021\021"
- "\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020\020"};
-
-char *translation_table = 0;
-
-#endif
-
-
 /* b pointer to begining of line */
 static void edit_draw_this_line (WEdit * edit, long b, long row, long start_col, long end_col)
 {
@@ -398,11 +306,6 @@ static void edit_draw_this_line (WEdit * edit, long b, long row, long start_col,
     unsigned int c;
     int i;
 
-#ifndef MIDNIGHT
-    if (!translation_table)
-	translation_table = basic_translation;
-#endif
-
     q = edit_move_forward3 (edit, b, start_col - edit->start_col, 0);
     start_col_real = (col = (int) edit_move_forward3 (edit, b, 0, q)) + edit->start_col;
 
@@ -416,6 +319,8 @@ static void edit_draw_this_line (WEdit * edit, long b, long row, long start_col,
 		    *p |= MOD_CURSOR * 256;
 		if (q >= m1 && q < m2)
 		    *p |= MOD_MARKED * 256;
+		if (q == edit->bracket)
+		    *p |= MOD_BOLD * 256;
 		if (q >= edit->found_start && q < edit->found_start + edit->found_len)
 		    *p |= MOD_HIGHLIGHTED * 256;
 		switch (c = edit_get_byte (edit, q++)) {
@@ -431,32 +336,15 @@ static void edit_draw_this_line (WEdit * edit, long b, long row, long start_col,
 		    while (--i)
 			*(p++) = c;
 		    break;
-#ifdef MIDNIGHT
 		case '\r':
 		    break;
-#endif
 		default:
-#ifdef MIDNIGHT
 		    if (is_printable (c)) {
 			*(p++) |= c;
 		    } else {
 			*(p++) = '.';
 			*p |= (256 * MOD_ABNORMAL);
 		    }
-#else
-		    if (option_international_characters) {
-			if (c < ' ' || (c > '~' && c < 160))
-			    *p |= (256 * MOD_ABNORMAL);
-			if (c < 160)
-			    *(p++) |= translation_table[c];
-			else
-			    *(p++) |= c;
-		    } else {
-			if (c < ' ' || c > '~')
-			    *p |= (256 * MOD_ABNORMAL);
-			*(p++) |= translation_table[c];
-		    }
-#endif
 		    col++;
 		    break;
 		}
@@ -467,32 +355,29 @@ static void edit_draw_this_line (WEdit * edit, long b, long row, long start_col,
     }
     *p = 0;
 
-#ifdef MIDNIGHT
     print_to_widget (edit, row, start_col, start_col_real, end_col, line);
-#else
-    xprint_to_widget (edit->widget->winid, row, start_col, start_col_real, end_col, line);
-#endif
 }
 
+#endif
+
 #ifdef MIDNIGHT
+
 #define key_pending(x) (!is_idle())
+
 #else
+
+int edit_mouse_pending (Window win);
+#define edit_draw_this_line edit_draw_this_line_proportional
+
 static int key_pending (WEdit * edit)
 {
-    int b = 0;
-    if (!(edit->force & REDRAW_COMPLETELY)) {
-	XEvent ev;
-	while (XCheckWindowEvent (CDisplay, edit->widget->winid, KeyPressMask, &ev)) {
-	    XSendEvent (CDisplay, edit->widget->winid, 0, KeyPressMask, &ev);	/* don't discard the event, but resend it */
-	    b = 1;
-	}
-    } else {
-	return 0;
-    }
-    return b | CWindowPending (edit->widget->scrollbar->winid);
+    if (!(edit->force & REDRAW_COMPLETELY))
+	return CCheckWindowEvent (edit->widget->winid, KeyPressMask, 0);
+    return 0;
 }
 #endif
 
+
 /* b for pointer to begining of line */
 static void drawthischar (WEdit * edit, long curs, long row)
 {
@@ -501,7 +386,6 @@ static void drawthischar (WEdit * edit, long curs, long row)
     edit_draw_this_line (edit, b, row, start_col, start_col);
 }
 
-
 /* cursor must be in screen for other than REDRAW_PAGE passed in force */
 void Crenderedittext (WEdit * edit, long start_row, long start_column, long end_row, long end_column)
 {
@@ -559,6 +443,7 @@ void Crenderedittext (WEdit * edit, long start_row, long start_column, long end_
 		    }
 		}
 	    }
+/*	    if (force & REDRAW_LINE) { 		---> default */
 	    b = edit_bol (edit, edit->curs1);
 	    if (curs_row >= start_row && curs_row <= end_row) {
 		if (key_pending (edit))
@@ -629,13 +514,13 @@ void Cedit_expose_to_area (XExposeEvent * xexpose, int *row1, int *col1, int *ro
     *row2 = (xexpose->y + xexpose->height - EDIT_TEXT_VERTICAL_OFFSET) / M_height;
 }
 
-void edit_render_tidbits (CWidget * widget)
+void edit_render_tidbits (CWidget * wdt)
 {
     int isfocussed;
-    int w = widget->width, h = widget->height;
+    int w = wdt->width, h = wdt->height;
     Window win;
 
-    win = widget->winid;
+    win = wdt->winid;
     isfocussed = (win == CGetFocus ());
 
     Csetcolor (C_FLAT);
@@ -648,14 +533,20 @@ void edit_render_tidbits (CWidget * widget)
     }
 }
 
+
 #endif
 
 void edit_render (WEdit * edit, int page, int row_start, int col_start, int row_end, int col_end)
 {
+    int f = 0;
     if (page)			/* if it was an expose event, 'page' would be set */
 	edit->force |= REDRAW_PAGE | REDRAW_IN_BOUNDS;
+    f = edit->force & (REDRAW_PAGE | REDRAW_COMPLETELY);
 
-#ifndef MIDNIGHT
+#ifdef MIDNIGHT
+    if (edit->force & REDRAW_COMPLETELY)
+    	redraw_labels (edit->widget.parent, (Widget *)edit); 
+#else
     edit_set_foreground_colors (
 				   Ccolor (option_editor_fg_normal),
 				   Ccolor (option_editor_fg_bold),
@@ -682,7 +573,11 @@ void edit_render (WEdit * edit, int page, int row_start, int col_start, int row_
 				   was left undrawn from a previous key press */
 	edit->force |= REDRAW_PAGE;
 #ifndef MIDNIGHT
-    edit_render_tidbits (edit->widget);
+    if (f) {
+        edit_render_tidbits (edit->widget);
+        Csetcolor (edit_normal_background_color);
+        Cline (edit->widget->winid, 3, 3, 3, edit->widget->height - 4);
+    }
 #endif
 }
 
@@ -706,7 +601,4 @@ void edit_render_keypress (WEdit * edit)
     edit_render (edit, 0, 0, 0, 0, 0);
 }
 
-
-
-
 #endif
