@@ -2,7 +2,7 @@
 
    Copyright (C) 1996 the Free Software Foundation
    
-   Authors: 1996 Paul Sheer
+   Authors: 1996, 1997 Paul Sheer
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -159,13 +159,21 @@ void menu_layout_cmd (void)		{  edit_layout_cmd(); }
 #endif
 #endif
 
-
 #ifdef MIDNIGHT
 void menu_exec_macro_cmd (void)		{  menu_key(XCTRL ('a')); }
 #else
 void menu_exec_macro_cmd (void)		{  menu_key(XK_a, ControlMask); }
 #endif
 
+void menu_exec_macro_delete_cmd (void)	{  menu_cmd(CK_Delete_Macro); }
+
+#ifndef MIDNIGHT
+void menu_xcut_cmd (void)		{  menu_cmd(CK_XCut); }
+void menu_xpaste_cmd (void)		{  menu_cmd(CK_XPaste); }
+void menu_selhist_cmd (void)		{  menu_cmd(CK_Selection_History); }
+void menu_xclipcopy_cmd (void)		{  menu_cmd(CK_XStore); }
+#endif
+
 #ifdef MIDNIGHT
 void menu_c_form_cmd (void)		{  menu_key(KEY_F(19)); }
 #endif
@@ -174,7 +182,9 @@ void menu_c_form_cmd (void)		{  menu_key(KEY_F(19)); }
 void menu_ispell_cmd (void)		{ menu_cmd(CK_Pipe_Block (1)); }
 #endif
 
+#ifdef MIDNIGHT
 void menu_sort_cmd (void)		{ menu_cmd(CK_Sort); }
+#endif
 
 void menu_date_cmd (void)		{  menu_cmd(CK_Date); }
 void menu_undo_cmd (void)		{  menu_cmd(CK_Undo); }
@@ -280,6 +290,7 @@ static menu_entry CmdMenu[] =
     {' ', "Start record macro      C-r", 'S', menu_begin_record_cmd},
     {' ', "Finish record macro...  C-r", 'F', menu_end_record_cmd},
     {' ', "Execute macro...   C-a, KEY", 'E', menu_exec_macro_cmd},
+    {' ', "Delete macro...            ", 'o', menu_exec_macro_delete_cmd},
     {' ', "", ' ', 0},
     {' ', "insert Date/time        C-d", 'D', menu_date_cmd},
     {' ', "", ' ', 0},
@@ -299,6 +310,7 @@ static menu_entry CmdMenuEmacs[] =
     {' ', "Start record macro      C-r", 'S', menu_begin_record_cmd},
     {' ', "Finish record macro...  C-r", 'F', menu_end_record_cmd},
     {' ', "Execute macro... C-x e, KEY", 'E', menu_exec_macro_cmd},
+    {' ', "Delete macro...            ", 'o', menu_exec_macro_delete_cmd},
     {' ', "", ' ', 0},
     {' ', "insert Date/time           ", 'D', menu_date_cmd},
     {' ', "", ' ', 0},
@@ -307,9 +319,12 @@ static menu_entry CmdMenuEmacs[] =
     {' ', "'indent' C Formatter    F19", 'C', menu_c_form_cmd}
 };
 
+extern void menu_save_mode_cmd (void);
+
 static menu_entry OptMenu[] =
 {
     {' ', "Word wrap...", 'W', menu_wrap_cmd},
+    {' ', "Save mode...", 'S', menu_save_mode_cmd}
 #if 0
     {' ', "Layout...", 'L', menu_layout_cmd}
 #endif
@@ -318,6 +333,7 @@ static menu_entry OptMenu[] =
 static menu_entry OptMenuEmacs[] =
 {
     {' ', "Word wrap...", 'W', menu_wrap_cmd},
+    {' ', "Save mode...", 'S', menu_save_mode_cmd}
 #if 0
     {' ', "Layout...", 'L', menu_layout_cmd}
 #endif
@@ -411,53 +427,57 @@ void CDrawEditMenuButtons (const char *ident, Window parent, Window focus_return
 
     Cdrawmenubutton (catstrs(ident, ".filemenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 8,
 		     " File ",
-		     "Open...          C-o", 'O', menu_load_cmd,
-		     "New              C-n", 'N', menu_new_cmd,
+		     "Open...\tC-o", 'O', menu_load_cmd,
+		     "New\tC-n", 'N', menu_new_cmd,
 		     "", ' ', 0,
-		     "Save              F2", 'S', menu_save_cmd,
-		     "save As...       F12", 'A', menu_save_as_cmd,
+		     "Save\tF2", 'S', menu_save_cmd,
+		     "save As...\tF12", 'A', menu_save_as_cmd,
 		     "", ' ', 0,
-		     "Insert file...   F15", 'I', menu_insert_file_cmd,
-		     "copy to File...  C-f", 'F', menu_cut_cmd
+		     "Insert file...\tF15", 'I', menu_insert_file_cmd,
+		     "copy to File...\tC-f", 'F', menu_cut_cmd
 	);
 
     Cgethintpos (&x, &d);
 
-    Cdrawmenubutton (catstrs (ident, ".editmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 9,
+    Cdrawmenubutton (catstrs (ident, ".editmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 14,
 		     " Edit ",
-		     "Toggle Mark         F3", 'T', menu_mark_cmd,
+		     "Toggle Mark\tF3", 'T', menu_mark_cmd,
 		     "", ' ', 0,
-		     "toggle Ins./overw.    ", 'I', menu_ins_cmd,
+		     "toggle Insert/overwrite\tIns", 'I', menu_ins_cmd,
 		     "", ' ', 0,
-		     "Copy                F5", 'C', menu_copy_cmd,
-		     "Move                F6", 'M', menu_move_cmd,
-		     "Delete              F8", 'D', menu_delete_cmd,
+		     "Copy block to cursor\tF5", 'C', menu_copy_cmd,
+		     "Move block to cursor\tF6", 'M', menu_move_cmd,
+		     "Delete block\tF8/C-Del", 'D', menu_delete_cmd,
 		     "", ' ', 0,
-		     "Undo       C-BackSpace", 'U', menu_undo_cmd
+		     "Copy block to clipbrd\tC-Ins", 'C', menu_xclipcopy_cmd,
+		     "Cut block to clipbrd\tS-Del", 'b', menu_xcut_cmd,
+		     "Paste block from clipbrd\tS-Ins", 'P', menu_xpaste_cmd,
+		     "Selection history\tM-Ins", 'h', menu_selhist_cmd,
+		     "", ' ', 0,
+		     "Undo\tC-BackSpace", 'U', menu_undo_cmd
 	);
 
     Cgethintpos (&x, &d);
 
     Cdrawmenubutton (catstrs (ident, ".searchmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 3,
-		     " Search/Replace ",
-		     "Search...         F7", 'S', menu_search_cmd,
-		     "search Again     F17", 'A', menu_search_again_cmd,
-		     "Replace...        F4", 'R', menu_replace_cmd
+		     " Srch/Replce ",
+		     "Search...\tF7", 'S', menu_search_cmd,
+		     "search Again\tF17", 'A', menu_search_again_cmd,
+		     "Replace...\tF4", 'R', menu_replace_cmd
 	);
 
     Cgethintpos (&x, &d);
 
-    Cdrawmenubutton (catstrs (ident, ".commandmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 9,
+    Cdrawmenubutton (catstrs (ident, ".commandmenu", 0), parent, focus_return, x, y, AUTO_WIDTH, AUTO_HEIGHT, 8,
 		     " Command ",
-		     "Goto line...            M-l", 'G', menu_goto_line,
+		     "Goto line...\tM-l", 'G', menu_goto_line,
 		     "", ' ', 0,
-		     "Start record macro      C-r", 'S', menu_begin_record_cmd,
-		     "Finish record macro...  C-r", 'F', menu_end_record_cmd,
-		     "Execute macro...   C-a, KEY", 'E', menu_exec_macro_cmd,
+		     "Start record macro\tC-r", 'S', menu_begin_record_cmd,
+		     "Finish record macro...\tC-r", 'F', menu_end_record_cmd,
+		     "Execute macro...\tC-a, KEY", 'E', menu_exec_macro_cmd,
+		     "Delete macro...\t", 'o', menu_exec_macro_delete_cmd,
 		     "", ' ', 0,
-		     "insert Date/time        C-d", 'D', menu_date_cmd,
-		     "", ' ', 0,
-		     "Sort...                 M-t", 'O', menu_sort_cmd
+		     "insert Date/time\tC-d", 'D', menu_date_cmd
 	);
 }
 
