@@ -34,13 +34,6 @@
 #  include <direct.h>
 #endif
 
-/* .ado: for OS/2 and NT */
-#if !defined(OS2_NT)                 
-#  define mc_mkdir mkdir              
-#else                                
-#  define mc_mkdir(s,m) mkdir(s)      
-#endif                               
-
 #ifndef MIDNIGHT
 
 extern int EditExposeRedraw;
@@ -62,14 +55,18 @@ extern int option_editor_bg_normal;
 /* starting_directory is for the filebrowser */
 CWidget *Cdraweditor (const char *identifier, Window parent, int x, int y,
 	   int width, int height, const char *text, const char *filename,
-		      const char *starting_directory)
+		    const char *starting_directory, unsigned int options)
 {
     static made_directory = 0;
-    char *scroll;
     CWidget *w;
     WEdit *e;
     wedit = w = Csetupwidget (identifier, parent, x, y,
-			      width + 7, height + 6, CEDITOR_WIDGET, INPUT_KEY | KeyReleaseMask | ButtonMotionMask, Ccolor (option_editor_bg_normal), 1);
+			      width + 7, height + 6, CEDITOR_WIDGET,
+		   ExposureMask | ButtonPressMask | ButtonReleaseMask | \
+		     KeyPressMask | KeyReleaseMask | ButtonMotionMask | \
+	   FocusChangeMask | StructureNotifyMask | PropertyChangeMask | \
+		   EnterWindowMask, Ccolor (option_editor_bg_normal), 1);
+    w->options = options;
 
     w->destroy = Cedit_destroy;
     if (filename)
@@ -78,7 +75,7 @@ CWidget *Cdraweditor (const char *identifier, Window parent, int x, int y,
 	w->label = strdup ("");
 
     if (!made_directory) {
-	mc_mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
+	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
 	made_directory = 1;
     }
     e = w->editor = Cmalloc (sizeof (WEdit));
@@ -95,12 +92,13 @@ CWidget *Cdraweditor (const char *identifier, Window parent, int x, int y,
     e->macro_i = -1;
     e->widget = w;
 
-    Cdrawtext (catstrs (identifier, ".text", 0), parent, x, y + height + 6 + WIDGET_SPACING, "%s", e->filename);
-    Cdrawvertscrollbar (scroll = catstrs (identifier, ".vsc", 0), parent,
+    if (!(options & EDITOR_NO_TEXT))
+	Cdrawtext (catstrs (identifier, ".text", 0), parent, x, y + height + 6 + WIDGET_SPACING, "%s", e->filename);
+    if (!(options & EDITOR_NO_SCROLL)) {
+	w->scrollbar = Cdrawvertscrollbar (catstrs (identifier, ".vsc", 0), parent,
 		x + width + 7 + WIDGET_SPACING, y, height + 6, 20, 0, 0);
-    w->scrollbar = Cwidget (scroll);
-    w->scrollbar->editor = (void *) w;
-
+	w->scrollbar->editor = (void *) w;
+    }
     return w;
 }
 
@@ -131,40 +129,6 @@ void update_scroll_bar (WEdit * e)
     }
 }
 
-
-void Cedit_update_screen (WEdit * e)
-{
-    XEvent ev;
-
-    if (!e)
-	return;
-
-    edit_scroll_screen_over_cursor (e);
-    update_curs_row (e);
-    update_curs_col (e);
-    update_scroll_bar (e);
-    edit_status (e);
-
-/* pop all events for this window for internal handling */
-    {
-	int b = 0;
-	while (XCheckWindowEvent (CDisplay, e->widget->winid, KeyPressMask | ButtonPressMask | ButtonReleaseMask | ButtonMotionMask, &ev)) {
-	    XSendEvent (CDisplay, e->widget->winid, 0, KeyPressMask, &ev);	/* don't discard the event, but resend it */
-	    b = 1;
-	}
-	b |= CWindowPending (e->widget->scrollbar->winid);
-	if (b && !(e->force & REDRAW_CHAR_ONLY) && !(e->force & REDRAW_COMPLETELY)) {
-	    e->force |= REDRAW_PAGE;
-	    return;
-	}
-    }
-
-    if (e->force & REDRAW_COMPLETELY)
-	e->force |= REDRAW_PAGE;
-    edit_render_keypress (e);
-}
-
-
 /* returns the position in the edit buffer of a window click */
 long edit_get_click_pos (WEdit * edit, int x, int y)
 {
@@ -185,8 +149,8 @@ long edit_get_click_pos (WEdit * edit, int x, int y)
 
 void edit_translate_xy (int xs, int ys, int *x, int *y)
 {
-    *x = (xs - EDIT_TEXT_HORIZONTAL_OFFSET) / TEXT_M_WIDTH + 1;
-    *y = (ys - EDIT_TEXT_VERTICAL_OFFSET) / TEXT_PIX_PER_LINE + 1;
+    *x = xs - EDIT_TEXT_HORIZONTAL_OFFSET;
+    *y = (ys - EDIT_TEXT_VERTICAL_OFFSET - option_text_line_spacing / 2 - 1) / TEXT_PIX_PER_LINE + 1;
 }
 
 static int just_dropped_something = 0;
@@ -245,18 +209,31 @@ static void edit_mouse_mark (WEdit * edit, XEvent * event, CEvent * ce)
 	    edit_mark_cmd (edit, 0);
 	    edit_left_word_move (edit);
 	    edit_mark_cmd (edit, 0);
+	    state = 0;
 	}
 	mouse_redraw (edit, click);
     } else if (event->type == ButtonRelease && state > 0 && win_press == edit->widget->winid) {
 	int x, y;
+	long start_mark, end_mark;
 	edit_translate_xy (event->xbutton.x, event->xbutton.y, &x, &y);
 	click = edit_get_click_pos (edit, x, y);
-	state = 0;
 	edit_cursor_move (edit, click - edit->curs1);
-	if (edit->mark2 < 0)
-	    edit_mark_cmd (edit, 0);
-	if (edit->mark2 != edit->mark1)
-	    XSetSelectionOwner (CDisplay, XA_PRIMARY, edit->widget->winid, event->xbutton.time);
+	if (state == 2)
+	    goto unhighlight;
+	if (!eval_marks (edit, &start_mark, &end_mark)) {
+	    if (start_mark <= click && click < end_mark) {	/* if clicked on highlighted text */
+	      unhighlight:
+		state = 0;
+		if (edit->mark2 < 0)
+		    edit_mark_cmd (edit, 0);
+		else
+		    edit_mark_cmd (edit, 1);
+		if (edit->mark2 != edit->mark1) {
+		    edit_get_selection (edit);
+		    XSetSelectionOwner (CDisplay, XA_PRIMARY, edit->widget->winid, event->xbutton.time);
+		}
+	    }
+	}
 	mouse_redraw (edit, click);
     } else if (event->type == MotionNotify && state > 0 && win_press == edit->widget->winid) {
 	int x, y;
@@ -273,36 +250,33 @@ static void edit_mouse_mark (WEdit * edit, XEvent * event, CEvent * ce)
 	mouse_redraw (edit, click);
     }
 }
-    
+
 void linkscrollbartoeditor (CWidget * w, CWidget * editor, XEvent * xevent, CEvent * cwevent, int whichscrbutton)
 {
-    int redrawtext = 0, i;
+    int i, start_line;
     WEdit *e;
     e = editor->editor;
     if (!e)
 	return;
-    if (w->firstline > 65535)
-	w->firstline = 65535;
-    if (xevent->type == MotionNotify && whichscrbutton == 3) {
+    if (!e->widget->scrollbar)
+	return;
+    start_line = e->start_line;
+    w->firstline = min (65535, w->firstline);
+    if ((xevent->type == ButtonRelease || xevent->type == MotionNotify) && whichscrbutton == 3) {
 	edit_move_display (e, (double) w->firstline * e->total_lines / 65535.0 + 1);
-	redrawtext = 1;
     } else if (xevent->type == ButtonPress && (cwevent->button == Button1 || cwevent->button == Button2)) {
 	switch (whichscrbutton) {
 	case 1:
 	    edit_move_display (e, e->start_line - e->num_widget_lines + 1);
-	    redrawtext = 1;
 	    break;
 	case 2:
 	    edit_move_display (e, e->start_line - 1);
-	    redrawtext = 1;
 	    break;
 	case 5:
 	    edit_move_display (e, e->start_line + 1);
-	    redrawtext = 1;
 	    break;
 	case 4:
 	    edit_move_display (e, e->start_line + e->num_widget_lines - 1);
-	    redrawtext = 1;
 	    break;
 	}
     }
@@ -317,38 +291,31 @@ void linkscrollbartoeditor (CWidget * w, CWidget * editor, XEvent * xevent, CEve
 	w->numlines = (double) 65535.0 *i / (e->total_lines + 1);
     else
 	w->numlines = 65535;
-    if (redrawtext) {
+    if (start_line != e->start_line) {
 	e->force |= REDRAW_PAGE;
 	Csetcursor (0, 0, 0, 0, 0, 0, 0, 0, 0);
-	{
-	    int b = 0;
-	    XEvent ev;
-	    while (XCheckWindowEvent (CDisplay, xevent->xany.window, ButtonMotionMask, &ev)) {
-		e->force |= REDRAW_PAGE;
-		XSendEvent (CDisplay, e->widget->winid, 0, KeyPressMask, &ev);	/* don't discard the event, but resend it */
-		b = 1;
-	    }
-	    if (b || CWindowPending (w->winid))
-		return;
-	}
+	if (CCheckWindowEvent (xevent->xany.window, ButtonReleaseMask | ButtonMotionMask, 0))
+	    return;
+    }
+    if (e->force) {
 	edit_render_keypress (e);
 	edit_status (e);
     }
 }
 
-#define Xdisplay CDisplay
-
 /* 
-   This function comes from rxvt-2.21b1/src/screen.c by
+   This section comes from rxvt-2.21b1/src/screen.c by
    Robert Nation <nation@rocket.sanders.lockheed.com> &
    mods by mj olesen <olesen@me.QueensU.CA>
+
+   Changes made for cooledit
  */
 void selection_send (XSelectionRequestEvent * rq)
 {
     XEvent ev;
     static Atom xa_targets = None;
     if (xa_targets == None)
-	xa_targets = XInternAtom (Xdisplay, "TARGETS", False);
+	xa_targets = XInternAtom (CDisplay, "TARGETS", False);
 
     ev.xselection.type = SelectionNotify;
     ev.xselection.property = None;
@@ -371,92 +338,141 @@ void selection_send (XSelectionRequestEvent * rq)
 	target_list[0] = (Atom32) xa_targets;
 	target_list[1] = (Atom32) XA_STRING;
 
-	XChangeProperty (Xdisplay, rq->requestor, rq->property,
+	XChangeProperty (CDisplay, rq->requestor, rq->property,
 		xa_targets, 8 * sizeof (target_list[0]), PropModeReplace,
-			 (char *) target_list,
+			 (unsigned char *) target_list,
 			 sizeof (target_list) / sizeof (target_list[0]));
 	ev.xselection.property = rq->property;
     } else if (rq->target == XA_STRING) {
-	XChangeProperty (Xdisplay, rq->requestor, rq->property,
+	XChangeProperty (CDisplay, rq->requestor, rq->property,
 			 XA_STRING, 8, PropModeReplace,
 			 selection.text, selection.len);
 	ev.xselection.property = rq->property;
     }
-    XSendEvent (Xdisplay, rq->requestor, False, 0, &ev);
+    XSendEvent (CDisplay, rq->requestor, False, 0, &ev);
 }
 
+/*{{{ paste selection */
+
+void paste_text (WEdit *edit, unsigned char *data, unsigned int nitems);
+
+/*
+ * Respond to a notification that a primary selection has been sent
+ */
+void selection_paste (WEdit * edit, Window win, unsigned prop, int delete)
+{
+    long nread;
+    unsigned long bytes_after;
+
+    if (prop == None)
+	return;
+
+    nread = 0;
+    do {
+	unsigned char *data;
+	Atom actual_type;
+	int actual_fmt;
+	unsigned long nitems;
+
+	if (XGetWindowProperty (CDisplay, win, prop,
+				nread / 4, 65536, delete,
+			      AnyPropertyType, &actual_type, &actual_fmt,
+				&nitems, &bytes_after,
+				&data) != Success) {
+	    XFree (data);
+	    return;
+	}
+	nread += nitems;
+	paste_text (edit, data, nitems);
+
+	XFree (data);
+    } while (bytes_after);
+}
+/*}}} */
+
+void selection_clear (void)
+{
+    selection.text = 0;
+    selection.len = 0;
+}
+
+void Cedit_update_screen (WEdit *e)
+{
+    if (!e)
+	return;
+    if (!e->force)
+	return;
+
+    edit_scroll_screen_over_cursor (e);
+    update_curs_row (e);
+    update_curs_col (e);
+    update_scroll_bar (e);
+    edit_status (e);
+
+    if (e->force & REDRAW_COMPLETELY)
+	e->force |= REDRAW_PAGE;
+
+/* pop all events for this window for internal handling */
+    if (e->force & (REDRAW_CHAR_ONLY | REDRAW_COMPLETELY)) {
+	edit_render_keypress (e);
+    } else if (CCheckWindowEvent (e->widget->winid, KeyPressMask | ButtonPressMask | ButtonReleaseMask | ButtonMotionMask, 0)) {
+	e->force |= REDRAW_PAGE;
+	return;
+    } else {
+	edit_render_keypress (e);
+    }
+}
 
 #define dnd_null_term_type(d) \
 	((d) == DndFile || (d) == DndText || (d) == DndDir || \
 	(d) == DndLink || (d) == DndExe || (d) == DndURL)
 
-int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
+/* handles drag and drop */
+void handle_client_message (CWidget * w, XEvent * xevent, CEvent * cwevent)
 {
+    int data_type;
+    unsigned char *data;
+    unsigned long size;
+    int xs, ys;
+    int x, y, r;
+    long click;
+    unsigned int state;
+    long start_mark = 0, end_mark = 0;
     WEdit *e = w->editor;
-    char xlat;
-    KeySym key;
-    long k = 1;
-    int cmd, ch;
-    static int old_tab_spacing = -1;
-
-    if (!e)
-	return 0;
-    if (!w->scrollbar)
-	return 0;
-
-    if (old_tab_spacing != option_tab_spacing)
-	e->force |= REDRAW_COMPLETELY;
-    old_tab_spacing = option_tab_spacing;
-
-    switch (xevent->type) {
-    case SelectionRequest:
-	edit_get_selection (e);
-	selection_send (&(xevent->xselectionrequest));
-	break;
-    case ClientMessage:{
-	    int data_type;
-	    unsigned char *data;
-	    unsigned long size;
-	    int xs, ys;
-	    int x, y, r;
-	    long click;
-	    unsigned int state;
-	    long start_mark = 0, end_mark = 0;
 
 /* see just below for a comment on what this is for: */
-	    if (CIsDropAcknowledge (xevent, &state) != DndNotDnd) {
-		if (!(state & Button1Mask) && just_dropped_something) {
-		    push_action (e, KEY_PRESS + e->start_display);
-		    edit_block_delete_cmd (e);
-		    Cedit_update_screen (e);
-		}
-		break;
-	    }
-	    data_type = CGetDrop (xevent, &data, &size, &xs, &ys);
-	    if (data_type == DndNotDnd || xs < 0 || ys < 0 || xs >= w->width || ys >= w->height)
-		break;
-	    edit_translate_xy (xs, ys, &x, &y);
-	    click = edit_get_click_pos (e, x, y);
+    if (CIsDropAcknowledge (xevent, &state) != DndNotDnd) {
+	if (!(state & Button1Mask) && just_dropped_something) {
+	    push_action (e, KEY_PRESS + e->start_display);
+	    edit_block_delete_cmd (e);
+	}
+	return;
+    }
+    data_type = CGetDrop (xevent, &data, &size, &xs, &ys);
+    if (data_type == DndNotDnd || xs < 0 || ys < 0 || xs >= w->width || ys >= w->height)
+	return;
+    edit_translate_xy (xs, ys, &x, &y);
+    click = edit_get_click_pos (e, x, y);
 
-	    r = eval_marks (e, &start_mark, &end_mark);
-	    if (r || click < start_mark || click >= end_mark) {		/* musn't be able to drop into a block,
-									   otherwise a single click will copy a block: */
-		push_action (e, KEY_PRESS + e->start_display);
-		edit_cursor_move (e, click - e->curs1);
-		if (data_type == DndFile) {
-		    edit_insert_file (e, data);
-		} else if (data_type != DndFiles) {
-		    if (dnd_null_term_type (data_type)) {
-			int len;
-			len = strlen (data);
-			size = min (len, size);
-		    }
-		    while (size--)
-			edit_insert_ahead (e, data[size]);
-		} else {
-		    while (size--)
-			edit_insert_ahead (e, data[size] ? data[size] : '\n');
-		}
+    r = eval_marks (e, &start_mark, &end_mark);
+    if (r || click < start_mark || click >= end_mark) {		/* musn't be able to drop into a block,
+								   otherwise a single click will copy a block: */
+	push_action (e, KEY_PRESS + e->start_display);
+	edit_cursor_move (e, click - e->curs1);
+	if (data_type == DndFile) {
+	    edit_insert_file (e, (char *) data);
+	} else if (data_type != DndFiles) {
+	    if (dnd_null_term_type (data_type)) {
+		int len;
+		len = strlen ((char *) data);
+		size = min (len, size);
+	    }
+	    while (size--)
+		edit_insert_ahead (e, data[size]);
+	} else {
+	    while (size--)
+		edit_insert_ahead (e, data[size] ? data[size] : '\n');
+	}
 /* The drop has now been successfully recieved. We can now send an acknowledge
    event back to the window that send the data. When this window recieves
    the acknowledge event, the app can decide whether or not to delete the data.
@@ -468,46 +484,81 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
    (for example) was passed to us, since the sender might take this to
    mean that all those file can be deleted! The two types we can acknowledge
    are: */
-		if (data_type == DndText || data_type == DndRawData)
-		    CDropAcknowledge (xevent);
-		e->force |= REDRAW_PAGE;
-		Cedit_update_screen (e);
-		break;
-	    } else {
-		push_action (e, KEY_PRESS + e->start_display);
-		edit_push_markers (e);
-		edit_set_markers (e, 0, 0);
-		e->force |= REDRAW_PAGE;
-		Cedit_update_screen (e);
-	    }
+	if (data_type == DndText || data_type == DndRawData)
+	    CDropAcknowledge (xevent);
+	e->force |= REDRAW_PAGE;
+	return;
+    }
+}
+
+int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
+{
+    WEdit *e = w->editor;
+    char xlat;
+    KeySym key;
+    int r = 0;
+    int cmd, ch;
+    static int old_tab_spacing = -1;
+
+    if (!e)
+	return 0;
+
+    if (old_tab_spacing != option_tab_spacing)
+	e->force |= REDRAW_COMPLETELY;
+    old_tab_spacing = option_tab_spacing;
+
+    if (xevent->type == KeyPress) {
+	if (xevent->xkey.keycode == 0x31 && xevent->xkey.state == 0xD) {
+	    Csetcolor (Ccolor(18));
+	    Crect (w->winid, 0, 0, w->width, w->height);
 	}
+    }
+
+    switch (xevent->type) {
+    case SelectionNotify:
+	selection_paste (e, xevent->xselection.requestor, xevent->xselection.property, True);
+	r = 1;
+	break;
+    case SelectionRequest:
+	r = 1;
+	selection_send (&(xevent->xselectionrequest));
+	break;
+/*  case SelectionClear:   ---> This is handled by coolnext.c: CNextEvent() */
+    case ClientMessage:
+	handle_client_message (w, xevent, cwevent);
+	r = 1;
 	break;
     case ResizeNotify:
+	if (w->options & EDITOR_NO_RESIZE)
+	    break;
 	Csetwidgetsize (w->ident, w->width + xevent->xconfigure.x, w->height + xevent->xconfigure.y);
 	e->num_widget_lines = (w->height - 6) / TEXT_PIX_PER_LINE;
 	e->num_widget_columns = (w->width - 7) / TEXT_M_WIDTH;
-	Csetwidgetsize (catstrs (w->ident, ".vsc", 0), 20, w->height);
-	Csetwidgetposition (catstrs (w->ident, ".vsc", 0), (Cwidget (catstrs (w->ident, ".vsc", 0)))->x + xevent->xconfigure.x, WIDGET_SPACING + 2);
-	Csetwidgetposition (catstrs (w->ident, ".text", 0), WIDGET_SPACING + 2, (Cwidget (catstrs (w->ident, ".text", 0)))->y + xevent->xconfigure.y);
+	if (!(w->options & EDITOR_NO_SCROLL)) {
+	    Csetwidgetsize (catstrs (w->ident, ".vsc", 0), 20, w->height);
+	    Csetwidgetposition (catstrs (w->ident, ".vsc", 0), (Cwidget (catstrs (w->ident, ".vsc", 0)))->x + xevent->xconfigure.x, w->y);
+	}
+	if (!(w->options & EDITOR_NO_TEXT))
+	    Csetwidgetposition (catstrs (w->ident, ".text", 0), w->x, (Cwidget (catstrs (w->ident, ".text", 0)))->y + xevent->xconfigure.y);
 	edit_status (e);
 	break;
     case ButtonPress:
-	Cresolvebutton (xevent, cwevent);
 	CFocusWindow (w->winid);
 	edit_render_tidbits (w);
     case ButtonRelease:
     case MotionNotify:
+	Cresolvebutton (xevent, cwevent);
 	edit_mouse_mark (e, xevent, cwevent);
 	break;
     case Expose:
 	EditExposeRedraw = 1;
 	edit_render_expose (e, &(xevent->xexpose));
 	EditExposeRedraw = 0;
-	return 0;
+	return 1;
     case FocusIn:
     case FocusOut:
+	edit_render_tidbits (w);
 	e->force |= REDRAW_CHAR_ONLY;
-	Cedit_update_screen (e);
 	break;
     case KeyRelease:
 	break;
@@ -515,31 +566,29 @@ int eh_editor (CWidget * w, XEvent * xevent, CEvent * cwevent)
 	xlat = 0;
 	key = CKeySym (xevent);
 	if (!key)
-	    return 0;
-	k = (long) k *key;
+	    break;
 	cwevent->ident = w->ident;
-	cwevent->key = k;
+	cwevent->key = key;
 	cwevent->xlat = xlat;
 	cwevent->state = xevent->xkey.state;
-	if (!edit_translate_key (e, xevent->xkey.keycode, k, xevent->xkey.state, &cmd, &ch)) {
-	    KeySym c;
-	    c = CKeySym (xevent);
+	if (!edit_translate_key (e, xevent->xkey.keycode, key, xevent->xkey.state, &cmd, &ch)) {
 	    cmd = CKeySymMod (xevent);
 	    if (cmd > 0)
 		cmd = CK_Macro (cmd);
 	    else
-		return 0;
+		break;
 	}
 	cwevent->command = cmd;
-	k = edit_execute_key_command (e, cmd, ch);
-	Cedit_update_screen (e);	/* command changed display, so update display */
-	return k;
+	r = edit_execute_key_command (e, cmd, ch);
+	break;
     case EditorCommand:
-	edit_execute_key_command (e, xevent->xkey.keycode, -1);
-	Cedit_update_screen (e);
-	return 1;
+	r = edit_execute_key_command (e, xevent->xkey.keycode, -1);
+	break;
+    default:
+	return 0;
     }
-    return 0;
+    Cedit_update_screen (e);
+    return r;
 }
 
 
@@ -551,7 +600,7 @@ Dlg_head *edit_dlg;
 WMenu *edit_menubar;
 
 
-static int edit_callback (Dlg_head * h, WEdit * edit, int msg, int par);
+static int edit_callback (Dlg_head * h, void * e, int msg, int par);
 
 static int edit_mode_callback (struct Dlg_head *h, int id, int msg)
 {
@@ -621,8 +670,14 @@ int edit (const char *_file)
     int midnight_colors[4];
     char *text = 0;
 
+    if (option_backup_ext_int != -1) {
+	option_backup_ext = malloc (sizeof(int) + 1);
+	option_backup_ext[sizeof(int)] = '\0';
+	memcpy (option_backup_ext, (char *) &option_backup_ext_int, sizeof (int));
+    }
+
     if (!made_directory) {
-	mc_mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
+	mkdir (catstrs (home_dir, EDIT_DIR, 0), 0777);
 	made_directory = 1;
     }
     if (_file) {
@@ -771,7 +826,7 @@ void edit_adjust_size (Dlg_head * h)
     WEdit *edit;
     WButtonBar *edit_bar;
 
-    edit = (WEdit *) find_widget_type (h, edit_callback);
+    edit = (WEdit *) find_widget_type (h, (void *) &edit_callback);	/* <---- ansi standard violation */
     edit_bar = (WButtonBar *) edit->widget.parent->current->next->widget;
     widget_set_size (&edit->widget, 0, 0, LINES - 1, COLS);
     widget_set_size (&edit_bar->widget, LINES - 1, 0, 1, COLS);
@@ -790,15 +845,15 @@ void Cedit_update_screen (WEdit * e)
 	e->force |= REDRAW_PAGE;
 	return;
     }
-    if (e->force & REDRAW_COMPLETELY) {
+    if (e->force & REDRAW_COMPLETELY)
 	e->force |= REDRAW_PAGE;
-	redraw_labels (e->widget.parent, (Widget *)e);
-    }
     edit_render_keypress (e);
 }
 
-static int edit_callback (Dlg_head * h, WEdit * e, int msg, int par)
+static int edit_callback (Dlg_head * h, void * ve, int msg, int par)
 {
+    WEdit *e = (WEdit *) ve;
+
     switch (msg) {
     case WIDGET_INIT:
 	e->force |= REDRAW_COMPLETELY;
