@@ -41,6 +41,7 @@
 #define HAVE_LIMITS_H
 #define HAVE_FCNTL_H
 #define NO_UNISTD_H
+#pragma include_alias(<utime.h>, <sys/utime.h>)			/* Only works for MSVC */
 
 // ---------------------------------------------------------------------------
 // "Standard" Library
@@ -118,6 +119,13 @@ typedef unsigned int nlink_t;
 #define W_OK    2       /*  Test for write permission   */
 #define X_OK    1       /*  Test for execute permission */
 #define F_OK    0       /*  Test for existence of file  */
+
+
+/* Missing Errno definitions */
+#define	ELOOP		40	/* Too many symbolic links encountered */
+
+/* Missing mask definition */
+#define O_ACCMODE	  0003
 #endif  //_MSC_VER
 
 
@@ -129,6 +137,6 @@ typedef unsigned int nlink_t;
 #define pclose  _pclose
 #define pipe(p)  _pipe(p, 4096, 0x8000 /*_O_BINARY*/)
 
-#define OS2_NT 1
+#define OS2_NT  1
 
 #endif //__CONFIG_HPP
