@@ -52,12 +52,14 @@
 #include <fs.h>
 #include <util.h>
 
-char *get_owner (int uid)
+char *
+get_owner (int uid)
 {
     return "none";
 }
 
-char *get_group (int gid)
+char *
+get_group (int gid)
 {
     return "none";
 }
@@ -72,17 +74,20 @@ static int old_error;		/* File descriptor of old standard error */
 /* Creates a pipe to hold standard error for a later analysis. */
 /* The pipe can hold 4096 bytes. Make sure no more is written */
 /* or a deadlock might occur. */
-void open_error_pipe (void)
+void 
+open_error_pipe (void)
 {
    return;
 }
 
-void close_error_pipe (int error, char *text)
+void 
+close_error_pipe (int error, char *text)
 {
    return;
 }
 
-void check_error_pipe (void)
+void 
+check_error_pipe (void)
 {
     char error[MAX_PIPE_SIZE];
     int len = 0;
@@ -104,7 +109,8 @@ void check_error_pipe (void)
 }
 
 
-static int StartWindowsProg(char *name, SHORT type)
+static int 
+StartWindowsProg (char *name, SHORT type)
 {
    PROGDETAILS  pDetails;
 
@@ -142,14 +148,16 @@ static int StartWindowsProg(char *name, SHORT type)
 }
 
 
-static int os2_system (int as_shell_command, const char *shell, const char *command, const char *parm);
+static int 
+os2_system (int as_shell_command, const char *shell, const char *command, const char *parm);
 
 /* 
   as_shell_command = 1: If a program is started during input line, CTRL-O
                         or RETURN 
                    = 0: F3, F4
 */
-int my_system (int as_shell_command, const char *shell, const char *command)
+int 
+my_system (int as_shell_command, const char *shell, const char *command)
 {
    char *sh;            /* This is the shell -- always! */
    char *cmd;           /* This is the command (only the command) */
@@ -195,7 +203,8 @@ int my_system (int as_shell_command, const char *shell, const char *command)
    os2_system (as_shell_command, sh, cmd, parm);
 }
 
-static int ux_startp (const char *shell, const char *command, const char *parm) 
+static int 
+ux_startp (const char *shell, const char *command, const char *parm) 
 {
     if (parm) {
          spawnlp (P_WAIT, 
@@ -217,7 +226,8 @@ static int ux_startp (const char *shell, const char *command, const char *parm)
 }
 
 
-static int os2_system (int as_shell_command, const char *shell, const char *command, const char *parm)
+static int 
+os2_system (int as_shell_command, const char *shell, const char *command, const char *parm)
 {
    register int i, j;
    ULONG        AppType = 0;                    /* Application type flags (returned) */
@@ -356,7 +366,8 @@ char *tilde_expand (char *directory)
     return strdup (directory);
 }
 
-void sleep(unsigned long dwMiliSecs)
+void 
+sleep(unsigned long dwMiliSecs)
 {
     DosSleep(dwMiliSecs);
 }
@@ -370,7 +381,8 @@ void sleep(unsigned long dwMiliSecs)
 	Trailing BACKSLASHs are removed.
 	Non-leading `../'s and trailing `..'s are handled by removing
 	portions of the path. */
-char *canonicalize_pathname (char *path)
+char *
+canonicalize_pathname (char *path)
 {
     int i, start;
     char stub_char;
@@ -447,7 +459,8 @@ char *canonicalize_pathname (char *path)
 }
 
 
-void my_statfs (struct my_statfs *myfs_stats, char *path)
+void 
+my_statfs (struct my_statfs *myfs_stats, char *path)
 {
     PFSALLOCATE pBuf;
     PFSINFO     pFsInfo;
@@ -517,23 +530,27 @@ void my_statfs (struct my_statfs *myfs_stats, char *path)
     free(pFsInfo);
 }
 
-int gettimeofday (struct timeval* tvp, void *p)
+int 
+gettimeofday (struct timeval* tvp, void *p)
 {
    DATETIME     pdt = {0};
-   if (p != NULL)		// what is "p"?
+   if (p != NULL)		/* what is "p"? */
 	return 0;	
 	
-    // Since MC only calls this func from get_random_hint we return 
-    // some value, not exactly the "correct" one
+    /* Since MC only calls this func from get_random_hint we return 
+     * some value, not exactly the "correct" one
+     */
     DosGetDateTime(&pdt);
     tvp->tv_usec = (pdt.hours * 60 + pdt.minutes) * 60 + pdt.seconds;
-    tvp->tv_sec = tvp->tv_usec * 1000 + pdt.hundredths * 10; 	// Number of milliseconds since Windows started
+    /* Number of milliseconds since Windows started */
+    tvp->tv_sec = tvp->tv_usec * 1000 + pdt.hundredths * 10;
     return 0;
 }
 
-// FAKE funcs
+/* FAKE functions */
 
-int look_for_exe(const char* pathname)
+int 
+look_for_exe(const char* pathname)
 {
    int j;
    char *p;
@@ -557,10 +574,11 @@ int look_for_exe(const char* pathname)
 }
 
 /* lstat - Because of symlinks in Unix, stat will give info 
-	   on the file pointed to and lstat on the symlink itself.
-	   We have no such a difference/trouble.
+ *   on the file pointed to and lstat on the symlink itself.
+ *   We have no such a difference/trouble.
  */
-int lstat (const char* pathname, struct stat *buffer)
+int 
+lstat (const char* pathname, struct stat *buffer)
 {
    int rc = stat (pathname, buffer);
    if (rc == 0) {
@@ -574,85 +592,106 @@ int lstat (const char* pathname, struct stat *buffer)
    return rc;
 }
 
-int getuid ()	      
+int 
+getuid ()	      
 {
     return 0;
 }
 
-int getgid ()	      
+int 
+getgid ()	      
 {
     return 0;
 }
 
-int readlink (char* path, char* buf, int size)
-{
-    return -1;
-}
-int symlink (char *n1, char *n2)
-{
-    return -1;
-}
-int link (char *p1, char *p2)
-{
-    return -1;
-}
-int chown (char *path, int owner, int group)
-{
-    return -1;
-}
-int mknod (char *path, int mode, int dev)
+int 
+readlink (char* path, char* buf, int size)
 {
     return -1;
 }
 
-void init_uid_gid_cache (void)
+int 
+symlink (char *n1, char *n2)
+{
+    return -1;
+}
+
+int 
+link (char *p1, char *p2)
+{
+    return -1;
+}
+
+int 
+chown (char *path, int owner, int group)
+{
+    return -1;
+}
+
+int 
+mknod (char *path, int mode, int dev)
+{
+    return -1;
+}
+
+void 
+init_uid_gid_cache (void)
 {
     return;
 }
 
-int mc_doublepopen (int inhandle, int inlen, pid_t *the_pid, char *command, ...)
+int 
+mc_doublepopen (int inhandle, int inlen, pid_t *the_pid, char *command, ...)
 {
 	return 0;
 }
 
-int mc_doublepclose (int pipe, pid_t pid)
+int 
+mc_doublepclose (int pipe, pid_t pid)
 {
 	return 0;
 }
 
 /*hacks to get it compile, remove these after vfs works */
-char *vfs_get_current_dir (void)
+char *
+vfs_get_current_dir (void)
 {
 	return NULL;
 }
 
-int vfs_current_is_extfs (void)
+int 
+vfs_current_is_extfs (void)
 {
 	return 0;
 }
 
-int vfs_file_is_ftp (char *filename)
+int 
+vfs_file_is_ftp (char *filename)
 {
 	return 0;
 }
 
-int mc_utime (char *path, struct utimbuf *times)
+int
+mc_utime (char *path, struct utimbuf *times)
 {
 	return 0;
 }
 
 
-void extfs_run (char *file)
+void
+extfs_run (char *file)
 {
    return;
 }
 
-int geteuid(void)
+int
+geteuid(void)
 {
    return 0;
 }
 
-char *getcwd(char *buffer, int size)
+char *
+getcwd(char *buffer, int size)
 {
    APIRET rc;
    ULONG  lgh = size;
@@ -679,34 +718,38 @@ char *getcwd(char *buffer, int size)
    } /* endif */
 }
 
-int mc_chdir(char *pathname)
+int
+mc_chdir(char *pathname)
 {
    APIRET       ret;
    register int lgh = strlen(pathname);
-   // Set the current drive
+
+   /* Set the current drive */
    if (lgh == 0) {
       return -1;
    } else {
-      // First set the default drive
+      /* First set the default drive */
       if (lgh > 1) {
          if (pathname[1] == ':') {
              ret = DosSetDefaultDisk(toupper(pathname[0]) - 'A' + 1); 
          } /* endif */
       } /* endif */
-      // After that, set the current dir!
+      /* After that, set the current dir! */
       ret = DosSetCurrentDir(pathname);
    } /* endif */
    return ret;
 }
  
-int mc_chmod(char *pathName, int unxmode)
+int
+mc_chmod(char *pathName, int unxmode)
 {
    /* OS/2 does not need S_REG */
    int os2Mode = unxmode & 0x0FFF;
    return chmod(pathName, os2Mode);
 }
 
-static int conv_os2_unx_rc(int os2rc)
+static int
+conv_os2_unx_rc(int os2rc)
 {
    int errCode;
    switch (os2rc) {
@@ -731,12 +774,14 @@ static int conv_os2_unx_rc(int os2rc)
    return errCode;
 }
 
-int mc_open (char *file, int flags, int pmode)
+int
+mc_open (char *file, int flags, int pmode)
 {
     return open(file, (flags | O_BINARY), pmode);
 }
 
-int mc_unlink(char *pathName)
+int
+mc_unlink(char *pathName)
 {
    /* Use OS/2 API to delete a file, if the file is set as read-only, 
       the file will be deleted without asking the user! */
@@ -760,7 +805,8 @@ int mc_unlink(char *pathName)
    } /* endif */
 }
 
-int strncasecmp (char *s, char *d, int count)
+int
+strncasecmp (char *s, char *d, int count)
 {
     register char result;
 
@@ -776,12 +822,14 @@ int strncasecmp (char *s, char *d, int count)
     return result;
 }
 
-FILE *_popen(const char *cmd, const char *mode)
+FILE *
+_popen(const char *cmd, const char *mode)
 {
    return NULL;
 }
 
-int _pclose(FILE *file)
+int
+_pclose(FILE *file)
 {
    return 1;
 }
@@ -813,12 +861,14 @@ get_default_editor (void)
    Get the default shell for the current hardware platform
    TODO: Get the value of %OS2_SHELL% or %SHELL%: which one?
 */
-char* get_default_shell()
+char *
+get_default_shell()
 {
     return getenv ("COMSPEC");
 }
 
-int getpid (void)
+int
+getpid (void)
 {
 	return 0;
 }
@@ -831,3 +881,13 @@ errno_dir_not_empty (int err)
     return 0;
 }
 
+char *
+get_mc_lib_dir ()
+{
+   char *mchome = getenv("MCHOME");
+   if (!mchome) {
+      return "D:\\mc";
+   } else {
+      return mchome;
+   } /* endif */
+}
