@@ -1,171 +1,182 @@
 #ifndef DAVIS_SLANG_H_
 #define DAVIS_SLANG_H_
+/* -*- mode: C; mode: fold; -*- */
 /* Copyright (c) 1992, 1995 John E. Davis
  * All rights reserved.
  * 
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
+#define SLANG_VERSION 9938
+/*{{{ System Dependent Macros and Typedefs */
 
-/* Midnight Commander change: some Unices do not define this, and slang
- * requires it:
- */
-
-#ifndef unix
-#    define unix
-#endif
-
-#define SLANG_VERSION 9929
 #if defined(__WATCOMC__) && !defined(__QNX__)
-#include <slconfig.h>
-#endif
+#  ifndef msdos
+#    define msdos
+#  endif
+#  ifndef  DOS386
+#    define  DOS386
+#  endif
+#  ifndef FLOAT_TYPE
+#    define FLOAT_TYPE
+#  endif
+#  ifndef pc_system
+#    define pc_system
+#  endif
+#endif /* __watcomc__ */
 
-/* Configure the Linux unicode support */
-#ifdef linux
-# include <sys/types.h>
-# include <linux/kd.h>
-# ifdef GIO_UNIMAP
-# define linux_unicode
+#ifdef unix
+# ifndef __unix__
+#  define __unix__ 1
 # endif
 #endif
 
-
-
-#ifdef FLOAT_TYPE
-#ifndef USE_DOUBLE
-#define USE_DOUBLE
-#endif
+#ifndef __GO32__
+# ifdef __unix__
+#  define REAL_UNIX_SYSTEM
+# endif
+# ifdef IS_AIX
+#  define REAL_UNIX_SYSTEM
+# endif
 #endif
 
-#ifdef pc_system
-#  ifndef  HAVE_MEMCPY
-#    define  HAVE_MEMCPY
-#  endif
-#  ifndef  HAVE_MEMCMP
-#    define  HAVE_MEMCMP
-#  endif
-#  ifndef  HAVE_MEMSET
-#    define  HAVE_MEMSET
-#  endif
-#  ifndef  HAVE_MEMCHR
-#    define  HAVE_MEMCHR
-#  endif
+/* Set of the various defines for pc systems.  This includes OS/2 */
+#ifdef __MSDOS__
+# ifndef msdos
+#   define msdos
+# endif
+# ifndef pc_system
+#   define pc_system
+# endif
 #endif
- 
+
+#ifdef __GO32__
+# ifndef pc_system
+#   define pc_system
+# endif
+# ifdef REAL_UNIX_SYSTEM
+#   undef REAL_UNIX_SYSTEM
+# endif
+#endif
+
+#if defined(__EMX__) && defined(OS2)
+# ifndef pc_system
+#   define pc_system
+# endif
+# ifndef __os2__
+#   define __os2__
+# endif
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
-
+#if 0
+}
+#endif
 /* ---------------------------- Generic Macros ----------------------------- */
 
 /*  __SC__ is defined for Symantec C++
    DOS386 is defined for -mx memory model, 32 bit DOS extender. */
 
 #ifdef VOID
-#undef VOID
+# undef VOID
 #endif
 
-#if defined(msdos) && !defined(DOS386) & !defined(WIN32)
-#  ifdef __SC__
-#    include <dos.h>
-#  endif
+#if defined(msdos) && !defined(DOS386) & !defined(__WIN32__) && !defined(__GO32__)
+# ifdef __SC__
+#  include <dos.h>
+# endif
+   typedef void *VOID_STAR;
+# define VOID void
+# include <alloc.h>
+#else
+# if defined (__cplusplus) || defined(__STDC__)
    typedef void *VOID_STAR;
 #  define VOID void
-#  include <alloc.h>
-#else
-#  ifndef NO_STDLIB_H
-#    include <stdlib.h>
-#  endif
-#  if defined (__cplusplus) || defined(__STDC__)
-   typedef void *VOID_STAR;
-#    define VOID void
-#  else
-     typedef unsigned char *VOID_STAR;
-#    define VOID unsigned char
-#  endif
+# else
+   typedef unsigned char *VOID_STAR;
+#  define VOID unsigned char
+# endif
 #endif
    
-#ifndef MALLOC_DEBUG
-#if defined(msdos) && !defined(DOS386)
-#  define SLFREE(buf)  farfree((void far *)(buf))
-#  define SLMALLOC(x) farmalloc((unsigned long) (x))
-#  define SLREALLOC(buf, n) farrealloc((void far *) (buf), (unsigned long) (n))
-#  define SLCALLOC(n, m) farcalloc((unsigned long) (n), (unsigned long) (m))
+#if 1
+   typedef int (*FVOID_STAR)(void);
 #else
-#  if defined(VMS) && !defined(__DECC)
-#    define SLFREE VAXC$FREE_OPT
-#    define SLMALLOC VAXC$MALLOC_OPT
-#    define SLREALLOC VAXC$REALLOC_OPT
-#    define SLCALLOC VAXC$CALLOC_OPT
-#  else
-#    define SLFREE(x) free((char *)(x))
-#    define SLMALLOC malloc
-#ifdef __cplusplus
-#    define SLREALLOC(p,n) realloc((malloc_t) (p), n)
-#else
-#    define SLREALLOC realloc
+# define FVOID_STAR VOID_STAR
 #endif
-#    define SLCALLOC calloc
+   
+#if defined(msdos) && !defined(DOS386) && !defined(__GO32__) && !defined(__WIN32__)
+# define SLFREE(buf)  farfree((void far *)(buf))
+# define SLMALLOC(x) farmalloc((unsigned long) (x))
+# define SLREALLOC(buf, n) farrealloc((void far *) (buf), (unsigned long) (n))
+# define SLCALLOC(n, m) farcalloc((unsigned long) (n), (unsigned long) (m))
+#else
+# if defined(VMS) && !defined(__DECC)
+#  define SLFREE VAXC$FREE_OPT
+#  define SLMALLOC VAXC$MALLOC_OPT
+#  define SLREALLOC VAXC$REALLOC_OPT
+#  define SLCALLOC VAXC$CALLOC_OPT
+# else
+#  define SLFREE(x) free((char *)(x))
+#  define SLMALLOC malloc
+#  if defined(__cplusplus) && !defined(__BEOS__)
+#   define SLREALLOC(p,n) realloc((malloc_t) (p), (n))
+#  else
+#   define SLREALLOC realloc
 #  endif
+#  define SLCALLOC calloc
+# endif
 #endif
 
-#else
-   /* MALLOC_DEBUG */
+#ifdef SL_MALLOC_DEBUG
+# undef SLMALLOC
+# undef SLCALLOC
+# undef SLREALLOC
+# undef SLFREE
 # define SLMALLOC(x) SLdebug_malloc((unsigned long) (x))
 # define SLFREE(x) SLdebug_free((unsigned char *)(x))
 # define SLCALLOC(n, m) SLdebug_calloc((unsigned long) (n), (unsigned long)(m))
 # define SLREALLOC(p, x) SLdebug_realloc((unsigned char *)(p), (unsigned long)(x))
-
+#endif /* SL_MALLOC_DEBUG */
+   
    extern unsigned char *SLdebug_malloc (unsigned long);
    extern unsigned char *SLdebug_calloc (unsigned long, unsigned long);
    extern unsigned char *SLdebug_realloc (unsigned char *, unsigned long);
    extern void SLdebug_free (unsigned char *);
    extern void SLmalloc_dump_statistics (void);
+   extern char *SLstrcpy(register char *, register char *);
+   extern int SLstrcmp(register char *, register char *);
+   extern char *SLstrncpy(char *, register char *, register  int);
    
-#endif /* MALLOC_DEBUG */
+   extern void SLmemset (char *, char, int);
+   extern char *SLmemchr (register char *, register char, register int);
+   extern char *SLmemcpy (char *, char *, int);
+   extern int SLmemcmp (char *, char *, int);
 
-/* If your compiler does not have memory.h, ignore this line */
-#if !defined(VMS) && !defined(__WATCOMC__)
-#include <memory.h>
-#endif 
-
-#ifdef HAVE_MEMSET
-#define MEMSET memset
-#else 
-#define MEMSET jed_memset
-extern void jed_memset(char *, char, int);
+#ifdef float64
+# undef float64
+#endif
+   
+#ifndef FLOAT64_TYPEDEFED
+# define FLOAT64_TYPEDEFED
+  typedef double float64;
 #endif
 
-#ifdef HAVE_MEMCHR
-#define MEMCHR memchr
-#else 
-#define MEMCHR jed_memchr
-extern char *jed_memchr(register char *, register char, register int);
-#endif
 
-#ifdef HAVE_MEMCPY
-#define MEMCPY memcpy
-#else
-#define MEMCPY jed_memcpy
-extern char *jed_memcpy(char *, char *, int);
-#endif
-
-#ifdef HAS_MEMCMP
-#define MEMCMP memcmp
-#else
-#define MEMCMP jed_memcmp
-extern int jed_memcmp(char *, char *, int);
-#endif
-
-/* -------------------  S-Lang Stuff -------------------------------*/
-
-#define FLOAT double
+/*}}}*/
 
+/*{{{ Interpreter Typedefs */
 
 #define SLANG_MAX_NAME_LEN 30
 /* maximum length of an identifier */
 /* first char in identifiers is the hash */
 
+   /* Note that long is used for addresses instead of void *.  The reason for
+    * this is that I have a gut feeling that sizeof (long) > sizeof(void *)
+    * on some machines.  This is certainly the case for MSDOS where addresses
+    * can be 16 bit.
+    */
 typedef struct SLang_Name_Type
   {
 #ifdef SLANG_STATS
@@ -174,6 +185,18 @@ typedef struct SLang_Name_Type
      char name[SLANG_MAX_NAME_LEN + 2]; /* [0] is hash */
 
      unsigned char sub_type;
+     
+/* Values for main_type may be as follows.  The particlular values are
+ * for compatability.
+ */
+#define SLANG_LVARIABLE		0x01
+#define SLANG_INTRINSIC 	0x06
+#define SLANG_FUNCTION  	0x07
+#define SLANG_GVARIABLE 	0x0D
+#define SLANG_IVARIABLE 	0x0E           /* intrinsic variables */
+/* Note!!! For Macro MAKE_VARIABLE below to work, SLANG_IVARIABLE Must
+   be 1 less than SLANG_RVARIABLE!!! */
+#define SLANG_RVARIABLE		0x0F	       /* read only variable */
      unsigned char main_type;
      long addr;
   }
@@ -200,6 +223,12 @@ typedef struct SLang_Load_Type
    int top_level;		       /* 1 if at top level of parsing */
 } SLang_Load_Type;
 
+#if defined(ultrix) && !defined(__GNUC__)
+# ifndef NO_PROTOTYPES
+#  define NO_PROTOTYPES
+# endif
+#endif
+   
 #ifndef NO_PROTOTYPES 
 # define _PROTO(x) x
 #else
@@ -217,7 +246,7 @@ typedef struct SL_OOBinary_Type
     * return zero if the operation is not defined.
     */
    int (*binary_function)_PROTO((int, unsigned char, unsigned char, 
-				  VOID *, VOID *));
+				  VOID_STAR, VOID_STAR));
 
    struct SL_OOBinary_Type *next;
 }
@@ -226,15 +255,15 @@ SL_OOBinary_Type;
 typedef struct
 {
    /* Methods */
-   void (*destroy)_PROTO((VOID *));
+   void (*destroy)_PROTO((VOID_STAR));
    /* called to delete/free the object */
-   char *(*string)_PROTO((VOID *));
+   char *(*string)_PROTO((VOID_STAR));
    /* returns a string representation of the object */
-   int (*unary_function)_PROTO((int, unsigned char, VOID *));
+   int (*unary_function)_PROTO((int, unsigned char, VOID_STAR));
    /* unary operation function */
    SL_OOBinary_Type *binary_ops;
    
-   int (*copy_function)_PROTO((unsigned char, VOID *));
+   int (*copy_function)_PROTO((unsigned char, VOID_STAR));
    /* This function is called do make a copy of the object */
 } SLang_Class_Type;
 
@@ -252,6 +281,9 @@ typedef struct
 SLuser_Object_Type;
 
    
+/*}}}*/
+/*{{{ Interpreter Function Prototypes */
+
   extern volatile int SLang_Error;
   /* Non zero if error occurs.  Must be reset to zero to continue. */
 
@@ -343,31 +375,38 @@ SLuser_Object_Type;
     * if the stack is empty or a type mismatch occurs, setting SLang_Error.
     */
 
+   extern int SLpop_string (char **);
    extern int SLang_pop_string(char **, int *);
    /* pops string *p0 from stack.  If *p1 is non-zero, the string must be
     * freed after its use.  DO NOT FREE p0 if *p1 IS ZERO! Returns 0 upon
     * success */
       
-   extern int SLang_pop_float(FLOAT *, int *, int *);
+   extern int SLang_pop_float(float64 *, int *, int *);
    /* Pops float *p1 from stack.  If *p3 is non-zero, *p1 was derived
       from the integer *p2. Returns zero upon success. */
       
    extern SLuser_Object_Type *SLang_pop_user_object (unsigned char);
    extern void SLang_free_user_object (SLuser_Object_Type *);
+   extern void SLang_free_intrinsic_user_object (SLuser_Object_Type *);
+   /* This is like SLang_free_user_object but is meant to free those
+    * that have been declared as intrinsic variables by the application.
+    * Normally an application would never need to call this.
+    */
+
    extern void SLang_push_user_object (SLuser_Object_Type *);
    extern SLuser_Object_Type *SLang_create_user_object (unsigned char);
    
-   extern int SLang_add_unary_op (unsigned char, VOID *);
-   extern int SLang_add_binary_op (unsigned char, unsigned char, VOID *);
-   extern int SLang_register_class (unsigned char, VOID *, VOID *);
-   extern int SLang_add_copy_operation (unsigned char, VOID *);
+   extern int SLang_add_unary_op (unsigned char, FVOID_STAR);
+   extern int SLang_add_binary_op (unsigned char, unsigned char, FVOID_STAR);
+   extern int SLang_register_class (unsigned char, FVOID_STAR, FVOID_STAR);
+   extern int SLang_add_copy_operation (unsigned char, FVOID_STAR);
 
    extern long *SLang_pop_pointer(unsigned char *, unsigned char *, int *);
    /* Returns a pointer to object of type *p1,*p2 on top of stack. 
       If *p3 is non-zero, the Object must be freed after use. */
 
 
-   extern void SLang_push_float(FLOAT);
+   extern void SLang_push_float(float64);
    /* Push Float onto stack */
 
    extern void SLang_push_string(char *);
@@ -453,55 +492,77 @@ extern void SLexecute_function(SLang_Name_Type *);
  * the C program.  The parameter must be non-NULL and must have been 
  * previously obtained by a call to 'SLang_get_function'.
  */
-
 extern void SLroll_stack (int *);
 /* If argument *p is positive, the top |*p| objects on the stack are rolled
-  * up.  If negative, the stack is rolled down.
-  */
+ * up.  If negative, the stack is rolled down.
+ */
+
+extern void SLmake_lut (unsigned char *, unsigned char *, unsigned char);
+
+   extern int SLang_guess_type (char *);
+
+
+/*}}}*/
+
+/*{{{ Misc Functions */
 
 extern char *SLmake_string (char *);
-extern char *SLmake_nstring (char *, int);
+extern char *SLmake_nstring (char *, unsigned int);
 /* Returns a null terminated string made from the first n characters of the
  * string.
  */
-extern void SLmake_lut (unsigned char *, unsigned char *, unsigned char);
-    
-extern int SLang_guess_type (char *);
-/* SLang tty interface */
-extern int SLtt_Baud_Rate;
-extern unsigned long SLtt_Num_Chars_Output;
+
+extern char *SLcurrent_time_string (void);
+
+extern int SLatoi(unsigned char *);
+
+extern int SLang_extract_token(char **, char *, int);
+/* returns 0 upon failure and non-zero upon success.  The first parameter 
+ * is a pointer to the input stream which this function will bump along.  
+ * The second parameter is the buffer where the token is placed.  The third 
+ * parameter is used internally by the S-Lang library and should be 0 for
+ * user applications.
+ */
+
+/*}}}*/
+
+/*{{{ SLang getkey interface Functions */
    
-#ifdef unix
+#ifdef REAL_UNIX_SYSTEM
 extern int SLang_TT_Baud_Rate;
 extern int SLang_TT_Read_FD;
 #endif
    
 extern int SLang_init_tty (int, int, int);
-/* Initializes the tty for single character input.  The first parameter *p1
- * is the value 0-255 to be used for the abort character.  If the second
- * parameter p2 is non-zero, flow control is enabled.  If the last 
- * parmeter p3 is zero, output processing is NOT turned on.  A value of 
- * zero is required for the screen management routines.
- * Returns 0 upon success.
- * In addition, if SLang_TT_Baud_Rate == 0 when this function is called, 
- * SLang will attempt to determine  the terminals baud rate.  As far as 
- * the SLang library is concerned, if SLang_TT_Baud_Rate is less than or 
- * equal to zero, the baud rate is effectively infinite.
+/* Initializes the tty for single character input.  If the first parameter *p1
+ * is in the range 0-255, it will be used for the abort character;
+ * otherwise, (unix only) if it is -1, the abort character will be the one
+ * used by the terminal.  If the second parameter p2 is non-zero, flow
+ * control is enabled.  If the last parmeter p3 is zero, output processing
+ * is NOT turned on.  A value of zero is required for the screen management
+ * routines. Returns 0 upon success. In addition, if SLang_TT_Baud_Rate ==
+ * 0 when this function is called, SLang will attempt to determine the
+ * terminals baud rate.  As far as the SLang library is concerned, if
+ * SLang_TT_Baud_Rate is less than or equal to zero, the baud rate is
+ * effectively infinite.
  */
 
 extern void SLang_reset_tty (void);
 /* Resets tty to what it was prior to a call to SLang_init_tty */
-#ifdef unix
+#ifdef REAL_UNIX_SYSTEM
 extern void SLtty_set_suspend_state (int);
    /* If non-zero argument, terminal driver will be told to react to the
     * suspend character.  If 0, it will not.
     */
+extern int (*SLang_getkey_intr_hook) (void);
 #endif
 
+#define SLANG_GETKEY_ERROR 0xFFFF
 extern unsigned int SLang_getkey (void);
-/* reads a single key from the tty.  If the read fails, 0xFFFF is returned. */
-extern void SLang_ungetkey_string (unsigned char *, int);
-extern void SLang_buffer_keystring (unsigned char *, int);
+/* reads a single key from the tty.  If the read fails,  0xFFFF is returned. */
+
+extern void SLang_ungetkey_string (unsigned char *, unsigned int);
+extern void SLang_buffer_keystring (unsigned char *, unsigned int);
 extern void SLang_ungetkey (unsigned char);
 extern void SLang_flush_input (void);
 extern int SLang_input_pending (int);
@@ -518,7 +579,14 @@ extern void SLang_set_abort_signal (void (*)(int));
  */
 
 extern volatile int SLKeyBoard_Quit;
-/* --------------- SLang_keymap interface ----------------------- */
+
+#ifdef VMS
+/* If this function returns -1, ^Y will be added to input buffer. */
+extern int (*SLtty_VMS_Ctrl_Y_Hook) (void);
+#endif
+/*}}}*/
+
+/*{{{ SLang Keymap routines */
 
 typedef struct SLKeymap_Function_Type
   {
@@ -530,8 +598,21 @@ SLKeymap_Function_Type;
 typedef struct SLang_Key_Type
   {
      unsigned char str[13];	       /* key sequence */
+#define SLKEY_F_INTERPRET	0x01
+#define SLKEY_F_INTRINSIC	0x02
+#define SLKEY_F_KEYSYM		0x03
      unsigned char type;	       /* type of function */
-     VOID *f;			       /* function to invoke */
+#ifdef SLKEYMAP_OBSOLETE
+     VOID_STAR f;			       /* function to invoke */
+#else
+     union
+       {
+	  char *s;
+	  FVOID_STAR f;
+	  unsigned int keysym;
+       }
+     f;
+#endif
      struct SLang_Key_Type *next;      /* */
   }
 SLang_Key_Type;
@@ -554,22 +635,27 @@ SLKeyMap_List_Type;
 
 extern SLKeyMap_List_Type SLKeyMap_List[SLANG_MAX_KEYMAPS];   /* these better be inited to 0! */
 
-#define SLKEY_F_INTERPRET 0x01
-#define SLKEY_F_INTRINSIC 0x02
 
 extern char *SLang_process_keystring(char *);
-extern int SLang_define_key1(char *, VOID *, unsigned char, SLKeyMap_List_Type *);
+
+#ifdef SLKEYMAP_OBSOLETE   
+extern int SLang_define_key1(char *, VOID_STAR, unsigned int, SLKeyMap_List_Type *);
 /* define key p1 in keymap p4 to invoke function p2.  If type p3 is given by
  * SLKEY_F_INTRINSIC, p2 is an intrinsic function, else it is a string to be
  * passed to the interpreter for evaluation.  The return value is important.
  * It returns 0 upon success, -1 upon malloc error, and -2 if the key is 
  * inconsistent.  SLang_Error is set upon error. */
-
+#else
+extern int SLkm_define_key (char *, FVOID_STAR, SLKeyMap_List_Type *);
+#endif
+   
 extern int SLang_define_key(char *, char *, SLKeyMap_List_Type *);
 /* Like define_key1 except that p2 is a string that is to be associated with 
  * a function in the functions field of p3.  This routine calls define_key1.
  */
 
+extern int SLkm_define_keysym (char *, unsigned int, SLKeyMap_List_Type *);
+
 extern void SLang_undefine_key(char *, SLKeyMap_List_Type *);
 
 extern SLKeyMap_List_Type *SLang_create_keymap(char *, SLKeyMap_List_Type *);
@@ -583,13 +669,23 @@ extern char *SLang_make_keystring(unsigned char *);
 extern SLang_Key_Type *SLang_do_key(SLKeyMap_List_Type *, int (*)(void));
 /* read a key using keymap p1 with getkey function p2 */
 
-extern VOID *SLang_find_key_function(char *, SLKeyMap_List_Type *);
+extern
+#ifdef SLKEYMAP_OBSOLETE
+     VOID_STAR
+#else
+     FVOID_STAR
+#endif
+     SLang_find_key_function(char *, SLKeyMap_List_Type *);
+
 extern SLKeyMap_List_Type *SLang_find_keymap(char *);
 
 extern int SLang_Last_Key_Char;
 extern int SLang_Key_TimeOut_Flag;
 
-/* --------------- SLang_read_line interface ----------------------- */
+
+/*}}}*/
+
+/*{{{ SLang Readline Interface */
 
 typedef struct SLang_Read_Line_Type
 {
@@ -619,7 +715,7 @@ typedef struct
    int dhscroll;		       /* amount to use for horiz scroll */
    char *prompt;
    
-   VOID *last_fun;		       /* last function executed by rl */
+   FVOID_STAR last_fun;		       /* last function executed by rl */
 
    /* These two contain an image of what is on the display */
    unsigned char upd_buf1[SLRL_DISPLAY_BUFFER_SIZE];
@@ -631,6 +727,8 @@ typedef struct
 
    /* tty variables */
    unsigned int flags;		       /*  */
+#define SL_RLINE_NO_ECHO	1
+#define SL_RLINE_USE_ANSI	2
    unsigned int (*getkey)(void);       /* getkey function -- required */
    void (*tt_goto_column)(int);
    void (*tt_insert)(char);
@@ -642,15 +740,7 @@ typedef struct
     */
 } SLang_RLine_Info_Type;
 
-#ifdef linux_unicode
-typedef unsigned long SLang_Char_Type;
-#else
-typedef unsigned short SLang_Char_Type;
-#endif
-extern int SLtt_Unicode;
-
 extern int SLang_RL_EOF_Char;
-#define SLRL_USE_ANSI 0x01
 
 extern SLang_Read_Line_Type * SLang_rline_save_line (SLang_RLine_Info_Type *);
 extern int SLang_init_readline (SLang_RLine_Info_Type *);
@@ -658,19 +748,13 @@ extern int SLang_read_line (SLang_RLine_Info_Type *);
 extern int SLang_rline_insert (char *);
 extern void SLrline_redraw (SLang_RLine_Info_Type *);   
 extern int SLang_Rline_Quit;
-/* ---------------------------------------------------------------------- */
-extern int SLatoi(unsigned char *);
 
-extern int SLang_extract_token(char **, char *, int);
-/* returns 0 upon failure and non-zero upon success.  The first parameter 
- * is a pointer to the input stream which this function will bump along.  
- * The second parameter is the buffer where the token is placed.  The third 
- * parameter is used internally by the S-Lang library and should be 0 for
- * user applications.
- */
+/*}}}*/
 
-/* ----------------------------------------------------------------------- */
-/* Screen Output functions */
+/*{{{ Low Level Screen Output Interface */
+
+extern unsigned long SLtt_Num_Chars_Output;
+extern int SLtt_Baud_Rate;
 
 typedef unsigned long SLtt_Char_Type;
 
@@ -686,19 +770,28 @@ extern int SLtt_Term_Cannot_Insert;
 extern int SLtt_Term_Cannot_Scroll;
 extern int SLtt_Use_Ansi_Colors;
 extern int SLtt_Ignore_Beep;
-extern int SLtt_Try_Termcap;
-extern int SLtt_Has_Alt_Charset;
+#if defined(REAL_UNIX_SYSTEM)
+extern int SLtt_Force_Keypad_Init;
+#endif
    
-#if defined(VMS) || defined(unix)
+#ifndef __GO32__
+#if defined(VMS) || defined(REAL_UNIX_SYSTEM)
 extern int SLtt_Blink_Mode;
 extern int SLtt_Use_Blink_For_ACS;
 extern int SLtt_Newline_Ok;
+extern int SLtt_Has_Alt_Charset;
+extern int SLtt_Has_Status_Line;       /* if 0, NO.  If > 0, YES, IF -1, ?? */
+# ifndef VMS
+extern int SLtt_Try_Termcap;
+# endif
+#endif
 #endif
 
 #ifdef msdos
 extern int SLtt_Msdos_Cheap_Video;
 #endif
-extern void SLtt_set_alt_char_set (int);
+   
+   
 extern int SLtt_flush_output (void);
 extern void SLtt_set_scroll_region(int, int);
 extern void SLtt_reset_scroll_region(void);
@@ -715,41 +808,46 @@ extern void SLtt_normal_video(void);
 extern void SLtt_cls(void);
 extern void SLtt_beep(void);
 extern void SLtt_reverse_index(int);
-extern void SLtt_smart_puts(SLang_Char_Type *, SLang_Char_Type *, int, int);
+extern void SLtt_smart_puts(unsigned short *, unsigned short *, int, int);
 extern void SLtt_write_string (char *);
 extern void SLtt_putchar(char);
 extern void SLtt_init_video (void);
 extern void SLtt_reset_video (void);
-extern void SLtt_want_unicode (void);
-extern unsigned short SLtt_fix_unicode (unsigned short);
 extern void SLtt_get_terminfo(void);
+extern void SLtt_get_screen_size (void);
 extern int SLtt_set_cursor_visibility (int);
-#if defined(VMS) || defined(unix)
-# ifndef __GO32__
+   
+#if defined(VMS) || defined(REAL_UNIX_SYSTEM)
 extern void SLtt_enable_cursor_keys(void);
 extern void SLtt_set_term_vtxxx(int *);
 extern void SLtt_set_color_esc (int, char *);
 extern void SLtt_wide_width(void);
 extern void SLtt_narrow_width(void);
-#  ifdef unix
+extern int SLtt_set_mouse_mode (int, int);
+extern void SLtt_set_alt_char_set (int);
+extern int SLtt_write_to_status_line (char *, int);
+extern void SLtt_disable_status_line (void);
+# ifdef REAL_UNIX_SYSTEM
    extern char *SLtt_tgetstr (char *);
    extern int SLtt_tgetnum (char *);
    extern int SLtt_tgetflag (char *);
-   extern char *SLtt_tigetstr (char *, char **);
    extern char *SLtt_tigetent (char *);
+   extern char *SLtt_tigetstr (char *, char **);
    extern int SLtt_tigetnum (char *, char **);
-   extern int SLtt_tigetflag (char *, char **);
-#  endif
 # endif
 #endif
+   
+extern SLtt_Char_Type SLtt_get_color_object (int);
 extern void SLtt_set_color_object (int, SLtt_Char_Type);
 extern void SLtt_set_color (int, char *, char *, char *);
 extern void SLtt_set_mono (int, char *, SLtt_Char_Type);
 extern void SLtt_add_color_attribute (int, SLtt_Char_Type);
+extern void SLtt_set_color_fgbg (int, SLtt_Char_Type, SLtt_Char_Type);
      
-/*-------------------------------------------------------------------------
- * Preprocessor
- */
+/*}}}*/
+
+/*{{{ SLang Preprocessor Interface */
+
 typedef struct 
 {
    int this_level;
@@ -757,6 +855,9 @@ typedef struct
    int prev_exec_level;
    char preprocess_char;
    char comment_char;
+   unsigned char flags;
+#define SLPREP_BLANK_LINES_OK	1
+#define SLPREP_COMMENT_LINES_OK	2
 }
 SLPreprocess_Type;
 
@@ -766,12 +867,12 @@ extern int SLprep_line_ok (char *, SLPreprocess_Type *);
    extern int SLdefine_for_ifdef (char *);
    /* Adds a string to the SLang #ifdef preparsing defines. SLang already
       defines MSDOS, UNIX, and VMS on the appropriate system. */
+extern int (*SLprep_exists_hook) (char *, char);
       
    
-   
+/*}}}*/
 
-/* ----------------------------------------------------------------------- */
-/* Screen management functions */
+/*{{{ SLsmg Screen Management Functions */
 
 #include <stdarg.h>
 extern void SLsmg_fill_region (int, int, int, int, unsigned char);
@@ -799,23 +900,19 @@ extern void SLsmg_refresh (void);
 extern void SLsmg_touch_lines (int, int);
 extern int SLsmg_init_smg (void);
 extern void SLsmg_reset_smg (void);
-extern SLang_Char_Type SLsmg_char_at(void);
+extern unsigned short SLsmg_char_at(void);
 extern void SLsmg_set_screen_start (int *, int *);
 extern void SLsmg_draw_hline (int);
 extern void SLsmg_draw_vline (int);
 extern void SLsmg_draw_object (int, int, unsigned char);
 extern void SLsmg_draw_box (int, int, int, int);
-extern void SLsmg_draw_double_hline (int);
-extern void SLsmg_draw_double_vline (int);
-extern void SLsmg_draw_double_box (int, int, int, int);
-extern void SLsmg_draw_unicode (int, int, unsigned short);
-extern void SLsmg_write_unicode (unsigned short);
 extern int SLsmg_get_column(void);
 extern int SLsmg_get_row(void);
-
 extern void SLsmg_forward (int);
 extern void SLsmg_write_color_chars (unsigned short *, unsigned int);
- 
+extern unsigned int SLsmg_read_raw (unsigned short *, unsigned int);
+extern unsigned int SLsmg_write_raw (unsigned short *, unsigned int);
+   
 extern int SLsmg_Display_Eight_Bit;
 extern int SLsmg_Tab_Width;
 extern int SLsmg_Newline_Moves;
@@ -828,39 +925,13 @@ extern int SLsmg_Backspace_Moves;
 # define SLSMG_URCORN_CHAR	0xBF
 # define SLSMG_LLCORN_CHAR	0xC0
 # define SLSMG_LRCORN_CHAR	0xD9
-/* There are several to choose from: 0xB0, 0xB1, and 0xB2 */
-# define SLSMG_CKBRD_CHAR	0xB0
-/* Double line chars */
-# define SLSMG_LTEE_CHAR	0xC3
 # define SLSMG_RTEE_CHAR	0xB4
-# define SLSMG_DHLINE_CHAR	0xCD
-# define SLSMG_DVLINE_CHAR	0xBA
-# define SLSMG_DULCORN_CHAR	0xC9
-# define SLSMG_DURCORN_CHAR	0xBB
-# define SLSMG_DLLCORN_CHAR	0xC8
-# define SLSMG_DLRCORN_CHAR	0xBC
-
-/* .ado: Some of the characters are used by me (Double line chars) */
-# define SLSMG_DHLINE_CHAR  0xCD
-# define SLSMG_DVLINE_CHAR  0xBA
-# define SLSMG_DULCORN_CHAR 0xC9
-# define SLSMG_DURCORN_CHAR 0xBB
-# define SLSMG_DLLCORN_CHAR 0xC8
-# define SLSMG_DLRCORN_CHAR 0xBC
-# define SLSMG_DRTEE_CHAR   0xB9
-# define SLSMG_DLTEE_CHAR   0xCC
-
-# define SLSMG_UTEE_CHAR    0xC2
-# define SLSMG_DUTEE_CHAR   0xCB
-# define SLSMG_DTEE_CHAR    0xC1
-# define SLSMG_DDTEE_CHAR   0xCA
-
+# define SLSMG_LTEE_CHAR	0xC3
+# define SLSMG_UTEE_CHAR	0xC2
+# define SLSMG_DTEE_CHAR	0xC1
 # define SLSMG_PLUS_CHAR	0xC5
-# define SLSMG_DPLUS_CHAR   0xCE
-
 /* There are several to choose from: 0xB0, 0xB1, and 0xB2 */
 # define SLSMG_CKBRD_CHAR	0xB0
-
 #else
 # define SLSMG_HLINE_CHAR	'q'
 # define SLSMG_VLINE_CHAR	'x'
@@ -873,53 +944,154 @@ extern int SLsmg_Backspace_Moves;
 # define SLSMG_LTEE_CHAR	't'
 # define SLSMG_UTEE_CHAR	'w'
 # define SLSMG_DTEE_CHAR	'v'
-# define SLSMG_CROSS_CHAR	'n'
+# define SLSMG_PLUS_CHAR	'n'
 #endif
-/* Just some characters from Unicode */
-# define SLUNI_HLINE_CHAR	0x2500
-# define SLUNI_VLINE_CHAR	0x2502
-# define SLUNI_ULCORN_CHAR	0x250C
-# define SLUNI_URCORN_CHAR	0x2510
-# define SLUNI_LLCORN_CHAR	0x2514
-# define SLUNI_LRCORN_CHAR	0x2518
-# define SLUNI_CKBRD_CHAR	0x2591
-# define SLUNI_RTEE_CHAR	0x2524
-# define SLUNI_LTEE_CHAR	0x251C
-# define SLUNI_UTEE_CHAR	0x252C
-# define SLUNI_DTEE_CHAR	0x2534
-# define SLUNI_CROSS_CHAR	0x253C
-# define SLUNI_DHLINE_CHAR	0x2550
-# define SLUNI_DVLINE_CHAR	0x2551
-# define SLUNI_DULCORN_CHAR	0x2554
-# define SLUNI_DURCORN_CHAR	0x2557
-# define SLUNI_DLLCORN_CHAR	0x255A
-# define SLUNI_DLRCORN_CHAR	0x255D
-# define SLUNI_DRTEE_CHAR	0x2563
-# define SLUNI_DLTEE_CHAR	0x2560
-# define SLUNI_DUTEE_CHAR	0x2566
-# define SLUNI_DDTEE_CHAR	0x2569
-# define SLUNI_DCROSS_CHAR	0x256C
-# define SLUNI_DSRTEE_CHAR	0x2562
-# define SLUNI_DSLTEE_CHAR	0x255F
-# define SLUNI_DSUTEE_CHAR	0x2564
-# define SLUNI_DSDTEE_CHAR	0x2567
-# define SLUNI_SDRTEE_CHAR	0x2561
-# define SLUNI_SDLTEE_CHAR	0x255E
-# define SLUNI_SDUTEE_CHAR	0x2565
-# define SLUNI_SDDTEE_CHAR	0x2568
-/* ----------------------------------------------------------------------- */
 
-/* subtypes of literal (static) and data (dynamic)*/
+#ifndef pc_system
+# define SLSMG_COLOR_BLACK		0x000000
+# define SLSMG_COLOR_RED		0x000001
+# define SLSMG_COLOR_GREEN		0x000002
+# define SLSMG_COLOR_BROWN		0x000003
+# define SLSMG_COLOR_BLUE		0x000004
+# define SLSMG_COLOR_MAGENTA		0x000005
+# define SLSMG_COLOR_CYAN		0x000006
+# define SLSMG_COLOR_LGRAY		0x000007
+# define SLSMG_COLOR_GRAY		0x000008
+# define SLSMG_COLOR_BRIGHT_RED		0x000009
+# define SLSMG_COLOR_BRIGHT_GREEN	0x00000A
+# define SLSMG_COLOR_BRIGHT_BROWN	0x00000B
+# define SLSMG_COLOR_BRIGHT_BLUE	0x00000C
+# define SLSMG_COLOR_BRIGHT_CYAN	0x00000D
+# define SLSMG_COLOR_BRIGHT_MAGENTA	0x00000E
+# define SLSMG_COLOR_BRIGHT_WHITE	0x00000F
+#endif
+
+/*}}}*/
+
+/*{{{ SLang Keypad Interface */
+
+#define SL_KEY_ERR		0xFFFF
+
+#define SL_KEY_UP		0x101
+#define SL_KEY_DOWN		0x102
+#define SL_KEY_LEFT		0x103
+#define SL_KEY_RIGHT		0x104
+#define SL_KEY_PPAGE		0x105
+#define SL_KEY_NPAGE		0x106
+#define SL_KEY_HOME		0x107
+#define SL_KEY_END		0x108
+#define SL_KEY_A1		0x109
+#define SL_KEY_A3		0x10A
+#define SL_KEY_B2		0x10B
+#define SL_KEY_C1		0x10C
+#define SL_KEY_C3		0x10D
+#define SL_KEY_REDO		0x10E
+#define SL_KEY_UNDO		0x10F
+#define SL_KEY_BACKSPACE	0x110
+#define SL_KEY_ENTER		0x111
+#define SL_KEY_IC		0x112
+#define SL_KEY_DELETE		0x113
+
+#define SL_KEY_F0		0x200
+#define SL_KEY_F(X)		(SL_KEY_F0 + X)
+
+/* I do not intend to use keysymps > 0x1000.  Applications can use those. */
+/* Returns 0 upon success or -1 upon error. */
+int SLkp_define_keysym (char *, unsigned int);
+
+/* This function must be called AFTER SLtt_get_terminfo and not before. */
+extern int SLkp_init (void);
+
+/* This function uses SLang_getkey and assumes that what ever initialization 
+ * is required for SLang_getkey has been performed.
+ */
+extern int SLkp_getkey (void);
+
+/*}}}*/
+
+/*{{{ SLang Scroll Interface */
+
+typedef struct _SLscroll_Type
+{
+   struct _SLscroll_Type *next;
+   struct _SLscroll_Type *prev;
+   unsigned int flags;
+}
+SLscroll_Type;
+
+typedef struct 
+{
+   unsigned int flags;
+   SLscroll_Type *top_window_line;   /* list element at top of window */
+   SLscroll_Type *bot_window_line;   /* list element at bottom of window */
+   SLscroll_Type *current_line;    /* current list element */
+   SLscroll_Type *lines;	       /* first list element */
+   unsigned int nrows;		       /* number of rows in window */
+   unsigned int hidden_mask;	       /* applied to flags in SLscroll_Type */
+   unsigned int line_num;	       /* current line number (visible) */
+   unsigned int num_lines;	       /* total number of lines (visible) */
+   unsigned int window_row;	       /* row of current_line in window */
+   unsigned int border;		       /* number of rows that form scroll border */
+   int cannot_scroll;		       /* should window scroll or recenter */
+}
+SLscroll_Window_Type;
+
+extern int SLscroll_find_top (SLscroll_Window_Type *);
+extern int SLscroll_find_line_num (SLscroll_Window_Type *);
+extern unsigned int SLscroll_next_n (SLscroll_Window_Type *, unsigned int);
+extern unsigned int SLscroll_prev_n (SLscroll_Window_Type *, unsigned int);
+extern int SLscroll_pageup (SLscroll_Window_Type *);
+extern int SLscroll_pagedown (SLscroll_Window_Type *);
+
+/*}}}*/
+
+/*{{{ Signal Routines */
+
+typedef void SLSig_Fun_Type (int);
+extern SLSig_Fun_Type *SLsignal (int, SLSig_Fun_Type *);
+extern SLSig_Fun_Type *SLsignal_intr (int, SLSig_Fun_Type *);
+#ifndef pc_system
+extern int SLsig_block_signals (void);
+extern int SLsig_unblock_signals (void);
+#endif
+/*}}}*/
+
+/*{{{ Interpreter Macro Definitions */
+
+/* This value is a main_type just like the other main_types defined
+ * near the definition of SLang_Name_Type.  Applications should avoid using
+ * this so if you do not understands its role, do not use it.
+ */
+#define SLANG_DATA		0x30           /* real objects which may be destroyed */
+
+/* Subtypes */
+
+/* The definitions here are for objects that may be on the run-time stack.
+ * They are actually sub_types of literal and data main_types.
+ */
 #define VOID_TYPE	1
 #define INT_TYPE 	2
 #ifdef FLOAT_TYPE
-#undef FLOAT_TYPE
-#define FLOAT_TYPE	3
+# undef FLOAT_TYPE
+# define FLOAT_TYPE	3
 #endif
 #define CHAR_TYPE	4
 #define INTP_TYPE	5
-#define SLANG_OBJ_TYPE	6
+/* An object of INTP_TYPE should never really occur on the stack.  Rather,
+ * the integer to which it refers will be there instead.  It is defined here
+ * because it is a valid type for MAKE_VARIABLE.
+ */
 
+#define SLANG_OBJ_TYPE	6
+/* SLANG_OBJ_TYPE refers to an object on the stack that is a pointer to
+ * some other object.
+ */
+
+#if 0
+/* This is not ready yet.  */
+# define SLANG_NOOP	9
+#endif
+   
 /* Everything above string should correspond to a pointer in the object 
  * structure.  See do_binary (slang.c) for exploitation of this fact.
  */
@@ -931,50 +1103,10 @@ extern int SLsmg_Backspace_Moves;
  */
 
 
-#define SLANG_LVARIABLE	0x01
-#define SLANG_LOGICAL	0x02
-#define SLANG_BINARY	0x03
-/* #define SLANG_LINTRINSIC	0x04
- * #define SLANG_LFUNCTION 	0x05 */
-#define SLANG_INTRINSIC 	0x06
-#define SLANG_FUNCTION  	0x07
-#define SLANG_LITERAL	0x08           /* constant objects */
-#define SLANG_BLOCK      0x09
-#define SLANG_DIRECTIVE	0x0A
-#define SLANG_EQS	0x0A
-#define SLANG_UNARY	0x0B
-#define SLANG_LUNARY	0x0C
-
-#define SLANG_GVARIABLE 	0x0D
-#define SLANG_IVARIABLE 	0x0E           /* intrinsic variables */
-/* Note!!! For Macro MAKE_VARIABLE below to work, SLANG_IVARIABLE Must
-   be 1 less than SLANG_RVARIABLE!!! */
-#define SLANG_RVARIABLE	0x0F	       /* read only variable */
-
-/* These 3 MUST be in this order too ! */
-#define SLANG_RETURN	0x10
-#define SLANG_BREAK	0x11
-#define SLANG_CONTINUE	0x12
-
-
-#define SLANG_EXCH	0x13
-#define SLANG_LABEL	0x14
-#define SLANG_LOBJPTR	0x15
-#define SLANG_GOBJPTR	0x16
-#define SLANG_X_ERROR	0x17
-/* These must be in this order */
-#define SLANG_X_USER0	0x18
-#define SLANG_X_USER1	0x19
-#define SLANG_X_USER2	0x1A
-#define SLANG_X_USER3	0x1B
-#define SLANG_X_USER4	0x1C
-
-
-#define SLANG_DATA	0x30           /* real objects which may be destroyed */
-
-
 /* Binary and Unary Subtypes */
-/* This group may be overloaded. */
+/* Since the application can define new types and can overload the binary
+ * and unary operators, these definitions must be present in this file.
+ */
 #define SLANG_PLUS	1
 #define SLANG_MINUS	2
 #define SLANG_TIMES	3
@@ -1039,7 +1171,11 @@ extern void SLadd_at_handler (long *, char *);
 
 #define SLANG_END_TABLE  MAKE_INTRINSIC("", 0, 0, 0)
 
-/* Uppercase, lowercase macros */
+
+/*}}}*/
+
+/*{{{ Upper/Lowercase Functions */
+
 extern void SLang_define_case(int *, int *);
 extern void SLang_init_case_tables (void);
 
@@ -1051,69 +1187,65 @@ extern unsigned char Chg_LCase_Lut[256];
 			Chg_UCase_Lut[(unsigned char) (x)] : Chg_LCase_Lut[(unsigned char) (x)])
 
 
+/*}}}*/
+
+/*{{{ Regular Expression Interface */
+
 typedef struct
 {
    unsigned char *pat;		       /* regular expression pattern */
    unsigned char *buf;		       /* buffer for compiled regexp */
-   int buf_len;			       /* length of buffer */
+   unsigned int buf_len;	       /* length of buffer */
    int case_sensitive;		       /* 1 if match is case sensitive  */
    int must_match;		       /* 1 if line must contain substring */
    int must_match_bol;		       /* true if it must match beginning of line */
    unsigned char must_match_str[16];   /* 15 char null term substring */
    int osearch;			       /* 1 if ordinary search suffices */
-   int min_length;		       /* minimum length the match must be */
+   unsigned int min_length;	       /* minimum length the match must be */
    int beg_matches[10];		       /* offset of start of \( */
-   int end_matches[10];		       /* length of nth submatch
+   unsigned int end_matches[10];       /* length of nth submatch
 					* Note that the entire match corresponds
 					* to \0 
 					*/
    int offset;			       /* offset to be added to beg_matches */
 } SLRegexp_Type;
 
-extern unsigned char *SLang_regexp_match(unsigned char *, int, SLRegexp_Type *);
+extern unsigned char *SLang_regexp_match(unsigned char *, 
+					 unsigned int, 
+					 SLRegexp_Type *);
 extern int SLang_regexp_compile (SLRegexp_Type *);
 extern char *SLregexp_quote_string (char *, char *, unsigned int);
 
-#ifdef SLKEYMAPS
-/* Keymap routines and definitions */
-typedef
-{
-   char *name;
-   int (*f) (void);
-}
-SLKeymap_Fun_Type;
-#endif
 
+/*}}}*/
 
-/* ------------------------------------------------------------------------ */
-/* cmd line definitions */
+/*{{{ SLang Command Interface */
 
 #define SLCMD_MAX_ARGS 10
-#ifdef __WATCOMC__
-       struct SLcmd_Cmd_Type; /* Pre-declaration is needed for the watcom c++ compiler */
-#endif
-typedef struct SLcmd_Cmd_Table_Type
+struct _SLcmd_Cmd_Type; /* Pre-declaration is needed below */
+typedef struct
 {
-   struct SLcmd_Cmd_Type *table;
+   struct _SLcmd_Cmd_Type *table;
    int argc;
    char *string_args[SLCMD_MAX_ARGS];
    int int_args[SLCMD_MAX_ARGS];
-   FLOAT float_args[SLCMD_MAX_ARGS];
+   float64 float_args[SLCMD_MAX_ARGS];
    unsigned char arg_type[SLCMD_MAX_ARGS];
 } SLcmd_Cmd_Table_Type;
 
 
-typedef struct SLcmd_Cmd_Type
+typedef struct _SLcmd_Cmd_Type
 {
-   int (*cmdfun)(int, struct SLcmd_Cmd_Table_Type *);
+   int (*cmdfun)(int, SLcmd_Cmd_Table_Type *);
    char cmd[32];
    char arg_type[SLCMD_MAX_ARGS];
 } SLcmd_Cmd_Type;
 
 extern int SLcmd_execute_string (char *, SLcmd_Cmd_Table_Type *);
-/* ------------------------------------------------------------------------ */
 
-/* search functions */
+/*}}}*/
+
+/*{{{ SLang Search Interface */
 
 typedef struct
 {
@@ -1143,9 +1275,13 @@ unsigned char *SLsearch (unsigned char *, unsigned char *, SLsearch_Type *);
  * NULL.
  */
 
+/*}}}*/
+
+#if 0
+{
+#endif
 #ifdef __cplusplus
 }
 #endif
 
-/* _davis_slang_h_ */
-#endif 
+#endif  /* _DAVIS_SLANG_H_ */
