@@ -5,11 +5,12 @@
  * You may distribute under the terms of either the GNU General Public
  * License or the Perl Artistic License.
  */
-#include <config.h>
+
+#include "config.h"
+
 #include <stdio.h>
 #include <string.h>
 
-#include "config.h"
 #include "slang.h"
 #include "_slang.h"
 
@@ -17,38 +18,13 @@ typedef struct Screen_Type
   {
      int n;                    /* number of chars written last time */
      int flags;                /* line untouched, etc... */
-     SLang_Char_Type *old, *neew;
+     unsigned short *old, *neew;
 #ifndef pc_system
      unsigned long old_hash, new_hash;
 #endif
   }
 Screen_Type;
 
-#ifdef linux_unicode
-struct _uniconv
-  {
-    unsigned char acs;
-    unsigned short uni;
-  }
-UniConv_Table [] = {
-{ SLSMG_HLINE_CHAR, SLUNI_HLINE_CHAR },
-{ SLSMG_VLINE_CHAR, SLUNI_VLINE_CHAR },
-{ SLSMG_ULCORN_CHAR, SLUNI_ULCORN_CHAR },
-{ SLSMG_URCORN_CHAR, SLUNI_URCORN_CHAR },
-{ SLSMG_LLCORN_CHAR, SLUNI_LLCORN_CHAR },
-{ SLSMG_LRCORN_CHAR, SLUNI_LRCORN_CHAR },
-{ SLSMG_CKBRD_CHAR, SLUNI_CKBRD_CHAR },
-{ SLSMG_RTEE_CHAR, SLUNI_RTEE_CHAR },
-{ SLSMG_LTEE_CHAR, SLUNI_LTEE_CHAR },
-{ SLSMG_UTEE_CHAR, SLUNI_UTEE_CHAR },
-{ SLSMG_DTEE_CHAR, SLUNI_DTEE_CHAR },
-{ SLSMG_CROSS_CHAR, SLUNI_CROSS_CHAR }
-};
-#define colorshift 16
-#else
-#define colorshift 8
-#endif
-
 #define TOUCHED 0x1
 #define TRASHED 0x2
 
@@ -75,15 +51,19 @@ static int This_Color;		       /* only the first 8 bits of this
 #define ALT_CHAR_FLAG 0x00
 #endif
 
+int SLsmg_Newline_Moves = 0;
+int SLsmg_Backspace_Moves = 0;
 
-static void blank_line (SLang_Char_Type *p, int n, unsigned char ch)
+static void blank_line (unsigned short *p, int n, unsigned char ch)
 {
-   register SLang_Char_Type *pmax = p + n;
+   register unsigned short *pmax = p + n;
+   register unsigned short color_ch;
+
+   color_ch = (This_Color << 8) | (unsigned short) ch;
    
    while (p < pmax)
      {
-	*p = (This_Color << colorshift) | (SLang_Char_Type) ch;
-	p++;
+	*p++ = color_ch;
      }
 }
 
@@ -106,13 +86,38 @@ static void clear_region (int row, int n)
 
 void SLsmg_erase_eol (void)
 {
-   int c = This_Col;
-   if ((This_Row < 0) || (This_Row >= Screen_Rows)) return;
+   int r, c;
+   
+   c = This_Col - Start_Col;
+   r = This_Row - Start_Row;
+   
+   if ((r < 0) || (r >= Screen_Rows)) return;
    if (c < 0) c = 0; else if (c >= Screen_Cols) return;
    blank_line (SL_Screen[This_Row].neew + c , Screen_Cols - c, ' ');
    SL_Screen[This_Row].flags |= TOUCHED;
 }
 
+static void scroll_up (void)
+{
+   unsigned int i, imax;
+   unsigned short *neew;
+
+   neew = SL_Screen[0].neew;
+   imax = Screen_Rows - 1;
+   for (i = 0; i < imax; i++)
+     {
+	SL_Screen[i].neew = SL_Screen[i + 1].neew;
+	SL_Screen[i].flags |= TOUCHED;
+     }
+   SL_Screen[i].neew = neew;
+   SL_Screen[i].flags |= TOUCHED;
+   blank_line (neew, Screen_Cols, ' ');
+   This_Row--;
+}
+
+   
+
+
 void SLsmg_gotorc (int r, int c)
 {
    This_Row = r;
@@ -140,14 +145,12 @@ static int This_Alt_Char;
 #ifndef pc_system
 void SLsmg_set_char_set (int i)
 {
-#ifdef linux_unicode
-   if (SLtt_Unicode) return; /* char set has no meaning :) */
-#endif   
    if (SLtt_Use_Blink_For_ACS) return; /* alt chars not used and the alt bit
 					* is used to indicate a blink.
 					*/
    if (i) This_Alt_Char = ALT_CHAR_FLAG;
    else This_Alt_Char = 0;
+   
    This_Color &= 0x7F;
    This_Color |= This_Alt_Char;
 }
@@ -271,11 +274,26 @@ int SLsmg_Display_Eight_Bit = 128;
 
 void SLsmg_write_nchars (char *str, int n)
 {
-   register SLang_Char_Type *p, old, neew;
+   register unsigned short *p, old, neew, color;
    unsigned char ch;
    unsigned int flags;
-   int len, start_len = Start_Col, max_len;
-   char *str_max = str + n;
+   int len, start_len, max_len;
+   char *str_max;
+   int newline_flag;
+#ifndef pc_system
+   int alt_char_set_flag;
+   
+   alt_char_set_flag = ((SLtt_Use_Blink_For_ACS == 0) 
+			&& (This_Color & ALT_CHAR_FLAG));
+#endif
+
+   str_max = str + n;
+   color = This_Color << 8;
+   
+   top:				       /* get here only on newline */
+   
+   newline_flag = 0;
+   start_len = Start_Col;
    
    if (point_visible (0) == 0) return;
    
@@ -289,23 +307,23 @@ void SLsmg_write_nchars (char *str, int n)
    while ((len < max_len) && (str < str_max))
      {	
 	ch = (unsigned char) *str++;
+
 #ifndef pc_system
-#ifdef linux_unicode
-        if (!SLtt_Unicode)
-#endif
-	  if (This_Alt_Char)
-	    {
-	       ch = Alt_Char_Set[ch & 0x7F];
-	    }
+	if (alt_char_set_flag)
+	  ch = Alt_Char_Set [ch & 0x7F];
 #endif
 	if (((ch >= ' ') && (ch < 127))
-	    || (ch >= (unsigned char) SLsmg_Display_Eight_Bit))
+	    || (ch >= (unsigned char) SLsmg_Display_Eight_Bit)
+#ifndef pc_system
+	    || alt_char_set_flag
+#endif
+	    )
 	  {
 	     len += 1;
 	     if (len > start_len)
 	       {
 		  old = *p;
-		  neew = (This_Color << colorshift) | (SLang_Char_Type) ch;
+		  neew = color | (unsigned short) ch;
 		  if (old != neew)
 		    {
 		       flags |= TOUCHED;
@@ -321,7 +339,7 @@ void SLsmg_write_nchars (char *str, int n)
 	     n += SLsmg_Tab_Width;
 	     n = SLsmg_Tab_Width - (n % SLsmg_Tab_Width);
 	     if (len + n > max_len) n = max_len - len;
-	     neew = (This_Color << colorshift) | (SLang_Char_Type) ' ';
+	     neew = color | (unsigned short) ' ';
 	     while (n--)
 	       {
 		  len += 1;
@@ -336,12 +354,20 @@ void SLsmg_write_nchars (char *str, int n)
 		    }
 	       }
 	  }
-	else if (ch == '\n') break;
+	else if (ch == '\n')
+	  {
+	     newline_flag = 1;
+	     break;
+	  }
+	else if ((ch == 0x8) && SLsmg_Backspace_Moves)
+	  {
+	     if (len != 0) len--;
+	  }
 	else
 	  {
 	     if (ch & 0x80)
 	       {
-		  neew = (This_Color << colorshift) | (SLang_Char_Type) '~';
+		  neew = color | (unsigned short) '~';
 		  len += 1;
 		  if (len > start_len)
 		    {
@@ -359,7 +385,7 @@ void SLsmg_write_nchars (char *str, int n)
 	     len += 1;
 	     if (len > start_len)
 	       {
-		  neew = (This_Color << colorshift) | (SLang_Char_Type) '^';
+		  neew = color | (unsigned short) '^';
 		  if (*p != neew)
 		    {
 		       *p = neew;
@@ -373,7 +399,7 @@ void SLsmg_write_nchars (char *str, int n)
 	     len++;
 	     if (len > start_len)
 	       {
-		  neew = (This_Color << colorshift) | (SLang_Char_Type) ch;
+		  neew = color | (unsigned short) ch;
 		  if (*p != neew)
 		    {
 		       *p = neew;
@@ -386,39 +412,31 @@ void SLsmg_write_nchars (char *str, int n)
    
    SL_Screen[This_Row].flags = flags;
    This_Col = len;
-}
-
-void SLsmg_write_unicode (unsigned short ch)
-{
-#ifdef linux_unicode
-   register SLang_Char_Type *p, old, neew;
    
-   if (ch < 0x100) 
+   if (SLsmg_Newline_Moves == 0)
+     return;
+   
+   if (newline_flag == 0)
      {
-        SLsmg_write_nchars ((char *) &ch, 1);
-        return;
+	while (str < str_max)
+	  {
+	     if (*str == '\n') break;
+	     str++;
+	  }
+	if (str == str_max) return;
+	str++;
      }
    
-   if (point_visible (0) == 0) return;
-   
-   p = SL_Screen[This_Row].neew;
-   if (This_Col > Start_Col) p += (This_Col - Start_Col);
-   if (This_Col >= Start_Col + Screen_Cols)
-        return;
-   This_Col++;
-   if (This_Col > Start_Col)
+   This_Row++;
+   This_Col = 0;
+   if (This_Row == Start_Row + Screen_Rows)
      {
-	old = *p;
-	neew = (This_Color << colorshift) | (SLang_Char_Type) ch;
-	if (old != neew)
-	  {
-	     SL_Screen[This_Row].flags |= TOUCHED;
-	     *p = neew;
-	  }
-     }	
-#endif
+	if (SLsmg_Newline_Moves > 0) scroll_up ();
+     }
+   goto top;
 }
 
+
 void SLsmg_write_char (char ch)
 {
    SLsmg_write_nchars (&ch, 1);
@@ -429,25 +447,27 @@ static int Cls_Flag;
 
 void SLsmg_cls (void)
 {
-   This_Color = This_Alt_Char;
+   This_Color = 0;
    clear_region (0, Screen_Rows);
+   This_Color = This_Alt_Char;
    Cls_Flag = 1;
 }
-
-static void do_copy (SLang_Char_Type *a, SLang_Char_Type *b)
+#if 0
+static void do_copy (unsigned short *a, unsigned short *b)
 {
-   SLang_Char_Type *amax = a + Screen_Cols;
+   unsigned short *amax = a + Screen_Cols;
    
    while (a < amax) *a++ = *b++;
 }
+#endif
 
 #ifndef pc_system
 int SLsmg_Scroll_Hash_Border = 0;
-static unsigned long compute_hash (SLang_Char_Type *s, int n)
+static unsigned long compute_hash (unsigned short *s, int n)
 {
    register unsigned long h = 0, g;
    register unsigned long sum = 0;
-   register SLang_Char_Type *smax, ch;
+   register unsigned short *smax, ch;
    int is_blank = 2;
    
    s += SLsmg_Scroll_Hash_Border;
@@ -455,64 +475,66 @@ static unsigned long compute_hash (SLang_Char_Type *s, int n)
    while (s < smax) 
      {
 	ch = *s++;
-#ifdef linux_unicode
-	if (is_blank && ((ch & 0xFFFF) != 32)) is_blank--;
-#else	
 	if (is_blank && ((ch & 0xFF) != 32)) is_blank--;
-#endif	
 	
-	sum += ch & 0xFFFF;
+	sum += ch;
 	
 	h = sum + (h << 3);
-	if ((g = h & 0xE0000000L) != 0)
+	if ((g = h & 0xE0000000UL) != 0)
 	  {
 	     h = h ^ (g >> 24);
 	     h = h ^ g;
 	  }
-#ifdef linux_unicode	  
-	sum += (ch >> 16) & 0xFFFF;
-	
-	h = sum + (h << 3);
-	if ((g = h & 0xE0000000L) != 0)
-	  {
-	     h = h ^ (g >> 24);
-	     h = h ^ g;
-	  }
-#endif	  
      }
    if (is_blank) return 0;
    return h;
 }
 
-unsigned long Blank_Hash;
+static unsigned long Blank_Hash;
 
 static void try_scroll (void)
 {
    int i, j, di, r1, r2, rmin, rmax;
    unsigned long hash;
    int color, did_scroll = 0;
-   SLang_Char_Type *tmp;
+   unsigned short *tmp;
    int ignore;
    
-   /* find region limits */
+   /* find region limits. */
+   
    for (rmax = Screen_Rows - 1; rmax > 0; rmax--)
      {
 	if (SL_Screen[rmax].new_hash != SL_Screen[rmax].old_hash)
-	  break;
+	  {
+	     r1 = rmax - 1;
+	     if ((r1 == 0)
+		 || (SL_Screen[r1].new_hash != SL_Screen[r1].old_hash))
+	       break;
+	     
+	     rmax = r1;
+	  }
      }
    
    for (rmin = 0; rmin < rmax; rmin++)
      {
 	if (SL_Screen[rmin].new_hash != SL_Screen[rmin].old_hash)
-	  break;
+	  {
+	     r1 = rmin + 1;
+	     if ((r1 == rmax)
+		 || (SL_Screen[r1].new_hash != SL_Screen[r1].old_hash))
+	       break;
+	     
+	     rmin = r1;
+	  }
      }
+
    
    for (i = rmax; i > rmin; i--)
      {
 	hash = SL_Screen[i].new_hash;
 	if (hash == Blank_Hash) continue;
 	
-	if ((hash == SL_Screen[i].old_hash)
+	if ((hash == SL_Screen[i].old_hash) 
 	    || ((i + 1 < Screen_Rows) && (hash == SL_Screen[i + 1].old_hash))
 	    || ((i - 1 > rmin) && (SL_Screen[i].old_hash == SL_Screen[i - 1].new_hash)))
 	  continue;
@@ -681,10 +703,7 @@ void SLsmg_refresh (void)
    
    if (Cls_Flag) 
      {
-	SLtt_normal_video ();  
-#ifndef SCO_FLAVOR
-	SLtt_cls ();
-#endif
+	SLtt_normal_video ();  SLtt_cls ();
      }
 #ifndef pc_system
    else if (SLtt_Term_Cannot_Scroll == 0) try_scroll ();
@@ -717,7 +736,10 @@ void SLsmg_refresh (void)
 	SL_Screen[i].neew[Screen_Cols] = 0;
 	
 	SLtt_smart_puts (SL_Screen[i].neew, SL_Screen[i].old, Screen_Cols, i);
-	do_copy (SL_Screen[i].old, SL_Screen[i].neew);
+
+	SLMEMCPY ((char *) SL_Screen[i].old, (char *) SL_Screen[i].neew, 
+		  Screen_Cols * sizeof (short));
+
 	SL_Screen[i].flags = 0;
 #ifndef pc_system
 	SL_Screen[i].old_hash = SL_Screen[i].new_hash;
@@ -767,11 +789,13 @@ static char Fake_Alt_Char_Pairs [] = "a:j+k+l+m+q-t+u+v+w+x|";
 
 static void init_alt_char_set (void)
 {
-   int i = 0;
+   int i;
    unsigned char *p, *pmax, ch;
    
    if (Alt_Char_Set[128] == 128) return;
 
+   i = 32;
+   memset ((char *)Alt_Char_Set, ' ', i);
    while (i <= 128) 
      {
 	Alt_Char_Set [i] = i;
@@ -798,27 +822,55 @@ static void init_alt_char_set (void)
 }
 #endif
 
+#ifndef pc_system
+# define BLOCK_SIGNALS SLsig_block_signals ();
+# define UNBLOCK_SIGNALS SLsig_unblock_signals ();
+#else
+# define BLOCK_SIGNALS
+# define UNBLOCK_SIGNALS
+#endif
+
+static int Smg_Suspended;
 void SLsmg_suspend_smg (void)
 {
-   SLtt_reset_video ();
+   BLOCK_SIGNALS
+
+   if (Smg_Suspended == 0)
+     {
+	SLtt_reset_video ();
+	Smg_Suspended = 1;
+     }
+   
+   UNBLOCK_SIGNALS
 }
 
 void SLsmg_resume_smg (void)
 {
    int i;
+   BLOCK_SIGNALS
+
+   if (Smg_Suspended == 0) 
+     {
+	UNBLOCK_SIGNALS
+	return;
+     }
+   
+   Smg_Suspended = 0;
    SLtt_init_video ();
    Cls_Flag = 1;
    for (i = 0; i < Screen_Rows; i++)
      SL_Screen[i].flags |= TRASHED;
    SLsmg_refresh ();
+   
+   UNBLOCK_SIGNALS
 }
 
-
 int SLsmg_init_smg (void)
 {
    int i, len;
-   SLang_Char_Type *old, *neew;
-   
+   unsigned short *old, *neew;
+   BLOCK_SIGNALS
+     
    if (Smg_Inited) SLsmg_reset_smg ();
    SLtt_init_video ();
    Screen_Cols = SLtt_Screen_Cols;
@@ -834,10 +886,11 @@ int SLsmg_init_smg (void)
    len = Screen_Cols + 3;
    for (i = 0; i < Screen_Rows; i++)
      {
-	if ((NULL == (old = (SLang_Char_Type *) SLMALLOC (sizeof(SLang_Char_Type) * len)))
-	    || ((NULL == (neew = (SLang_Char_Type *) SLMALLOC (sizeof(SLang_Char_Type) * len)))))
+	if ((NULL == (old = (unsigned short *) SLMALLOC (sizeof(short) * len)))
+	    || ((NULL == (neew = (unsigned short *) SLMALLOC (sizeof(short) * len)))))
 	  {
 	     SLang_Error = SL_MALLOC_ERROR;
+	     UNBLOCK_SIGNALS
 	     return 0;
 	  }
 	blank_line (old, len, ' ');
@@ -851,6 +904,7 @@ int SLsmg_init_smg (void)
 #endif
      }
    Smg_Inited = 1;
+   UNBLOCK_SIGNALS
    return 1;
 }
 
@@ -858,8 +912,13 @@ int SLsmg_init_smg (void)
 void SLsmg_reset_smg (void)
 {
    int i;
-   
-   if (Smg_Inited == 0) return;
+   BLOCK_SIGNALS
+     
+   if (Smg_Inited == 0) 
+     {
+	UNBLOCK_SIGNALS
+	return;
+     }
    for (i = 0; i < Screen_Rows; i++)
      {
 	if (SL_Screen[i].old != NULL) SLFREE (SL_Screen[i].old);
@@ -869,10 +928,12 @@ void SLsmg_reset_smg (void)
    SLtt_reset_video ();
    This_Alt_Char = This_Color = 0;
    Smg_Inited = 0;
+   
+   UNBLOCK_SIGNALS
 }
 
 
-SLang_Char_Type SLsmg_char_at (void)
+unsigned short SLsmg_char_at (void)
 {
    if (point_visible (1))
      {
@@ -881,6 +942,7 @@ SLang_Char_Type SLsmg_char_at (void)
    return 0;
 }
 
+
 void SLsmg_vprintf (char *fmt, va_list ap)
 {
    char p[1000];
@@ -911,56 +973,25 @@ void SLsmg_set_screen_start (int *r, int *c)
 void SLsmg_draw_object (int r, int c, unsigned char object)
 {
    This_Row = r;  This_Col = c;
-
-#ifdef linux_unicode
-   if (SLtt_Unicode)
-     {
-        int i;
-        unsigned short ch = 0xfffd;
-        
-        for (i = 0; i < sizeof (UniConv_Table) / sizeof (UniConv_Table [0]);
-            i++)
-          if (object == UniConv_Table [i].acs)
-            {
-               ch = UniConv_Table [i].uni;
-               break;
-            }
-        if (point_visible (1))
-            SLsmg_write_unicode (SLtt_fix_unicode (ch));
-        This_Col = c + 1;
-        return;
-     }
-#endif
+   
    if (point_visible (1))
      {
-	int tac = This_Alt_Char;
-	This_Alt_Char = ALT_CHAR_FLAG;
-	This_Color |= This_Alt_Char;
+	int color = This_Color;
+	This_Color |= ALT_CHAR_FLAG;
 	SLsmg_write_char (object);
-	This_Alt_Char = tac;
-	This_Color = (This_Color & 0x7F) | This_Alt_Char;
+	This_Color = color;
      }
 
    This_Col = c + 1;
 }
 
-void SLsmg_draw_unicode (int r, int c, unsigned short object)
-{
-#ifdef linux_unicode
-    This_Row = r; This_Col = c;
-    
-    SLsmg_write_unicode (SLtt_fix_unicode (object));
-    This_Col = c + 1;
-#endif
-}
-
 void SLsmg_draw_hline (int n)
 {
    static unsigned char hbuf[16];
    int count;
    int cmin, cmax;
    int final_col = This_Col + n;
-   int tac = This_Alt_Char;
+   int save_color;
    
    if ((This_Row < Start_Row) || (This_Row >= Start_Row + Screen_Rows) 
        || (0 == compute_clip (This_Col, n, Start_Col, Start_Col + Screen_Cols,
@@ -969,28 +1000,18 @@ void SLsmg_draw_hline (int n)
 	This_Col = final_col;
 	return;
      }
-
-   n = cmax - cmin;
-   This_Col = cmin;
    
-#ifdef linux_unicode   
-   if (SLtt_Unicode)
-     {
-        unsigned short ch = SLtt_fix_unicode (SLUNI_HLINE_CHAR);
-        while (n-- > 0)
-           SLsmg_write_unicode (ch);
-        This_Col = final_col;
-        return;
-     }
-#endif     
    if (hbuf[0] == 0)
      {
-	MEMSET ((char *) hbuf, SLSMG_HLINE_CHAR, 16);
+	SLMEMSET ((char *) hbuf, SLSMG_HLINE_CHAR, 16);
      }
    
+   n = cmax - cmin;
    count = n / 16;
-   This_Alt_Char = ALT_CHAR_FLAG;
-   This_Color |= This_Alt_Char;
+   
+   save_color = This_Color;
+   This_Color |= ALT_CHAR_FLAG;
+   This_Col = cmin;
    
    SLsmg_write_nchars ((char *) hbuf, n % 16);
    while (count-- > 0)
@@ -998,51 +1019,17 @@ void SLsmg_draw_hline (int n)
 	SLsmg_write_nchars ((char *) hbuf, 16);
      }
    
-   This_Alt_Char = tac;
-   This_Color = (This_Color & 0x7F) | This_Alt_Char;
+   This_Color = save_color;
    This_Col = final_col;
 }
 
-void SLsmg_draw_double_hline (int n)
-{
-#ifdef linux_unicode
-   int cmin, cmax;
-   int final_col = This_Col + n;
-   
-   if (SLtt_Unicode)
-     {
-        unsigned short ch;
-        
-        if ((This_Row < Start_Row) || (This_Row >= Start_Row + Screen_Rows) 
-            || (0 == compute_clip (This_Col, n, Start_Col, Start_Col + Screen_Cols,
-			           &cmin, &cmax)))
-          {
-	     This_Col = final_col;
-	     return;
-          }
-
-        n = cmax - cmin;
-        This_Col = cmin;
-        ch = SLtt_fix_unicode (SLUNI_DHLINE_CHAR);
-   
-        while (n-- > 0)
-           SLsmg_write_unicode (ch);
-        This_Col = final_col;
-        return;
-     }
-   else
-#endif
-     SLsmg_draw_hline (n);
-}
-
 void SLsmg_draw_vline (int n)
 {
    unsigned char ch = SLSMG_VLINE_CHAR;
-   int tac = This_Alt_Char;
-   int c = This_Col;   
-   int rmin, rmax;
+   int c = This_Col, rmin, rmax;
    int final_row = This_Row + n;
-
+   int save_color;
+   
    if (((c < Start_Col) || (c >= Start_Col + Screen_Cols)) ||
        (0 == compute_clip (This_Row, n, Start_Row, Start_Row + Screen_Rows,
 			  &rmin, &rmax)))
@@ -1050,65 +1037,18 @@ void SLsmg_draw_vline (int n)
 	This_Row = final_row;
 	return;
      }
-
-#ifdef linux_unicode
-   if (SLtt_Unicode)
-     {
-        unsigned short ch = SLtt_fix_unicode (SLUNI_VLINE_CHAR);
-        
-        for (This_Row = rmin; This_Row < rmax; This_Row++)
-          {
-             This_Col = c;
-             SLsmg_write_unicode (ch);
-          }
-     }
-   else
-#endif   
-     {
-        This_Alt_Char = ALT_CHAR_FLAG;
-        This_Color |= This_Alt_Char;
-    
-        for (This_Row = rmin; This_Row < rmax; This_Row++)
-          {
-	     This_Col = c;
-	     SLsmg_write_nchars ((char *) &ch, 1);
-          }
    
-        This_Alt_Char = tac;
-        This_Color = (This_Color & 0x7F) | This_Alt_Char;
+   save_color = This_Color;
+   This_Color |= ALT_CHAR_FLAG;
+    
+   for (This_Row = rmin; This_Row < rmax; This_Row++)
+     {
+	This_Col = c;
+	SLsmg_write_nchars ((char *) &ch, 1);
      }
    
    This_Col = c;  This_Row = final_row;
-}
-
-void SLsmg_draw_double_vline (int n)
-{
-#ifdef linux_unicode
-   int c = This_Col, rmin, rmax;
-   int final_row = This_Row + n;
-
-   if (SLtt_Unicode)
-     {   
-        unsigned short ch = SLtt_fix_unicode (SLUNI_DVLINE_CHAR);
-        
-        if (((c < Start_Col) || (c >= Start_Col + Screen_Cols)) ||
-            (0 == compute_clip (This_Row, n, Start_Row, Start_Row + Screen_Rows,
-			       &rmin, &rmax)))
-          {
-	     This_Row = final_row;
-	     return;
-          }
-        
-        for (This_Row = rmin; This_Row < rmax; This_Row++)
-          {
-             This_Col = c;
-             SLsmg_write_unicode (ch);
-          }
-        This_Col = c;  This_Row = final_row;
-     }
-   else
-#endif
-     SLsmg_draw_vline (n);   
+   This_Color = save_color;
 }
 
 void SLsmg_draw_box (int r, int c, int dr, int dc)
@@ -1121,51 +1061,10 @@ void SLsmg_draw_box (int r, int c, int dr, int dc)
    This_Row = r;  This_Col = c;
    SLsmg_draw_vline (dr);
    SLsmg_draw_hline (dc);   
-#ifdef linux_unicode
-   if (SLtt_Unicode)
-     {
-        SLsmg_draw_unicode (r, c, SLUNI_ULCORN_CHAR);
-        SLsmg_draw_unicode (r, c + dc, SLUNI_URCORN_CHAR);
-        SLsmg_draw_unicode (r + dr, c, SLUNI_LLCORN_CHAR);
-        SLsmg_draw_unicode (r + dr, c + dc, SLUNI_LRCORN_CHAR);
-     }
-   else
-#endif
-     {
-        SLsmg_draw_object (r, c, SLSMG_ULCORN_CHAR);
-        SLsmg_draw_object (r, c + dc, SLSMG_URCORN_CHAR);
-        SLsmg_draw_object (r + dr, c, SLSMG_LLCORN_CHAR);
-        SLsmg_draw_object (r + dr, c + dc, SLSMG_LRCORN_CHAR);
-     }
-   This_Row = r; This_Col = c;
-}
-   
-void SLsmg_draw_double_box (int r, int c, int dr, int dc)
-{
-   if (!dr || !dc) return; 
-   This_Row = r;  This_Col = c;
-   dr--; dc--;
-   SLsmg_draw_double_hline (dc);  
-   SLsmg_draw_double_vline (dr);
-   This_Row = r;  This_Col = c;
-   SLsmg_draw_double_vline (dr);
-   SLsmg_draw_double_hline (dc);   
-#ifdef linux_unicode
-   if (SLtt_Unicode)
-     {
-        SLsmg_draw_unicode (r, c, SLUNI_DULCORN_CHAR);
-        SLsmg_draw_unicode (r, c + dc, SLUNI_DURCORN_CHAR);
-        SLsmg_draw_unicode (r + dr, c, SLUNI_DLLCORN_CHAR);
-        SLsmg_draw_unicode (r + dr, c + dc, SLUNI_DLRCORN_CHAR);
-     }
-   else
-#endif
-     {
-        SLsmg_draw_object (r, c, SLSMG_ULCORN_CHAR);
-        SLsmg_draw_object (r, c + dc, SLSMG_URCORN_CHAR);
-        SLsmg_draw_object (r + dr, c, SLSMG_LLCORN_CHAR);
-        SLsmg_draw_object (r + dr, c + dc, SLSMG_LRCORN_CHAR);
-     }
+   SLsmg_draw_object (r, c, SLSMG_ULCORN_CHAR);
+   SLsmg_draw_object (r, c + dc, SLSMG_URCORN_CHAR);
+   SLsmg_draw_object (r + dr, c, SLSMG_LLCORN_CHAR);
+   SLsmg_draw_object (r + dr, c + dc, SLSMG_LRCORN_CHAR);
    This_Row = r; This_Col = c;
 }
    
@@ -1187,10 +1086,10 @@ void SLsmg_fill_region (int r, int c, int dr, int dc, unsigned char ch)
    rmax = This_Row + dr;
    if (rmax > Screen_Rows) rmax = Screen_Rows;
 
-#ifndef pc_system
+#if 0
    ch = Alt_Char_Set[ch];
 #endif
-   if (ch != hbuf[0]) MEMSET ((char *) hbuf, (char) ch, 16);
+   if (ch != hbuf[0]) SLMEMSET ((char *) hbuf, (char) ch, 16);
    
    for (This_Row = r; This_Row < rmax; This_Row++)
      {
@@ -1206,3 +1105,81 @@ void SLsmg_fill_region (int r, int c, int dr, int dc, unsigned char ch)
    This_Row = r;
 }
 
+void SLsmg_forward (int n)
+{
+   This_Col += n;
+}
+
+void SLsmg_write_color_chars (unsigned short *s, unsigned int len)
+{
+   unsigned short *smax, sh;
+   char buf[32], *b, *bmax;
+   int color, save_color;
+   
+   smax = s + len;
+   b = buf;
+   bmax = b + sizeof (buf);
+   
+   save_color = This_Color;
+
+   while (s < smax)
+     {
+	sh = *s++;
+	
+	color = sh >> 8;
+	if ((color != This_Color) || (b == bmax))
+	  {
+	     if (b != buf) 
+	       {
+		  SLsmg_write_nchars (buf, (int) (b - buf));
+		  b = buf;
+	       }
+	     This_Color = color;
+	  }
+	*b++ = (char) (sh & 0xFF);
+     }
+   
+   if (b != buf)
+     SLsmg_write_nchars (buf, (int) (b - buf));
+	
+   This_Color = save_color;
+}
+
+unsigned int SLsmg_read_raw (unsigned short *buf, unsigned int len)
+{
+   unsigned int r, c;
+
+   if (0 == point_visible (1)) return 0;
+   
+   r = (unsigned int) (This_Row - Start_Row);
+   c = (unsigned int) (This_Col - Start_Col);
+   
+   if (c + len > (unsigned int) Screen_Cols)
+     len = (unsigned int) Screen_Cols - c;
+   
+   memcpy ((char *) buf, (char *) (SL_Screen[r].neew + c), len * sizeof (short));
+   return len;
+}
+
+unsigned int SLsmg_write_raw (unsigned short *buf, unsigned int len)
+{
+   unsigned int r, c;
+   unsigned short *dest;
+   
+   if (0 == point_visible (1)) return 0;
+   
+   r = (unsigned int) (This_Row - Start_Row);
+   c = (unsigned int) (This_Col - Start_Col);
+   
+   if (c + len > (unsigned int) Screen_Cols)
+     len = (unsigned int) Screen_Cols - c;
+   
+   dest = SL_Screen[r].neew + c;
+   
+   if (0 != memcmp ((char *) dest, (char *) buf, len * sizeof (short)))
+     {	
+	memcpy ((char *) dest, (char *) buf, len * sizeof (short));
+	SL_Screen[r].flags |= TOUCHED;
+     }
+   return len;
+}
