@@ -9,8 +9,13 @@
  * License or the Perl Artistic License.
  */
 
-#include <config.h>
+#include "config.h"
+
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
+# include <stdlib.h>
+#endif
+
 #ifndef USE_SETUPTERM
 #include "slang.h"
 #include "_slang.h"
@@ -70,19 +75,19 @@ typedef struct
 #define SLTERMCAP  2
    unsigned int flags;
 
-   int name_section_size;
+   unsigned int name_section_size;
    char *terminal_names;
    
-   int boolean_section_size;
+   unsigned int boolean_section_size;
    unsigned char *boolean_flags;
    
-   int num_numbers;
+   unsigned int num_numbers;
    unsigned char *numbers;
    
-   int num_string_offsets;
+   unsigned int num_string_offsets;
    unsigned char *string_offsets;
    
-   int string_table_size;
+   unsigned int string_table_size;
    char *string_table;
    
 } Terminfo_Type;
@@ -124,7 +129,7 @@ static FILE *open_terminfo (char *file, Terminfo_Type *h)
  */
 
 /* returns pointer to malloced space */
-static unsigned char *read_terminfo_section (FILE *fp, int size)
+static unsigned char *read_terminfo_section (FILE *fp, unsigned int size)
 {
    char *s;
    
@@ -157,7 +162,7 @@ static unsigned char *read_boolean_flags (FILE *fp, Terminfo_Type *t)
     * boundary.
     */
    
-   int size = (t->name_section_size + t->boolean_section_size) % 2;
+   unsigned int size = (t->name_section_size + t->boolean_section_size) % 2;
    size += t->boolean_section_size;
    
    return t->boolean_flags = read_terminfo_section (fp, size);
@@ -213,13 +218,16 @@ static char *read_string_table (FILE *fp, Terminfo_Type *t)
  * are implemented by multiple links to the same compiled file.
  */
 
-#define MAX_TI_DIRS 4
+#define MAX_TI_DIRS 7
 static char *Terminfo_Dirs [MAX_TI_DIRS] = 
 {
    NULL,
    "/usr/lib/terminfo",
    "/usr/share/lib/terminfo",
-   "/usr/local/lib/terminfo"
+   "/usr/local/lib/terminfo",
+   "/lib/terminfo",
+   "/usr/local/share/terminfo",
+   "/usr/share/terminfo"
 };
 
 char *SLtt_tigetent (char *term)
@@ -230,13 +238,27 @@ char *SLtt_tigetent (char *term)
    char file[256];
    Terminfo_Type *ti;
 
-   if (term == NULL) return NULL;
+   if (
+       (term == NULL) 
+#ifdef SLANG_UNTIC
+       && (SLang_Untic_Terminfo_File == NULL)
+#endif
+       )
+     return NULL;
    
    if (NULL == (ti = (Terminfo_Type *) SLMALLOC (sizeof (Terminfo_Type))))
      {
 	return NULL;
      }
    
+#ifdef SLANG_UNTIC
+   if (SLang_Untic_Terminfo_File != NULL)
+     {
+	fp = open_terminfo (SLang_Untic_Terminfo_File, ti);
+	goto fp_open_label;
+     }
+   else
+#endif
    /* If we are on a termcap based system, use termcap */
    if (0 == tcap_getent (term, ti)) return (char *) ti;
        
@@ -252,6 +274,9 @@ char *SLtt_tigetent (char *term)
 	  }
 	i++;
      }
+#ifdef SLANG_UNTIC
+   fp_open_label:
+#endif
    
    if (fp != NULL) 
      {
@@ -285,104 +310,115 @@ char *SLtt_tigetent (char *term)
    return NULL;
 }
 
+#ifdef SLANG_UNTIC
+# define UNTIC_COMMENT(x) ,x
+#else
+# define UNTIC_COMMENT(x)
+#endif
+
 typedef struct 
 {
-   char *name;
+   char name[3];
    int offset;
+#ifdef SLANG_UNTIC
+   char *comment;
+#endif
 }
 Tgetstr_Map_Type;
-   
-Tgetstr_Map_Type Tgetstr_Map [] =
+
+/* I need to add: K1-5, %0-5(not important), @8, &8... */
+static Tgetstr_Map_Type Tgetstr_Map [] =
 {
-   {"@7", 164},			       /* KEY End */
-   {"AL", 110},			       /* parm_insert_line */
-   {"DL", 106},			       /* parm_delete_line */
-   {"RI", 112},			       /* parm_right_cursor */
-   {"ac", 146},			       /* acs_chars */
-   {"ae", 38},			       /* exit_alt_charset_mode */
-   {"as", 25},			       /* enter_alt_charset_mode */
-   {"ce", 6},			       /* clr_eol */
-   {"cl", 5},			       /* clear_screen */
-   {"cm", 10},			       /* cursor_address */
-   {"cs", 3},			       /* change_scroll_region */
-   {"dc", 21},			       /* delete_character */
-   {"ei", 42},			       /* exit_insert_mode */
-   {"eA", 155},			       /* enable alt char set */
-   {"im", 31},			       /* enter_insert_mode */
-   {"k0", 65},			       /* key_f0 */
-   {"k1", 66},			       /* key_f1 */
-   {"k2", 68},			       /* key_f2 */
-   {"k3", 69},			       /* key_f3 */
-   {"k4", 70},			       /* key_f4 */
-   {"k5", 71},			       /* key_f5 */
-   {"k6", 72},			       /* key_f6 */
-   {"k7", 73},			       /* key_f7 */
-   {"k8", 74},			       /* key_f8 */
-   {"k9", 75},			       /* key_f9 */
-   {"kA", 78},			       /* key_il */
-   {"kC", 57},			       /* key_clear */
-   {"kD", 59},			       /* key_dc */
-   {"kE", 63},			       /* key_eol, */
-   {"kF", 84},			       /* key_sf */
-   {"kH", 80},			       /* key_ll */
-   {"kI", 77},			       /* key_ic */
-   {"kL", 60},			       /* key_dl */
-   {"kM", 62},			       /* key_eic, */
-   {"kN", 81},			       /* key_npage */
-   {"kP", 82},			       /* key_ppage */
-   {"kR", 85},			       /* key_sr */
-   {"kS", 64},			       /* key_eos, */
-   {"kT", 86},			       /* key_stab */
-   {"ka", 56},			       /* key_catab */
-   {"k;", 67},			       /* key_f10 */
-   {"kb", 55},			       /* key_backspace */
-   {"kd", 61},			       /* key_down */
-   {"kh", 76},			       /* key_home */
-   {"kl", 79},			       /* key_left */
-   {"kr", 83},			       /* key_right */
-   {"kt", 58},			       /* key_ctab */
-   {"ku", 87},			       /* key_up */
-   {"mb", 26},			       /* enter_blink_mode */
-   {"md", 27},			       /* enter_bold_mode */
-   {"me", 39},			       /* exit_attribute_mode */
-   {"mr", 34},			       /* enter_reverse_mode */
-   {"se", 43},			       /* exit_standout_mode */
-   {"so", 35},			       /* enter_standout_mode */
-   {"sr", 130},			       /* scroll_reverse */
-   {"te", 40},			       /* end cursor addressing */
-   {"ti", 28},			       /* begin cursor addressing */
-   {"up", 19},			       /* cursor_up */
-   {"us", 36},			       /* enter_underline_mode */
-   {"vb", 45},			       /* flash_screen */
-   {"vi", 13},			       /* make cursor invisible */
-   {"vs", 20},			       /* make cursor visible */
-   {"ke", 88},			       /* out of keypad transmit mode */
-   {"ks", 89},			       /* keypad transmit mode */
-   {"F1", 216},			       /* key_f11 */
-   {"F2", 217},			       /* key_f12 */
-   {"F3", 218},			       /* key_f13 */
-   {"F4", 219},			       /* key_f14 */
-   {"F5", 220},			       /* key_f15 */
-   {"F6", 221},			       /* key_f16 */
-   {"F7", 222},			       /* key_f17 */
-   {"F8", 223},			       /* key_f18 */
-   {"F9", 224},			       /* key_f19 */
-   {"FA", 225},			       /* key_f20 */
-   {"op", 297},                        /* original pair */
-   {NULL, 0}
+   {"@7", 164 		UNTIC_COMMENT("KEY End")},
+   {"AB", 360 		UNTIC_COMMENT("set a color background")},
+   {"AF", 359 		UNTIC_COMMENT("set a color foreground")},
+   {"AL", 110 		UNTIC_COMMENT("parm_insert_line")},
+   {"DL", 106 		UNTIC_COMMENT("parm_delete_line")},
+   {"RI", 112 		UNTIC_COMMENT("parm_right_cursor")},
+   {"Sf", 302 		UNTIC_COMMENT("set foreground (color)")},
+   {"Sb", 303 		UNTIC_COMMENT("set background (color)")},
+   {"ac", 146 		UNTIC_COMMENT("acs_chars")},
+   {"ae", 38 		UNTIC_COMMENT("exit_alt_charset_mode")},
+   {"as", 25 		UNTIC_COMMENT("enter_alt_charset_mode")},
+   {"ce", 6 		UNTIC_COMMENT("clr_eol")},
+   {"cl", 5 		UNTIC_COMMENT("clear_screen")},
+   {"cm", 10 		UNTIC_COMMENT("cursor_address")},
+   {"cs", 3 		UNTIC_COMMENT("change_scroll_region")},
+   {"dc", 21 		UNTIC_COMMENT("delete_character")},
+   {"ds", 23 		UNTIC_COMMENT("disable status line")},
+   {"eA", 155 		UNTIC_COMMENT("enable alt char set")},
+   {"ei", 42 		UNTIC_COMMENT("exit_insert_mode")},
+   {"fs", 47 		UNTIC_COMMENT("return from status line")},
+   {"im", 31 		UNTIC_COMMENT("enter_insert_mode")},
+   {"k0", 65 		UNTIC_COMMENT("key_f0")},
+   {"k1", 66 		UNTIC_COMMENT("key_f1")},
+   {"k2", 68 		UNTIC_COMMENT("key_f2")},
+   {"k3", 69 		UNTIC_COMMENT("key_f3")},
+   {"k4", 70 		UNTIC_COMMENT("key_f4")},
+   {"k5", 71 		UNTIC_COMMENT("key_f5")},
+   {"k6", 72 		UNTIC_COMMENT("key_f6")},
+   {"k7", 73 		UNTIC_COMMENT("key_f7")},
+   {"k8", 74 		UNTIC_COMMENT("key_f8")},
+   {"k9", 75 		UNTIC_COMMENT("key_f9")},
+   {"kA", 78 		UNTIC_COMMENT("key_il")},
+   {"kC", 57 		UNTIC_COMMENT("key_clear")},
+   {"kD", 59 		UNTIC_COMMENT("key_dc")},
+   {"kE", 63 		UNTIC_COMMENT("key_eol,")},
+   {"kF", 84 		UNTIC_COMMENT("key_sf")},
+   {"kH", 80 		UNTIC_COMMENT("key_ll")},
+   {"kI", 77 		UNTIC_COMMENT("key_ic")},
+   {"kL", 60 		UNTIC_COMMENT("key_dl")},
+   {"kM", 62 		UNTIC_COMMENT("key_eic,")},
+   {"kN", 81 		UNTIC_COMMENT("key_npage")},
+   {"kP", 82 		UNTIC_COMMENT("key_ppage")},
+   {"kR", 85 		UNTIC_COMMENT("key_sr")},
+   {"kS", 64 		UNTIC_COMMENT("key_eos,")},
+   {"kT", 86 		UNTIC_COMMENT("key_stab")},
+   {"ka", 56 		UNTIC_COMMENT("key_catab")},
+   {"k;", 67 		UNTIC_COMMENT("key_f10")},
+   {"kb", 55 		UNTIC_COMMENT("key_backspace")},
+   {"kd", 61 		UNTIC_COMMENT("key_down")},
+   {"ke", 88 		UNTIC_COMMENT("End keypad transmit mode")},
+   {"kh", 76 		UNTIC_COMMENT("key_home")},
+   {"kl", 79 		UNTIC_COMMENT("key_left")},
+   {"kr", 83 		UNTIC_COMMENT("key_right")},
+   {"ks", 89 		UNTIC_COMMENT("Start keypad transmit mode")},
+   {"kt", 58 		UNTIC_COMMENT("key_ctab")},
+   {"ku", 87 		UNTIC_COMMENT("key_up")},
+   {"mb", 26 		UNTIC_COMMENT("enter_blink_mode")},
+   {"md", 27 		UNTIC_COMMENT("enter_bold_mode")},
+   {"me", 39 		UNTIC_COMMENT("exit_attribute_mode")},
+   {"mr", 34 		UNTIC_COMMENT("enter_reverse_mode")},
+   {"op", 297 		UNTIC_COMMENT("orig_pair (color)")},
+   {"pf", 119 		UNTIC_COMMENT("turn OFF printer")},
+   {"po", 120 		UNTIC_COMMENT("turn ON printer")},
+   {"se", 43 		UNTIC_COMMENT("exit_standout_mode")},
+   {"so", 35 		UNTIC_COMMENT("enter_standout_mode")},
+   {"sr", 130 		UNTIC_COMMENT("scroll_reverse")},
+   {"te", 40 		UNTIC_COMMENT("end cursor addressing")},
+   {"ti", 28 		UNTIC_COMMENT("begin cursor addressing")},
+   {"ts", 135 		UNTIC_COMMENT("goto to status line")},
+   {"up", 19 		UNTIC_COMMENT("cursor_up")},
+   {"us", 36 		UNTIC_COMMENT("enter_underline_mode")},
+   {"vb", 45 		UNTIC_COMMENT("flash_screen")},
+   {"ve", 16 		UNTIC_COMMENT("make cursor very visible")},
+   {"vi", 13 		UNTIC_COMMENT("make cursor invisible")},
+   {"vs", 20 		UNTIC_COMMENT("make cursor very visible")},
+   {"", 0 		UNTIC_COMMENT(NULL)}
 };
 
-static int compute_cap_offset (char *cap, Terminfo_Type *t, Tgetstr_Map_Type *map, int max_ofs)
+static int compute_cap_offset (char *cap, Terminfo_Type *t, Tgetstr_Map_Type *map, unsigned int max_ofs)
 {
    char cha, chb;
-   
+
+   (void) t;
    cha = *cap++; chb = *cap;
    
-   while (map->name != NULL)
+   while (*map->name != 0)
      {
 	if ((cha == *map->name) && (chb == *(map->name + 1)))
 	  {
-	     if (map->offset >= max_ofs) return -1;
+	     if (map->offset >= (int) max_ofs) return -1;
 	     return map->offset;
 	  }
 	map++;
@@ -407,11 +443,15 @@ char *SLtt_tigetstr (char *cap, char **pp)
    return t->string_table + offset;
 }
 
-Tgetstr_Map_Type Tgetnum_Map[] =
+static Tgetstr_Map_Type Tgetnum_Map[] =
 {
-   {"co", 0},			       /* columns */
-   {"li", 2},			       /* lines */
-   {NULL, -1}
+   {"co", 0		UNTIC_COMMENT("columns")},
+   {"li", 2		UNTIC_COMMENT("lines")},
+   {"Co", 13		UNTIC_COMMENT("max colors")},
+   {"pa", 14		UNTIC_COMMENT("max pairs")},
+   {"sg", 4		UNTIC_COMMENT("magic cookie glitch")},
+   {"ws", 7		UNTIC_COMMENT("num columns in status line")},
+   {"", -1		UNTIC_COMMENT(NULL)}
 };
 
 int SLtt_tigetnum (char *cap, char **pp)
@@ -428,13 +468,15 @@ int SLtt_tigetnum (char *cap, char **pp)
    return make_integer (t->numbers + 2 * offset);
 }
 
-Tgetstr_Map_Type Tgetflag_Map[] =
+static Tgetstr_Map_Type Tgetflag_Map[] =
 {
-   {"am", 1},			       /* auto_right_margin */
-   {"ms", 14},			       /* move_standout_mode */
-   {"xs", 3},			       /* ceol_standout_glitch */
-   {"sg", 4},			       /* magic cookie glitch */
-   {NULL, -1}
+   {"am", 1		UNTIC_COMMENT("auto right margin")},
+   {"hs", 9 		UNTIC_COMMENT("has status line")},
+   {"ms", 14		UNTIC_COMMENT("move standout mode")},
+   {"xs", 3		UNTIC_COMMENT("ceol standout glitch")},
+   {"xn", 4		UNTIC_COMMENT("NEWLINE ignored after 80 columns")},
+   {"es", 16   		UNTIC_COMMENT("status line esc ok")},
+   {"", -1		UNTIC_COMMENT(NULL)}
 };
 
 int SLtt_tigetflag (char *cap, char **pp)
@@ -478,7 +520,7 @@ static int tcap_getflag (char *cap, Terminfo_Type *t)
    return 0;
 }
 
-static char *tcap_get_cap (unsigned char *cap, unsigned char *caps, int len)
+static char *tcap_get_cap (unsigned char *cap, unsigned char *caps, unsigned int len)
 {
    unsigned char c0, c1;
    unsigned char *caps_max;
@@ -520,16 +562,22 @@ static int tcap_extract_field (unsigned char *t0)
    return -1;
 }
 
-int SLtt_try_termcap = 1;
+int SLtt_Try_Termcap = 1;
 static int tcap_getent (char *term, Terminfo_Type *ti)
 {
    unsigned char *termcap, ch;
    unsigned char *buf, *b;
    unsigned char *t;
    int len;
-
-   if (!SLtt_try_termcap == 0) return -1;
    
+   if (SLtt_Try_Termcap == 0) return -1;
+#if 1
+   /* XFREE86 xterm sets the TERMCAP environment variable to an invalid
+    * value.  Specifically, it lacks the tc= string.
+    */
+   if (!strncmp (term, "xterm", 5))
+     return -1;
+#endif
    termcap = (unsigned char *) getenv ("TERMCAP");
    if ((termcap == NULL) || (*termcap == '/')) return -1;
    
@@ -550,7 +598,7 @@ static int tcap_getent (char *term, Terminfo_Type *ti)
    
    /* malloc some extra space just in case it is needed. */
    len = strlen ((char *) termcap) + 256;
-   if (NULL == (buf = (unsigned char *) SLMALLOC (len))) return -1;
+   if (NULL == (buf = (unsigned char *) SLMALLOC ((unsigned int) len))) return -1;
 
    b = buf;
    
@@ -566,7 +614,7 @@ static int tcap_getent (char *term, Terminfo_Type *ti)
 	SLFREE (buf);
 	return -1;
      }
-   strncpy ((char *) b, (char *) t, len);
+   strncpy ((char *) b, (char *) t, (unsigned int) len);
    b[len] = 0;
    b += len + 1;
    ti->name_section_size = len;
@@ -676,7 +724,7 @@ static int tcap_getent (char *term, Terminfo_Type *ti)
 #include <curses.h>
 #include <term.h>
 
-int SLtt_try_termcap = 1;
+int SLtt_Try_Termcap = 1;
 
 char *SLtt_tigetent (char *term)
 {
