@@ -301,17 +301,27 @@ static void do_enter_key (Dlg_head *h, int f_pos)
 	run_dlg (chl_dlg);
 	
 	if (b_pos != chl_list->pos){
+	    int ok = 0;
 	    if (f_pos == 3){
 		chl_pass = getpwnam (chl_list->current->text);
-		sf_stat->st_uid = chl_pass->pw_uid;
+		if (chl_pass){
+		    ok = 1;
+		    sf_stat->st_uid = chl_pass->pw_uid;
+		}
 	    } else {
 		chl_grp = getgrnam (chl_list->current->text);
-		sf_stat->st_gid = chl_grp->gr_gid;
+		if (chl_grp){
+		    sf_stat->st_gid = chl_grp->gr_gid;
+		    ok = 1;
+		}
+	    }
+	    if (ok){
+		ch_flags [f_pos + 6] = '+';
+		get_ownership ();
 	    }
-	    ch_flags [f_pos + 6] = '+';
-	    get_ownership ();
 	    dlg_focus (h);
-	    print_flags ();
+	    if (ok)
+		print_flags ();
 	}
 	if (chl_dlg->ret_value == KEY_LEFT){
 	    if (f_pos == 4)
