@@ -24,6 +24,7 @@
 #include <config.h>
 #include <stdarg.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #    include <unistd.h>
@@ -35,7 +36,9 @@
 #include "dlg.h"
 #include "widget.h"
 #include "wtools.h"
-/* We currenlty only support one way of comunicating the background
+
+/*
+ * We currenlty only support one way of comunicating the background
  * and foreground process by using the socketpair system call
  */
 #ifdef USE_NETCODE
@@ -47,6 +50,7 @@
 #include "file.h"
 #include "background.h"
 #include "mad.h"
+#include "key.h"	/* For add_select_channel(), delete_select_channel() */
 
 /* If true, this is a background process */
 int we_are_background = 0;
@@ -59,12 +63,56 @@ extern int recursive_result;
 /* If set background tasks wait to be attached */
 int background_wait = 0;
 
+#ifndef HAVE_SOCKETPAIR
+int socketpair(int, int, int, int fd[2]);
+#endif
+
 /* File descriptor for talking to our parent */
 static int parent_fd;
 
 #define MAXCALLARGS 4		/* Number of arguments supported */
 #define mymsg "Desde el hijo\n\r"
 
+struct TaskList *task_list = NULL;
+    
+void
+register_task_running (pid_t pid, int fd, char *info)
+{
+    TaskList *new;
+
+    new = xmalloc (sizeof (TaskList), "note_task_running");
+    new->pid   = pid;
+    new->info  = info;
+    new->state = Task_Running;
+    new->next  = task_list;
+    new->fd    = fd;
+    task_list  = new;
+
+    add_select_channel (fd, background_attention, (void *) pid);
+}
+
+void
+unregister_task_running (pid_t pid, int fd)
+{
+    TaskList *p = task_list;
+    TaskList *prev = 0;
+
+    while (p){
+	if (p->pid == pid){
+	    if (prev)
+		prev->next = p->next;
+	    else
+		task_list = p->next;
+	    free (p->info);
+	    free (p);
+	    break;
+	}
+	prev = p;
+	p = p->next;
+    }
+    delete_select_channel (fd);
+}
+
 /*
  * Try to make the Midnight Commander a background job
  *
@@ -74,7 +122,7 @@ static int parent_fd;
  * -1 on failure
  */
 int
-do_background (void)
+do_background (char *info)
 {
     int comm [2];		/* control connection stream */
     int pid;
@@ -86,7 +134,7 @@ do_background (void)
 	return -1;
     
     if (pid == 0){
-	int i, nullfd;
+	int nullfd;
 
 	parent_fd = comm [1];
 	we_are_background = 1;
@@ -116,7 +164,7 @@ do_background (void)
 	return 0;
     } else {
 	close (comm [1]);
-	add_select_channel (comm [0], background_attention, (void *) pid);
+	register_task_running (pid, comm [0], info);
 	return 1;
     }
 }
@@ -188,7 +236,7 @@ background_attention (int fd, void *xpid)
     int  argc, i, result, status;
     char *data [MAXCALLARGS];
     char *resstr;
-    int  pid = (int) xpid;
+    pid_t  pid = (pid_t) xpid;
     int bytes;
     enum ReturnType type;
     
@@ -198,14 +246,14 @@ background_attention (int fd, void *xpid)
 	    message (1, " Background process error ", " Child died unexpectedly ");
 	else
 	    message (1, " Background process error ", " Unknown error in child ");
-	delete_select_channel (fd);
+	unregister_task_running (pid, fd);
 	waitpid (pid, &status, 0);
 	return 0;
     }
 
     /* If the routine is zero, then the child is telling us that he is dying */
     if ((int) routine == MSG_CHILD_EXITING){
-	delete_select_channel (fd);
+	unregister_task_running (pid, fd);
 	waitpid (pid, &status, 0);
 	return 0;
     }
@@ -315,8 +363,6 @@ int
 parent_call (void *routine, int argc, ...)
 {
     va_list ap;
-    enum ReturnType type = Return_Integer;
-    void *a;
     int i;
     
     va_start (ap, argc);
@@ -343,7 +389,6 @@ char *
 parent_call_string (void *routine, int argc, ...)
 {
     va_list ap;
-    void *a;
     char *str;
     int i;
     
@@ -377,7 +422,7 @@ int
 call_1s (int (*routine)(enum OperationMode, char *), char *str)
 {
     if (we_are_background)
-	return parent_call (routine, 1, strlen (str), str);
+	return parent_call ((void *)routine, 1, strlen (str), str);
     else 
 	return (*routine)(Foreground, str);
 }
@@ -386,7 +431,7 @@ void
 message_1s (int flags, char *title, char *str1)
 {
     if (we_are_background)
-	parent_call (real_message_1s, 3, sizeof (flags), &flags,
+	parent_call ((void *)real_message_1s, 3, sizeof (flags), &flags,
 		    strlen (title), title, strlen (str1), str1);
     else
 	real_message_1s (&flags, title, str1);
@@ -396,7 +441,7 @@ void
 message_2s (int flags, char *title, char *str1, char *str2)
 {
     if (we_are_background)
-	parent_call (real_message_2s, 4, sizeof (flags), &flags,
+	parent_call ((void *)real_message_2s, 4, sizeof (flags), &flags,
 		     strlen (title), title, strlen (str1), str1,
 	             strlen (str2), str2);
     else
@@ -407,7 +452,7 @@ void
 message_3s (int flags, char *title, char *str1, char *str2, const char *str3)
 {
     if (we_are_background)
-	parent_call (real_message_3s, 3, sizeof (flags), &flags,
+	parent_call ((void *)real_message_3s, 3, sizeof (flags), &flags,
 		     strlen (title), title, strlen (str1), str1,
 	             strlen (str2), str2, strlen (str3), str3);
     else
@@ -419,7 +464,7 @@ input_dialog_help (char *header, char *text, char *help, char *def_text)
 {
     extern char *real_input_dialog_help (char *header, char *text, char *help, char *def_text);
     if (we_are_background)
-	return parent_call_string (real_input_dialog_help, 4,
+	return parent_call_string ((void *)real_input_dialog_help, 4,
 				   strlen (header),   header,
 				   strlen (text),     text,
 				   strlen (help),     help,
