@@ -356,19 +356,22 @@ void confirm_box ()
     }
 }
 
-#define DISPY 9
+#define DISPY 11
 #define DISPX 46
 
 static int new_mode;
+static int new_meta;
 
 char *display_bits_str [] =
-{ "Full 8 bits", "ISO 8859-1", "7 bits" };
+{ "Full 8 bits output", "ISO 8859-1", "7 bits" };
 
 static QuickWidget display_widgets [] = {
-{ quick_button,   4,  6,    3, DISPY, " [ Ok ]",
-      'o', 3, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "o" },
 { quick_button,   4,  6,    4, DISPY, " [ Cancel ]",
       'c', 3, B_CANCEL, 0, 0, XV_WLAY_CENTERROW, "c" },
+{ quick_button,   4,  6,    3, DISPY, " [ Ok ]",
+      'o', 3, B_ENTER, 0, 0, XV_WLAY_CENTERROW, "o" },
+{ quick_checkbox, 4, DISPX, 7, DISPY, "Full 8 bits input",
+      'u', 0, 0, &new_meta, 0, XV_WLAY_BELOWCLOSE, "u" },
 { quick_radio,    4, DISPX, 3, DISPY, "", 3, 0, 0,
       &new_mode, display_bits_str, XV_WLAY_BELOWCLOSE, "r" },
 { 0,              0, 0, 0, 0, 0, 0, 0, 0, 0, 0, XV_WLAY_DONTCARE }
@@ -380,7 +383,6 @@ static QuickDialog display_bits =
 
 void display_bits_box ()
 {
-    int v;
     int current_mode;
 
     if (full_eight_bits)
@@ -390,10 +392,10 @@ void display_bits_box ()
     else
 	current_mode = 2;
     
-    display_widgets [2].value = current_mode;
-    v = quick_dialog (&display_bits);
-    if (v != B_ENTER)
-	return;
+    display_widgets [3].value = current_mode;
+    new_meta = !use_8th_bit_as_meta;
+    if (quick_dialog (&display_bits) != B_ENTER)
+	    return;
 
     eight_bit_clean = new_mode < 2;
     full_eight_bits = new_mode == 0;
@@ -402,6 +404,7 @@ void display_bits_box ()
 #else
     SLsmg_Display_Eight_Bit = full_eight_bits ? 128 : 160;
 #endif
+    use_8th_bit_as_meta=!new_meta;
 }
 
 #define TREE_Y 20
@@ -433,6 +436,7 @@ char *tree (char *current_dir)
     WTree    *mytree;
     Dlg_head *dlg;
     char     *val;
+    WButtonBar *bar;
 
     tree_colors [3] = dialog_colors [0];
     tree_colors [1] = dialog_colors [1];
@@ -442,7 +446,11 @@ char *tree (char *current_dir)
 		      tree_callback, "[Directory Tree]", "tree", DLG_CENTER);
     mytree = tree_new (0, 2, 2, TREE_Y - 6, TREE_X - 5);
     add_widget (dlg, mytree);
-
+    bar = buttonbar_new(1);
+    add_widget (dlg, bar);
+    bar->widget.x = 0;
+    bar->widget.y = LINES - 1;
+    
     run_dlg (dlg);
     if (dlg->ret_value == B_ENTER)
 	val = strdup (mytree->selected_ptr->name);
@@ -645,7 +653,7 @@ char *cd_dialog (void)
 	quick_widgets [INPUT_INDEX].y_divisions =
 	        Quick_input.ylen  = 5;
     Quick_input.xpos = 2;
-    Quick_input.ypos = ((Widget *) cmdline)->y - Quick_input.ylen;
+    Quick_input.ypos = LINES - 2 - Quick_input.ylen;
     quick_widgets [INPUT_INDEX].relative_y = 2;
     quick_widgets [INPUT_INDEX].str_result = &my_str;
     
@@ -729,7 +737,7 @@ task_cb (int action, void *ignored)
     int sig;
     
     if (!bg_list->list)
-	return;
+	return 0;
 
     /* Get this instance information */
     tl = (TaskList *) bg_list->current->data;
