@@ -222,7 +222,7 @@ static inline void do_chown (uid_t u, gid_t g)
 	message (1, " Error ", " Couldn't chown \"%s\" \n %s ",
 	     cpanel->dir.list [current_file].fname, unix_error_string (errno));
 
-    file_mark (cpanel, current_file, 0);
+    do_file_mark (cpanel, current_file, 0);
 }
 
 static void apply_chowns (uid_t u, gid_t g)
@@ -231,13 +231,11 @@ static void apply_chowns (uid_t u, gid_t g)
   
     need_update = end_chown = 1;
     do_chown (u,g);
-    cpanel->marked--;
   
     do {
 	fname = next_file ();
     
 	do_chown (u,g);
-	cpanel->marked--;
     } while (cpanel->marked);
 }
 
@@ -338,8 +336,7 @@ void chown_cmd (void)
 	}
 	
 	if (cpanel->marked && ch_dlg->ret_value != B_CANCEL){
-	    file_mark (cpanel, current_file, 0);
-	    cpanel->marked--;
+	    do_file_mark (cpanel, current_file, 0);
 	    need_update = 1;
 	}
 	destroy_dlg (ch_dlg);
