@@ -76,7 +76,7 @@
 #include "color.h"
 #define MIDNIGHT
 #ifdef  USE_INTERNAL_EDIT
-#include "../edit/edit.h"	/* edit */
+    extern int edit (const char *file);
 #endif
 #include "../vfs/vfs.h"
 #define WANT_WIDGETS
@@ -573,6 +573,7 @@ static int check_for_default(char *default_file, char *file)
 void ext_cmd (void)
 {
     char *buffer;
+    char *extdir;
     int  dir;
 
     dir = 0;
@@ -581,13 +582,17 @@ void ext_cmd (void)
 			    " Which extension file you want to edit? ", 0, 2,
 			    " User ", " System Wide ");
     }
+    extdir = concat_dir_and_file (mc_home, "mc.ext");
+
     if (dir == 0){
 	buffer = concat_dir_and_file (home_dir, ".mc.ext");
-	check_for_default (LIBDIR "mc.ext", buffer);
+	check_for_default (extdir, buffer);
 	do_edit (buffer);
 	free (buffer);
     } else if (dir == 1)
-	do_edit (LIBDIR "mc.ext");
+	do_edit (extdir);
+
+   free (extdir);
    flush_extension_file ();
 }
 
@@ -596,6 +601,7 @@ void menu_edit_cmd (void)
 #define MC_MENU "mc.menu"
 
     char *buffer, *home_file;
+    char *menufile;
     int dir = 0;
     
     if (geteuid () == 0)
@@ -608,11 +614,13 @@ void menu_edit_cmd (void)
     switch (dir){
     case 0:
 	buffer = strdup (home_file);
-	check_for_default (LIBDIR MC_MENU, buffer);
+        menufile = concat_dir_and_file(mc_home, MC_MENU);
+	check_for_default (menufile, buffer);
+        free (menufile);
 	break;
 
     case 1:
-	buffer = copy_strings (LIBDIR, MC_MENU, 0);
+	buffer = copy_strings (mc_home, MC_MENU, 0);
 	break;
 
     default:
@@ -860,6 +868,7 @@ void view_other_cmd (void)
 	reset_shell_mode ();
 	noecho ();
 #endif
+	keypad(stdscr, FALSE);
 	endwin ();
 	if (!status_using_ncurses)
 	    do_exit_ca_mode ();
@@ -892,7 +901,7 @@ void view_other_cmd (void)
 	    fprintf (stderr, "\033="); fflush (stderr);
 	}
 	reset_prog_mode ();
-	
+	keypad(stdscr, TRUE);
 #ifndef HAVE_X	
 	if (use_mouse_p)
 	    init_mouse ();
@@ -1073,7 +1082,9 @@ void other_symlink_cmd (void)
 
 void help_cmd (void)
 {
-    interactive_display (LIBDIR "mc.hlp", "[main]");
+   char *hlpfile = concat_dir_and_file (mc_home, "mc.hlp");
+   interactive_display (hlpfile, "[main]");
+   free (hlpfile);
 }
 
 void view_panel_cmd (void)
@@ -1095,6 +1106,7 @@ void mkdir_panel_cmd (void)
 char *get_random_hint (void)
 {
     char *data, *result, *eol;
+    char *hintfile;
     int  len;
     int start;
     
@@ -1118,7 +1130,9 @@ char *get_random_hint (void)
     last_sec = tv.tv_sec;
 #endif
 
-    data = load_file (LIBDIR "mc.hint");
+    hintfile = concat_dir_and_file (mc_home, "mc.hint");
+    data = load_file (hintfile);
+    free (hintfile);
     if (!data)
 	return 0;
 
