@@ -5,6 +5,8 @@ void ftplink_cmd (void);
 void undelete_cmd (void);
 void help_cmd (void);
 void dirsizes_cmd (void);
+int view_file_at_line (char *filename, int plain_view, int internal, 
+                       int start_line);
 int view_file (char *filename, int normal, int internal);
 void view_cmd (WPanel *panel);
 void view_simple_cmd ();
@@ -48,6 +50,4 @@ void view_panel_cmd (void);
 void quick_cd_cmd (void);
 char *get_random_hint (void);
 void source_routing (void);
-
-extern int use_internal_edit;
 #endif /* __CMD_H */
