@@ -63,22 +63,30 @@ static struct colorpair color_map [] = {
 
     /* Dialog colors */
     { "dnormal=",    0, 0 },	/* Dialog normal */
-    { "dfocus=",     0, 0  },	/* Dialog focused */
+    { "dfocus=",     0, 0 },	/* Dialog focused */
     { "dhotnormal=", 0, 0 },	/* Dialog normal/hot */
     { "dhotfocus=",  0, 0 },	/* Dialog focused/hot */
     
     { "viewunderline=", 0, 0 },	/* _\b? sequence in view */
-    { "menusel=", 0, 0 },	/* Menu selected color */
-    { "menuhot=", 0, 0 },       /* Color for menu hotkeys */
+    { "menusel=",    0, 0 },	/* Menu selected color */
+    { "menuhot=",    0, 0 },    /* Color for menu hotkeys */
     { "menuhotsel=", 0, 0 },    /* Menu hotkeys/selected entry */
     
-    { "helpnormal=", 0, 0},     /* Help normal */
-    { "helpitalic=", 0, 0},     /* Italic in help */
-    { "helpbold=", 0, 0},       /* Bold in help */
-    { "helplink=", 0, 0},       /* Not selected hyperlink */
-    { "helpslink=", 0, 0},      /* Selected hyperlink */
+    { "helpnormal=", 0, 0 },    /* Help normal */
+    { "helpitalic=", 0, 0 },    /* Italic in help */
+    { "helpbold=",   0, 0 },    /* Bold in help */
+    { "helplink=",   0, 0 },    /* Not selected hyperlink */
+    { "helpslink=",  0, 0 },    /* Selected hyperlink */
     
-    { "gauge=", 0, 0},		/* Color of the progress bar (percentage) */
+    { "gauge=",      0, 0 },    /* Color of the progress bar (percentage) */
+    { "input=",      0, 0 },
+ 
+    /* Per file types colors */
+    { "directory=",  0, 0 },
+    { "execute=",    0, 0 },
+    { "link=",       0, 0 },
+    { "device=",     0, 0 },
+    { "special=",    0, 0 },
 };
 
 struct color_table_s {
@@ -89,6 +97,7 @@ struct color_table_s {
 
 struct color_table_s color_table [] = {
     { "black",         COLOR_BLACK   },
+    { "gray",         COLOR_BLACK   | A_BOLD },
     { "red",           COLOR_RED     },
     { "brightred",     COLOR_RED     | A_BOLD },
     { "green",         COLOR_GREEN   },
@@ -186,7 +195,13 @@ static char *default_colors =
 "helpbold=blue,white:"
 "helplink=black,cyan:"
 "helpslink=yellow,blue:"
-"gauge=white,black";
+"gauge=white,black:"
+"input=black,cyan:"
+"directory=white,blue:"
+"execute=brightgreen,blue:"
+"link=lightgray,blue:"
+"device=brightmagenta,blue:"
+"special=black,blue";
 
 static void configure_colors (void)
 {
