@@ -116,10 +116,7 @@ static int examine_cd (char *path)
             c = *s;
             *s = 0;
             if (*p) {
-                if (*(s - 1) == PATH_SEP)
-                    r = copy_strings (p, q, NULL);
-                else
-                    r = copy_strings (p, PATH_SEP_STR, q, NULL);
+		r = concat_dir_and_file (p, q);
                 result = do_cd (r, cd_parse_command);
                 free (r);
             }
@@ -167,7 +164,7 @@ void do_cd_command (char *cmd)
 	} else {
 	    char *old = cpanel->cwd;
 	    char *new;
-	    new = copy_strings (old, PATH_SEP_STR, cmd+3, 0);
+	    new = concat_dir_and_file (old, cmd+3);
 	    tree_chdir (the_tree, new);
 	    free (new);
 	}
