@@ -491,10 +491,10 @@ char *command_completion_function (char *text, int state)
 	    	path = xmalloc (strlen (p) + 2, "Command completion");
 	    	strcpy (path, p);
 	    	path [strlen (p) + 1] = 0;
-	    	p = strchr (path, ':');
+	    	p = strchr (path, PATH_ENV_SEP);
 	    	while (p){
 	    	    *p = 0;
-	    	    p = strchr (p + 1, ':');
+	    	    p = strchr (p + 1, PATH_ENV_SEP);
 	    	}
 	    }
 	}
