@@ -20,6 +20,8 @@
 #include <config.h>
 
 int    cons_saver_pid = 1;
+extern int rxvt_extensions;
+signed char console_flag = 0;
 
 #if defined(linux) || defined(__linux__)
 
@@ -33,8 +35,6 @@ int    cons_saver_pid = 1;
 #include "win.h"
 #include "cons.saver.h"
 
-signed char console_flag = 0;
-
 static int pipefd1 [2] = {-1, -1}, pipefd2 [2] = {-1, -1};
 
 void show_console_contents (int starty, unsigned char begin_line, unsigned char end_line)
@@ -44,6 +44,12 @@ void show_console_contents (int starty, unsigned char begin_line, unsigned char
     int i;
 
     standend ();
+
+    if (look_for_rxvt_extensions ()) {
+	show_rxvt_contents (starty, begin_line, end_line);
+	return;
+    }
+
     /* Is tty console? */
     if (!console_flag)
 	return;
@@ -87,6 +93,8 @@ void handle_console (unsigned char action)
 
     switch (action){
     case CONSOLE_INIT:
+	if (look_for_rxvt_extensions ())
+	    return;
 	/* Close old pipe ends in case it is the 2nd time we run cons.saver */
 	close (pipefd1[1]);
 	close (pipefd2[0]);
@@ -146,6 +154,8 @@ void handle_console (unsigned char action)
     case CONSOLE_DONE:
     case CONSOLE_SAVE:
     case CONSOLE_RESTORE:
+	if (look_for_rxvt_extensions ())
+	    return;
 	/* Is tty console? */
 	if (!console_flag)
 	    return;
@@ -195,7 +205,6 @@ void handle_console (unsigned char action)
 #include "cons.saver.h"
 
 static int FD_OUT = 2;
-signed char console_flag = 0;
 
 static unsigned short* vidbuf = NULL;
 static unsigned short* screen = NULL;
@@ -341,6 +350,8 @@ console_init()
 void
 handle_console (unsigned char action)
 {
+	if (look_for_rxvt_extensions ())
+		return;
 	switch (action){
 		case CONSOLE_INIT:
 			console_init();
@@ -367,6 +378,10 @@ show_console_contents (int starty, unsigned char begin_line, unsigned char end_l
 {
 	register int i, len = (end_line - begin_line) * width;
 
+	if (look_for_rxvt_extensions ()) {
+		show_rxvt_contents (starty, begin_line, end_line);
+		return;
+	}
 	attrset(DEFAULT_COLOR);
 	for (i = 0; i < len; i++)
 	{
@@ -377,3 +392,28 @@ show_console_contents (int starty, unsigned char begin_line, unsigned char end_l
 }
 
 #endif /* SCO_FLAVOR */
+
+
+#if !defined(linux) && !defined(__linux__) && !defined(SCO_FLAVOR)
+
+#include "tty.h"
+
+void show_console_contents (int starty, unsigned char begin_line, unsigned char end_line)
+{
+    standend ();
+
+    if (look_for_rxvt_extensions ()) {
+	show_rxvt_contents (starty, begin_line, end_line);
+	return;
+    }
+    console_flag = 0;
+}
+
+void handle_console (unsigned char action)
+{
+    look_for_rxvt_extensions ();
+}
+
+#endif				/* !defined(linux) && !defined(__linux__) && !defined(SCO_FLAVOR) */
+
+
