@@ -30,8 +30,9 @@
 #include "win.h"
 #include "mouse.h"
 #include "main.h"
-#include "key.h"	/* For mi_getch() */
-#include "dlg.h"	/* draw_box, yes I know, it's silly */
+#include "key.h"		/* For mi_getch() */
+#include "dlg.h"		/* draw_box, yes I know, it's silly */
+#include "background.h"		/* we_are_background definition */
 
 /* "$Id: dialog.c,v 1.15 1995/02/21 19:05:31 miguel Exp $" */
 
@@ -64,7 +65,6 @@ void pop_refresh (void)
 
 static void do_complete_refresh (Refresh *refresh_list)
 {
-#ifndef HAVE_X
     if (!refresh_list)
 	return;
 
@@ -72,12 +72,12 @@ static void do_complete_refresh (Refresh *refresh_list)
 	do_complete_refresh (refresh_list->next);
     
     (*(refresh_list->refresh_fn))(refresh_list->parameter);
-#endif    
 }
 
 void do_refresh (void)
 {
-#ifndef HAVE_X
+    if (we_are_background)
+	return;
     if (!refresh_list)
 	return;
     else {
@@ -87,7 +87,6 @@ void do_refresh (void)
 	    do_complete_refresh (refresh_list);
 	}
     }
-#endif    
 }
 
 /* Poor man's window puts, it doesn't handle auto-wrap */
