@@ -47,7 +47,7 @@ int mix_all_files = 0;
 static int reverse = 1;
 
 /* Are the files sorted case sensitively? */
-static int case_sensitive = 1;
+static int case_sensitive = OS_SORT_CASE_SENSITIVE_DEFAULT;
 
 #define MY_ISDIR(x) ( (S_ISDIR (x->buf.st_mode) || x->f.link_to_dir) ? 1 : 0)
 
@@ -301,25 +301,37 @@ void clean_dir (dir_list *list, int count)
     }
 }
 
-/* Used to set up a directory list when there is no access to a directory */
-int set_zero_dir (dir_list *list)
+static int 
+add_dotdot_to_list (dir_list *list, int index)
 {
     char buffer [MC_MAXPATHLEN + MC_MAXPATHLEN];
     char *p, *s;
     int i = 0;
     
-    (list->list) [0].fname = strdup ("..");
-    (list->list) [0].cache = NULL;
+    /* Need to grow the *list? */
+    if (index == list->size) {
+	list->list = realloc (list->list, sizeof (file_entry) *
+			      (list->size + RESIZE_STEPS));
+	if (!list->list)
+	    return 0;
+	list->size += RESIZE_STEPS;
+    }
+
+    (list->list) [index].fnamelen = 2;
+    (list->list) [index].fname = strdup ("..");
+    (list->list) [index].cache = NULL;
+    (list->list) [index].f.link_to_dir = 0;
+    (list->list) [index].f.stalled_link = 0;
 
     /* FIXME: We need to get the panel definition! to use file_mark */
-    (list->list) [0].f.marked = 0;
+    (list->list) [index].f.marked = 0;
     mc_get_current_wd (buffer, sizeof (buffer) - 1 );
     if (buffer [strlen (buffer) - 1] == PATH_SEP)
     	buffer [strlen (buffer) - 1] = 0;
     for (;;) {
     	strcat (buffer, PATH_SEP_STR "..");
         p = vfs_canon (buffer);
-        if (mc_stat (p, &((list->list) [0].buf)) != -1){
+        if (mc_stat (p, &((list->list) [index].buf)) != -1){
 	    free (p);
             break;
 	}
@@ -331,12 +343,24 @@ int set_zero_dir (dir_list *list)
 	strcpy (buffer, p);
 	free (p);
     }
+
+/* Commented out to preserve a usable '..'. What's the purpose of this
+ * three lines? (Norbert) */
+#if 0
     if (i) { /* So there is bogus information on the .. directory's stat */
-        (list->list) [0].buf.st_mode &= ~0444;
+        (list->list) [index].buf.st_mode &= ~0444;
     }
+#endif
     return 1;
 }
 
+/* Used to set up a directory list when there is no access to a directory */
+int set_zero_dir (dir_list *list)
+{
+    return (add_dotdot_to_list (list, 0));
+}
+
+/* If you change handle_dirent then check also handle_path. */
 /* Return values: -1 = failure, 0 = don't add, 1 = add to the list */
 int handle_dirent (dir_list *list, char *filter, struct dirent *dp,
 		   struct stat *buf1, int next_free, int *link_to_dir,
@@ -384,6 +408,45 @@ int handle_dirent (dir_list *list, char *filter, struct dirent *dp,
     return 1;
 }
 
+/* handle_path is a simplified handle_dirent. The difference is that 
+   handle_path doesn't pay attention to show_dot_files and show_backups.
+   Moreover handle_path can't be used with a filemask. 
+   If you change handle_path then check also handle_dirent. */
+/* Return values: -1 = failure, 0 = don't add, 1 = add to the list */
+int handle_path (dir_list *list, char *path,
+		   struct stat *buf1, int next_free, int *link_to_dir,
+		   int *stalled_link)
+{
+    if (path [0] == '.' && path [1] == 0)
+	return 0;
+    if (mc_lstat (path, buf1) == -1)
+        return 0;
+
+    if (S_ISDIR (buf1->st_mode))
+	tree_check (path);
+    
+    /* A link to a file or a directory? */
+    *link_to_dir = 0;
+    *stalled_link = 0;
+    if (S_ISLNK(buf1->st_mode)){
+	struct stat buf2;
+	if (!mc_stat (path, &buf2))
+	    *link_to_dir = S_ISDIR(buf2.st_mode) != 0;
+	else
+	    *stalled_link = 1;
+    }
+
+    /* Need to grow the *list? */
+    if (next_free == list->size){
+	list->list = realloc (list->list, sizeof (file_entry) *
+			      (list->size + RESIZE_STEPS));
+	if (!list->list)
+	    return -1;
+	list->size += RESIZE_STEPS;
+    }
+    return 1;
+}
+
 int do_load_dir(dir_list *list, sortfn *sort, int reverse, int case_sensitive, char *filter)
 {
     DIR           *dirp;
@@ -391,6 +454,7 @@ int do_load_dir(dir_list *list, sortfn *sort, int reverse, int case_sensitive, c
     int           status, link_to_dir, stalled_link;
     int           next_free = 0;
     struct stat   buf;
+    int dotdot_found = 0;
 
     start_tree_check (NULL);
     
@@ -412,12 +476,18 @@ int do_load_dir(dir_list *list, sortfn *sort, int reverse, int case_sensitive, c
 	list->list [next_free].f.link_to_dir = link_to_dir;
 	list->list [next_free].f.stalled_link = stalled_link;
 	list->list [next_free].buf = buf;
+	if (strcmp (dp->d_name, ".." ) == 0)
+	    dotdot_found = 1;
 	next_free++;
 	if (!(next_free % 32))
 	    rotate_dash ();
     }
-    if (next_free)
+
+    if (next_free) {
+	if (!dotdot_found)
+	    add_dotdot_to_list (list, next_free++);
 	do_sort (list, sort, next_free-1, reverse, case_sensitive);
+    }
     else
 	return set_zero_dir (list);
     
@@ -487,6 +557,7 @@ int do_reload_dir (dir_list *list, sortfn *sort, int count, int rev,
     int           i, found, status, link_to_dir, stalled_link;
     struct stat   buf;
     int		  tmp_len;  /* For optimisation */
+    int 	  dotdot_found = 0; 
 
     start_tree_check (NULL);
     dirp = mc_opendir (".");
@@ -543,14 +614,19 @@ int do_reload_dir (dir_list *list, sortfn *sort, int count, int rev,
 	list->list [next_free].f.link_to_dir = link_to_dir;
 	list->list [next_free].f.stalled_link = stalled_link;
 	list->list [next_free].buf = buf;
+	if (strcmp (dp->d_name, ".." ) == 0)
+	    dotdot_found = 1;
 	next_free++;
 	if (!(next_free % 16))
 	    rotate_dash ();
     }
     mc_closedir (dirp);
     end_tree_check (NULL);
-    if (next_free)
+    if (next_free) {
+	if (!dotdot_found)
+	    add_dotdot_to_list (list, next_free++);
 	do_sort (list, sort, next_free-1, rev, case_sensitive);
+    }
     else
 	next_free = set_zero_dir (list);
     clean_dir (&dir_copy, count);
