@@ -38,6 +38,8 @@ int set_zero_dir (dir_list *list);
 #ifdef DIR_H_INCLUDE_HANDLE_DIRENT
 int handle_dirent (dir_list *list, char *filter, struct dirent *dp,
 		   struct stat *buf1, int next_free, int *link_to_dir, int *stalled_link);
+int handle_path (dir_list *list, char *path, struct stat *buf1, int next_free, 
+		   int *link_to_dir, int *stalled_link);
 #endif
 
 /* Sorting functions */
