@@ -125,7 +125,7 @@ void dlg_erase (Dlg_head *h)
     int x, y;
 
     for (y = 0; y < h->lines; y++){
-	move (y+h->y, h->x);
+	move (y+h->y, h->x);	/* FIXME: should test if ERR */
 	for (x = 0; x < h->cols; x++){
 	    addch (' ');
 	}
@@ -513,6 +513,8 @@ void dlg_refresh (void *parameter)
 
 static INLINE void dialog_handle_key (Dlg_head *h, int d_key)
 {
+   char *hlpfile;
+
     switch (d_key){
     case KEY_LEFT:
     case KEY_UP:
@@ -525,7 +527,9 @@ static INLINE void dialog_handle_key (Dlg_head *h, int d_key)
 	break;
 
     case KEY_F(1):
-	interactive_display (LIBDIR "mc.hlp", h->help_ctx);
+        hlpfile = concat_dir_and_file (mc_home, "mc.hlp");
+	interactive_display (hlpfile, h->help_ctx);
+        free (hlpfile);
 	do_refresh ();
 	break;
 
@@ -535,7 +539,7 @@ static INLINE void dialog_handle_key (Dlg_head *h, int d_key)
 	    
     case XCTRL('l'):
 #ifndef HAVE_SLANG
-	/* Use this is the refreshes fail */
+	/* Use this if the refreshes fail */
 	clr_scr ();
 	do_refresh ();
 #else
@@ -583,7 +587,11 @@ static int dlg_try_hotkey (Dlg_head *h, int d_key)
     if (d_key & ALT(0) && c >= 'A' && c <= 'z')
 	d_key = c;
 
-     
+#ifdef _OS_NT
+	/* .ado: fix problem with file_permission under Win95 */
+    if (d_key == 0) return 0;
+#endif
+	
     handled = 0;
     if (h->current->widget->options & W_WANT_HOTKEY)
 	handled = callback (h) (h, h->current->widget, WIDGET_HOTKEY, d_key);
@@ -797,8 +805,8 @@ void run_dlg (Dlg_head *h)
 #else
     event.x = -1;
     while (h->running) {
-#ifdef HAVE_SLANG
-	/* It does not work with ncurses, it will break */
+#if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
+	/* It does not work with ncurses before 1.9.9g, it will break */
 	if (winch_flag)
 	    change_screen_size ();
 #endif
