@@ -328,7 +328,7 @@ char *regex_command (char *filename, char *action, char *drops, int *move_dir)
 	    extension_file = buffer;
         else
 check_stock_mc_ext:
-	    extension_file = LIBDIR "mc.ext";
+	    extension_file = concat_dir_and_file (mc_home, "mc.ext");
         if ((data = load_file (extension_file)) == NULL) {
 	    free (buffer);
 	    return 0;
@@ -342,24 +342,37 @@ check_stock_mc_ext:
 		    home_error = 1;
 		    goto check_stock_mc_ext;            
 	        } else {
-	            message (1, " " LIBDIR "mc.ext file error", "\
-Format of the " LIBDIR "mc.ext file has changed\n\
+                    char *msg;
+                    char *msg2;
+                    msg = copy_strings(" ", mc_home, "mc.ext file error");
+                    msg2 = copy_strings("Format of the ", 
+                                         mc_home, 
+"mc.ext file has changed\n\
 with version 3.0. It seems that installation\n\
 failed. Please fetch a fresh new copy from the\n\
 Midnight Commander package or in case you don't\n\
-have any, get it from ftp://ftp.nuclecu.unam.mx.");
+have any, get it from ftp://ftp.nuclecu.unam.mx.", 0);
+	            message (1, msg, msg2);
+                    free (msg);
+                    free (msg2);
 		    free (buffer);
 		    return 0;
 	        }
 	    }
 	}
-	if (home_error)
-	    message (1, " ~/" MC_EXT " file error ", "\
-Format of the ~/" MC_EXT " file has changed\n\
+	if (home_error) {
+            char *msg;
+            char *msg2;
+            msg = copy_strings(" ~/" MC_EXT " file error ");
+            msg2 = copy_strings("Format of the ~/" MC_EXT " file has changed\n\
 with version 3.0. You may want either to\n\
-copy it from " LIBDIR "mc.ext or use that\n\
+copy it from ", mc_home, "mc.ext or use that\n\
 file as an example of how to write it.\n\
-" LIBDIR "mc.ext will be used for this moment.");
+", mc_home,  "mc.ext will be used for this moment.", 0);
+	    message (1, msg, msg2);
+            free (msg);
+            free (msg2);
+        }
         free (buffer);
     }
     mc_stat (filename, &mystat);
