@@ -7,4 +7,10 @@ char *regex_command (char *filename, char *action, char *drops, int *move_dir);
  * to flush the cached mc.ext file
  */
 void flush_extension_file (void);
+
+#ifdef OS2_NT
+# define MC_EXT "mc.ext"
+#else
+# define MC_EXT ".mc.ext"
+#endif
 #endif
