@@ -127,8 +127,8 @@ int dive_into_subdirs = 1;
 /* If running as root, preserve the original uid/gid on file copy */
 int preserve_uidgid = 1;
 
-/* If off, it gets a little scrict with dangerous operations */
-int know_what_am_i_doing = 1;
+/* If on, it gets a little scrict with dangerous operations */
+int know_not_what_am_i_doing = 0;
 
 int stable_symlinks = 0;
 
@@ -194,8 +194,10 @@ enum {
 };
 
 /* Pointer to the operate dialog */
-static Dlg_head *op_dlg;
-int    showing_eta;
+static   Dlg_head *op_dlg;
+int      showing_eta;
+int      showing_bps;
+unsigned long bps = 0, bps_time = 0;
 
 static char *op_names [] = { " Copy ", " Move ", " Delete " };
 static int selected_button;
@@ -217,6 +219,7 @@ static WLabel *FileString [2];
 static WLabel *ProgressLabel [3];
 static WGauge *ProgressGauge [3];
 static WLabel *eta_label;
+static WLabel *bps_label;
 static WLabel *stalled_label;
 
 /* }}} */
@@ -258,6 +261,14 @@ check_buttons (void)
     if (c == ERR)
         return FILE_CONT;
 
+    /* discard mouse events */
+    /* FIXME: make the skip and abort button mouse aware */
+    if (c == MCKEY_MOUSE) {
+	while (get_key_code (1) != ERR)
+	   ;
+	return FILE_CONT;
+    }
+	
     switch (c){
     case 's':
 	selected_button = FILE_SKIP;
@@ -337,6 +348,7 @@ create_op_win (int op, int with_eta)
     replace_result = 0;
     recursive_result = 0;
     showing_eta = with_eta;
+    showing_bps = with_eta;
     
     op_dlg = create_dlg (0, 0, WY+4, x_size, dialog_colors,
 			 op_win_callback, "", "opwin", DLG_CENTER);
@@ -357,17 +369,18 @@ create_op_win (int op, int with_eta)
         XV_WLAY_CENTERROW);
 #endif
     tk_new_frame (op_dlg, "2.");
-    add_widgetl (op_dlg, ProgressGauge [2] = gauge_new (8, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
+    add_widgetl (op_dlg, ProgressGauge [2] = gauge_new (7, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [2] = label_new (8, FCOPY_LABEL_X, fifteen, NULL), 
+    add_widgetl (op_dlg, ProgressLabel [2] = label_new (7, FCOPY_LABEL_X, fifteen, NULL), 
         XV_WLAY_NEXTROW);
+    add_widgetl (op_dlg, bps_label = label_new (7, ETA_X, "", NULL), XV_WLAY_NEXTROW);
 
-    tk_new_frame (op_dlg, "1.");
-    add_widgetl (op_dlg, ProgressGauge [1] = gauge_new (7, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
+        tk_new_frame (op_dlg, "1.");
+    add_widgetl (op_dlg, ProgressGauge [1] = gauge_new (8, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, ProgressLabel [1] = label_new (7, FCOPY_LABEL_X, fifteen, NULL), 
+    add_widgetl (op_dlg, ProgressLabel [1] = label_new (8, FCOPY_LABEL_X, fifteen, NULL), 
         XV_WLAY_RIGHTOF);
-    add_widgetl (op_dlg, stalled_label = label_new (7, ETA_X, "", NULL), XV_WLAY_NEXTROW);
+    add_widgetl (op_dlg, stalled_label = label_new (8, ETA_X, "", NULL), XV_WLAY_NEXTROW);
 	
     tk_new_frame (op_dlg, "0.");
     add_widgetl (op_dlg, ProgressGauge [0] = gauge_new (6, FCOPY_GAUGE_X, 0, 100, 0, NULL), 
@@ -495,6 +508,24 @@ file_eta_show ()
     label_set_text (eta_label, eta_buffer);
 }
 
+static void
+file_bps_show ()
+{
+    char bps_buffer [30];
+
+    if (!showing_bps)
+	return;
+
+    if (bps > 1024){
+	    if (bps > 1024*1024){
+		    sprintf (bps_buffer, "%.2f MBS", bps / (1024*1024.0));
+	    } else
+		    sprintf (bps_buffer, "%.2f KBS", bps / 1024.0);
+    } else 
+	    sprintf (bps_buffer, "%ld BPS", bps);
+    label_set_text (bps_label, bps_buffer);
+}
+
 static int
 show_file_progress (long done, long total)
 {
@@ -503,6 +534,7 @@ show_file_progress (long done, long total)
     if (total > 0){
 	label_set_text (ProgressLabel [0], "File");
 	file_eta_show ();
+	file_bps_show ();
 	return show_bar (0, done, total);
     } else
 	return show_no_bar (0);
@@ -1082,10 +1114,19 @@ copy_file_file (char *src_path, char *dst_path, int toplevel, int ask_overwrite)
 		
 		if (n_read_total){
 		    eta_secs = ((dt / (double) n_read_total) * file_size) - dt;
-/*		    eta_secs = ((dt * file_size) / n_read_total) - dt; */
+		    bps = n_read_total / ((dt < 1) ? 1 : dt);
 		} else
 		    eta_secs = 0.0;
 	    }
+
+	    /* 4. Compute BPS rate */
+	    if (secs > 2){
+		bps_time = (tv_current.tv_sec - tv_transfer_start.tv_sec);
+		if (bps_time < 1)
+		    bps_time = 1;
+		bps = n_read_total / bps_time;
+	    }
+	    
 	    label_set_text (stalled_label, stalled_msg);
 
 	    return_status = show_file_progress (n_read_total, file_size);
@@ -1187,6 +1228,7 @@ copy_dir_dir (char *s, char *d, int toplevel, int move_over)
     DIR    *reading;
     char   *path, *mdpath, *dest_file, *dest_dir;
     int    return_status = FILE_CONT;
+    struct utimbuf utb;
 
     /* First get the mode of the source dir */
  retry_src_stat:
@@ -1268,6 +1310,19 @@ copy_dir_dir (char *s, char *d, int toplevel, int move_over)
 	return return_status;
     }
 
+#ifndef OS2_NT 
+    if (preserve_uidgid) {
+     retry_dst_chown:
+        if (mc_chown (dest_dir, cbuf.st_uid, cbuf.st_gid)){
+	    return_status = file_error
+	        (" Cannot chown target directory \"%s\" \n %s ", dest_dir);
+	    if (return_status == FILE_RETRY)
+	        goto retry_dst_chown;
+	    return return_status;
+        }
+    }
+#endif
+
  dont_mkdir:
     /* open the source dir for reading */
     reading = mc_opendir (s);
@@ -1321,6 +1376,9 @@ copy_dir_dir (char *s, char *d, int toplevel, int move_over)
     /* .ado: Directories can not have permission set in OS/2 */
 #ifndef __os2__
     mc_chmod (dest_dir, cbuf.st_mode);
+    utb.actime = cbuf.st_atime;
+    utb.modtime = cbuf.st_mtime;
+    mc_utime(dest_dir, &utb);
 #endif
     free (dest_dir);
     return return_status;
@@ -1848,6 +1906,8 @@ panel_operate (int operation, char *thedefault)
     
     /* Show confirmation dialog */
     if (operation == OP_DELETE && confirm_delete){
+        if (know_not_what_am_i_doing)
+	    query_set_sel (1);
 	i = query_dialog (op_names [operation], cmd_buf,
 			  3, 2, " Yes ", " No ");
 	if (i != 0)
@@ -2254,17 +2314,16 @@ real_query_recursive (enum OperationMode mode, char *s)
 	                       : "\n   Background process: Directory not empty \n   Delete it recursively? ";
 	text = copy_strings (" Delete: ", name_trunc (s, 30), " ", 0);
 
-        if (know_what_am_i_doing){
+        if (know_not_what_am_i_doing)
 	    query_set_sel (1);
-	    recursive_result = query_dialog (text, msg, 3 | WITH_HOTKEYS, 5,
-					     "y Yes ", "n No ", "l alL ", "e nonE ", "a Abort ");
-	} else
-	    recursive_result = query_dialog (text, msg, 3 | WITH_HOTKEYS, 5, 
-					     "y Yes ", "n No ", "l alL ", "e nonE ", "a Abort ");
+        recursive_result = query_dialog (text, msg, 3 | WITH_HOTKEYS, 5,
+				     "y Yes ", "n No ", "l alL ", "e nonE ", "a Abort ");
+	
+	
 	if (recursive_result != RECURSIVE_ABORT)
 	    do_refresh ();
 	free (text);
-	if (!know_what_am_i_doing && (recursive_result == RECURSIVE_YES
+	if (know_not_what_am_i_doing && (recursive_result == RECURSIVE_YES
 	    || recursive_result == RECURSIVE_ALWAYS)){
 	    text = copy_strings (" Please type 'yes' to confirm recursive delete of ",
 				 recursive_result == RECURSIVE_YES
