@@ -5,7 +5,7 @@ enum { OP_COPY, OP_MOVE, OP_DELETE };
 enum { FILE_CONT, FILE_RETRY, FILE_SKIP, FILE_ABORT };
 
 extern int verbose;
-extern int know_what_am_i_doing;
+extern int know_not_what_am_i_doing;
 
 int copy_file_file (char *s, char *d, int toplevel, int ask_overwrite);
 int erase_dir (char *s);
