@@ -377,7 +377,11 @@ search_content (Dlg_head *h, char *directory, char *filename)
     if (!egrep_path)
 	egrep_path = locate_egrep ();
 
+#ifndef GREP_STDIN
+    pipe = mc_doublepopen (file_fd, -1, &pid, egrep_path, egrep_path, "-n", content_pattern, NULL);
+#else /* GREP_STDIN */
     pipe = mc_doublepopen (file_fd, -1, &pid, egrep_path, egrep_path, "-n", content_pattern, "-", NULL);
+#endif /* GREP STDIN */
 	
     if (pipe == -1){
 	mc_close (file_fd);
@@ -826,15 +830,14 @@ do_find (void)
 	v = find_file (start_dir, pattern, content, &dirname, &filename);
 	free (start_dir);
 	free (pattern);
-	if (content)
-	    free (content);
-
+	
 	if (v == B_ENTER){
 	    if (dirname || filename){
 		if (dirname){
 		    do_cd (dirname, cd_exact);
 		    if (filename)
-			try_to_select (current_panel, filename + 4);
+			try_to_select (current_panel, filename + (content ? 
+			   (strchr (filename + 4, ':') - filename + 1) : 4) );
 		} else if (filename)
 		    do_cd (filename, cd_exact);
 		paint_panel (current_panel);
@@ -846,6 +849,8 @@ do_find (void)
 		free (filename);
 	    break;
 	}
+	if (content) 
+	    free (content);
 	dir_and_file_set = dirname && filename;
 	if (dirname)  free (dirname);
 	if (filename) free (filename);
