@@ -57,6 +57,10 @@
 #include "dlg.h"		/* For Dlg_head */
 #include "widget.h"		/* For Widget */
 
+#ifdef HAVE_TK
+#    include "tkmain.h"
+#endif
+
 #ifndef USE_NCURSES
 #   define ACS_MAP(x) '*'
 #else
