@@ -1,10 +1,11 @@
 /* Directory hotlist -- for the Midnight Commander
-   Copyright (C) 1994, 1995, 1996 the Free Software Foundation.
+   Copyright (C) 1994, 1995, 1996, 1997 the Free Software Foundation.
 
    Written by:
     1994 Radek Doulik
     1995 Janne Kukonlehto
     1996 Andrej Borsenkow
+    1997 Norbert Warmuth
 
    Janne did the original Hotlist code, Andrej made the groupable
    hotlist; the move hotlist and revamped the file format and made
@@ -36,9 +37,17 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/stat.h>
+#ifdef SCO_FLAVOR
+#	include <sys/timeb.h>	/* alex: for struct timeb, used in time.h */
+#endif /* SCO_FLAVOR */
 #include <time.h>
-#include <grp.h>
-#include <pwd.h>
+#ifndef __os2__
+#    include <grp.h>
+#    include <pwd.h>
+#else
+#    include <io.h>
+#    include <direct.h>
+#endif
 #include "tty.h"
 #include "mad.h"
 #include "util.h"		/* Needed for the externs */
@@ -59,6 +68,12 @@
 #include "main.h"
 #include "global.h"
 #include "hotlist.h"
+#include "key.h"
+#include "command.h"
+
+#ifdef HAVE_TK
+#    include "tkwidget.h"
+#endif
 
 #define UX		5
 #define UY		2
@@ -85,9 +100,6 @@ static Dlg_head *movelist_dlg;
 
 static WLabel *pname, *pname_group, *movelist_group;
 
-static char *hotlist_section = "Hotlist";
-static char *hotlist_group_section = "Hotlist.Groups";
-
 enum HotListType {
     HL_TYPE_GROUP,
     HL_TYPE_ENTRY,
@@ -111,6 +123,7 @@ static struct {
     				   update listbox */
     int		moving;		/* we are in moving hotlist currently */
     int		modified;	/* hotlist was modified */
+    int		type;		/* LIST_HOTLIST || LIST_VFSLIST */
 }   hotlist_state;
 
 struct _hotlist_but {
@@ -413,8 +426,10 @@ static int hotlist_callback (Dlg_head * h, int Par, int Msg)
 
     case DLG_UNHANDLED_KEY:
 	switch (Par) {
-	case KEY_ENTER:
 	case '\n':
+	    if (ctrl_pressed())
+		goto l1;
+	case KEY_ENTER:
 	case KEY_RIGHT:
 	    if (hotlist_button_callback (B_ENTER, 0)) {
 		h->ret_value = B_ENTER;
@@ -423,8 +438,31 @@ static int hotlist_callback (Dlg_head * h, int Par, int Msg)
 	    return 1;
 	    break;
 	case KEY_LEFT:
-	    return !hotlist_button_callback (B_UP_GROUP, 0);
+	    if (hotlist_state.type != LIST_VFSLIST )
+    		return !hotlist_button_callback (B_UP_GROUP, 0);
+	    else
+		return 0;
 	    break;
+l1:
+	case ALT('\n'):
+	case ALT('\r'):
+	    if (!hotlist_state.moving)
+	    {
+    		if (l_hotlist->current){
+		    if (l_hotlist->current->data) {
+			struct hotlist *hlp = (struct hotlist*) l_hotlist->current->data;
+			if (hlp->type == HL_TYPE_ENTRY) {
+			    char *tmp = copy_strings( "cd ", hlp->directory, NULL);
+			    stuff (input_w (cmdline), tmp, 0);
+			    free (tmp);
+			    h->running = 0;
+			    h->ret_value = B_CANCEL;
+			    return 1;
+		        }
+		    }
+		}
+	    }
+	    return 1; /* ignore key */
 	default:
 	    return 0;
 	}
@@ -490,7 +528,8 @@ static void init_hotlist (int list_type)
 			    "expanded_view_of_groups", 0, profile_name);
 
     hotlist_dlg = create_dlg (0, 0, LINES-2, COLS-6, dialog_colors,
-			      hotlist_callback, "[Hotlist]",
+			      hotlist_callback, 
+			      list_type == LIST_VFSLIST ? "[vfshot]" : "[Hotlist]",
 			      list_type == LIST_VFSLIST ? "vfshot" : "hotlist",
 			      DLG_CENTER|DLG_GRID);
     x_set_dialog_title (hotlist_dlg,
@@ -775,7 +814,7 @@ static int add_new_group_input (char *header, char *label, char **result)
 
 void add_new_group_cmd (void)
 {
-    char   *label, *grp_name;
+    char   *label;
     int		ret;
 
     ret = add_new_group_input (" New hotlist group ", "Name of new group", &label);
@@ -808,7 +847,6 @@ void add2hotlist_cmd (void)
 static void remove_group (struct hotlist *grp)
 {
     struct hotlist *current = grp->head;
-    char   *grp_section;
 
     while (current) {
 	struct hotlist *next = current->next;
@@ -816,9 +854,6 @@ static void remove_group (struct hotlist *grp)
 	if (current->type == HL_TYPE_GROUP)
 	    remove_group (current);
 
-	/* FIXME: Commented out because freeing seems to cause corruption
-	   after some add and remove operations */
-	/* bor: I will try it :) */
 	if (current->label)
 	    free (current->label);
 	if (current->directory)
@@ -832,8 +867,6 @@ static void remove_group (struct hotlist *grp)
 	
 static void remove_from_hotlist (struct hotlist *entry)
 {
-    struct hotlist *current = current_group->head;
-
     if (entry->type == HL_TYPE_GROUP) {
 	if (entry->head) {
 	    char *header;
@@ -857,9 +890,6 @@ static void remove_from_hotlist (struct hotlist *entry)
 
     unlink_entry (entry);
 
-    /* FIXME: Commented out because freeing seems to cause corruption
-       after some add and remove operations */
-    /* bor: I will try it :) */
     if (entry->label)
           free (entry->label);
     if (entry->directory)
@@ -874,7 +904,9 @@ char *hotlist_cmd (int vfs_or_hotlist)
 {
     char *target = NULL;
 
-again:
+    hotlist_state.type = vfs_or_hotlist;
+    load_hotlist ();
+
     init_hotlist (vfs_or_hotlist);
 
     /* display file info */
@@ -883,6 +915,7 @@ again:
     hotlist_state.running = 1;
     run_dlg (hotlist_dlg);
     hotlist_state.running = 0;
+    save_hotlist ();
 
     switch (hotlist_dlg->ret_value) {
     case B_CANCEL:
@@ -920,7 +953,6 @@ void load_group (struct hotlist *grp)
     }
     free (group_section);
 
-no_groups:
     profile_keys = profile_init_iterator (grp->directory, profile_name);
 
     while (profile_keys){
@@ -948,6 +980,7 @@ static int  tkn_length;
 
 static char *hotlist_file_name;
 static FILE *hotlist_file;
+static time_t hotlist_file_mtime;
 
 static int hot_skip_blanks ()
 {
@@ -1172,12 +1205,18 @@ void load_hotlist (void)
     char	*grp_section;
     int		has_old_list = 0;
     int		remove_old_list = 0;
+    struct stat stat_buf;
 
-    if (hotlist_state.loaded)
-	return;
+    if (hotlist_state.loaded) {
+	stat (hotlist_file_name, &stat_buf);
+	if (hotlist_file_mtime < stat_buf.st_mtime) 
+	    done_hotlist ();
+	else
+	    return;
+    }
 
     if (!hotlist_file_name)
-	hotlist_file_name = copy_strings (home_dir, PATH_SEP_STR ".mc.hot", 0);
+	hotlist_file_name = concat_dir_and_file (home_dir, HOTLIST_FILENAME);
     
     hotlist	       = new_hotlist ();
     hotlist->type      = HL_TYPE_GROUP;
@@ -1197,8 +1236,8 @@ void load_hotlist (void)
 	int	result;
 
 	message (0, " Hotlist Load ",
-		 "Hotlist is now kept in file ~/.mc.hot",
-		 "MC will load hotlist from ~/mc.ini and then",
+		 "Hotlist is now kept in file ~/" HOTLIST_FILENAME,
+		 "MC will load hotlist from ~/" PROFILE_NAME " and then",
 		 "delete [Hotlist] section there"
 	);
 	load_group (hotlist);
@@ -1213,7 +1252,7 @@ void load_hotlist (void)
 	    remove_old_list = 1;
 	} else {
 	    message (D_ERROR, " Hotlist Load ",
-		     "MC was unable to write ~/.mc.hot file"
+		     "MC was unable to write ~/" HOTLIST_FILENAME " file"
 		     "your old hotlist entries were not deleted");
 	}
     } else {
@@ -1224,15 +1263,15 @@ void load_hotlist (void)
 	    int		result;
 
 	    result = query_dialog (" Hotlist Load ", 
-	   "You have ~/.mc.hot file and [Hotlist] section in ~/.mc.ini\n" /**/
-	   "Your ~/.mc.hot most probably was created\n" /**/
+	   "You have ~/" HOTLIST_FILENAME " file and [Hotlist] section in ~/" PROFILE_NAME "\n" /**/
+	   "Your ~/" HOTLIST_FILENAME " most probably was created\n" /**/
 	   "by an earlier development version of MC\n" /**/
-	   "and is more actual than ~/.mc.ini entries\n\n" /**/
+	   "and is more actual than ~/" PROFILE_NAME " entries\n\n" /**/
 	   "You can choose between\n\n" /**/
-	   "  Remove - remove old hotlist entries from ~/.mc.ini\n" /**/
+	   "  Remove - remove old hotlist entries from ~/" PROFILE_NAME "\n" /**/
 	   "  Keep   - keep your old entries; you will be asked\n" /**/
 	   "           the same question next time\n" /**/
-	   "  Merge  - add old entries to hotlist as group \"Entries from ~/.mc.ini\"\n\n",
+	   "  Merge  - add old entries to hotlist as group \"Entries from ~/" PROFILE_NAME "\"\n\n",
 				   D_ERROR, 3, " Remove ", " Keep ", " Merge "
 		      );
 	    if (result == 0)
@@ -1246,7 +1285,7 @@ void load_hotlist (void)
 
 		old            = new_hotlist ();
 		old->type      = HL_TYPE_GROUP;
-		old->label     = strdup (" Entries from ~/.mc.ini ");
+		old->label     = strdup (" Entries from ~/" PROFILE_NAME);
 		old->up	       = hotlist;
 		old->head      = hotlist->head;
 		old->next      = grp;
@@ -1254,7 +1293,7 @@ void load_hotlist (void)
 		hotlist_state.modified = 1;
 		if (!save_hotlist ())
 		    message (D_ERROR, " Hotlist Load ",
-			 "MC was unable to write ~/.mc.hot file"
+			 "MC was unable to write ~/" HOTLIST_FILENAME " file"
 			 "your old hotlist entries were not deleted");
 		else
 		    remove_old_list = 1;
@@ -1268,6 +1307,8 @@ void load_hotlist (void)
 	sync_profiles ();
     }
 
+    stat (hotlist_file_name, &stat_buf);
+    hotlist_file_mtime = stat_buf.st_mtime;
     current_group = hotlist;
 }
 
@@ -1306,7 +1347,6 @@ static int  list_level = 0;
 void hot_save_group (struct hotlist *grp)
 {
     struct hotlist *current = grp->head;
-    char           *group_section;
     int		    i;
     char	   *s;
 
@@ -1365,15 +1405,22 @@ do { \
 int save_hotlist (void)
 {
     int		saved = 0;
-
+    struct      stat stat_buf;
+    
     if (!hotlist_state.readonly && hotlist_state.modified && hotlist_file_name) {
 	char	*fbak = copy_strings (hotlist_file_name, ".bak", 0);
 
 	rename (hotlist_file_name, fbak);
 	if ((hotlist_file = fopen (hotlist_file_name, "w")) != 0) {
+	    if (stat (fbak, &stat_buf) == 0)
+		chmod (hotlist_file_name, stat_buf.st_mode);
+	    else
+		chmod (hotlist_file_name, S_IRUSR | S_IWUSR);
 	    hot_save_group (hotlist);
 	    fflush (hotlist_file);
 	    fclose (hotlist_file);
+	    stat (hotlist_file_name, &stat_buf);
+	    hotlist_file_mtime = stat_buf.st_mtime;
 	    saved = 1;
 	    hotlist_state.modified = 0;
 	} else
@@ -1387,10 +1434,18 @@ int save_hotlist (void)
 void done_hotlist (void)
 {
     remove_group (hotlist);
+    hotlist_state.loaded = 0;
     if (hotlist->label)
         free (hotlist->label);
     if (hotlist->directory)
         free (hotlist->directory);
     free (hotlist);
+    if (hotlist_file_name)
+        free (hotlist_file_name);
+    hotlist_file_name = 0;
     hotlist = current_group = 0;
+    l_hotlist = 0;
+    current_group = 0;
 }
+
+
