@@ -17,6 +17,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <config.h>
+#include <stdio.h>
 #include "tty.h"
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -72,10 +73,10 @@ void info_show_info (WInfo *info)
 	return;
 
     info_box (info->widget.parent, info);
-    set_attr (0, 1);
+    attrset (MARKED_COLOR);
     widget_move (&info->widget, 1, 3);
     printw ("Midnight Commander %s", VERSION);
-    set_attr (0, 0);
+    attrset (NORMAL_COLOR);
     widget_move (&info->widget, 2, 1);
     /* .ado: info->widget.x has wrong value (==0) on NT and OS/2, why? */
 #if !defined(OS2_NT)
@@ -181,7 +182,7 @@ void info_show_info (WInfo *info)
 	
     case 5:
 	widget_move (&info->widget, 5, 3);
-	printw ("Mode:      %s (%o)",
+	printw ("Mode:      %s (%04o)",
 		string_perm (buf.st_mode), buf.st_mode & 07777);
 	
     case 4:
