@@ -47,9 +47,15 @@
 #include "main.h"
 #include "file.h"
 #include "../vfs/vfs.h"
+
 #ifdef __linux__
-#  include <linux/termios.h> /* This is needed for TIOCLINUX */
+#    if defined(__GLIBC__) && (__GLIBC__ < 2)
+#        include <linux/termios.h>	/* This is needed for TIOCLINUX */
+#    else
+#        include <termios.h>
+#    endif
 #endif
+
 #ifdef HAVE_TK
 #   include "tkmain.h"
 #endif
@@ -217,6 +223,10 @@ key_define_t mc_bindings [] = {
 
 /* Broken terminfo and termcap databases on xterminals */
 key_define_t xterm_key_defines [] = {
+    { KEY_F(1),   ESC_STR "OP",   MCKEY_NOACTION },
+    { KEY_F(2),   ESC_STR "OQ",   MCKEY_NOACTION },
+    { KEY_F(3),   ESC_STR "OR",   MCKEY_NOACTION },
+    { KEY_F(4),   ESC_STR "OS",   MCKEY_NOACTION },
     { KEY_F(1),   ESC_STR "[11~", MCKEY_NOACTION },
     { KEY_F(2),   ESC_STR "[12~", MCKEY_NOACTION },
     { KEY_F(3),   ESC_STR "[13~", MCKEY_NOACTION },
@@ -721,7 +731,7 @@ int get_event (Gpm_Event *event, int redo_event)
 		     */
 
 		    timeout.tv_sec = seconds;
-		    timeout.tv_sec = 0;
+		    timeout.tv_usec = 0;
 		    time_addr = &timeout;
 		} else
 		    time_addr = NULL;
