@@ -2,11 +2,16 @@
 #define __KEY_H
 
 void init_key (void);
+void done_key (void);
 int mi_getch (void);
 int get_event (Gpm_Event *event, int redo_event);
 int is_idle (void);
 int ctrl_pressed ();
 
+/* Possible return values from get_event: */
+#define EV_MOUSE   -2
+#define EV_NONE    -1
+
 /* Used to get the modifier information          */
 /* Currently, it just works on the Linux console */
 #define SHIFT_PRESSED 1
@@ -26,6 +31,7 @@ typedef int (*select_fn)(int fd, void *info);
 
 /* Channel manipulation */
 void add_select_channel    (int fd, select_fn callback, void *info);
+void delete_select_channel (int fd);
 void remove_select_channel (int fd);
 
 /* Activate/deactivate the channel checking */
@@ -42,6 +48,10 @@ int is_quit_char (int c);
 /* To define sequences and return codes */
 #define MCKEY_NOACTION	0
 #define MCKEY_ESCAPE	1
+
+/* Return code for the mouse sequence */
+#define MCKEY_MOUSE     -2
+
 void do_define_key (int code, char *strcap);
 void define_sequence (int code, char *seq, int action);
 
