@@ -415,6 +415,12 @@ void layout_change (void)
 {
     setup_panels ();
     layout_do_change = 0;
+#ifndef HAVE_X
+    /* re-init the menu, because perhaps there was a change in the way 
+	how the panel are split (horizontal/vertical). */
+    done_menu();
+    init_menu();
+#endif
 }
 
 void layout_cmd (void)
@@ -615,29 +621,44 @@ void edit_adjust_size (Dlg_head * h);
 void change_screen_size (void)
 {
 #ifndef HAVE_X
-#ifdef HAVE_SLANG
+#if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
 #if defined TIOCGWINSZ && !defined SCO_FLAVOR
     struct winsize winsz;
     extern Dlg_head *view_dlg;
     extern Dlg_head *edit_dlg;
     
+#ifndef NCURSES_VERSION
     mc_noraw_mode ();
     endwin ();
+#endif
     winsz.ws_col = winsz.ws_row = 0;
 
     /* Ioctl on the STDIN_FILENO */
     ioctl (0, TIOCGWINSZ, &winsz);
     if (winsz.ws_col && winsz.ws_row){
+#if defined(NCURSES_VERSION) && defined(HAVE_RESIZETERM)
+	resizeterm(winsz.ws_row, winsz.ws_col);
+	clearok(stdscr,TRUE);	/* FIXME: sigwinch's should use a semaphore! */
+#else
 	COLS = winsz.ws_col;
 	LINES = winsz.ws_row;
+#endif
 #ifdef HAVE_SUBSHELL_SUPPORT
 	    resize_subshell ();
 #endif
     }
 
     check_split ();
+#ifndef NCURSES_VERSION
+    /* XSI Curses spec states that portable applications shall not invoke
+     * initscr() more than once.  This kludge could be done within the scope
+     * of the specification by using endwin followed by a refresh (in fact,
+     * more than one curses implementation does this); it is guaranteed to work
+     * only with slang.
+     */
     initscr ();
     init_curses ();
+#endif
     setup_panels ();
     if (current_dlg == view_dlg)
 	view_adjust_size (view_dlg);
@@ -650,7 +671,7 @@ void change_screen_size (void)
     do_refresh ();
     touchwin (stdscr);
 #endif /* TIOCGWINSZ && !SCO_FLAVOR */
-#endif /* HAVE_SLANG */
+#endif /* defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4 */
 #endif /* HAVE_X */
     winch_flag = 0;
 }
