@@ -19,6 +19,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <config.h>
+#include <locale.h>
 
 #ifdef _OS_NT
 #    include <windows.h>
@@ -230,6 +231,14 @@ int navigate_with_arrows = 0;
 /* If it is set, the commander will iconify itself when executing a program */
 int iconify_on_exec = 1;
 
+/* If true use +, -, | for line drawing */
+int force_ugly_line_drawing = 0;
+
+/* If true program softkeys (HP terminals only) on startup and after every 
+   command ran in the subshell to the description found in the termcap/terminfo 
+   database */
+int reset_hp_softkeys = 0;
+
 /* The prompt */
 char *prompt = 0;
 
@@ -338,6 +347,9 @@ int stdout_fd = 0;
 /* The user's shell */
 char *shell;
 
+/* mc_home: The home of MC */
+char *mc_home;
+
 /* Ugly hack in order to distinguish between left and right panel in menubar */
 int is_right;
 #define MENU_PANEL_IDX  (is_right ? 1 : 0)
@@ -372,6 +384,10 @@ static const int status_using_old_tools =
     0;
 #endif
 
+#if defined(OS2_NT)
+char *get_mc_lib_dir ();
+#endif
+
 int panel_event    (Gpm_Event *event, WPanel *panel);
 int menu_bar_event (Gpm_Event *event, void *);
 
@@ -967,7 +983,11 @@ int do_panel_cd (WPanel *panel, char *new_dir, enum cd_enum cd_type)
 {
     char *directory, *olddir;
     char temp [MC_MAXPATHLEN];
-
+#ifdef USE_VFS    
+    vfs *oldvfs;
+    vfsid oldvfsid;
+    struct vfs_stamping *parent;
+#endif    
     olddir = strdup (panel->cwd);
 
     /* Convert *new_path to a suitable pathname, handle ~user */
@@ -992,11 +1012,18 @@ int do_panel_cd (WPanel *panel, char *new_dir, enum cd_enum cd_type)
     /* Success: save previous directory, shutdown status of previous dir */
     directory_history_add (olddir);
     strcpy (panel->lwd, olddir);
-    free (olddir);
     free_completions (input_w (cmdline));
     
     mc_get_current_wd (panel->cwd, sizeof (panel->cwd) - 2);
 
+#ifdef USE_VFS
+    oldvfs = vfs_type (olddir);
+    oldvfsid = vfs_ncs_getid (oldvfs, olddir, &parent);
+    vfs_add_noncurrent_stamps (oldvfs, oldvfsid, parent);
+    vfs_rm_parents (parent);
+#endif
+    free (olddir);
+
     subshell_chdir (panel->cwd);
 
     /* Reload current panel */
@@ -1036,8 +1063,8 @@ int load_prompt (int fd, void *unused)
 	    prompt [COLS - 8 ] = 0;
 	    prompt_len = COLS - 8;
 	}
-	winput_set_origin ((WInput *)cmdline, prompt_len, COLS-prompt_len);
 	label_set_text (the_prompt, prompt);
+	winput_set_origin ((WInput *)cmdline, prompt_len, COLS-prompt_len);
 
 	/* since the prompt has changed, and we are called from one of the 
 	 * get_event channels, the prompt updating does not take place
@@ -1119,7 +1146,7 @@ static void switch_to_listing (int panel_index)
 static void listing_cmd (void)
 {
     int   view_type, use_msformat;
-    char  *user, *status;
+    char  *err, *user, *status;
     WPanel *p;
     int   display_type;
 
@@ -1146,10 +1173,17 @@ static void listing_cmd (void)
     p->user_mini_status = use_msformat; 
     p->list_type = view_type;
     
-    if (set_panel_format (p, panel_format (p))){
+    if (err=set_panel_format (p, panel_format (p))){
+        free (err);
 	free (p->user_format);
 	p->user_format  = strdup (DEFAULT_USER_FORMAT);
-	set_panel_format (p, DEFAULT_USER_FORMAT);
+	err = set_panel_format (p, DEFAULT_USER_FORMAT);
+	if (err && use_msformat){
+	    free (err);
+	    free (p->mini_status_format);
+	    p->mini_status_format  = strdup (DEFAULT_USER_FORMAT);
+	    set_panel_format (p, DEFAULT_USER_FORMAT);
+	}
     }
     do_refresh ();
 }
@@ -1249,7 +1283,7 @@ static void sort_cmd (void)
     do_re_sort (p);
 }
 
-#ifdef OS2_NT
+#if 0
 /* .ado: 28.05.97:
    jump to the directory which is .. relative to dir_name.
    e.g.: dir_name = /a/b/c      ==> jump to /a/b
@@ -1268,21 +1302,21 @@ jump2dotdot(char *dir_name)
 	}
     }
     strcpy (tmp_name, dir_name);
-    if (i == 0){                                             /* On the top PATH_SEP */
+    if (i == 0)                                             /* On the top PATH_SEP */
 	tmp_name [1] = 0;
-    } else {
+    else {
+#ifdef OS2_NT	
         /* Important for NT and OS/2: */
 	if (dir_name [i] == ':') {       /* on top <DRV_LETTER>: */
 	    tmp_name [i+2] = 0;
-	} else {
+	} else 
+#endif    
 	    tmp_name [i+1] = 0;
-	}
+
     }
     do_cd (tmp_name, cd_exact);
     return;
 }
-#else
-#define jump2dotdot(x)
 #endif
 
 static void
@@ -1292,8 +1326,12 @@ tree_box (void)
 
     sel_dir = tree (selection (cpanel)->fname);
     if (sel_dir){
+#if 0	
        	jump2dotdot(sel_dir);
 	try_to_select (cpanel, sel_dir);
+#else
+	do_cd(sel_dir, cd_exact);
+#endif
 	free (sel_dir);
     }
 }
@@ -1438,25 +1476,32 @@ Menu MenuBar [5];
 static Menu MenuBarEmpty [5];
 #endif
 
-static void init_menu (void)
+void init_menu (void)
 {
     int i;
-    
+
+#ifdef HAVE_X    
     MenuBar [0] = create_menu (" Left ", PanelMenu, menu_entries (PanelMenu));
+#else
+    MenuBar [0] = create_menu ( horizontal_split ? " Above " : " Left ", 
+                                PanelMenu, menu_entries (PanelMenu));
+#endif
     MenuBar [1] = create_menu (" File ", FileMenu, menu_entries (FileMenu));
     MenuBar [2] = create_menu (" Command ", CmdMenu, menu_entries (CmdMenu));
     MenuBar [3] = create_menu (" Options ", OptMenu, menu_entries (OptMenu));
 #ifndef HAVE_XVIEW
+#ifdef HAVE_X
     MenuBar [4] = create_menu (" Right ", RightMenu, menu_entries (PanelMenu));
-
-#ifndef HAVE_X
+#else
+    MenuBar [4] = create_menu (horizontal_split ? " Below " : " Right ", 
+			       RightMenu, menu_entries (PanelMenu));
     for (i = 0; i < 5; i++)
 	MenuBarEmpty [i] = create_menu (MenuBar [i]->name, 0, 0);
-#endif /* ! HAVE_X */
+#endif /* HAVE_X */
 #endif /* ! HAVE_XVIEW */
 }
 
-static void done_menu (void)
+void done_menu (void)
 {
     int i;
 
@@ -1880,7 +1925,7 @@ static void setup_sigwinch ()
 #ifndef OS2_NT
     struct sigaction act, oact;
     
-#   ifdef HAVE_SLANG
+#   if defined(HAVE_SLANG) || NCURSES_VERSION_MAJOR >= 4
 #       ifdef SIGWINCH
             act.sa_handler = flag_winch;
             sigemptyset (&act.sa_mask);
@@ -1942,15 +1987,16 @@ static void setup_mc (void)
 
 static void setup_dummy_mc (const char *file)
 {
-    setup_pre ();
-    setup_post ();
+    char d[MC_MAXPATHLEN];
+
+    mc_get_current_wd (d, MC_MAXPATHLEN);
+    setup_mc ();
+    mc_chdir (d);
 
     /* Create a fake current_panel, this is needed because the
      * expand_format routine will use current panel.
      */
-    cpanel = xmalloc (sizeof (WPanel), "fake_panel");
-    strcpy (cpanel->cwd, ".");
-    cpanel->dir.list = (file_entry *) xmalloc (sizeof (file_entry), "fake_p");
+    strcpy (cpanel->cwd, d);
     cpanel->selected = 0;
     cpanel->count = 1;
     cpanel->dir.list[0].fname = (char *) file;
@@ -2262,7 +2308,7 @@ static void do_nc (void)
 #endif
 	free (path);
 	midnight_shutdown = 1;
-	done_screen ();
+        done_mc ();
 	return;
     }
     setup_mc ();
@@ -2280,8 +2326,9 @@ static void do_nc (void)
 	    last_wd_string = strdup (cpanel->cwd);
     }
     done_mc ();
-
+    
     destroy_dlg (midnight_dlg);
+    cpanel = 0;
     done_mc_profile ();
 }
 #include "features.inc"
@@ -2296,7 +2343,7 @@ static void version (int verbose)
     fprintf (stderr,
 	    "with mouse support on xterm%s%s%s.\n",
 	     status_mouse_support ? " and the Linux console" : "",
-	     status_using_ncurses ? "\nusing the ncurses library" : "\n",
+	     status_using_ncurses ? "\nusing the ncurses library" : "",
 	     status_using_old_tools ? " and compiled with old tools" : "");
 #endif /* HAVE_X */
 
@@ -2320,6 +2367,8 @@ void OS_Setup ()
     
     /* Default opening mode for files is binary, not text (CR/LF translation) */
     _fmode = _O_BINARY;
+
+    mc_home = get_mc_lib_dir ();
 }
 
 static void sigchld_handler_no_subshell (int sig)
@@ -2330,6 +2379,13 @@ void init_sigchld (void)
 {
 }
 
+void
+init_sigfatals (void)
+{
+	/* Nothing to be done on the OS/2, Windows/NT */
+}
+
+
 #elif defined (__os2__)
 /* OS/2 Version */
 #   define CONTROL_FILE "\\mc.%d.control"
@@ -2345,6 +2401,8 @@ OS_Setup (void)
     shell = getenv ("COMSPEC");
     if (!shell || !*shell)
 	shell = get_default_shell ();
+
+    mc_home = get_mc_lib_dir ();
 }
 
 static void
@@ -2392,11 +2450,15 @@ OS_Setup ()
 
     sprintf (control_file, CONTROL_FILE, getpid ());
     my_putenv ("MC_CONTROL_FILE", control_file);
+    
+    /* This is the directory, where MC was installed, on Unix this always is LIBDIR */
+    mc_home = strdup(LIBDIR);
 }
 
 static void
 sigchld_handler_no_subshell (int sig)
 {
+#ifndef HAVE_X
     int pid, status;
 
     if (!console_flag)
@@ -2424,9 +2486,10 @@ sigchld_handler_no_subshell (int sig)
 	}
 	/* }}} */
     }
-#endif /* SCO_FLAVOR */
+#endif /* ! SCO_FLAVOR */
 
     /* If we get here, some other child exited; ignore it */
+#endif /* ! HAVE_X  */
 }
 
 #if 0
@@ -2525,15 +2588,20 @@ static void handle_args (int argc, char *argv [])
 #else
     extern
 #endif
-    int SLtt_try_termcap;
+    int SLtt_Try_Termcap;
 #endif
 
 #ifdef USE_TERMCAP
-    SLtt_try_termcap = 1;
+    SLtt_Try_Termcap = 1;
 #endif
 
-    while ((c = getopt (argc, argv, "VbsdPcfC:hmxtuUe:v:l:NXB" ARCH_FLAGS)) != -1) {
+    while ((c = getopt (argc, argv, "aVbsdPcfC:hkmxtuUe:v:l:XB" ARCH_FLAGS)) != -1) {
 	switch (c) {
+#ifdef HAVE_SLANG
+	case 'a':
+    	    force_ugly_line_drawing = 1;
+	    break;
+#endif
 	case 'V':
 	    version (1);
 	    finish_program = 1;
@@ -2556,13 +2624,16 @@ static void handle_args (int argc, char *argv [])
 #endif
 	    break;
 
+#ifndef OS2_NT
 	case 'f':
-	    fprintf (stderr, "Library directory for the Midnight Commander: " LIBDIR "\n");
+	    fprintf (stderr, "Library directory for the Midnight Commander: %s\n", LIBDIR);
 	    finish_program = 1;
 	    break;
-
+#endif
+	    
 	case 'm':
-	    use_8th_bit_as_meta = 0;
+	    fprintf (stderr, "Option -m is obsolete. Please look at Display Bits... in the Option's menu\n");
+	    finish_program = 1;
 	    break;
 
 	case 'e':
@@ -2607,14 +2678,8 @@ static void handle_args (int argc, char *argv [])
 
 #ifdef HAVE_SLANG
 	case 't':
-	    SLtt_try_termcap = 1;
+	    SLtt_Try_Termcap = 1;
 	    break;
-
-#ifndef OS2_NT
-	case 'N':
-	    SLtt_want_unicode ();
-	    break;
-#endif				/* _OS_NT */
 #endif				/* HAVE_SLANG */
 
 	case 'u':
@@ -2634,6 +2699,9 @@ static void handle_args (int argc, char *argv [])
 	    use_subshell = 1;
 #endif
 	    break;
+	case 'k':
+	    reset_hp_softkeys = 1;
+	    break;
 
 	case '?':
 	case 'h':
@@ -2641,7 +2709,10 @@ static void handle_args (int argc, char *argv [])
 	    version (0);
 	    fprintf (stderr,
 		"The Midnight Commander " VERSION "\nUsage is:\n\n"
-		"mc [-bCcdfhlmNPstuUVvx?" ARCH_FLAGS "] [other_panel_dir] [this-dir]\n"
+		"mc [-abCcdfhklmPstuUVvx?" ARCH_FLAGS "] [other_panel_dir] [this-dir]\n"
+#ifdef HAVE_SLANG
+		     "-a Force use of +, -, | for line drawing\n"
+#endif
 		     "-b Force black and white\n"
 		     "-C <keyword>=<fore>,<back> Color usage keywords:\n"
 	     "   normal, selected, marked, markselect, errors, reverse\n"
@@ -2656,12 +2727,11 @@ static void handle_args (int argc, char *argv [])
 #endif
 		     "-d Disable mouse support\n"
 		     "-f Print configured paths\n"
-		  "-m Don't use the highest character bit as meta flag\n"
+		     "-k Reset softkeys (HP terminals only) to their terminfo/termcap default\n"
 		     "-P At exit, print last working directory\n"
 		   "-s Disables verbose operation (for slow terminals)\n"
 #ifdef HAVE_SLANG
 		     "-t Activate support for the TERMCAP variable\n"
-		     "-N Use Unicode if available\n"
 #endif
 #ifdef USE_NETCODE
 		     "-l file Log ftpfs commands to the file\n"
@@ -2682,15 +2752,20 @@ static void handle_args (int argc, char *argv [])
 	    printf (".");
 	exit (1);
     }
-    OS_Setup ();
 
-    /* sets the current dir and the other dir */
+    /* Check for special invocation names mcedit and mcview,
+     * if none apply then set the current directory and the other
+     * directory from the command line arguments
+     */
     if (strstr (argv[0], "mcedit") == argv[0] + strlen(argv[0]) - 6) {
 	edit_one_file = "";
 	for (; optind < argc; optind++) {
 	    edit_one_file = strdup (argv[optind]);
 	}
+    } else if (strstr (argv [0], "mcview") == argv [0] + strlen (argv [0]) - 6) {
+	view_one_file = argv [optind];
     } else {
+       	/* sets the current dir and the other dir */
 	for (; optind < argc; optind++) {
 	    if (this_dir) {
 		other_dir = strdup (argv[optind]);
@@ -2717,6 +2792,13 @@ int main (int argc, char *argv [])
 #endif
     vfs_init ();
 
+    /* Initialize list of all user group for timur_clr_mode */
+    init_groups ();
+
+    setlocale (LC_CTYPE, "");
+
+    OS_Setup ();
+
 #ifdef HAVE_X
     /* NOTE: This call has to be before any our argument handling :) */
     if (xtoolkit_init (&argc, argv) == -1)
@@ -2760,7 +2842,7 @@ int main (int argc, char *argv [])
      * that happends to have cons_saver_pid (a random startup value).
      * and PID 1 is init, unlikely we could be the parent of it.
      */
-    cons_saver_pid = 1;
+/*    cons_saver_pid = 1; */
 #   else
     /* Must be done before installing the SIGCHLD handler [[FIXME]] */
     handle_console (CONSOLE_INIT);
@@ -2856,6 +2938,9 @@ int main (int argc, char *argv [])
     /* Virtual File System shutdown */
     vfs_shut ();
 
+    /* Delete list of all user groups*/
+    delete_groups ();
+
     flush_extension_file (); /* does only free memory */
 
 #   ifndef HAVE_X
