@@ -35,6 +35,14 @@ void free (void *ptr);
 #include <strings.h>
 #endif
 
+#ifdef __NetBSD__
+#   define xBSD
+#endif
+
+#ifdef __OpenBSD__
+#   define xBSD
+#endif
+
 #if defined(SCO_FLAVOR) && defined(__GNUC__)
 extern char* strdup(const char*);
 #endif
@@ -122,7 +130,7 @@ static int xatoi (char *cp)
 }
 #endif				/* MOUNTED_GETMNTENT1.  */
 
-#if defined (MOUNTED_GETMNTINFO) && !defined (__NetBSD__)
+#if defined (MOUNTED_GETMNTINFO) && !defined (xBSD)
 static char *fstype_to_string (short t)
 {
     switch (t) {
@@ -239,7 +247,7 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
 	    me = (struct mount_entry *) malloc (sizeof (struct mount_entry));
 	    me->me_devname = strdup (fsp->f_mntfromname);
 	    me->me_mountdir = strdup (fsp->f_mntonname);
-#ifdef __NetBSD__
+#ifdef xBSD
 	    me->me_type = strdup (fsp->f_fstypename);
 #else
 	    me->me_type = fstype_to_string (fsp->f_type);
@@ -458,3 +466,84 @@ struct mount_entry *read_filesystem_list (int need_fs_type, int all_fs)
     return mount_list;
 }
 #endif /* NO_INFOMOUNT */
+
+#if defined(NO_INFOMOUNT) && defined(__QNX__)
+/*
+** QNX has no [gs]etmnt*(), [gs]etfs*(), or /etc/mnttab, but can do
+** this via the following code.
+** Note that, as this is based on CWD, it only fills one mount_entry
+** structure. See my_statfs() in utilunix.c for the "other side" of
+** this hack.
+*/
+
+#include <fcntl.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#include <sys/disk.h>
+#include <sys/fsys.h>
+#include <sys/statfs.h>
+
+#include "mountlist.h"
+
+struct mount_entry *read_filesystem_list(int need_fs_type, int all_fs)
+{
+	struct _disk_entry	de;
+	struct statfs		fs;
+	int					i, fd;
+	char				*tp, dev[_POSIX_NAME_MAX], dir[_POSIX_PATH_MAX];
+
+	static struct mount_entry	*me = NULL;
+
+	if (me)
+	{
+		if (me->me_devname) free(me->me_devname);
+		if (me->me_mountdir) free(me->me_mountdir);
+		if (me->me_type) free(me->me_type);
+	}
+	else
+		me = (struct mount_entry *)malloc(sizeof(struct mount_entry));
+
+	if (!getcwd(dir, _POSIX_PATH_MAX)) return (NULL);
+
+	if ((fd = open(dir, O_RDONLY)) == -1) return (NULL);
+
+	i = disk_get_entry(fd, &de);
+
+	close(fd);
+
+	if (i == -1) return (NULL);
+
+	switch (de.disk_type)
+	{
+		case _UNMOUNTED:	tp = "unmounted";	break;
+		case _FLOPPY:		tp = "Floppy";		break;
+		case _HARD:			tp = "Hard";		break;
+		case _RAMDISK:		tp = "Ram";			break;
+		case _REMOVABLE:	tp = "Removable";	break;
+		case _TAPE:			tp = "Tape";		break;
+		case _CDROM:		tp = "CDROM";		break;
+		default:			tp = "unknown";
+	}
+
+	if (fsys_get_mount_dev(dir, &dev) == -1) return (NULL);
+
+	if (fsys_get_mount_pt(dev, &dir) == -1) return (NULL);
+
+	me->me_devname = strdup(dev);
+	me->me_mountdir = strdup(dir);
+	me->me_type = strdup(tp);
+	me->me_dev = de.disk_type;
+
+#ifdef DEBUG
+	fprintf(stderr, "disk_get_entry():\n\tdisk_type=%d (%s)\n\tdriver_name='%-*.*s'\n\tdisk_drv=%d\n",
+		de.disk_type, tp, _DRIVER_NAME_LEN, _DRIVER_NAME_LEN, de.driver_name, de.disk_drv);
+	fprintf(stderr, "fsys_get_mount_dev():\n\tdevice='%s'\n", dev);
+	fprintf(stderr, "fsys_get_mount_pt():\n\tmount point='%s'\n", dir);
+#endif
+
+	return (me);
+}
+#endif /* __QNX__ */
+
