@@ -1,7 +1,11 @@
 #ifndef __MYSLANG_H
 #define __MYSLANG_H
 
-#include "../slang/slang.h"
+#ifdef SLANG_H_INSIDE_SLANG_DIR
+#    include <slang/slang.h>
+#else
+#    include "slang.h"
+#endif
 
 enum {
     KEY_BACKSPACE = 400,
@@ -24,6 +28,24 @@ enum {
 #    define acs()   ;
 #    define noacs() ;
 #    define baudrate() 19200
+#    define SLSMG_CKBRD_CHAR   0xB0
+#    define SLSMG_DHLINE_CHAR  0xCD
+#    define SLSMG_DVLINE_CHAR  0xBA
+#    define SLSMG_DULCORN_CHAR 0xC9
+#    define SLSMG_DURCORN_CHAR 0xBB
+#    define SLSMG_DLLCORN_CHAR 0xC8
+#    define SLSMG_DLRCORN_CHAR 0xBC
+#    define SLSMG_DHLINE_CHAR  0xCD
+#    define SLSMG_DVLINE_CHAR  0xBA
+#    define SLSMG_DULCORN_CHAR 0xC9
+#    define SLSMG_DURCORN_CHAR 0xBB
+#    define SLSMG_DLLCORN_CHAR 0xC8
+#    define SLSMG_DLRCORN_CHAR 0xBC
+#    define SLSMG_DRTEE_CHAR   0xB9
+#    define SLSMG_DLTEE_CHAR   0xCC
+#    define SLSMG_DUTEE_CHAR   0xCB
+#    define SLSMG_DDTEE_CHAR   0xCA
+#    define SLSMG_DPLUS_CHAR   0xCE
 #else
 #    define ACS_LTEE 't'
 #    define acs()   SLsmg_set_char_set(1)
@@ -84,6 +106,38 @@ int has_colors (void);
 /* Internal function prototypes */
 void load_terminfo_keys ();
 
+/* copied from slcurses.h (MC version 4.0.7) */
+#define move SLsmg_gotorc
+#define clreol SLsmg_erase_eol
+#define printw SLsmg_printf
+#define mvprintw(x, y, z) SLsmg_gotorc(x, y); SLsmg_printf(z)
+#define COLS SLtt_Screen_Cols
+#define LINES SLtt_Screen_Rows
+#define clrtobot SLsmg_erase_eos
+#define clrtoeol SLsmg_erase_eol
+#define standout SLsmg_reverse_video
+#define standend  SLsmg_normal_video
+#define addch SLsmg_write_char
+#define addstr SLsmg_write_string
+#define initscr() do { extern int force_ugly_line_drawing; \
+	     extern int SLtt_Has_Alt_Charset; \
+	     SLtt_get_terminfo (); \
+	     if (force_ugly_line_drawing) \
+		SLtt_Has_Alt_Charset = 0; \
+             SLsmg_init_smg (); \
+          } while(0)
+#define refresh SLsmg_refresh
+#define clear SLsmg_cls
+#define erase SLsmg_cls
+#define mvaddstr(y, x, s) SLsmg_gotorc(y, x); SLsmg_write_string(s)
+#define touchline SLsmg_touch_lines
+#define inch SLsmg_char_at
+#define endwin SLsmg_reset_smg
+
+#ifndef OS2_NT
+#define SLsmg_draw_double_box(r,c,dr,dc) SLsmg_draw_box ((r), (c), (dr), (dc))
+#endif
+
 #if 0
 #   define one_vline() {acs (); addch (ACS_VLINE); noacs ();}
 #   define one_hline() {acs (); addch (ACS_HLINE); noacs ();}
