@@ -28,11 +28,20 @@ enum panel_display_enum {
 #define J_LEFT  0
 #define J_RIGHT 1
 
-typedef struct {
+#define NORMAL         0
+#define SELECTED       1
+#define MARKED         2
+#define MARKED_SELECTED        3
+#define STATUS         5
+
+
+typedef struct format_e {
+    struct format_e *next;
     int  field_len;
     char *(*string_fn)(file_entry *, int len);
     int  just_mode;
     char *title;
+    char *id;
 } format_e;
 
 typedef struct {
@@ -58,12 +67,15 @@ typedef struct {
     char     *filter;		/* File name filter */
 
     int      dirty;		/* Should we redisplay the panel? */
-    /* We should cleanup this */
-    char     *user_format;	/* User format */
+
+
     int	     user_mini_status;  /* Is mini_status_format used */
-    char    *mini_status_format;/* User format for mini status line */
+    char     *user_format;      /* User format */
+    char     *mini_status_format;/* User format for mini status line */
+
     format_e *format;		/* Display format */
-    int      fmt_count;		/* Number of formats */
+    format_e *status_format;    /* Mini status format */
+
     int      format_modified;	/* If the format was changed this is set */
     
     char     *panel_name;	/* The panel name */
@@ -89,10 +101,10 @@ WPanel *panel_new ();
 void panel_set_size (WPanel *panel, int x1, int y1, int x2, int y2);
 void paint_paint (WPanel *panel);
 void panel_refresh (WPanel *panel);
-void set_attr (int hilight, int marked);
 void Xtry_to_select (WPanel *panel, char *name);
 
 extern int torben_fj_mode;
+extern int highlight_mode;
 extern int show_mini_info;
 extern int panel_scroll_pages;
 
@@ -118,8 +130,8 @@ extern int extra_info;
 
 void set_colors (WPanel *panel);
 void paint_panel (WPanel *panel);
-void format_file (char *dest, WPanel *panel, int file_index, int panel_width);
-void repaint_file (WPanel *panel, int file_index, int move);
+void format_file (char *dest, WPanel *panel, int file_index, int panel_width, int attr, int isstatus);
+void repaint_file (WPanel *panel, int file_index, int move, int attr, int isstatus);
 void display_mini_info (WPanel *panel);
 void panel_reload (WPanel *panel);
 void paint_dir (WPanel *panel);
@@ -141,6 +153,7 @@ void panel_update_format (WPanel *panel);
 format_e *parse_display_format (WPanel *panel, char *format, 
 				char **error, int isstatus);
 char *panel_format (WPanel *panel);
+char *mini_status_format (WPanel *panel);
 char *set_panel_format (WPanel *p, char *format);
 
 extern WPanel *current_panel;
@@ -164,6 +177,7 @@ void unselect_item (WPanel *panel);
 extern Hook *select_file_hook;
 
 char *string_file_type (file_entry *fe, int len);
+char *string_file_size_brief (file_entry *fe, int len);
 char *string_file_permission (file_entry *fe, int len);
 char *string_file_nlinks (file_entry *fe, int len);
 char *string_file_owner (file_entry *fe, int len);
@@ -174,6 +188,7 @@ char *string_file_atime (file_entry *fe, int len);
 char *string_file_ctime (file_entry *fe, int len);
 char *string_file_name (file_entry *fe, int len);
 char *string_space (file_entry *fe, int len);
+char *string_dot (file_entry *fe, int len);
 char *string_marked (file_entry *fe, int len);
 char *string_file_perm_octal (file_entry *fe, int len);
 char *string_inode (file_entry *fe, int len);
