@@ -11,6 +11,9 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
+   Written by: 1995 Miguel de Icaza
+              1997 Timur Bakeyev
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
@@ -77,6 +80,13 @@ int torben_fj_mode = 0;
 /* If true, up/down keys scroll the pane listing by pages */
 int panel_scroll_pages = 1;
 
+/* If 1, we use permission hilighting, if 2 - then add per file type hilighting */
+int highlight_mode = 0;
+
+/* This gives abilitiy to determine colored user priveleges */
+extern user_in_groups *current_user_gid;
+extern uid_t current_user_uid;
+
 /* If we have an info panel, this points to it */
 WPanel *the_info_panel = 0;
 
@@ -105,27 +115,78 @@ set_colors (WPanel *panel)
     if (hascolors)
 	attrset (NORMAL_COLOR);
 }
+#endif
 
+/* Delete format string, it is a linked list */
 void
-set_attr (int hilight, int marked)
+delete_format (format_e *format)
 {
-    int color;
+    format_e *next;
+    
+    while (format){
+        next = format->next;
+        free (format);
+        format = next;
+     }
+}
 
-    color = (marked * 2 + hilight);
-    standend ();
-
-    /* Does this work on B&W terminals? */
-    if (hascolors){
-	attrset (sel_mark_color [color]);
+/* This code relies on the default justification!!! */
+void
+add_permission_string (char *dest, int width, file_entry *fe, int attr, int color, int is_octal)
+{
+    int i, r, l;
+    
+    l = get_user_rights (&fe->buf);
+    
+    if (is_octal){
+	/* Place of the access bit in octal mode */
+        l = width + l - 3;
+	r = l + 1;
     } else {
-	if (hilight)
-	    attrset (A_REVERSE | (marked ? A_BOLD : 0));
-	else
-	    if (marked)
-		attrset (A_BOLD);
+	/* The same to the triplet in string mode */
+        l = l * 3 + 1;
+	r = l + 3;
+    }
+    
+    for(i = 0; i < width; i++){
+	if (i >= l && i < r){
+            if (attr == SELECTED || attr == MARKED_SELECTED)
+                attrset (MARKED_SELECTED_COLOR);
+            else
+                attrset (MARKED_COLOR);
+        } else
+            attrset (color);
+	
+	addch (dest[i]);
     }
 }
-#endif
+
+int
+normal_color (file_entry *fe)
+{
+    if (highlight_mode == 2){
+        if (S_ISDIR (fe->buf.st_mode))
+            return (DIRECTORY_COLOR);
+        else if (S_ISLNK (fe->buf.st_mode)) {
+            if (fe->f.link_to_dir)
+                return (DIRECTORY_COLOR);
+            else if (fe->f.stalled_link)
+                return (STALLED_COLOR);
+            else
+                return (LINK_COLOR);
+        } else if (S_ISSOCK (fe->buf.st_mode))
+            return (SPECIAL_COLOR);
+        else if (S_ISCHR (fe->buf.st_mode))
+            return (DEVICE_COLOR);
+        else if (S_ISBLK (fe->buf.st_mode))
+            return (DEVICE_COLOR);
+        else if (S_ISFIFO (fe->buf.st_mode))
+            return (SPECIAL_COLOR);
+        else if (is_exe (fe->buf.st_mode))
+            return (EXECUTABLE_COLOR);
+    }
+    return (NORMAL_COLOR);
+}
 
 /* This functions return a string representation of a file entry */
 char *
@@ -158,6 +219,19 @@ string_file_type (file_entry *fe, int len)
     return buffer;
 }
 
+char *
+string_file_size_brief (file_entry *fe, int len)
+{
+    static char buffer [8];
+
+    if (S_ISDIR (fe->buf.st_mode)){
+       strcpy (buffer, (strcmp (fe->fname, "..") ? "SUB-DIR" : "UP--DIR"));
+       return buffer;
+    }
+
+    return string_file_size (fe, len);
+}
+
 char *
 string_file_permission (file_entry *fe, int len)
 {
@@ -198,15 +272,15 @@ string_file_size (file_entry *fe, int len)
     else
 #endif 
     {   
-        sprintf (buffer, "%d", (int) fe->buf.st_size);
+        sprintf (buffer, "%lu", (unsigned long) fe->buf.st_size);
         if (len && (i = strlen (buffer)) > len) {
             if (i - 2 > len) {
                 if (i - 5 > len)
-                    sprintf (buffer, "%dG", (int) ((fe->buf.st_size) >> 30));
+                    sprintf (buffer, "%luG", (unsigned long) ((fe->buf.st_size) >> 30));
                 else
-                    sprintf (buffer, "%dM", (int) ((fe->buf.st_size) >> 20));
+                    sprintf (buffer, "%luM", (unsigned long) ((fe->buf.st_size) >> 20));
             } else
-                sprintf (buffer, "%dK", (int) ((fe->buf.st_size) >> 10));
+                sprintf (buffer, "%luK", (unsigned long) ((fe->buf.st_size) >> 10));
         }
     }
     return buffer;
@@ -245,6 +319,12 @@ string_space (file_entry *fe, int len)
     return " ";
 }
 
+char *
+string_dot (file_entry *fe, int len)
+{
+    return ".";
+}
+
 char *
 string_marked (file_entry *fe, int len)
 {
@@ -300,8 +380,9 @@ static struct {
 { "size",  7,  0, J_RIGHT, "Size",       string_file_size,       (sortfn *) sort_size },
 { "type",  1,  0, J_LEFT,  "",           string_file_type,       (sortfn *) sort_type },
 { "mtime", 12, 0, J_RIGHT, "MTime",      string_file_mtime,      (sortfn *) sort_time },
-{ "perm",  10, 0, J_RIGHT, "Permission", string_file_permission, NULL},
-{ "mode",  7,  0, J_RIGHT, "OctM",       string_file_perm_octal, NULL },
+{ "bsize", 7,  0, J_RIGHT, "Size",      string_file_size_brief, (sortfn *) sort_size },
+{ "perm",  10, 0, J_LEFT,  "Permission", string_file_permission, NULL },
+{ "mode",  6,  0, J_RIGHT, "Perm",       string_file_perm_octal, NULL },
 { "|",     1,  0, J_RIGHT, "|",          0,                      NULL },
 { "nlink", 2,  0, J_RIGHT, "Nl",         string_file_nlinks,     (sortfn *) sort_links },
 { "ngid",  5,  0, J_RIGHT, "GID",        string_file_ngid,       (sortfn *) sort_ngid },
@@ -311,6 +392,7 @@ static struct {
 { "atime", 12, 0, J_RIGHT, "ATime",      string_file_atime,      (sortfn *) sort_atime },
 { "ctime", 12, 0, J_RIGHT, "CTime",      string_file_ctime,      (sortfn *) sort_ctime },
 { "space", 1,  0, J_RIGHT, " ",          string_space,           NULL },
+{ "dot",   1,  0, J_RIGHT, " ",          string_dot,             NULL },
 { "mark",  1,  0, J_RIGHT, " ",          string_marked,          NULL },
 { "inode", 5,  0, J_RIGHT, "Inode",      string_inode,           (sortfn *) sort_inode },
 };
@@ -344,34 +426,66 @@ to_buffer (char *dest, int just_mode, int len, char *txt)
 
 /* Formats the file number file_index of panel in the buffer dest */
 void
-format_file (char *dest, WPanel *panel, int file_index, int width)
+format_file (char *dest, WPanel *panel, int file_index, int width, int attr, int isstatus)
 {
-    int   i, length = 0;
-    int   top = panel->fmt_count;
-    char  *txt;
-    char  *old_pos;
-    char  *cdest = dest;
-    int   just_mode; 
-    int   empty_line = file_index >= panel->count;
-   
-    for (i = 0; i < top; i++){
-	file_entry *fe = &panel->dir.list [file_index];
+    int      i, color, length, empty_line;
+    char     *txt;
+    char     *old_pos;
+    char     *cdest = dest;
+    format_e *format, *home;
+    file_entry *fe;
+    
+    length     = 0;
+    empty_line = (file_index >= panel->count);
+    home       = (isstatus) ? panel->status_format : panel->format;   
+    fe         = &panel->dir.list [file_index];
 
-	if (panel->format [i].string_fn){
+    switch (attr){
+    case SELECTED:
+    case MARKED:
+    case MARKED_SELECTED:
+	color = sel_mark_color [attr];
+	break;
+    case STATUS:
+	color = NORMAL_COLOR;
+	break;
+    case NORMAL:
+    default:
+	color = normal_color(fe);
+    }
+
+    for (format = home; format; format = format->next){
+	if (format->string_fn){
 	    if (empty_line)
 		txt = " ";
 	    else
-		txt = (*panel->format [i].string_fn)(fe, panel->format [i].field_len);
-	    just_mode = panel->format [i].just_mode;
+		txt = (*format->string_fn)(fe, format->field_len);
 
 	    old_pos = cdest;
-	    cdest = to_buffer (cdest, just_mode, panel->format [i].field_len, txt);
-	    length += panel->format [i].field_len;
+	    cdest = to_buffer (cdest, format->just_mode, format->field_len, txt);
+	    length += format->field_len;
+
 #ifndef HAVE_X
-	    addstr (old_pos);
+            /* What shall we do? Will we color each line according to
+	     * the file type? Any suggestions to mc@timur.kazan.su
+	     */
+	    
+            attrset (color);
+ 
+            if (highlight_mode && !strcmp(format->id, "perm"))
+                add_permission_string (old_pos, format->field_len, fe, attr, color, 0);
+            else if (highlight_mode && !strcmp(format->id, "mode"))
+                add_permission_string (old_pos, format->field_len, fe, attr, color, 1);
+            else
+		addstr (old_pos);
 #endif
 	} else {
 #ifndef HAVE_X
+            /* I'm preffer the view without this 3 lines, try to kill it :-) */
+            if (attr == SELECTED || attr == MARKED_SELECTED)
+                attrset (SELECTED_COLOR);
+            else
+                attrset (NORMAL_COLOR);
 	    one_vline ();
 #else
 	    *cdest++ = ' ';
@@ -379,6 +493,7 @@ format_file (char *dest, WPanel *panel, int file_index, int width)
 	    length++;
 	}
     }
+    
     if (length < width){
 	int still = width - length;
 	while (still--)
@@ -393,14 +508,14 @@ format_file (char *dest, WPanel *panel, int file_index, int width)
 
 #ifndef HAVE_X
 void
-repaint_file (WPanel *panel, int file_index, int mv)
+repaint_file (WPanel *panel, int file_index, int mv, int attr, int isstatus)
 {
     int    second_column = 0;
     int	   width, offset;
     char   buffer [255];
 
     offset = 0;
-    if (panel->split){
+    if (!isstatus && panel->split){
 	
 	second_column = (file_index - panel->top_file) / llines (panel);
 	width = (panel->widget.cols/2 - 2) - 1;
@@ -413,7 +528,7 @@ repaint_file (WPanel *panel, int file_index, int mv)
         width = (panel->widget.cols - 2);
 
     if (mv){
-	if (panel->split){
+	if (!isstatus && panel->split){
 	    widget_move (&panel->widget,
 			 (file_index - panel->top_file) %
 			 llines (panel) + 2,
@@ -422,9 +537,9 @@ repaint_file (WPanel *panel, int file_index, int mv)
 	    widget_move (&panel->widget, file_index - panel->top_file + 2, 1);
     }
     
-    format_file (buffer, panel, file_index, width);
+    format_file (buffer, panel, file_index, width, attr, isstatus);
     
-    if (panel->split){
+    if (!isstatus && panel->split){
 	if (second_column)
 	    addch (' ');
 	else {
@@ -435,34 +550,6 @@ repaint_file (WPanel *panel, int file_index, int mv)
 }
 #endif
 
-/* FIXME: This function uses hard coded constants */
-/* panel->cols - 22 is not a wise idea */
-void
-mini_info_brief (WPanel *panel)
-{
-    int   isdir = 0;
-    char  *sdir_txt = "";
-    
-#define entry (panel->dir.list [panel->selected].buf)
-    
-    if (S_ISDIR (entry.st_mode)){
-	isdir = 1;
-	sdir_txt = strcmp (panel->dir.list [panel->selected].fname, "..")
-	    ? ">SUB-DIR<" : ">UP--DIR<";
-    }
-
-    printw ("%s%-*s", string_file_type (&panel->dir.list [panel->selected], 1),
-	     panel->widget.cols-24, split_extension
-	     (name_trunc (panel->dir.list [panel->selected].fname,
-			  panel->widget.cols-24), panel->widget.cols-24));
-
-    one_vline ();
-    printw ("%*s", 7, isdir ? sdir_txt : size_trunc (entry.st_size));
-    one_vline ();
-
-    printw ("%10s", string_perm (entry.st_mode));
-}
-
 void
 display_mini_info (WPanel *panel)
 {
@@ -479,6 +566,7 @@ display_mini_info (WPanel *panel)
 	return;
     }
 
+    /* Status displays total marked size */
     if (panel->marked){
 	char buffer [100];
 	char *p;
@@ -498,6 +586,7 @@ display_mini_info (WPanel *panel)
 	return;
     }
 
+    /* Status resolves links and show them */
     set_colors (panel);
 #ifndef OS2_NT    
     if (S_ISLNK (panel->dir.list [panel->selected].buf.st_mode)){
@@ -516,49 +605,11 @@ display_mini_info (WPanel *panel)
 	return;
     }
 #endif
-    if (panel->user_mini_status){
-	char *err;
-
-	if (panel->format)
-	    free (panel->format);
-	panel->format = parse_display_format (panel, panel->mini_status_format,
-					      &err, 1);
-	if (err){
-	    beep ();	/* For debugging */
-	    free (panel->mini_status_format);
-	    panel->mini_status_format = strdup (DEFAULT_USER_FORMAT);
-	    panel->format =
-		parse_display_format (panel, panel->mini_status_format,
-				      &err, 1);
-	}
-#ifndef HAVE_XVIEW	
-	repaint_file (panel, panel->selected, 0);
+    /* Default behaviour */
+#ifndef HAVE_XVIEW
+    repaint_file (panel, panel->selected, 0, STATUS, 1);
 #endif	
-	
-	/* This clears the second half of the mini status line, 'cause
-	   repaint_file writes only the first half */
-	if (panel->split){
-	    widget_move (&panel->widget, llines (panel)+3,
-			 (panel->widget.cols-2)/2);
-	    printw ("%*s", (panel->widget.cols-2) -
-		    (panel->widget.cols-2)/2 + 1, "");
-	}
-	
-	if (panel->format)
-	    free (panel->format);
-	panel->format = parse_display_format (panel, panel_format (panel),
-					      &err, 0);
-	return;
-    }
-
-    if (panel->list_type == list_brief){
-	mini_info_brief (panel);
-	return;
-    }
-    
-    printw ("%-*s", panel->widget.cols-2,
-	    name_trunc(panel->dir.list [panel->selected].fname,
-		       panel->widget.cols-2));
+    return;
 #endif
 }
 
@@ -567,21 +618,19 @@ void
 paint_dir (WPanel *panel)
 {
     int i;
-    int hilight;		/* Color used for hilighting */
-    int marked;			/* Color used for marked file */
+    int color;			/* Color value of the line */
     int items;			/* Number of items */
 
     items = llines (panel) * (panel->split ? 2 : 1);
     
     for (i = 0; i < items; i++){
 	if (i+panel->top_file >= panel->count)
-	    set_attr (0, 0);
+	    color = 0;
 	else {
-	    hilight = panel->selected==i+panel->top_file && panel->active;
-	    marked  = panel->dir.list [i+panel->top_file].f.marked;
-	    set_attr (hilight, marked);
+	    color = 2 * (panel->dir.list [i+panel->top_file].f.marked);
+	    color += (panel->selected==i+panel->top_file && panel->active);
 	}
-	repaint_file (panel, i+panel->top_file, 1);
+	repaint_file (panel, i+panel->top_file, 1, color, 0);	
     }
     standend ();
     panel->dirty = 0;
@@ -649,7 +698,7 @@ show_dir (WPanel *panel)
 
     widget_move (&panel->widget, 0, 1);
 
-    trim (strip_home (panel->cwd), tmp, panel->widget.cols-5);
+    trim (strip_home_and_password (panel->cwd), tmp, panel->widget.cols-5);
     addstr (tmp);
     
     if (panel->active)
@@ -807,7 +856,8 @@ panel_destroy (WPanel *p)
 
     free (p->user_format);
     free (p->mini_status_format);
-    free (p->format);
+    delete_format (p->format);
+    delete_format (p->status_format);
     free (p->dir.list);
     free (p->panel_name);
     free (name);
@@ -827,9 +877,18 @@ panel_update_format (WPanel *panel)
     
     panel->format = parse_display_format (panel, panel_format (panel),
 					  &err, 0);
-    panel_format_modified (panel);
     if (err)
 	free (err);
+    
+    if (show_mini_info){
+        panel->status_format = parse_display_format (panel, mini_status_format (panel),
+						     &err, 1);
+	if (err)
+	    free (err);
+    }
+    
+    panel_format_modified (panel);
+    
     panel_update_cols (&(panel->widget), panel->frame_size);
 }
 
@@ -840,6 +899,7 @@ panel_new (char *panel_name)
 {
     WPanel *panel;
     char *section;
+    char *err;
 
     panel = xmalloc (sizeof (WPanel), "panel_new");
 
@@ -854,21 +914,19 @@ panel_new (char *panel_name)
     mc_get_current_wd (panel->cwd, sizeof (panel->cwd)-2);
     strcpy (panel->lwd, ".");
 
-    panel->dir.list  = (file_entry *) malloc (MIN_FILES * sizeof (file_entry));
-    panel->dir.size  = MIN_FILES;
-    panel->active    = 0;
-    panel->filter    = 0;
-    panel->split     = 0;
-    panel->top_file  = 0;
-    panel->selected  = 0;
-    panel->marked    = 0;
-    panel->total     = 0;
-    panel->reverse   = 0;
-    panel->format    = 0;
-    panel->fmt_count = 0;
-    panel->dirty     = 1;
-    panel->searching = 0;
-    panel->dirs_marked = 0;
+    panel->dir.list    = (file_entry *) malloc (MIN_FILES * sizeof (file_entry));
+    panel->dir.size    = MIN_FILES;
+    panel->active      = 0;
+    panel->filter      = 0;
+    panel->split       = 0;
+    panel->top_file    = 0;
+    panel->selected    = 0;
+    panel->marked      = 0;
+    panel->total       = 0;
+    panel->reverse     = 0;
+    panel->dirty       = 1;
+    panel->searching   = 0;
+    panel->dirs_marked         = 0;
     panel->is_panelized = 0;
     panel->has_dir_sizes = 0;
     panel->format_modified = 1;
@@ -886,7 +944,19 @@ panel_new (char *panel_name)
     }
     panel_load_setup (panel, section);
     free (section);
+
+    /* Load format strings */
+    panel->format = parse_display_format (panel, panel_format (panel), &err, 0);
+    if (err) free (err);
     
+    if (show_mini_info){
+        panel->status_format = parse_display_format (panel, mini_status_format (panel), &err, 1);
+        if (err)
+	    free (err);
+    } else
+        panel->status_format = 0;
+    
+
     /* Load the default format */
     
     panel->count = do_load_dir (&panel->dir, panel->sort_type,
@@ -944,32 +1014,34 @@ panel_reload (WPanel *panel)
 void
 paint_frame (WPanel *panel)
 {
-    const int top = panel->fmt_count;
-    char *txt, buffer [30], new_buffer [80];
-    int header_len, spaces, extra;
-    int i = 0, j;
+    char     *txt, buffer [30], new_buffer [80];
+    int      header_len, spaces, extra;
+    int      i = 0, j;
+    format_e *format;
     
     show_dir (panel);
     x_sort_label_start (panel);
-    for (i = 0; i < top; i++){ 
-	txt = panel->format [i].title;
+
+    for (format = panel->format; format; format = format->next){
+	txt = format->title;
+
 	header_len = strlen (txt);
-	if (header_len > panel->format [i].field_len){
+	if (header_len > format->field_len){
 	    strcpy (buffer, txt);
 	    txt = buffer;
-	    txt [panel->format [i].field_len] = 0;
+	    txt [format->field_len] = 0;
 	    header_len = strlen (txt);
 	}
 	
-	spaces = (panel->format [i].field_len - header_len) / 2;
-	extra  = (panel->format [i].field_len - header_len) % 2;
+	spaces = (format->field_len - header_len) / 2;
+	extra  = (format->field_len - header_len) % 2;
 	sprintf (new_buffer, "%*s%-s%*s", spaces, "",
 		 txt, spaces+extra, "");
 
 	for (j = 0; j < ELEMENTS(formats); j++){
-	    if (strcmp (panel->format [i].title, formats [j].title))
+	    if (strcmp (format->id, formats [j].id))
 		continue;
-	    x_add_sort_label (panel, i, new_buffer, panel->format [i].title, formats [j].sort_routine);
+	    x_add_sort_label (panel, i, new_buffer, format->id, formats [j].sort_routine);
 	}
     }
     panel->format_modified = 0;
@@ -979,12 +1051,11 @@ paint_frame (WPanel *panel)
 void
 paint_frame (WPanel *panel)
 {
-    const int top = panel->fmt_count;
     int  header_len;
     int  spaces, extra;
-    int  i, side, width;
+    int  side, width;
+    
     char *txt, buffer[30]; /*Hope that this is enough ;-) */
-	
     if (!panel->split)
 	adjust_top_file (panel);
     
@@ -994,8 +1065,10 @@ paint_frame (WPanel *panel)
     widget_move (&panel->widget, 1, 1);
 
     for (side = 0; side <= panel->split; side++){
+	format_e *format;
+	
 	if (side){
-	    set_attr (0, 0);
+	    attrset (NORMAL_COLOR);
 	    one_vline ();
 	    width = panel->widget.cols - panel->widget.cols/2 - 1;
 	} else if (panel->split)
@@ -1003,31 +1076,32 @@ paint_frame (WPanel *panel)
 	else
 	    width = panel->widget.cols - 2;
 	
-	for (i = 0; i < top; i++){
-
-	    if (panel->format [i].string_fn){
-		txt = panel->format [i].title;
+	for (format = panel->format; format; format = format->next){
+            if (format->string_fn){
+                txt = format->title;
+		
 		header_len = strlen (txt);
-		if (header_len > panel->format [i].field_len){
+		if (header_len > format->field_len){
 		    strcpy (buffer, txt);
 		    txt = buffer;
-		    txt [panel->format [i].field_len] = 0;
+		    txt [format->field_len] = 0;
 		    header_len = strlen (txt);
 		}
 		
-		set_attr (0, 1);
-		spaces = (panel->format [i].field_len - header_len) / 2;
-		extra  = (panel->format [i].field_len - header_len) % 2;
+                attrset (MARKED_COLOR);
+                spaces = (format->field_len - header_len) / 2;
+                extra  = (format->field_len - header_len) % 2;
 		printw ("%*s%-s%*s", spaces, "",
 			 txt, spaces+extra, "");
 		width -= 2 * spaces + extra + header_len;
 	    } else {
-		set_attr (0, 0);
+		attrset (NORMAL_COLOR);
 		one_vline ();
 		width --;
 		continue;
 	    }
 	}
+	
 	if (width > 0)
 	    printw ("%*s", width, "");
     }
@@ -1035,31 +1109,35 @@ paint_frame (WPanel *panel)
 #endif
 
 static char *
-parse_panel_size (WPanel *panel, char *format)
+parse_panel_size (WPanel *panel, char *format, int isstatus)
 {
+    int frame = frame_half;
     format = skip_separators (format);
 
-    panel->frame_size = frame_half;
-    panel->split = 0;
-    
-    if (strncmp (format, "full", 4) == 0){
-	panel->frame_size = frame_full;
+    if (!strncmp (format, "full", 4)){
+	frame = frame_full;
 	format += 4;
-    } else if (strncmp (format, "half", 4) == 0){
+    } else if (!strncmp (format, "half", 4)){
+	frame = frame_half;
 	format += 4;
     }
-    panel_update_cols (&(panel->widget), panel->frame_size);
+
+    if (!isstatus){
+        panel->frame_size = frame;
+        panel->split = 0;
+    }
     
     /* Now, the optional column specifier */
     format = skip_separators (format);
     
     if (*format == '1' || *format == '2'){
-	panel->split = *format == '2';
+	if (!isstatus)
+	    panel->split = *format == '2';
 	format++;
     }
 
-    if (horizontal_split)
-	panel->widget.cols = COLS;
+    if (!isstatus)
+        panel_update_cols (&(panel->widget), panel->frame_size);
     
     return skip_separators (format);
 }
@@ -1084,51 +1162,48 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
 {
 #define MAX_EXPAND 4
     
-    format_e *darr;		/* The formats we return */
-    int  fields;		/* Number of fields */
-    int  expand_list [MAX_EXPAND]; /* Expand at most 4 fields. */
-    int  expand_top = 0;	/* Max used element in expand */
-    int  field;
-    int  usable_columns;	/* Usable columns in the panel */
-    int  total_cols = 0;	/* Used columns by the format */
-    char *s;
+    format_e *darr, *old, *home = 0;   /* The formats we return */
+    int  expand_top = 0;               /* Max used element in expand */
+    int  usable_columns;               /* Usable columns in the panel */
+    int  total_cols = 0;               /* Used columns by the format */
+    char *expand_list [MAX_EXPAND];    /* Expand at most 4 fields. */
     int  i, j;
-    int  set_justify;		/* flag: set justification mode? */
-    int  justify = 0;		/* Which mode. */
+    int  set_justify;                  /* flag: set justification mode? */
+    int  justify = 0;                  /* Which mode. */
     int  cols_save;
 
     *error = 0;
     
     if (!format)
-	format = "half type,name,|,size,|,perm";
-
-    /* Determine the panel size */
-    cols_save = panel->widget.cols;
-    format = parse_panel_size (panel, format);
+	format = DEFAULT_USER_FORMAT;
     
     /* This makes sure that the panel and mini status full/half mode
        setting is equal */
-    if (isstatus){
-    	panel->widget.cols = cols_save;
-    }
+    format = parse_panel_size (panel, format, isstatus);
 
-    usable_columns = ((panel->widget.cols-2)/(panel->split+1)) - panel->split;
+    /* Status needn't to be split */
+    usable_columns = ((panel->widget.cols-2)/((isstatus)
+					      ? 1
+					      : (panel->split+1))) - (!isstatus && panel->split);
     
-    /* Count the number of fields */
-    fields = 1;
-    for (s = format; *s; s++)
-	if (*s == ',')
-	    fields++;
-
-    panel->fmt_count = fields;
+    /* Clean expand list */
+    for (i = 0; i < MAX_EXPAND; i++)
+        expand_list [i] = '\0';
     
-    /* Allocate the space */
-    darr = xmalloc (sizeof (format_e) * fields, "parse_display_format");
-    
-    for (field = 0; field < fields; field++){
+    while (*format){
 	int found = 0;
 	int expands = 1;
 
+        darr = xmalloc (sizeof (format_e), "parse_display_format");
+	
+        /* I'm so ugly, don't look at me :-) */
+        if (!home)
+            home = old = darr;
+	
+        old->next = darr;
+        darr->next = 0;
+        old = darr;
+	
 	format = skip_separators (format);
 
 	if (*format == '<' || *format == '>'){
@@ -1146,14 +1221,16 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
 
 	    format += klen;
 
-	    darr [field].field_len = formats [i].min_size;
-	    darr [field].string_fn = formats [i].string_fn;
-	    darr [field].title     = formats [i].title;
+            darr->field_len = formats [i].min_size;
+            darr->string_fn = formats [i].string_fn;
+            darr->title     = formats [i].title;
+            darr->id        = formats [i].id;
+
 
 	    if (set_justify)
-		darr [field].just_mode = justify;
+		darr->just_mode = justify;
 	    else
-		darr [field].just_mode = formats [i].default_just;
+		darr->just_mode = formats [i].default_just;
 
 	    found = 1;
 
@@ -1170,22 +1247,24 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
 		format++;
 		req_length = atoi (format);
 #if 0		
-		if (req_length > darr [field].field_len)
+		if (req_length > darr->field_len)
 #endif
-		    darr [field].field_len = req_length;
+		    darr->field_len = req_length;
 
 		format = skip_numbers (format);
 
 		/* Now, if they insist on expansion */
-		if (*format == '+')
+		if (*format == '+'){
 		    expands = 1;
-		format++;
+		    format++;
+		}
+
 	    }
 
 	    if (expands) {
 		/* Take note if it's expandable */
 		if (formats [i].expands && expand_top < MAX_EXPAND)
-		    expand_list [expand_top++] = field;
+		    expand_list [expand_top++] = formats [i].id;
 	    }
 	    break;
 	}
@@ -1193,30 +1272,34 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
 	    char old_char;
 	    
 	    int pos = min (8, strlen (format));
-	    free (darr);
+	    delete_format (home);
 	    old_char = format [pos];
 	    format [pos] = 0;
 	    *error = copy_strings("Unknow tag on display format: ", format, 0);
 	    format [pos] = old_char;
 	    return 0;
 	}
-	total_cols += darr [field].field_len;
-
+	total_cols += darr->field_len;
     }
     
     /* If we used more columns than the available columns, adjust that */
     if (total_cols > usable_columns){
-	int dif   = total_cols - usable_columns;
+	int pdif, dif = total_cols - usable_columns;
 	
-	while (dif)
-	    for (j = 0; j < field; j++){
-		if (darr [j].field_len-1){
-		    darr [j].field_len--;
-		    if (dif)
-			dif--;
+        while (dif){
+	    pdif = dif;
+	    for (darr = home; darr; darr = darr->next){
+		if (dif && darr->field_len - 1){
+		    darr->field_len--;
+		    dif--;
 		}
 	    }
-	total_cols = usable_columns;
+
+	    /* avoid endless loop if num fields > 40 */
+	    if (pdif == dif)
+		break; 
+	}
+	total_cols  = usable_columns; /* give up, the rest should be truncated */
     }
 
     /* Expand the available space */
@@ -1224,14 +1307,16 @@ parse_display_format (WPanel *panel, char *format, char **error, int isstatus)
 	int spaces = (usable_columns - total_cols) / expand_top;
 	int extra  = (usable_columns - total_cols) % expand_top;
 	
-	for (i = 0; i < expand_top; i++)
-	    darr [expand_list [i]].field_len += spaces;
-
-	/* The modulo goes to the first one */
-	darr [expand_list [0]].field_len += extra;
+        for (i = 0; darr && i < expand_top; i++){
+	    for (darr = home; darr; darr = darr->next){
+		if (!strcmp(darr->id, expand_list [i])){
+		    darr->field_len += ( spaces + ((i == 0) ? extra : 0 ));
+		    break;
+                }
+            }
+        }
     }
-    
-    return darr;
+    return home;
 }
 
 /* Switches the panel to the mode specified in the format */
@@ -1241,12 +1326,23 @@ set_panel_format (WPanel *p, char *format)
     char *err;
 
     if (p->format)
-	free (p->format);
+	delete_format (p->format);
     
     p->format = parse_display_format (p, format, &err, 0);
+
     if (err)
 	return err;
 
+    if (show_mini_info){
+	if (p->status_format)
+	    delete_format (p->status_format);
+	
+	p->status_format = parse_display_format (p, mini_status_format (p), &err, 1);
+	
+	if (err)
+	    return err;
+    }
+    
     panel_format_modified (p);
     panel_update_cols (&(p->widget), p->frame_size);
     paint_frame (p);
@@ -1277,6 +1373,29 @@ panel_format (WPanel *panel)
     }
 }
 
+char *
+mini_status_format (WPanel *panel)
+{
+    if (panel->user_mini_status)
+       return panel->mini_status_format;
+
+    switch (panel->list_type){
+
+    case list_long:
+       return "full perm,space,nlink,space,owner,space,group,space,size,space,mtime,space,name";
+
+    case list_brief:
+       return "half type,name";
+
+    case list_full:
+       return "half type,name";
+
+    default:
+    case list_user:
+       return panel->user_format;
+    }
+}
+
 /*                          */
 /* Panel operation commands */
 /*                          */
@@ -1345,11 +1464,10 @@ select_item (WPanel *panel)
     }
 
 #ifndef HAVE_X    
-    set_attr (1, selection (panel)->f.marked);
     if (repaint)
 	paint_panel (panel);
     else
-	repaint_file (panel, panel->selected, 1);
+	repaint_file (panel, panel->selected, 1, 2*selection (panel)->f.marked+1, 0);
 #else
     if (old_top != panel->top_file)
 	x_adjust_top_file (panel);
@@ -1387,8 +1505,7 @@ unselect_item (WPanel *panel)
 void
 unselect_item (WPanel *panel)
 {
-    set_attr (0, selection (panel)->f.marked);
-    repaint_file (panel, panel->selected, 1);
+    repaint_file (panel, panel->selected, 1, 2*selection (panel)->f.marked, 0);
 }
 #endif
 
@@ -1698,8 +1815,7 @@ do_mark_file (WPanel *panel, int do_move)
     int idx = panel->selected;
     do_file_mark (panel, idx, selection (panel)->f.marked ? 0 : 1);
 #ifndef HAVE_XVIEW	
-    set_attr  (1, panel->dir.list [idx].f.marked);
-    repaint_file (panel, idx, 1);
+    repaint_file (panel, idx, 1, 2*panel->dir.list [idx].f.marked+1, 0);
 #endif        
 
     if (mark_moves_down && do_move)
@@ -1764,7 +1880,10 @@ static void
 start_search (WPanel *panel)
 {
     if (panel->searching){
-	move_down (panel);
+	if (panel->selected+1 == panel->count)
+	    panel->selected = 0;
+	else
+	    move_down (panel);
 	do_search (panel, 0);
     } else {
 	panel->searching = 1;
@@ -1783,14 +1902,19 @@ do_enter (WPanel *panel)
     } else { 
 	if (is_exe (selection (panel)->buf.st_mode) &&
 	    if_link_is_exe (selection (panel))) {
-	    if (vfs_current_is_local ()) {
+#ifdef USE_VFS	    
+	    if (vfs_current_is_local ()) 
+#endif	    
+	    {
 	        char *tmp = copy_strings (".", PATH_SEP_STR, selection (panel)->fname, 0);
 	        if (!confirm_execute || (query_dialog (" The Midnight Commander ",
 	                                    " Do you really want to execute? ",
 	                                         0, 2, " Yes ", " No ") == 0))
 	            execute (tmp);
 	        free (tmp);
-	    } else if (vfs_current_is_extfs ()) {
+	    } 
+#ifdef USE_VFS	    	    
+	    else if (vfs_current_is_extfs ()) {
 	        char *tmp = vfs_get_current_dir();
 	        char *tmp2;
 
@@ -1798,6 +1922,7 @@ do_enter (WPanel *panel)
 	        extfs_run(tmp2);
 	        free (tmp2);
 	    }
+#endif /* USE_VFS */	    
 	    return;
 	} else
 	    regex_command (selection (panel)->fname, "Open", NULL, 0);
@@ -1973,6 +2098,7 @@ panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
     switch (msg){
     case WIDGET_INIT:
 #ifdef HAVE_X
+	define_label (h, (Widget *)panel, 1, "Help", help_cmd);
 	define_label (h, (Widget *)panel, 2, "Menu", user_menu_cmd);
 	define_label (h, (Widget *)panel, 3, "View", view_panel_cmd);
 	define_label (h, (Widget *)panel, 4, "Edit", edit_panel_cmd);
@@ -2004,6 +2130,7 @@ panel_callback (Dlg_head *h, WPanel *panel, int msg, int par)
 	show_dir (panel);
 	select_item (panel);
 #ifndef HAVE_X	
+	define_label (h, (Widget *)panel, 1, "Help", help_cmd);
 	define_label (h, (Widget *)panel, 2, "Menu", user_menu_cmd);
 	define_label (h, (Widget *)panel, 3, "View", view_panel_cmd);
 	define_label (h, (Widget *)panel, 4, "Edit", edit_panel_cmd);
@@ -2106,7 +2233,7 @@ get_sort_fn (char *name)
 
     /* Then try the short name options, from our local table */
     for (i = 0; i < ELEMENTS (formats); i++){
-	if (strcmp (name, formats [i].title) == 0 && formats [i].sort_routine)
+	if (strcmp (name, formats [i].id) == 0 && formats [i].sort_routine)
 	    return formats [i].sort_routine;
     }
     return NULL;
