@@ -66,6 +66,7 @@ char *profile_name;
 
 char setup_color_string [4096];
 char term_color_string [4096];
+char color_terminal_string [512];
 
 #define load_int(a,b,c) GetPrivateProfileInt(a,b,c,profile_name)
 #define load_string(a,b,c,d,e) GetPrivateProfileString(a,b,c,d,e,profile_name)
@@ -157,10 +158,12 @@ static struct {
     { "confirm_overwrite", &confirm_overwrite },
     { "confirm_execute", &confirm_execute },
     { "confirm_exit", &confirm_exit },
+    { "safe_delete", &know_not_what_am_i_doing },
     { "mouse_repeat_rate", &mou_auto_repeat },
     { "double_click_speed", &double_click_speed },
     { "eight_bit_clean", &eight_bit_clean },
     { "full_eight_bits", &full_eight_bits },
+    { "use_8th_bit_as_meta", &use_8th_bit_as_meta },
     { "confirm_view_dir", &confirm_view_dir },
     { "tree_navigation_flag", &tree_navigation_flag },
     { "mouse_move_pages", &mouse_move_pages },
@@ -177,6 +180,7 @@ static struct {
     { "have_fast_cpu", &have_fast_cpu },
     { "iconify_on_exec", &iconify_on_exec },
     { "torben_fj_mode", &torben_fj_mode },
+    { "highlight_mode", &highlight_mode },
     { "use_file_to_guess_type", &use_file_to_check_type },
     { "alternate_plus_minus", &alternate_plus_minus },
     { "only_leading_plus_minus", &only_leading_plus_minus },
@@ -205,6 +209,8 @@ static struct {
     { "editor_return_does_auto_indent", &option_return_does_auto_indent },
     { "editor_backspace_through_tabs", &option_backspace_through_tabs },
     { "editor_fake_half_tabs", &option_fake_half_tabs },
+    { "editor_option_save_mode", &option_save_mode },
+    { "editor_option_backup_ext_int", &option_backup_ext_int },
 #endif
     { 0, 0 }
 };
@@ -424,20 +430,22 @@ void load_setup (void)
 {
     static char *buffer;
     char   *profile;
+    char   *inifile;
     int    i;
 #ifdef USE_NETCODE
     extern char *ftpfs_proxy_host;
 #endif
     buffer = concat_dir_and_file (home_dir, PROFILE_NAME);
-    
+    inifile = concat_dir_and_file (mc_home, "mc.ini");
     if (exist_file (buffer)){
 	profile = buffer;
-    } else if (exist_file (LIBDIR "mc.ini")){
-	profile = strdup (LIBDIR "mc.ini");
+    } else if (exist_file (inifile)){
+	profile = strdup (inifile);
 	free (buffer);
     } else {
 	profile = buffer;
     }
+    free (inifile);
     
     profile_name = profile;
 
@@ -482,6 +490,8 @@ void load_setup (void)
 			     sizeof (setup_color_string));
     load_string ("Colors", getenv ("TERM"), "",
 			     term_color_string, sizeof (term_color_string));
+    load_string ("Colors", "color_terminals", "",
+			     color_terminal_string, sizeof (color_terminal_string));
 
     /* Load the directory history */
     directory_history_load ();
@@ -549,12 +559,14 @@ void load_keys_from_section (char *terminal, char *profile_name)
 
 void load_key_defs (void)
 {
+    char *libfile = concat_dir_and_file (mc_home, "mc.lib");
     load_keys_from_section (getenv ("TERM"), profile_name);
     load_keys_from_section ("general", profile_name);
 
-    load_keys_from_section (getenv ("TERM"), LIBDIR "mc.lib");
-    load_keys_from_section ("general", LIBDIR "mc.lib");
+    load_keys_from_section (getenv ("TERM"), libfile);
+    load_keys_from_section ("general", libfile);
 
     /* We don't want a huge database loaded in core */
-    free_profile_name (LIBDIR "mc.lib");
+    free_profile_name (libfile);
+    free (libfile);
 }
