@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <termios.h>
 #include <signal.h>
+#include <string.h>
 #include "tty.h"
 #include "mad.h"
 #include "color.h"
@@ -33,6 +34,7 @@
 #include "main.h"		/* extern: force_colors */
 #include "win.h"		/* do_exit_ca_mode */
 #include "background.h"		/* we_are_background definition */
+#include "setup.h"
 
 #ifdef HAVE_SLANG
 
@@ -60,6 +62,47 @@ static int slinterrupt;
 /* Controls whether we should wait for input in getch */
 static int no_slang_delay;
 
+/* {{{  Copied from ../slang/slgetkey.c, removed the DEC_8Bit_HACK, */
+extern unsigned int SLang_Input_Buffer_Len;
+extern unsigned char SLang_Input_Buffer [];
+extern unsigned int SLsys_getkey (void);
+extern int SLsys_input_pending (int);
+
+static unsigned int SLang_getkey2 (void)
+{
+   unsigned int imax;
+   unsigned int ch;
+   
+   if (SLang_Input_Buffer_Len)
+     {
+	ch = (unsigned int) *SLang_Input_Buffer;
+	SLang_Input_Buffer_Len--;
+	imax = SLang_Input_Buffer_Len;
+   
+	memcpy ((char *) SLang_Input_Buffer, 
+		(char *) (SLang_Input_Buffer + 1), imax);
+	return(ch);
+     }
+   else return(SLsys_getkey ());
+}
+
+static int SLang_input_pending2 (int tsecs)
+{
+   int n;
+   unsigned char c;
+   if (SLang_Input_Buffer_Len) return (int) SLang_Input_Buffer_Len;
+   
+   n = SLsys_input_pending (tsecs);
+   
+   if (n <= 0) return 0;
+   
+   c = (unsigned char) SLang_getkey2 ();
+   SLang_ungetkey_string (&c, 1);
+   
+   return n;
+}
+/* }}} */
+
 static void
 slang_intr (int signo)
 {
@@ -112,6 +155,9 @@ got_interrupt ()
 void
 slang_init (void)
 {
+    extern int SLtt_Blink_Mode;
+    extern int SLtt_Has_Alt_Charset;
+    extern int force_ugly_line_drawing;
     struct sigaction act, oact;
     
     Read_FD = fileno (stdin);
@@ -119,6 +165,8 @@ slang_init (void)
     tcgetattr (Read_FD, &boot_mode);
     /* 255 = ignore abort char; XCTRL('g') for abort char = ^g */
     SLang_init_tty (XCTRL('c'), 1, 0);	
+    if (force_ugly_line_drawing)
+	SLtt_Has_Alt_Charset = 0;
     tcgetattr (Read_FD, &new_mode);
     slang_prog_mode ();
     load_terminfo_keys ();
@@ -126,6 +174,7 @@ slang_init (void)
     sigemptyset (&act.sa_mask);
     act.sa_flags = SA_RESTART;
     sigaction (SIGINT, &act, &oact);
+    SLtt_Blink_Mode = 0;
 }
 
 void
@@ -169,15 +218,52 @@ slang_shutdown ()
     }
 }
 
+/* HP Terminals have capabilities (pfkey, pfloc, pfx) to program function keys.
+   elm 2.4pl15 invoked with the -K option utilizes these softkeys and the
+   consequence is that function keys don't work in MC sometimes. 
+   Unfortunately I don't now the one and only escape sequence to turn off 
+   softkeys (elm uses three different capabilities to turn on softkeys and two 
+   capabilities to turn them off). 
+   Among other things elm uses the pair we already use in slang_keypad. That's 
+   the reason why I call slang_reset_softkeys from slang_keypad. In lack of
+   something better the softkeys are programmed to their defaults from the
+   termcap/terminfo database.
+   The escape sequence to program the softkeys is taken from elm and it is 
+   hardcoded because neither slang nor ncurses 4.1 know how to 'printf' this 
+   sequence. -- Norbert
+ */   
+   
+void
+slang_reset_softkeys (void)
+{
+    int key;
+    char *send;
+    char *display = "                ";
+    char tmp[100];
+    
+    for ( key = 1; key < 9; key++ ) {
+	sprintf ( tmp, "k%d", key);
+	send = (char *) SLtt_tgetstr (tmp);
+	if (send) {
+            sprintf(tmp, "\033&f%dk%dd%dL%s%s", key,
+                    strlen(display), strlen(send), display, send);
+    	    SLtt_write_string (tmp);
+	}
+    }
+}
+
 /* keypad routines */
 void
 slang_keypad (int set)
 {
     char *keypad_string;
+    extern int reset_hp_softkeys;
     
     keypad_string = (char *) SLtt_tgetstr (set ? "ks" : "ke");
     if (keypad_string)
 	SLtt_write_string (keypad_string);
+    if (set && reset_hp_softkeys)
+	slang_reset_softkeys ();
 }
 
 void
@@ -246,6 +332,7 @@ char *color_terminals [] = {
 int has_colors ()
 {
     char *terminal = getenv ("TERM");
+    char *cts = color_terminal_string, *s;
     int  i;
 
     SLtt_Use_Ansi_Colors = force_colors;
@@ -254,9 +341,34 @@ int has_colors ()
 
     /* We want to allow overwriding */
     if (!disable_colors){
+      if (!*cts)
+      {
+      /* check hard-coded terminals */
 	for (i = 0; color_terminals [i]; i++)
 	    if (strcmp (color_terminals [i], terminal) == 0)
 		SLtt_Use_Ansi_Colors = 1;
+      }
+      else
+      /* check color_terminal_string */
+      {
+        while (*cts)
+        {
+          while (*cts == ' ' || *cts == '\t') cts++;
+
+          s = cts;
+          i = 0;
+
+          while (*cts && *cts != ',')
+          {
+            cts++;
+            i++;
+          }
+          if (i && i == strlen(terminal) && strncmp(s, terminal, i) == 0)
+            SLtt_Use_Ansi_Colors = 1;
+
+          if (*cts == ',') cts++;
+        }
+      }
     }
     
     /* Setup emulated colors */
@@ -355,35 +467,11 @@ load_terminfo_keys ()
 
 int getch ()
 {
-    unsigned int ch;
-
     if (no_slang_delay)
-	if (SLang_input_pending (0) == 0)
+	if (SLang_input_pending2 (0) == 0)
 	    return -1;
 
-    ch = SLang_getkey ();
-
-#ifndef __GO32__
-#define DEC_8BIT_HACK 64
-
-   /* This has the effect of mapping all characters in the range 128-159 to
-    * ESC [ something 
-    */
-   if ( (use_8th_bit_as_meta == 1) && (ch & 0x80) )
-     {
-	unsigned char i;
-	i = (unsigned char) (ch & 0x7F);
-	if (i < ' ')
-	  {
-	     i += DEC_8BIT_HACK;
-	     SLang_ungetkey (i);
-	     ch = 27;
-	  }
-     }
-
-#endif    
-
-   return (ch);
+    return (SLang_getkey2 ());
 }
 
 extern int slow_terminal;
