@@ -857,6 +857,7 @@ void sigchld_handler (int sig)
 	/* }}} */
     }
 
+#ifndef HAVE_X
 #ifndef SCO_FLAVOR
     pid = waitpid (cons_saver_pid, &status, WUNTRACED | WNOHANG);
     
@@ -876,8 +877,8 @@ void sigchld_handler (int sig)
 
 	/* }}} */
     }
-#endif /* SCO_FLAVOR */
-
+#endif /* ! SCO_FLAVOR */
+#endif /* ! HAVE_X */
     /* If we get here, some other child exited; ignore it */
 }
 
