@@ -110,6 +110,25 @@ static tree_entry *forw_ptr (tree_entry *ptr, int *count)
     return ptr;
 }
 
+int
+pathcmp (const char *p1, const char *p2)
+{
+    for ( ;*p1 && *p2 && *p1 == *p2; p1++, p2++)
+        ;
+
+    if (*p1 == '\0' && *p2 == '\0' )
+        return 0;
+    if (*p1 == '\0')
+        return -1;
+    if (*p2 == '\0')
+        return 1;
+    if (*p1 == PATH_SEP)
+        return -1;
+    if (*p2 == PATH_SEP)
+        return 1;
+    return (*p1 - *p2);
+}
+
 /* Searches for specified directory */
 static tree_entry *whereis (WTree *tree, char *name)
 {
@@ -129,8 +148,7 @@ static tree_entry *whereis (WTree *tree, char *name)
 	}
     }
 #endif
-    while (current && flag < 0
-	   && (flag = strcmp (current->name, name)) < 0)
+    while (current && (flag = pathcmp (current->name, name)) < 0)
 	current = current->next;
 
     if (flag == 0)
@@ -170,8 +188,7 @@ tree_entry *tree_add_entry (WTree *tree, char *name)
     }
 #endif
     /* Search for the correct place */
-    while (current && flag < 0
-	   && (flag = strcmp (current->name, name)) < 0){
+    while (current && (flag = pathcmp (current->name, name)) < 0){
 	old = current;
 	current = current->next;
     }
@@ -254,6 +271,7 @@ tree_entry *tree_add_entry (WTree *tree, char *name)
     return new;
 }
 
+#if 0
 /* Append a directory to the list of directories */
 static tree_entry *tree_append_entry (WTree *tree, char *name)
 {
@@ -299,6 +317,7 @@ static tree_entry *tree_append_entry (WTree *tree, char *name)
     /* The entry has now been appended */
     return new;
 }
+#endif
 
 static void remove_entry (WTree *tree, tree_entry *entry)
 {
@@ -359,7 +378,7 @@ void tree_remove_entry (WTree *tree, char *name)
     current = base->next;
     while (current
 	   && strncmp (current->name, base->name, len) == 0
-	   && current->sublevel >= base_sublevel){
+	   && (current->name[len] == '\0' || current->name[len] == PATH_SEP)){
 	old = current;
 	current = current->next;
 	remove_entry (tree, old);
@@ -408,18 +427,13 @@ void start_tree_check (WTree *tree)
     }
 
     /* Mark old subdirectories for delete */
-    /* Here is the bug reported by Michael */
     tree->check_start = current->next;
     len = strlen (tree->check_name);
-    if (tree->check_name [0] == PATH_SEP && tree->check_name [1] == 0)
-	tree->check_sublevel = current->sublevel;
-    else
-	tree->check_sublevel = current->sublevel + 1;
 
     current = tree->check_start;
     while (current
 	   && strncmp (current->name, tree->check_name, len) == 0
-	   && current->sublevel >= tree->check_sublevel){
+	   && (current->name[len] == '\0' || current->name[len] == PATH_SEP || len == 1)){
 	current->mark = 1;
 	current = current->next;
     }
@@ -443,7 +457,7 @@ void do_tree_check (WTree *tree, const char *subname)
 
     /* Search for the subdirectory */
     current = tree->check_start;
-    while (current && (flag = strcmp (current->name, name)) < 0)
+    while (current && (flag = pathcmp (current->name, name)) < 0)
 	current = current->next;
     
     if (flag != 0)
@@ -459,7 +473,7 @@ void do_tree_check (WTree *tree, const char *subname)
 	current = base->next;
 	while (current
 	       && strncmp (current->name, base->name, len) == 0
-	       && current->sublevel >= tree->check_sublevel){
+	       && (current->name[len] == '\0' || current->name[len] == PATH_SEP || len == 1)){
 	    current->mark = 0;
 	    current = current->next;
 	}
@@ -497,7 +511,7 @@ void end_tree_check (WTree *tree)
     current = tree->check_start;
     while (current
 	   && strncmp (current->name, tree->check_name, len) == 0
-	   && current->sublevel >= tree->check_sublevel){
+	   && (current->name[len] == '\0' || current->name[len] == PATH_SEP || len == 1)){
 	old = current;
 	current = current->next;
 	if (old->mark)
@@ -538,7 +552,7 @@ void load_tree (WTree *tree)
                  isalpha(name[0]) && 
                  (name[1] == ':') && 
                  (name[2] == '\\')) {
-        		tree_append_entry (tree, name);
+        		tree_add_entry (tree, name);
         		strcpy (oldname, name);
             } else
 #endif
@@ -552,11 +566,11 @@ void load_tree (WTree *tree)
 		    different = strtok (NULL, "");
 		    if (different){
 			strcpy (oldname + common, different);
-			tree_append_entry (tree, oldname);
+			tree_add_entry (tree, oldname);
 		    }
 		}
 	    } else {
-		tree_append_entry (tree, name);
+		tree_add_entry (tree, name);
 		strcpy (oldname, name);
 	    }
 	}
@@ -712,10 +726,14 @@ void show_tree (WTree *tree)
 	
 	tree->tree_shown [i] = current;
 	if (current->sublevel == topsublevel){
+
 	    /* Top level directory */
 	    if (tree->active && current == tree->selected_ptr)
-		if (!use_colors && !tree->is_panel) set_attr (0, 1);
-		else set_attr (1, 0);
+		if (!use_colors && !tree->is_panel)
+			attrset (MARKED_COLOR);
+		else
+			attrset (SELECTED_COLOR);
+
 	    /* Show full name */
 	    addstr (name_trunc (current->name, tree_cols - 6));
 	} else{
@@ -743,8 +761,10 @@ void show_tree (WTree *tree)
 	    
 	    if (tree->active && current == tree->selected_ptr)
 		/* Selected directory -> change color */
-		if (!use_colors && !tree->is_panel) set_attr (0, 1);
-		else set_attr (1, 0);
+		if (!use_colors && !tree->is_panel)
+		    attrset (MARKED_COLOR);
+		else
+		    attrset (SELECTED_COLOR);
 
 	    /* Show sub-name */
 	    addch (' ');
@@ -1015,7 +1035,7 @@ int tree_rescan_cmd (WTree *tree)
 	return 0;
     if (!mc_get_current_wd (old_dir, MC_MAXPATHLEN))
 	return 0;
-    if (chdir (tree->selected_ptr->name))
+    if (mc_chdir (tree->selected_ptr->name))
 	return 0;
     
     start_tree_check (tree);
@@ -1029,7 +1049,7 @@ int tree_rescan_cmd (WTree *tree)
 	closedir (dirp);
     }
     end_tree_check (tree);
-    chdir (old_dir);
+    mc_chdir (old_dir);
     return 1;
 }
 
@@ -1056,7 +1076,7 @@ void tree_copy (WTree *tree, char *default_dest)
     if (!tree->selected_ptr)
 	return;
     sprintf (cmd_buf, "Copy \"%s\" directory to:",
-	     name_trunc (tree->selected_ptr->name, 60));
+	     name_trunc (tree->selected_ptr->name, 50));
     dest = input_expand_dialog (" Copy ", cmd_buf, default_dest);
     if (!dest || !*dest){
 	return;
@@ -1069,7 +1089,9 @@ void tree_copy (WTree *tree, char *default_dest)
 
 static void tree_help_cmd (void)
 {
-    interactive_display (LIBDIR "mc.hlp", "[Directory Tree]");
+    char *hlpfile = concat_dir_and_file (mc_home, "mc.hlp");
+    interactive_display (hlpfile,  "[Directory Tree]");
+    free (hlpfile);
 }
 
 static int tree_copy_cmd (WTree *tree)
@@ -1086,7 +1108,7 @@ void tree_move (WTree *tree, char *default_dest)
     if (!tree->selected_ptr)
 	return;
     sprintf (cmd_buf, "Move \"%s\" directory to:",
-	     name_trunc (tree->selected_ptr->name, 60));
+	     name_trunc (tree->selected_ptr->name, 50));
     dest = input_expand_dialog (" Move ", cmd_buf, default_dest);
     if (!dest || !*dest){
 	return;
@@ -1139,7 +1161,7 @@ static int tree_rmdir_cmd (WTree *tree)
     if (tree->selected_ptr){
 	if (!mc_get_current_wd (old_dir, MC_MAXPATHLEN))
 	    return 0;
-	if (chdir (PATH_SEP_STR))
+	if (mc_chdir (PATH_SEP_STR))
 	    return 0;
 	if (confirm_delete){
 	    char *cmd_buf;
@@ -1158,7 +1180,7 @@ static int tree_rmdir_cmd (WTree *tree)
 	if (erase_dir (tree->selected_ptr->name) == FILE_CONT)
 	    tree_forget_cmd (tree);
 	destroy_op_win ();
-	chdir (old_dir);
+	mc_chdir (old_dir);
 	return 1;
     } else
 	return 0;
@@ -1279,15 +1301,20 @@ static void start_search (WTree *tree)
     int i;
 
     if (tree->searching){
+    
+	if (tree->selected_ptr == tree->tree_last)
+	    tree_move_to_top(tree);
+	else {
 	/* set navigation mode temporarily to 'Static' because in 
 	 * dynamic navigation mode tree_move_forward will not move
 	 * to a lower sublevel if necessary (sequent searches must
 	 * start with the directory followed the last found directory)
          */	 
-	i = tree_navigation_flag;
-	tree_navigation_flag = 0;
-	tree_move_forward (tree, 1);
-	tree_navigation_flag = i;
+	    i = tree_navigation_flag;
+	    tree_navigation_flag = 0;
+	    tree_move_forward (tree, 1);
+	    tree_navigation_flag = i;
+	}
 	tree_do_search (tree, 0);
     }
     else {
@@ -1315,6 +1342,7 @@ static key_map tree_keymap [] = {
     { XCTRL('p'), move_down    },
     { XCTRL('s'), start_search },
     { ALT('s'),   start_search },
+    { XCTRL('r'), tree_rescan_cmd },
     { 0, 0 }
     };
 
@@ -1363,7 +1391,7 @@ static inline int tree_key (WTree *tree, int key)
 
 static void tree_frame (Dlg_head *h, WTree *tree)
 {
-    set_attr (0, 0);
+    attrset (NORMAL_COLOR);
     widget_erase ((Widget*) tree);
     if (tree->is_panel)
 	draw_double_box (h, tree->widget.y, tree->widget.x, tree->widget.lines,
@@ -1398,8 +1426,13 @@ static int tree_callback (Dlg_head *h, WTree *tree, int msg, int par)
 	    5, "Copy",   (buttonbarfn) tree_copy_cmd, tree);
 	define_label_data (h, (Widget *)tree, 
 	    6, "RenMov", (buttonbarfn) tree_move_cmd, tree);
+#if 0
+	/* FIXME: mkdir is currently defunct */
 	define_label_data (h, (Widget *)tree, 
 	    7, "Mkdir",  (buttonbarfn) tree_mkdir_cmd, tree);
+#else
+        define_label (h, (Widget *) NULL, 7, "", 0);
+#endif
 	define_label_data (h, (Widget *)tree, 
 	    8, "Rmdir",  (buttonbarfn) tree_rmdir_cmd, tree);
 	set_navig_label (h);
@@ -1435,6 +1468,7 @@ WTree *tree_new (int is_panel, int y, int x, int lines, int cols)
     tree->topdiff = tree->widget.lines / 2;
     tree->searching = 0;
     tree->done = 0;
+    tree->active = 0;
     
     /* We do not want to keep the cursor */
     widget_want_cursor (tree->widget, 0);
