@@ -571,7 +571,7 @@ void user_menu_cmd (void)
     menu = concat_dir_and_file (home_dir, MC_MENU);
     if (!exist_file (menu)){
 	free (menu);
-	menu = copy_strings (LIBDIR, "mc.menu", 0);
+	menu = copy_strings (mc_home, "mc.menu", 0);
     }
 
     if ((data = load_file (menu)) == NULL){
