@@ -55,9 +55,14 @@
 #endif
 
 #ifdef __linux__
-#  include <linux/termios.h>	/* This is needed for TIOCLINUX */
+#    if defined(__GLIBC__) && (__GLIBC__ < 2)
+#        include <linux/termios.h>	/* This is needed for TIOCLINUX */
+#    else
+#        include <termios.h>
+#    endif
 #  include <sys/ioctl.h>
 #endif
+
 #include "fs.h"
 #include "mountlist.h"
 
@@ -117,7 +122,7 @@ int is_printable (int c)
     if (eight_bit_clean){
         if (full_eight_bits){
 	    if (xterm_flag)
-	        return xterm_printable [c & 0xff];
+	        return xterm_printable [c];
 	    else
 	        return (c > 31 && c != 127);
 	} else
@@ -276,7 +281,30 @@ char *string_perm (mode_t mode_bits)
     return mode;
 }
 
-char *strip_home(char *dir)
+static char *
+strip_password (char *path)
+{
+    char *at, *inner_colon, *dir;
+    
+    if (dir = strchr (path, PATH_SEP))
+	*dir = '\0';
+    /* search for any possible user */
+    at = strchr (path, '@');
+
+    /* We have a username */
+    if (at) {
+        *at = 0;
+        inner_colon = strchr (path, ':');
+	*at = '@';
+        if (inner_colon)
+            strcpy (inner_colon, at);
+    }
+    if (dir)
+	*dir = PATH_SEP;
+    return (path);
+}
+
+char *strip_home_and_password(char *dir)
 {
     static char newdir [MC_MAXPATHLEN], *p, *q;
 
@@ -287,36 +315,40 @@ char *strip_home(char *dir)
     } 
 #ifdef USE_NETCODE    
     else if (!strncmp (dir, "ftp://", 6)) {
-        if ((p = strchr (dir + 6, PATH_SEP)) != NULL) {
+	strip_password (strcpy (newdir, dir) + 6);
+        if ((p = strchr (newdir + 6, PATH_SEP)) != NULL) {
             *p = 0;
-            if ((q = ftpfs_gethome (dir)) == NULL)
-		return dir;
-            *p = PATH_SEP;
-            if (q != NULL && strcmp (q, PATH_SEP_STR) && !strncmp (p, q, strlen (q) - 1)) {
-                strncpy (newdir, dir, p - dir);
-                strcpy (newdir + (p - dir), "/~");
+	    q = ftpfs_gethome (newdir);
+	    *p = PATH_SEP;
+            if (q != 0 && strcmp (q, PATH_SEP_STR) && !strncmp (p, q, strlen (q) - 1)) {
+                strcpy (p, "/~");
                 strcat (newdir, p + strlen (q) - 1);
-                return newdir;
             }
         }
+        return newdir;
     } else if (!strncmp (dir, "mc:", 3)) {
-        char  *pth = (dir[3] == '/' && dir[4] == '/') ? dir + 5 : dir + 3;
-
+        char  *pth;
+	strcpy (newdir, dir);
+	if (newdir[3] == '/' && newdir[4] == '/') { 
+	    pth = newdir + 5;
+    	    strip_password ( newdir + 5);
+	} else { 
+	    pth = newdir + 3;
+	    strip_password (newdir + 3);
+	}
         if ((p = strchr (pth, PATH_SEP)) != NULL) {
             *p = 0;
-            if ((q = mcfs_gethome (dir)) == NULL)
-		return dir;
+	    q = mcfs_gethome (newdir);
             *p = PATH_SEP;
-            if (q != NULL && strcmp (q, PATH_SEP_STR) && !strncmp (p, q, strlen (q) - 1)) {
-                strncpy (newdir, dir, p - dir);
-                strcpy (newdir + (p - dir), "/~");
-                strcat (newdir, p + strlen (q) - 1);
+            if (q != NULL ) { 
+		if (strcmp (q, PATH_SEP_STR) && !strncmp (p, q, strlen (q) - 1)) {
+                   strcpy (p, "/~");
+                   strcat (newdir, p + strlen (q) - 1);
+		}
                 free (q);
-                return newdir;
-            } else if (q != NULL) {
-                free (q);
-            }
+            }	    
         }
+	return (newdir);
     }
 #endif    
     return dir;
@@ -572,24 +604,31 @@ char *extract_line (char *s, char *top)
 }
 
 /* FIXME: I should write a faster version of this (Aho-Corasick stuff) */
-char *icase_search (char *text, char *data)
+char * _icase_search (char *text, char *data, int *lng)
 {
     char *d = text;
     char *e = data;
-    register char c, c1;
-    
-    for (;*e; e++){
-    	c = *d | 0x20;
-    	c1 = *e | 0x20;
-	if (c == c1 && ((c >= 'a' && c <= 'z' && c1 >= 'a' && c1 <= 'z') ||
-	    *d == *e))
+    int dlng = 0;
+
+    if (lng)
+	*lng = 0;
+    for (;*e; e++) {
+	while (*(e+1) == '\b' && *(e+2)) {
+	    e += 2;
+	    dlng += 2;
+	}
+	if (toupper((unsigned char) *d) == toupper((unsigned char) *e))
 	    d++;
 	else {
 	    e -= d - text;
 	    d = text;
+	    dlng = 0;
 	}
-	if (!*d)
+	if (!*d) {
+	    if (lng)
+		*lng = strlen (text) + dlng;
 	    return e+1;
+	}
     }
     return 0;
 }
