@@ -18,7 +18,7 @@ char *size_trunc (long int size);
 char *size_trunc_sep (long int size);
 int is_exe (mode_t mode);
 char *string_perm (mode_t mode_bits);
-char *strip_home(char *dir);
+char *strip_home_and_password(char *dir);
 char *extension (char *);
 char *split_extension (char *, int pad);
 char *get_full_name (char *dir, char *file);
@@ -67,6 +67,15 @@ char *load_file (char *filename);
 #endif
 
 /* uid/gid managing */
+typedef struct user_in_groups{
+    struct user_in_groups *next;
+    int gid;
+} user_in_groups;
+
+void init_groups (void);
+void delete_groups (void);
+int get_user_rights (struct stat *buf);
+
 void init_uid_gid_cache (void);
 char *get_group (int);
 char *get_owner (int);
@@ -77,7 +86,8 @@ int exist_file (char *name);
 
 /* Returns a copy of *s until a \n is found and is below top */
 char *extract_line (char *s, char *top);
-char *icase_search (char *text, char *data);
+char *_icase_search (char *text, char *data, int *lng);
+#define icase_search(T,D) _icase_search((T), (D), NULL)
 
 /* Matching */
 enum { match_file, match_normal };
@@ -161,12 +171,14 @@ void tell_parent (int msg);
 #if defined(_OS_NT) || defined(__os2__)
 #    define PATH_SEP '\\'
 #    define PATH_SEP_STR "\\"
+#    define PATH_ENV_SEP ';'
 #    define OS_SORT_CASE_SENSITIVE_DEFAULT 0
      char *get_default_shell (void);
      char *get_default_editor (void);
 #else
 #    define PATH_SEP '/'
 #    define PATH_SEP_STR "/"
+#    define PATH_ENV_SEP ':'
 #    define get_default_editor() "vi"
 #    define OS_SORT_CASE_SENSITIVE_DEFAULT 1
 #endif
