@@ -51,7 +51,11 @@
 #   include <sys/timeb.h>
 #endif
 #ifdef __linux__
-#  include <linux/termios.h>	/* This is needed for TIOCLINUX */
+#    if defined(__GLIBC__) && (__GLIBC__ < 2)
+#        include <linux/termios.h>	/* This is needed for TIOCLINUX */
+#    else
+#        include <termios.h>
+#    endif
 #  include <sys/ioctl.h>
 #endif
 #include "util.h"
@@ -66,6 +70,67 @@
 struct sigaction startup_handler;
 extern struct mount_entry *mount_list;
 
+uid_t current_user_uid;
+user_in_groups *current_user_gid;
+
+void init_groups (void)
+{
+    int i;
+    struct passwd *pwd;
+    struct group *grp;
+    user_in_groups *cug, *pug;
+
+    current_user_uid = 0;      /* is this root.wheel */
+    current_user_gid = 0;      /* by default ? :-))  */
+
+    pwd = getpwuid (current_user_uid=getuid ());
+
+    setgrent ();
+    while (grp = getgrent ())
+       for (i = 0; grp->gr_mem[i]; i++)
+           if (!strcmp (pwd->pw_name,grp->gr_mem[i]))
+               {
+               cug = xmalloc (sizeof (user_in_groups), "init_groups");
+               if (!current_user_gid)
+                   current_user_gid = pug = cug;
+
+               cug->gid  = grp->gr_gid;
+               pug->next = cug;
+               cug->next = 0;
+               pug = cug;
+               break;
+               }
+    endgrent ();
+}
+
+/* Return the index of permission triplet */
+int
+get_user_rights (struct stat *buf)
+{
+    user_in_groups *cug;
+
+    if (buf->st_uid == current_user_uid || current_user_uid == 0)
+       return 0;
+
+    for (cug = current_user_gid; cug; cug = cug->next)
+       if (cug->gid == buf->st_gid) return 1;
+
+    return 2;
+}
+
+
+void
+delete_groups (void)
+{
+    user_in_groups *pug, *cug = current_user_gid;
+
+    while (cug){
+       pug = cug->next;
+       free (cug);
+       cug = pug;
+    }
+}
+
 #define UID_CACHE_SIZE 200
 #define GID_CACHE_SIZE 30
 
@@ -490,6 +555,31 @@ char *canonicalize_pathname (char *path)
       	if (!path[start])
 	    break;
 
+#if defined(__QNX__)
+		/*
+		** QNX accesses the directories of nodes on its peer-to-peer
+		** network by prefixing their directories with "//[nid]".
+		** We don't want to collapse two '/'s if they're at the start
+		** of the path, followed by digits, and ending with a '/'.
+		*/
+		if (start == 0 && i == 1)
+		{
+			char *p = path + 2;
+			char *q = strchr(p, PATH_SEP);
+
+			if (q > p)
+			{
+				*q = 0;
+				if (!strcspn(p, "0123456789"))
+				{
+					start = q - path;
+					i = start + 1;
+				}
+				*q = PATH_SEP;
+			}
+		}
+#endif
+
         /* Handle multiple `/'s in a row. */
         while (path[i] == PATH_SEP)
 	    i++;
@@ -654,6 +744,32 @@ void my_statfs (struct my_statfs *myfs_stats, char *path)
 	myfs_stats->nfree = fs_use.fsu_ffree;
 	myfs_stats->nodes = fs_use.fsu_files;
     } else
+#endif
+#if defined(NO_INFOMOUNT) && defined(__QNX__)
+/*
+** This is the "other side" of the hack to read_filesystem_list() in
+** mountlist.c.
+** It's not the most efficient approach, but consumes less memory. It
+** also accomodates QNX's ability to mount filesystems on the fly.
+*/
+	struct mount_entry	*entry;
+    struct fs_usage		fs_use;
+
+	if ((entry = read_filesystem_list(0, 0)) != NULL)
+	{
+		get_fs_usage(entry->me_mountdir, &fs_use);
+
+		myfs_stats->type = entry->me_dev;
+		myfs_stats->typename = entry->me_type;
+		myfs_stats->mpoint = entry->me_mountdir;
+		myfs_stats->device = entry->me_devname;
+
+		myfs_stats->avail = fs_use.fsu_bfree / 2;
+		myfs_stats->total = fs_use.fsu_blocks / 2;
+		myfs_stats->nfree = fs_use.fsu_ffree;
+		myfs_stats->nodes = fs_use.fsu_files;
+	}
+	else
 #endif
     {
 	myfs_stats->type = 0;
