@@ -598,7 +598,7 @@ do_view_init (WView *view, char *_command, char *_file, int start_line)
 	/* wrap_mode = 1; */
 	view->start_col = 0;
     }
-    if (_command && view->viewer_magic_flag)
+    if (_command && (view->viewer_magic_flag || _file[0] == '\0'))
 	error = init_growing_view (view, _command, _file);
     else
 	error = load_view_file (view, _file);
@@ -617,7 +617,7 @@ do_view_init (WView *view, char *_command, char *_file, int start_line)
     	view->command = strdup (_command);
     else
     	view->command = 0;
-    view->search_start = view->start_display = view->first;
+    view->search_start = view->start_display = view->start_save = view->first;
     view->found_len = 0;
     view->start_col = 0;
     view->last_search = 0;            /* Start a new search */
@@ -1630,12 +1630,12 @@ move_left (WView *view)
 static int
 icase_search_p (WView *view, char *text, char *data, int nothing)
 {
-    int p = 0;
+    int p = 0, lng;
     char *q;
 
-    p = (q = icase_search (text, data)) != 0; 
-    if (p){
-	view->found_len = strlen (text);
+    p = (q = _icase_search (text, data, &lng)) != 0; 
+    if (p) {
+	view->found_len = lng;
 	view->search_start = q - data - view->found_len;
     }
     return p;
@@ -1650,9 +1650,12 @@ grow_string_buffer (char *text, int *size)
     /* The grow steps */
     *size += 160;
     new = xmalloc (*size, "grow_string_buffer");
-    strncpy (new, text, old_size);
-    if (text)
+    if (text){
+	strncpy (new, text, old_size);
 	free (text);
+    } else {
+        *new = 0;
+    }
     return new;
 }
 
@@ -1983,7 +1986,8 @@ static int regexp_view_search (WView *view, char *pattern, char *string, int mat
     static regex_t r;
     static char *old_pattern = NULL;
     static int old_type;
-    int startpos, found_len, len;
+    regmatch_t pmatch[1];
+    int i, flags = REG_ICASE;
 
     if (!old_pattern || strcmp (old_pattern, pattern) || old_type != match_type){
 	if (old_pattern){
@@ -1991,20 +1995,24 @@ static int regexp_view_search (WView *view, char *pattern, char *string, int mat
 	    free (old_pattern);
 	    old_pattern = 0;
 	}
-	if (regcomp (&r, pattern, REG_EXTENDED|REG_NOSUB)){
+	for (i = 0; pattern[i] != 0; i++){
+	    if (isupper ((unsigned char) pattern[i])){
+		flags = 0;
+		break;
+	    }
+	}
+	flags |= REG_EXTENDED;
+	if (regcomp (&r, pattern, flags)){
 	    message (1, " Error ", " Invalid regular expression ");
 	    return -1;
 	}
 	old_pattern = strdup (pattern);
 	old_type = match_type;
     }
-    len = strlen (string);
-    startpos = re_search (&r, string, len, 0, len, NULL);
-    if (startpos < 0)
-    	return 0;
-    found_len = re_match (&r, string, len, startpos, NULL);
-    view->found_len = found_len;
-    view->search_start = startpos;
+    if (regexec (&r, string, 1, pmatch, 0) != 0)
+	return 0;
+    view->found_len = pmatch[0].rm_eo - pmatch[0].rm_so;
+    view->search_start = pmatch[0].rm_so;
     return 1;
 }
 
@@ -2039,7 +2047,9 @@ static void do_normal_search (void *xview, char *text)
 /* Real view only */
 static void help_cmd (void)
 {
-    interactive_display (LIBDIR "mc.hlp", "[Internal File Viewer]");
+    char *hlpfile = concat_dir_and_file (mc_home, "mc.hlp");
+    interactive_display (hlpfile, "[Internal File Viewer]");
+    free (hlpfile);
     /*
     view_refresh (0);
     */
@@ -2700,9 +2710,10 @@ view_hook (void *v)
 }
 
 static int
-view_callback (Dlg_head *h, WView *view, int msg, int par)
+view_callback (Dlg_head *h, WView *v, int msg, int par)
 {
-    int v;
+    WView *view = (WView *) v;
+    int i;
     
     switch (msg){
     case WIDGET_INIT:
@@ -2724,13 +2735,13 @@ view_callback (Dlg_head *h, WView *view, int msg, int par)
 	break;
 
     case WIDGET_KEY:
-	v = view_handle_key ((WView *)view, par);
+	i = view_handle_key ((WView *)view, par);
 	if (view->view_quit)
 	    h->running = 0;
 	else {
 	    view_update (view);
 	}
-	return v;
+	return i;
 
     case WIDGET_IDLE:
 	/* This event is generated when the user is using the 'F' flag */
