@@ -51,6 +51,10 @@
 #include "dialog.h"	/* For do_refresh() and my_wputs() */
 #include "complete.h"   /* INPUT_COMPLETE_CD */
 
+#if HAVE_TK
+#   include "tkmain.h"
+#endif
+
 /* }}} */
 
 /* {{{ Common dialog callback */
