@@ -28,6 +28,7 @@
 #include <stdarg.h>
 #include <string.h>
 #include <ctype.h>
+#include "main.h"
 #include "tkmain.h"
 #include "key.h"
 #include "global.h"
@@ -87,7 +88,7 @@ xtoolkit_init (int *argc, char *argv[])
 
     Tcl_SetVar(interp, "tcl_interactive", "0", TCL_GLOBAL_ONLY);
     
-    Tcl_SetVar (interp, "LIBDIR", LIBDIR, TCL_GLOBAL_ONLY);
+    Tcl_SetVar (interp, "LIBDIR", mc_home, TCL_GLOBAL_ONLY);
     Tcl_SetVar (interp, "mc_running", "1", TCL_GLOBAL_ONLY);
     
     Tcl_SetVar(interp, "one_window",
@@ -120,14 +121,20 @@ give an error here
 #endif
     
     Tcl_CreateCommand (interp, "tkmc", tkmc_callback, 0, 0);
-    if (Tcl_EvalFile (interp, LIBDIR "mc.tcl") == TCL_OK)
-	return 0;
-    else {
-	printf ("%s\n", interp->result);
-	exit (1);
+    {
+        char *mc_tcl = concat_dir_and_file (mc_home, "mc.tcl");
+	
+	if (Tcl_EvalFile (interp, mc_tcl) == TCL_OK){
+	    free (mc_tcl);
+	    return 0;
+	} else {
+	    printf ("%s\n", interp->result);
+	    exit (1);
+	}
     }
 }
 
+
 int
 xtoolkit_end (void)
 {
