@@ -392,9 +392,10 @@ char *extfs_analysis (char *name, char **archive, int *fstype, int is_dir)
 	    *p = 0;
 	}
 	archive_name = strdup (name + strlen (extfs_prefixes [*fstype]) + 1);
-	*p = c;
+	if (p)
+	    *p = c;
 	*archive = archive_name;
-	return strdup (p+1);
+	return p ? strdup (p+1) : NULL;
     }
     for (p = name + strlen (name); p > name; p--)
     	if (*p == '/' || (is_dir && !*p))
@@ -477,9 +478,11 @@ static char *extfs_get_path (char *inname, struct extfs_archive **archive, int i
 	} else {
 	    parent = xmalloc (sizeof (struct vfs_stamping), "vfs stamping");
 	    parent->v = v;
+	    parent->next = 0;
 	    parent->id = (*v->getid) (archive_name, &(parent->parent));
 	}
 	vfs_add_noncurrent_stamps (&extfs_vfs_ops, (vfsid) parc, parent);
+	vfs_rm_parents (parent);
     }
  return_success:
     *archive = parc;
@@ -782,9 +785,11 @@ static int extfs_close (void *data)
 	} else {
 	    parent = xmalloc (sizeof (struct vfs_stamping), "vfs stamping");
 	    parent->v = v;
+	    parent->next = 0;
 	    parent->id = (*v->getid) (file->archive->name, &(parent->parent));
 	}
         vfs_add_noncurrent_stamps (&extfs_vfs_ops, (vfsid) (file->archive), parent);
+	vfs_rm_parents (parent);
     }
 	
     free (data);
