@@ -380,7 +380,7 @@ ftpfs_connection_destructor(void *data)
 static int
 changetype (struct ftpfs_connection *bucket, int binary)
 {
-    if (binary != bucket->isbinary) {
+    if (binary || binary != bucket->isbinary) {
         if (command (bucket, WAIT_REPLY, "TYPE %c", binary ? 'I' : 'A') != COMPLETE) {
 	    ftpfserrno = EIO;
             return -1;
@@ -523,16 +523,21 @@ login_server (struct ftpfs_connection *bucket, char *netrcpass)
 	}
 #endif
 	print_vfs_message("ftpfs: sending login name");
-        if ((code = command (bucket, WAIT_REPLY, "USER %s", name)) == CONTINUE) {
+	code = command (bucket, WAIT_REPLY, "USER %s", name);
+
+	switch (code){
+	case CONTINUE:
 	    print_vfs_message("ftpfs: sending user password");
-            if (command (bucket, WAIT_REPLY, "PASS %s", pass) == COMPLETE)
-	        {
-		    print_vfs_message("ftpfs: logged in");
-		    wipe_password (pass);
-		    free (name);
-		    return 1;
-		}
-	} else {
+            if (command (bucket, WAIT_REPLY, "PASS %s", pass) != COMPLETE)
+		break;
+
+	case COMPLETE:
+	    print_vfs_message("ftpfs: logged in");
+	    wipe_password (pass);
+	    free (name);
+	    return 1;
+
+	default:
 	    bucket->failed_on_login = 1;
 	    /* ftpfserrno = E; */
 	    if (bucket->password)
@@ -2116,10 +2121,8 @@ static int ftpfs_close (void *data)
     int result = 0;
 
     if (fp->has_changed) {
-	if (!store_file(fp->fe)) {
-	    ftpfserrno = EIO;
+	if (!store_file(fp->fe))
 	    result = -1;
-	}
 	if (ftpfs_normal_flush)
 	    flush_all_directory(fp->fe->bucket);
     }
