@@ -102,7 +102,11 @@ static int local_stat (char *path, struct stat *buf)
 
 static int local_lstat (char *path, struct stat *buf)
 {
-    return lstat (path, buf);
+#ifndef HAVE_STATLSTAT
+    return lstat (path,buf);
+#else
+    return statlstat (path, buf);
+#endif
 }
 
 static int local_fstat (void *data, struct stat *buf)
