@@ -1,6 +1,10 @@
 /* Virtual File System: Midnight Commander file system.
    
-   Copyright (C) 1995 Miguel de Icaza
+   Copyright (C) 1995, 1996, 1997 The Free Software Foundation
+
+   Written by Miguel de Icaza
+              Andrej Borsenkow
+	      Norbert Warmuth
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -31,6 +35,9 @@
 #include <sys/socket.h>		/* AF_INET */
 #include <netinet/in.h>		/* struct in_addr */
 #include <arpa/inet.h>
+#ifdef SCO_FLAVOR
+#include <sys/timeb.h>	/* alex: for struct timeb definition */
+#endif /* SCO_FLAVOR */
 #include <time.h>
 
 #ifdef USE_TERMNET
@@ -200,7 +207,7 @@ static int mcfs_login_server (int my_socket, char *user, int port,
 static int mcfs_open_tcp_link (char *host, char *user, 
     int *port, char *netrcpass, int *version)
 {
-    int my_socket, result;
+    int my_socket;
     int old_port = *port;
     
     my_socket = open_tcp_link (host, port, version, " MCfs ");
@@ -434,7 +441,6 @@ char *mcfs_gethome (char *servername)
     char *remote_file;
     char *buffer;
     mcfs_connection *mc;
-    int  i;
 
     if (!(remote_file = mcfs_get_path (&mc, servername)))
 	return 0;
@@ -884,7 +890,7 @@ int mcfs_chown (char *path, int owner, int group)
 int mcfs_utime (char *path, struct utimbuf *times)
 {
     mcfs_connection   *mc;
-    int   i, status;
+    int status;
     char *file;
 
     if (!(file = mcfs_get_path (&mc, path)))
