@@ -579,7 +579,7 @@ static int read_header (struct tarfs_archive *archive, int tard)
 			     ? next_long_link
 			     : strdup (header->header.arch_linkname));
 	len = strlen (current_link_name);
-	if (current_link_name [len - 1] == '/')
+	if (len && current_link_name [len - 1] == '/')
 	    current_link_name[len - 1] = 0;
 
 	next_long_link = next_long_name = NULL;
@@ -643,6 +643,24 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	inode->archive = archive;
 	inode->data_offset = data_position;
 	inode->mode = from_oct (8, header->header.mode);
+
+        /* Adjust inode->mode because there are tar-files with
+         * linkflag==LF_SYMLINK and S_ISLNK(mod)==0. I don't 
+         * know about the other modes but I think I cause no new
+         * problem when I adjust them, too. -- Norbert.
+         */
+        if (header->header.linkflag == LF_DIR) {
+            inode->mode |= S_IFDIR;
+        } else if (header->header.linkflag == LF_SYMLINK) {
+            inode->mode |= S_IFLNK;
+        } else if (header->header.linkflag == LF_CHR) {
+            inode->mode |= S_IFCHR;
+        } else if (header->header.linkflag == LF_BLK) {
+            inode->mode |= S_IFBLK;
+        } else if (header->header.linkflag == LF_FIFO) {
+            inode->mode |= S_IFIFO;
+        }
+
 	inode->rdev = 0;
 	if (!strcmp (header->header.magic, TMAGIC)) {
 	    inode->uid = *header->header.uname ? finduid (header->header.uname) :
@@ -676,6 +694,7 @@ static int read_header (struct tarfs_archive *archive, int tard)
 	    inode->mode |= S_IFDIR;
 	    make_dot_doubledot (entry);
 	}
+
 	if (header->header.isextended) {
 	    entry->extended_offset = data_position;
 	    while (get_next_record (archive, tard)->ext_hdr.isextended);
@@ -824,9 +843,11 @@ static char *tarfs_get_path (char *inname, struct tarfs_archive **archive, int i
     } else {
 	parent = xmalloc (sizeof (struct vfs_stamping), "vfs stamping");
 	parent->v = v;
+	parent->next = 0;
 	parent->id = (*v->getid) (archive_name, &(parent->parent));
     }
     vfs_add_noncurrent_stamps (&tarfs_vfs_ops, (vfsid) parc, parent);
+    vfs_rm_parents (parent);
 return_success:
     *archive = parc;
     free (archive_name);
@@ -1085,9 +1106,11 @@ static int tar_close (void *data)
 	} else {
 	    parent = xmalloc (sizeof (struct vfs_stamping), "vfs stamping");
 	    parent->v = v;
+	    parent->next = 0;
 	    parent->id = (*v->getid) (file->archive->name, &(parent->parent));
 	}
         vfs_add_noncurrent_stamps (&tarfs_vfs_ops, (vfsid) (file->archive), parent);
+	vfs_rm_parents (parent);
     }
     (file->entry->inode->is_open)--;
 	
