@@ -241,7 +241,7 @@ void rpc_add_get_callback (int sock, void (*cback)(int))
     sock_callbacks = new;
 }
 
-#if defined(IS_AIX) || defined(linux) || defined(SCO_FLAVOR)
+#if defined(IS_AIX) || defined(linux) || defined(SCO_FLAVOR) || defined(__QNX__)
 static void sig_pipe (int unused)
 #else
 static void sig_pipe (void)
