@@ -4,10 +4,11 @@
    ext2fs library to recover files from ext2fs file systems.
 
    Parts of this program were taken from the lsdel.c and dump.c files
-   written by Ted Ts'o (tytso@lurch.mit.edu) for the ext2fs package.
+   written by Ted Ts'o (tytso@mit.edu) for the ext2fs package.
    
-   Copyright (C) 1995 the Free Software Foundation
+   Copyright (C) 1995, 1997 the Free Software Foundation
    Written by: 1995 Miguel de Icaza.
+               1997 Norbert Warmuth.
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,8 +44,12 @@
 #include <stdio.h>
 #include <malloc.h>
 #include <fcntl.h>
+#include <stdlib.h>
 #include <linux/ext2_fs.h>
 #include <ext2fs/ext2fs.h>
+#include <ctype.h>
+
+void print_vfs_message(char *, ...);
 
 struct deleted_info {
     ino_t   ino;
@@ -65,7 +70,7 @@ struct lsdel_struct {
 };
 
 /* We only allow one opened ext2fs */
-static char *ext2_fname = "";
+static char *ext2_fname;
 static ext2_filsys fs = NULL;
 static struct lsdel_struct     lsd;
 static struct deleted_info     *delarray;
@@ -73,6 +78,7 @@ static int num_delarray, max_delarray;
 static char *block_buf;
 static char *undelfserr = " undelfs: error ";
 static int readdir_ptr;
+static int undelfs_usage;
 
 /* To generate the . and .. entries use -2 */
 #define READDIR_PTR_INIT 0
@@ -80,7 +86,18 @@ static int readdir_ptr;
 static void
 undelfs_shutdown ()
 {
-    /* should shutdown the ext2 toolkit */
+    if (fs)
+	ext2fs_close (fs);
+    fs = 0;
+    if (ext2_fname)
+	free (ext2_fname);
+    ext2_fname = 0;
+    if (delarray)
+        free (delarray);
+    delarray = 0;
+    if (block_buf)
+	free (block_buf);
+    block_buf = 0;
 }
 
 static void
@@ -165,7 +182,7 @@ undelfs_loaddel ()
     }
     if ((retval = ext2fs_get_next_inode(scan, &ino, &inode))){
 	message_1s1d (1, undelfserr, " while starting inode scan %d ", retval);
-	goto free_block_buf;
+	goto error_out;
     }
 
     count = 0;
@@ -196,7 +213,7 @@ undelfs_loaddel ()
 				   max_delarray * sizeof(struct deleted_info));
 		if (!delarray) {
 		    message_1s (1, undelfserr, " no more memory while reallocating array ");
-		    goto free_block_buf;
+		    goto error_out;
 		}
 	    }
 	    
@@ -219,9 +236,11 @@ undelfs_loaddel ()
 	}
     }
     readdir_ptr = READDIR_PTR_INIT;
+    ext2fs_close_inode_scan (scan);
     return 1;
     
 error_out:    
+    ext2fs_close_inode_scan (scan);
 free_block_buf:
     free (block_buf);
     block_buf = 0;
@@ -236,7 +255,7 @@ void com_err (const char *str, long err_code, const char *s2, ...)
     char *cptr;
 
     cptr = xmalloc (strlen (s2) + strlen (str) + 60, "com_err");
-    sprintf (cptr, " %s (%s: %d) ", s2, str, err_code);
+    sprintf (cptr, " %s (%s: %ld) ", s2, str, err_code);
     message_1s (1, " Ext2lib error ", cptr);
     free (cptr);
 }
@@ -253,7 +272,7 @@ undelfs_opendir (char *dirname)
     /* We don't use the file name */
     free (f);
     
-    if (strcmp (ext2_fname, file)){
+    if (!ext2_fname || strcmp (ext2_fname, file)){
 	undelfs_shutdown ();
 	ext2_fname = file;
     } else {
@@ -319,8 +338,8 @@ undelfs_readdir (void *vfs_info)
     if (readdir_ptr < 0)
 	sprintf (dirent_dest, "%s", readdir_ptr == -2 ? "." : "..");
     else
-	sprintf (dirent_dest, "%ld:%ld",
-		 delarray [readdir_ptr].ino,
+	sprintf (dirent_dest, "%ld:%d",
+		 (long)delarray [readdir_ptr].ino,
 		 delarray [readdir_ptr].num_blocks);
     readdir_ptr++;
     
@@ -343,7 +362,7 @@ typedef struct {
     int  pos;			/* file position */
     int  current;		/* used to determine current position in itereate */
     int  finished;
-    long inode;
+    ino_t inode;
     int  bytes_read;
     long size;
     
@@ -357,7 +376,7 @@ static void *
 undelfs_open (char *fname, int flags, int mode)
 {
     char *file, *f;
-    int  inode, i;
+    ino_t  inode, i;
     undelfs_file *p;
     
     /* Only allow reads on this file system */
@@ -365,7 +384,7 @@ undelfs_open (char *fname, int flags, int mode)
     if (!file)
 	return 0;
     
-    if (strcmp (ext2_fname, file)){
+    if (!ext2_fname || strcmp (ext2_fname, file)){
 	message_1s (1, undelfserr, " You have to chdir to extract files first ");
 	free (file);
 	free (f);
@@ -399,19 +418,25 @@ undelfs_open (char *fname, int flags, int mode)
 	p->pos = 0;
 	p->size = delarray [i].size;
     }
+    free (file);
+    free (f);
+    undelfs_usage++;
     return p;
 }
 
 static 	int
 undelfs_close (void *vfs_info)
 {
+    undelfs_file *p = vfs_info;
+    free (p->buf);
+    free (p);
+    undelfs_usage--;
     return 0;
 }
 
 static int
 dump_read(ext2_filsys fs, blk_t *blocknr, int blockcnt, void *private)
 {
-    ssize_t nbytes;
     int     copy_count;
     undelfs_file *p = (undelfs_file *) private;
     
@@ -502,7 +527,7 @@ undelfs_write (void *vfs_info, char *buf, int count)
 static long
 undelfs_getindex (char *path)
 {
-    long inode = atol (path);
+    ino_t inode = atol (path);
     int i;
 
     for (i = 0; i < num_delarray; i++){
@@ -537,14 +562,28 @@ undelfs_lstat(char *path, struct stat *buf)
     undelfs_get_path (path, &file, &f);
     if (!file)
 	return 0;
-    
-    if (strcmp (ext2_fname, file)){
+
+    /* When called from save_cwd_stats we get an incorrect file and f here:
+       e.g. incorrect                         correct
+            path = "undel:/dev/sda1"          path="undel:/dev/sda1/401:1"
+            file = "/dev"                     file="/dev/sda1"
+	    f    = "sda1"                     f   ="401:1"
+       If the first char in f is no digit -> return error */
+    if (!isdigit (*f)) {
+	free (file);
+	free (f);
+	return -1;
+    }
+	
+    if (!ext2_fname || strcmp (ext2_fname, file)){
 	message_1s (1, undelfserr, " You have to chdir to extract files first ");
 	free (file);
 	free (f);
 	return 0;
     }
     inode_index = undelfs_getindex (f);
+    free (file);
+    free (f);
 
     if (inode_index == -1)
 	return -1;
@@ -624,6 +663,8 @@ undelfs_chdir(char *path)
     /* our vfs, but that is left as an excercise for the reader */
     if ((fd = open (file, O_RDONLY)) == -1){
 	message_2s (1, undelfserr, " Could not open file: %s ", file);
+	free (f);
+	free (file);
 	return -1;
     }
     close (fd);
@@ -653,20 +694,29 @@ undelfs_mknod(char *path, int mode, int dev)
 static vfsid
 undelfs_getid(char *path, struct vfs_stamping **parent)
 {
+    char *ext2_fname, *file;
+
     /* We run only on the local fs */
     *parent = NULL;
-    return (vfsid) -1;
+    undelfs_get_path (path, &ext2_fname, &file);
+
+    if (!ext2_fname)
+        return (vfsid) -1;
+    free (ext2_fname);
+    free (file);
+    return (vfsid)0;
 }
 
 static int
 undelfs_nothingisopen(vfsid id)
 {
-    return 0;
+    return !undelfs_usage;
 }
 
 static void
 undelfs_free(vfsid id)
 {
+    undelfs_shutdown ();
 }
 
 static char *
