@@ -178,9 +178,9 @@ int vfs_timeouts ()
 void vfs_addstamp (vfs *v, vfsid id, struct vfs_stamping *parent)
 {
     if (v != &local_vfs_ops && id != (vfsid)-1) {
-        struct vfs_stamping *stamp;
+        struct vfs_stamping *stamp, *st1;
         
-        for (stamp = stamps; stamp != NULL; stamp = stamp->next)
+        for (stamp = stamps; stamp != NULL; st1 = stamp, stamp = stamp->next)
             if (stamp->v == v && stamp->id == id) {
 		gettimeofday(&(stamp->time), NULL);
                 return;
@@ -188,10 +188,24 @@ void vfs_addstamp (vfs *v, vfsid id, struct vfs_stamping *parent)
         stamp = xmalloc (sizeof (struct vfs_stamping), "vfs stamping");
         stamp->v = v;
         stamp->id = id;
-        stamp->parent = parent;
+	if (parent) {
+	    struct vfs_stamping *st = stamp;
+	    for ( ; parent; ) {
+		st->parent = xmalloc (sizeof (struct vfs_stamping), "vfs stamping");
+		*st->parent = *parent;
+		parent = parent->parent;
+		st = st->parent;
+	    }
+	    st->parent = 0;
+	}
+	else
+    	    stamp->parent = 0;
         gettimeofday (&(stamp->time), NULL);
-        stamp->next = stamps;
-        stamps = stamp;
+        stamp->next = 0;
+	if (stamps)
+	    st1->next = stamp;
+	else
+    	    stamps = stamp;
     }
 }
 
@@ -208,18 +222,26 @@ void vfs_stamp (vfs *v, vfsid id)
         }
 }
 
+void vfs_rm_parents (struct vfs_stamping *stamp)
+{
+    struct vfs_stamping *st2, *st3;
+    if (stamp) {
+	for (st2 = stamp, st3 = st2->parent; st3 != NULL; st2 = st3, st3 = st3->parent)
+	    free (st2);
+	free (st2);
+    }
+}
+
 void vfs_rmstamp (vfs *v, vfsid id, int removeparents)
 {
-    struct vfs_stamping *stamp, *st1, *st2, *st3;
+    struct vfs_stamping *stamp, *st1;
     
     for (stamp = stamps, st1 = NULL; stamp != NULL; st1 = stamp, stamp = stamp->next)
         if (stamp->v == v && stamp->id == id) {
             if (stamp->parent != NULL) {
                 if (removeparents)
                     vfs_rmstamp (stamp->parent->v, stamp->parent->id, 1);
-                for (st2 = stamp->parent, st3 = st2->next; st3 != NULL; st2 = st3, st3 = st3->next)
-                    free (st2);
-                free (st2);
+		vfs_rm_parents (stamp->parent);
             }
             if (st1 == NULL) {
                 stamps = stamp->next;
@@ -315,7 +337,7 @@ int mc_close (int handle)
     vfs *vfs;
     int result;
 
-    if (handle == -1)
+    if (handle == -1 || !vfs_info (handle))
 	return -1;
     
     vfs = vfs_op (handle);
@@ -324,13 +346,15 @@ int mc_close (int handle)
 
     result = (*vfs->close)(vfs_info (handle));
     vfs_free_bucket (handle);
+    if (result == -1)
+	errno = (*vfs->ferrno)();
     
     return result;
 }
 
 DIR *mc_opendir (char *dirname)
 {
-    int  handle;
+    int  handle, *handlep;
     void *info;
     vfs  *vfs;
     char *p = NULL;
@@ -361,7 +385,10 @@ DIR *mc_opendir (char *dirname)
     free (dirname);
     if (p)
         free (p);
-    return (DIR *) handle;
+    
+    handlep = (int *) xmalloc (sizeof (int), "opendir handle");
+    *handlep = handle;
+    return (DIR *) handlep;
 }
 
 /* This should strip the non needed part of a path name */
@@ -380,19 +407,20 @@ struct dirent *mc_readdir(DIR *dirp)
 #endif
 	return NULL;
     }
-    handle = (int) dirp;
+    handle = *(int *) dirp;
     vfs = vfs_op (handle);
     return (*vfs->readdir)(vfs_info (handle));
 }
 
 int mc_closedir (DIR *dirp)
 {
-    int handle = (int) dirp;
+    int handle = *(int *) dirp;
     vfs *vfs = vfs_op (handle);
     int result;
 
     result = (*vfs->closedir)(vfs_info (handle));
     vfs_free_bucket (handle);
+    free (dirp);
     return result; 
 }
 
@@ -891,7 +919,7 @@ char *vfs_canon (char *path)
     return 0;
 }
 
-static vfsid vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par)
+vfsid vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par)
 {
     vfsid nvfsid;
     int freeit = 0;
@@ -906,79 +934,117 @@ static vfsid vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par)
     return nvfsid;
 }
 
-void vfs_add_noncurrent_stamps (vfs *oldvfs, vfsid oldvfsid, struct vfs_stamping *parent)
+static int is_parent (vfs * nvfs, vfsid nvfsid, struct vfs_stamping *parent)
 {
-    vfs *nvfs, *n2vfs;
-    vfsid nvfsid, n2vfsid;
-    struct vfs_stamping *par;
+    struct vfs_stamping *stamp;
+    for (stamp = parent; stamp; stamp = stamp->parent)
+	if (stamp->v == nvfs && stamp->id == nvfsid)
+	    break;
+    return (stamp ? 1 : 0);
+}
 
-    	/* FIXME: As soon as we convert to multiple panels, this stuff
-    	   has to change. It works like this: We do not time out the
-    	   vfs's which are current in any panel and on the other
-    	   side we add the old directory with all its parents which
-    	   are not in any panel (if we find such one, we stop adding
-    	   parents to the time-outing structure. */
+void vfs_add_noncurrent_stamps (vfs * oldvfs, vfsid oldvfsid, struct vfs_stamping *parent)
+{
+    vfs *nvfs, *n2vfs, *n3vfs;
+    vfsid nvfsid, n2vfsid, n3vfsid;
+    struct vfs_stamping *par, *stamp;
+    int f;
 
+    /* FIXME: As soon as we convert to multiple panels, this stuff
+       has to change. It works like this: We do not time out the
+       vfs's which are current in any panel and on the other
+       side we add the old directory with all its parents which
+       are not in any panel (if we find such one, we stop adding
+       parents to the time-outing structure. */
+
+    /* There are three directories we have to take care of: current_dir,
+       cpanel->cwd and opanel->cwd. Athough most of the time either
+       current_dir and cpanel->cwd or current_dir and opanel->cwd are the
+       same, it's possible that all three are different -- Norbert */
+       
     if (!cpanel)
 	return;
-    
+
     nvfs = vfs_type (current_dir);
     nvfsid = vfs_ncs_getid (nvfs, current_dir, &par);
     vfs_rmstamp (nvfs, nvfsid, 1);
-    if (nvfs != oldvfs || nvfsid != oldvfsid) {
-        struct vfs_stamping *stamp;
 
-	if (get_other_type () == view_listing){
-	    n2vfs = vfs_type (opanel->cwd);
-    	    n2vfsid = vfs_ncs_getid (n2vfs, opanel->cwd, &par);
-	} else {
-	    n2vfs = (vfs *) -1;
-	    n2vfsid = (vfs *) -1;
+    f = is_parent (oldvfs, oldvfsid, par);
+    vfs_rm_parents (par);
+    if ((nvfs == oldvfs && nvfs == oldvfsid) || oldvfsid == (vfsid *)-1 || f) {
+	return;
+    }
+
+    if (get_current_type () == view_listing) {
+	n2vfs = vfs_type (cpanel->cwd);
+	n2vfsid = vfs_ncs_getid (n2vfs, cpanel->cwd, &par);
+        f = is_parent (oldvfs, oldvfsid, par);
+	vfs_rm_parents (par);
+	if ((n2vfs == oldvfs && n2vfsid == oldvfsid) || f) 
+	    return;
+    } else {
+	n2vfs = (vfs *) -1;
+	n2vfsid = (vfs *) -1;
+    }
+    
+    if (get_other_type () == view_listing) {
+	n3vfs = vfs_type (opanel->cwd);
+	n3vfsid = vfs_ncs_getid (n3vfs, opanel->cwd, &par);
+        f = is_parent (oldvfs, oldvfsid, par);
+	vfs_rm_parents (par);
+	if ((n3vfs == oldvfs && n3vfsid == oldvfsid) || f)
+	    return;
+    } else {
+	n3vfs = (vfs *)-1;
+	n3vfsid = (vfs *)-1;
+    }
+    
+    if ((*oldvfs->nothingisopen) (oldvfsid)) {
+	if (oldvfs == &extfs_vfs_ops && ((struct extfs_archive *) oldvfsid)->name == 0) {
+	    /* Free the resources immediatly when we leave a mtools fs
+	       ('cd a:') instead of waiting for the vfs-timeout */
+	    (oldvfs->free) (oldvfsid);
+	} else
+	    vfs_addstamp (oldvfs, oldvfsid, parent);
+	for (stamp = parent; stamp != NULL; stamp = stamp->parent) {
+	    if ((stamp->v == nvfs && stamp->id == nvfsid) ||
+		(stamp->v == n2vfs && stamp->id == n2vfsid) ||
+		(stamp->v == n3vfs && stamp->id == n3vfsid) ||
+		stamp->id == (vfsid) - 1 ||
+		!(*stamp->v->nothingisopen) (stamp->id))
+		break;
+	    if (stamp->v == &extfs_vfs_ops && ((struct extfs_archive *) stamp->id)->name == 0) {
+		(stamp->v->free) (stamp->id);
+		vfs_rmstamp (stamp->v, stamp->id, 0);
+	    } else
+		vfs_addstamp (stamp->v, stamp->id, stamp->parent);
 	}
-	
-        if ((n2vfs != oldvfs || n2vfsid != oldvfsid) &&
-            oldvfsid != (vfsid)-1 && 
-            (*oldvfs->nothingisopen) (oldvfsid)) {
-	    if (oldvfs == &extfs_vfs_ops && ((struct extfs_archive *)oldvfsid)->name == 0) {
-		/* Free the resources immediatly when we leave a mtools fs
-		   ('cd a:') instead of waiting for the vfs-timeout */ 
-		(oldvfs->free)(oldvfsid);
-	    }
-	    else
-        	vfs_addstamp (oldvfs, oldvfsid, parent);
-            for (stamp = parent; stamp != NULL; stamp = stamp->parent) {
-                if ((stamp->v == nvfs && stamp->id == nvfsid) ||
-                    (stamp->v == n2vfs && stamp->id == n2vfsid) ||
-                    stamp->id == (vfsid)-1 ||
-                    !(*stamp->v->nothingisopen) (stamp->id))
-                    break;
-	    if (oldvfs == &extfs_vfs_ops && ((struct extfs_archive *)oldvfsid)->name == 0)
-		(oldvfs->free)(oldvfsid);
-	    else
-                vfs_addstamp (stamp->v, stamp->id, stamp->parent);
-            }
-        }
     }
 }
 
+
 static void vfs_stamp_path (char *path)
 {
     vfs *vfs;
     vfsid id;
-    struct vfs_stamping *par;
+    struct vfs_stamping *par, *stamp;
     
     vfs = vfs_type (path);
     id  = vfs_ncs_getid (vfs, path, &par);
-    vfs_addstamp (vfs, id, par ? par->parent : par);
+    vfs_addstamp (vfs, id, par);
+    
+    for (stamp = par; stamp != NULL; stamp = stamp->parent) 
+	vfs_addstamp (stamp->v, stamp->id, stamp->parent);
+    vfs_rm_parents (par);
 }
 
 void vfs_add_current_stamps (void)
 {
-    vfs_stamp_path (cpanel->cwd);
-    if (get_other_type () == view_listing){
-	WPanel *p = (WPanel *) get_panel_widget (get_other_index ());
-	vfs_stamp_path (p->cwd);
-    }
+    vfs_stamp_path (current_dir);
+    if (get_current_type () == view_listing)
+        vfs_stamp_path (cpanel->cwd);
+    if (get_other_type () == view_listing)
+	vfs_stamp_path (opanel->cwd);
 }
 
 /* This function is really broken */
@@ -1012,9 +1078,6 @@ int mc_chdir (char *path)
 	last_current_dir = current_dir;
     }	                               
 
-    oldvfs = vfs_type (current_dir);
-    oldvfsid = vfs_ncs_getid (oldvfs, current_dir, &parent);
-    
     a = current_dir; /* Save a copy for case of failure */
     if (last_current_dir != current_dir)
 	free (last_current_dir);
@@ -1028,9 +1091,12 @@ int mc_chdir (char *path)
     	current_vfs = vfs_type (a);
     	current_dir = a;
     } else {
-    	free (a);
     	last_current_dir = current_dir;
+        oldvfs = vfs_type (a);
+	oldvfsid = vfs_ncs_getid (oldvfs, a, &parent);
+	free (a);
         vfs_add_noncurrent_stamps (oldvfs, oldvfsid, parent);
+	vfs_rm_parents (parent);
     }
     if (p) {
         free (p);
@@ -1337,8 +1403,10 @@ void vfs_free_resources (char *path)
     vid = vfs_ncs_getid (vfs, path, &parent);
     if (vid != (vfsid) -1)
 	(*vfs->free)(vid);
+    vfs_rm_parents (parent);
 }
 
+#if 0
 /* Shutdown a vfs given a path name */
 void vfs_shut_path (char *p)
 {
@@ -1348,17 +1416,24 @@ void vfs_shut_path (char *p)
     the_vfs = vfs_type (p);
     vfs_ncs_getid (the_vfs, p, &par);
     (*par->v->free)(par->id);
+    vfs_rm_parents (par);
 }
+#endif
 
 void vfs_shut (void)
 {
-    struct vfs_stamping *stamp;
+    struct vfs_stamping *stamp, *st;
 
-    for (stamp = stamps; stamp != NULL; stamp = stamp->next) {
+    for ( stamp = stamps, stamps = 0; stamp != NULL; ) {
 	(*stamp->v->free)(stamp->id);
+	st = stamp->next;
 	free (stamp);
+	stamp = st;
     }
 
+    if (stamps)
+	vfs_rmstamp (stamps->v, stamps->id, 1);
+    
     if (last_current_dir && last_current_dir != current_dir)
 	free (last_current_dir);
     if (current_dir)
