@@ -89,6 +89,8 @@
 
     extern int vfs_type_absolute;
     vfs *vfs_type (char *path);
+    vfsid vfs_ncs_getid (vfs *nvfs, char *dir, struct vfs_stamping **par);
+    void vfs_rm_parents (struct vfs_stamping *stamp);
     char *vfs_path (char *path);
     char *vfs_canon (char *path);
     char *mc_get_current_wd (char *buffer, int bufsize);
@@ -183,6 +185,8 @@
 #   define vfs_current_is_local() 1
 #   define vfs_file_is_local(x) 1
 #   define vfs_file_is_ftp(x) 0
+#   define vfs_current_is_tarfs() 0
+#   define vfs_current_is_extfs() 0
 #   define vfs_path(x) x
 #   define mc_close close
 #   define mc_read read
@@ -244,6 +248,7 @@
 #   undef mc_rename
 #   undef mc_mkdir
 #   undef mc_ctl
+#   undef mc_unlink
 #   define mc_ctl(a,b,c) 0
 #endif
 
