@@ -453,8 +453,8 @@ static int xv_get_text_len (WPanel *panel, char *text)
 
 static void xv_insert_panel_item (Canvas canvas, WPanel *panel, int idx)
 {
-    int top = panel->fmt_count;
-    int just_mode, i, len;
+    format_e *format;
+    int i, len;
     char *txt;
     file_entry *fe = & panel->dir.list [idx];
     int marked = fe->f.marked;
@@ -462,22 +462,22 @@ static void xv_insert_panel_item (Canvas canvas, WPanel *panel, int idx)
     static char buffer [1024];
     char *p = buffer;
     
-    for (i = 0; i < top; i++) {
-        just_mode = panel->format [i].just_mode;
-        if (panel->format [i].string_fn) {
-            txt = (*panel->format [i].string_fn) (fe, 0);
+    for (format = panel->format; format; format = format->next) {
+        if (format->string_fn) {
+            txt = (*format->string_fn) (fe, 0);
         } else
             txt = " ";
-        if (panel->format [i].string_fn == string_file_type) {
-	    panel->format [i].field_len = 18;
+        if (format->string_fn == string_file_type) {
+	    format->field_len = 18;
 	    p [0] = 3 + ((*txt == '/') ? 1 : 0);
 	    p [1] = 0;
 	    p += 2;
         } else {
 	    len = xv_get_text_len (panel, txt) + 2;
-	    if (panel->format [i].field_len < len)
-	        panel->format [i].field_len = len;
-	    p [0] = just_mode + 1;
+	    if (format->field_len < len)
+	        format->field_len = len;
+	    p [0] = format->just_mode + 1;
+
 	    strcpy (p + 1, txt);
 	    p = index (p + 1, 0) + 1;
 	}                
@@ -513,16 +513,17 @@ void x_fill_panel (WPanel *panel)
     Canvas canvas = (Canvas) panel->widget.wdata;
     Scrollbar scrollbar;
     Rect rect;
+    format_e *format;
 
-    for (i = 0; i < panel->fmt_count; i++)
-        panel->format [i].field_len = 2;
+    for (format = panel->format; format; format = format->next)
+        format->field_len = 2;
         
     for (i = 0; i < panel->count; i++)
         xv_insert_panel_item (canvas, panel, i);
         
-    for (i = 0, panel->total_width = 2 * PANELITEMBORDER; 
-         i < panel->fmt_count; i++)
-        panel->total_width += panel->format [i].field_len;
+    for (format = panel->format, panel->total_width = 2 * PANELITEMBORDER; 
+         format; format = format->next)
+        panel->total_width += format->field_len;
         
     w = panel->total_width + 2 * PANELBORDER;
     h = panel->item_height * panel->count + 2 * PANELBORDER;
@@ -573,9 +574,11 @@ static void xv_panel_repaint_item (WPanel *panel, int idx, Xv_Window paint_windo
     char *cache = fe->cache;
     XPoint xp [3];
     char *p;
-    int i, w, x, h, y;
+    int w, x, h, y;
     Server_image image, mask;
+    format_e *format;
     
+        
     if (cache == NULL)
         return;
     if (fe->f.marked)
@@ -621,7 +624,7 @@ static void xv_panel_repaint_item (WPanel *panel, int idx, Xv_Window paint_windo
     XSetBackground (dpy, gc, bg);
     rect.r_left += PANELITEMBORDER;
     
-    for (i = 0, p = fe->cache; *p; p = strchr (p, 0) + 1, i++) {
+    for (format=panel->format , p = fe->cache; *p; p = strchr (p, 0) + 1, format=format->next) {
         if (*p <= 2) { /* i.e. either left or right justified text */
             w = xv_get_text_len (panel, p + 1);
             h = panel->ascent + panel->descent;
@@ -629,7 +632,7 @@ static void xv_panel_repaint_item (WPanel *panel, int idx, Xv_Window paint_windo
             if (*p == J_LEFT + 1)
                 x = rect.r_left + 1;
             else
-            	x = rect.r_left + panel->format [i].field_len - w - 1;
+            	x = rect.r_left + format->field_len - w - 1;
             XDrawString (dpy, xid, gc, x, y, p + 1, strlen (p + 1));
         } else {
             switch (*p) {
@@ -647,7 +650,7 @@ static void xv_panel_repaint_item (WPanel *panel, int idx, Xv_Window paint_windo
                        x, y);
             XSetClipMask (dpy, gc, None);
         }
-        rect.r_left += panel->format [i].field_len;
+        rect.r_left += format->field_len;
     }
 }
 
